@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2010, 2019, Oracle and/or its affiliates. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -1411,6 +1411,17 @@ public abstract class NativeMediaPlayer implements MediaPlayer, MarkerStateListe
         }
     }
 
+    @Override
+    public boolean isErrorEventCached() {
+        synchronized (cachedErrorEvents) {
+            if (cachedErrorEvents.isEmpty()) {
+                return false;
+            } else {
+                return true;
+            }
+        }
+    }
+
     //**************************************************************************
     //***** Non-JNI methods called by the native layer. These methods are called
     //***** from the native layer via the invocation API. Their purpose is to
