@@ -277,16 +277,20 @@ gst_video_encoder_get_type (void)
       0,
       (GInstanceInitFunc) gst_video_encoder_init,
     };
+#ifndef GSTREAMER_LITE
     const GInterfaceInfo preset_interface_info = {
       NULL,                     /* interface_init */
       NULL,                     /* interface_finalize */
       NULL                      /* interface_data */
     };
+#endif // GSTREAMER_LITE
 
     _type = g_type_register_static (GST_TYPE_ELEMENT,
         "GstVideoEncoder", &info, G_TYPE_FLAG_ABSTRACT);
+#ifndef GSTREAMER_LITE
     g_type_add_interface_static (_type, GST_TYPE_PRESET,
         &preset_interface_info);
+#endif // GSTREAMER_LITE
     g_once_init_leave (&type, _type);
   }
   return type;
