@@ -143,7 +143,7 @@ static const int AVCODEC_LIBAV_EXPLICIT_VERSIONS[] = { 54, 56 };
 // For ffmpeg (libavcodec-ffmpeg.so)
 static const int AVCODEC_FFMPEG_EXPLICIT_VERSIONS[] = { 56 };
 // For libav or ffmpeg (libavcodec.so)
-static const int AVCODEC_EXPLICIT_VERSIONS[] = { 57 };
+static const int AVCODEC_EXPLICIT_VERSIONS[] = { 57, 58 };
 
 /*
  * Callback passed to dl_iterate_phdr(): finds the path of
@@ -1899,6 +1899,7 @@ scan_and_update_registry (GstRegistry * default_registry,
 #endif // other platforms
 #endif // GSTREAMER_LITE
 
+#ifndef GSTREAMER_LITE
   init_scan_context (&context, default_registry);
 
   /* It sounds tempting to just compare the mtime of directories with the mtime
@@ -1992,8 +1993,11 @@ scan_and_update_registry (GstRegistry * default_registry,
     g_strfreev (list);
   }
 
+
   clear_scan_context (&context);
+
   changed |= context.changed;
+#endif // GSTREAMER_LITE
 
   /* Remove cached plugins so stale info is cleared. */
   changed |= gst_registry_remove_cache_plugins (default_registry);
