@@ -992,7 +992,8 @@ public final class WebPage {
                 log.fine("getClientSelectedText() request for a disposed web page.");
                 return "";
             }
-            return twkGetSelectedText(getPage());
+            final String selectedText = twkGetSelectedText(getPage());
+            return selectedText != null ? selectedText : "";
 
         } finally {
             unlockPage();
