@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2018 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,6 +31,11 @@
 #include "ProbeContext.h"
 #include <wtf/InlineASM.h>
 
+#if OS(LINUX)
+#include <asm/hwcap.h>
+#include <sys/auxv.h>
+#endif
+
 namespace JSC {
 
 #if ENABLE(MASM_PROBE)
@@ -39,11 +44,16 @@ extern "C" void ctiMasmProbeTrampoline();
 
 using namespace ARM64Registers;
 
-#if COMPILER(GCC_OR_CLANG)
+#if COMPILER(GCC_COMPATIBLE)
 
 // The following are offsets for Probe::State fields accessed
 // by the ctiMasmProbeTrampoline stub.
+#if CPU(ADDRESS64)
 #define PTR_SIZE 8
+#else
+#define PTR_SIZE 4
+#endif
+
 #define PROBE_PROBE_FUNCTION_OFFSET (0 * PTR_SIZE)
 #define PROBE_ARG_OFFSET (1 * PTR_SIZE)
 #define PROBE_INIT_STACK_FUNCTION_OFFSET (2 * PTR_SIZE)
@@ -126,8 +136,8 @@ using namespace ARM64Registers;
 #define PROBE_CPU_Q31_OFFSET (PROBE_FIRST_FPREG_OFFSET + (31 * FPREG_SIZE))
 #define PROBE_SIZE (PROBE_FIRST_FPREG_OFFSET + (32 * FPREG_SIZE))
 
-#define SAVED_PROBE_RETURN_PC_OFFSET        (PROBE_SIZE + (0 * PTR_SIZE))
-#define PROBE_SIZE_PLUS_EXTRAS              (PROBE_SIZE + (3 * PTR_SIZE))
+#define SAVED_PROBE_RETURN_PC_OFFSET        (PROBE_SIZE + (0 * GPREG_SIZE))
+#define PROBE_SIZE_PLUS_EXTRAS              (PROBE_SIZE + (3 * GPREG_SIZE))
 
 // These ASSERTs remind you that if you change the layout of Probe::State,
 // you need to change ctiMasmProbeTrampoline offsets above to match.
@@ -216,7 +226,7 @@ static_assert(PROBE_OFFSETOF(cpu.fprs[ARM64Registers::q31]) == PROBE_CPU_Q31_OFF
 static_assert(sizeof(Probe::State) == PROBE_SIZE, "Probe::State's size matches ctiMasmProbeTrampoline");
 
 // Conditions for using ldp and stp.
-static_assert(PROBE_CPU_PC_OFFSET == PROBE_CPU_SP_OFFSET + PTR_SIZE, "PROBE_CPU_SP_OFFSET and PROBE_CPU_PC_OFFSET must be adjacent");
+static_assert(PROBE_CPU_PC_OFFSET == PROBE_CPU_SP_OFFSET + GPREG_SIZE, "PROBE_CPU_SP_OFFSET and PROBE_CPU_PC_OFFSET must be adjacent");
 static_assert(!(PROBE_SIZE_PLUS_EXTRAS & 0xf), "PROBE_SIZE_PLUS_EXTRAS should be 16 byte aligned"); // the Probe::State copying code relies on this.
 
 #undef PROBE_OFFSETOF
@@ -224,21 +234,21 @@ static_assert(!(PROBE_SIZE_PLUS_EXTRAS & 0xf), "PROBE_SIZE_PLUS_EXTRAS should be
 #define FPR_OFFSET(fpr) (PROBE_CPU_##fpr##_OFFSET - PROBE_CPU_Q0_OFFSET)
 
 struct IncomingProbeRecord {
-    uintptr_t x24;
-    uintptr_t x25;
-    uintptr_t x26;
-    uintptr_t x27;
-    uintptr_t x28;
-    uintptr_t x30; // lr
+    UCPURegister x24;
+    UCPURegister x25;
+    UCPURegister x26;
+    UCPURegister x27;
+    UCPURegister x28;
+    UCPURegister x30; // lr
 };
 
-#define IN_X24_OFFSET (0 * PTR_SIZE)
-#define IN_X25_OFFSET (1 * PTR_SIZE)
-#define IN_X26_OFFSET (2 * PTR_SIZE)
-#define IN_X27_OFFSET (3 * PTR_SIZE)
-#define IN_X28_OFFSET (4 * PTR_SIZE)
-#define IN_X30_OFFSET (5 * PTR_SIZE)
-#define IN_SIZE       (6 * PTR_SIZE)
+#define IN_X24_OFFSET (0 * GPREG_SIZE)
+#define IN_X25_OFFSET (1 * GPREG_SIZE)
+#define IN_X26_OFFSET (2 * GPREG_SIZE)
+#define IN_X27_OFFSET (3 * GPREG_SIZE)
+#define IN_X28_OFFSET (4 * GPREG_SIZE)
+#define IN_X30_OFFSET (5 * GPREG_SIZE)
+#define IN_SIZE       (6 * GPREG_SIZE)
 
 static_assert(IN_X24_OFFSET == offsetof(IncomingProbeRecord, x24), "IN_X24_OFFSET is incorrect");
 static_assert(IN_X25_OFFSET == offsetof(IncomingProbeRecord, x25), "IN_X25_OFFSET is incorrect");
@@ -250,21 +260,21 @@ static_assert(IN_SIZE == sizeof(IncomingProbeRecord), "IN_SIZE is incorrect");
 static_assert(!(sizeof(IncomingProbeRecord) & 0xf), "IncomingProbeStack must be 16-byte aligned");
 
 struct OutgoingProbeRecord {
-    uintptr_t nzcv;
-    uintptr_t fpsr;
-    uintptr_t x27;
-    uintptr_t x28;
-    uintptr_t fp;
-    uintptr_t lr;
+    UCPURegister nzcv;
+    UCPURegister fpsr;
+    UCPURegister x27;
+    UCPURegister x28;
+    UCPURegister fp;
+    UCPURegister lr;
 };
 
-#define OUT_NZCV_OFFSET (0 * PTR_SIZE)
-#define OUT_FPSR_OFFSET (1 * PTR_SIZE)
-#define OUT_X27_OFFSET  (2 * PTR_SIZE)
-#define OUT_X28_OFFSET  (3 * PTR_SIZE)
-#define OUT_FP_OFFSET   (4 * PTR_SIZE)
-#define OUT_LR_OFFSET   (5 * PTR_SIZE)
-#define OUT_SIZE        (6 * PTR_SIZE)
+#define OUT_NZCV_OFFSET (0 * GPREG_SIZE)
+#define OUT_FPSR_OFFSET (1 * GPREG_SIZE)
+#define OUT_X27_OFFSET  (2 * GPREG_SIZE)
+#define OUT_X28_OFFSET  (3 * GPREG_SIZE)
+#define OUT_FP_OFFSET   (4 * GPREG_SIZE)
+#define OUT_LR_OFFSET   (5 * GPREG_SIZE)
+#define OUT_SIZE        (6 * GPREG_SIZE)
 
 static_assert(OUT_NZCV_OFFSET == offsetof(OutgoingProbeRecord, nzcv), "OUT_NZCV_OFFSET is incorrect");
 static_assert(OUT_FPSR_OFFSET == offsetof(OutgoingProbeRecord, fpsr), "OUT_FPSR_OFFSET is incorrect");
@@ -276,12 +286,12 @@ static_assert(OUT_SIZE == sizeof(OutgoingProbeRecord), "OUT_SIZE is incorrect");
 static_assert(!(sizeof(OutgoingProbeRecord) & 0xf), "OutgoingProbeStack must be 16-byte aligned");
 
 struct LRRestorationRecord {
-    uintptr_t lr;
-    uintptr_t unusedDummyToEnsureSizeIs16ByteAligned;
+    UCPURegister lr;
+    UCPURegister unusedDummyToEnsureSizeIs16ByteAligned;
 };
 
-#define LR_RESTORATION_LR_OFFSET (0 * PTR_SIZE)
-#define LR_RESTORATION_SIZE      (2 * PTR_SIZE)
+#define LR_RESTORATION_LR_OFFSET (0 * GPREG_SIZE)
+#define LR_RESTORATION_SIZE      (2 * GPREG_SIZE)
 
 static_assert(LR_RESTORATION_LR_OFFSET == offsetof(LRRestorationRecord, lr), "LR_RESTORATION_LR_OFFSET is incorrect");
 static_assert(LR_RESTORATION_SIZE == sizeof(LRRestorationRecord), "LR_RESTORATION_SIZE is incorrect");
@@ -343,7 +353,7 @@ asm (
 
     "str       x30, [sp, #" STRINGIZE_VALUE_OF(SAVED_PROBE_RETURN_PC_OFFSET) "]" "\n" // Save a duplicate copy of return pc (in lr).
 
-    "add       x30, x30, #" STRINGIZE_VALUE_OF(2 * PTR_SIZE) "\n" // The PC after the probe is at 2 instructions past the return point.
+    "add       x30, x30, #" STRINGIZE_VALUE_OF(2 * GPREG_SIZE) "\n" // The PC after the probe is at 2 instructions past the return point.
     "str       x30, [sp, #" STRINGIZE_VALUE_OF(PROBE_CPU_PC_OFFSET) "]" "\n"
 
     "stp       x0, x1, [sp, #" STRINGIZE_VALUE_OF(PROBE_CPU_NZCV_OFFSET) "]" "\n" // Store nzcv and fpsr (preloaded into x0 and x1 above).
@@ -371,7 +381,7 @@ asm (
     // Note: we haven't changed the value of fp. Hence, it is still pointing to the frame of
     // the caller of the probe (which is what we want in order to play nice with debuggers e.g. lldb).
     "mov       x0, sp" "\n" // Set the Probe::State* arg.
-    "blr       x28" "\n" // Call the probe handler.
+    CALL_WITH_PTRTAG("blr", "x28", CFunctionPtrTag) // Call the probe handler.
 
     // Make sure the Probe::State is entirely below the result stack pointer so
     // that register values are still preserved when we call the initializeStack
@@ -407,7 +417,7 @@ asm (
     "cbz       x2, " LOCAL_LABEL_STRING(ctiMasmProbeTrampolineRestoreRegisters) "\n"
 
     "mov       x0, x27" "\n" // Set the Probe::State* arg.
-    "blr       x2" "\n" // Call the initializeStackFunction (loaded into x2 above).
+    CALL_WITH_PTRTAG("blr", "x2", CFunctionPtrTag) // Call the initializeStackFunction (loaded into x2 above).
 
     LOCAL_LABEL_STRING(ctiMasmProbeTrampolineRestoreRegisters) ":" "\n"
 
@@ -467,7 +477,7 @@ asm (
     "ldr       x30, [sp, #" STRINGIZE_VALUE_OF(PROBE_CPU_SP_OFFSET) "]" "\n" // preload the target sp.
     "ldr       x27, [sp, #" STRINGIZE_VALUE_OF(SAVED_PROBE_RETURN_PC_OFFSET) "]" "\n"
     "ldr       x28, [sp, #" STRINGIZE_VALUE_OF(PROBE_CPU_PC_OFFSET) "]" "\n"
-    "add       x27, x27, #" STRINGIZE_VALUE_OF(2 * PTR_SIZE) "\n"
+    "add       x27, x27, #" STRINGIZE_VALUE_OF(2 * GPREG_SIZE) "\n"
     "cmp       x27, x28" "\n"
     "bne     " LOCAL_LABEL_STRING(ctiMasmProbeTrampolineEnd) "\n"
 
@@ -497,14 +507,14 @@ asm (
     "mov       sp, x30" "\n"
 
     // Restore the remaining registers and pop the OutgoingProbeRecord.
-    "ldp       x27, x28, [sp], #" STRINGIZE_VALUE_OF(2 * PTR_SIZE) "\n"
+    "ldp       x27, x28, [sp], #" STRINGIZE_VALUE_OF(2 * GPREG_SIZE) "\n"
     "msr       nzcv, x27" "\n"
     "msr       fpsr, x28" "\n"
-    "ldp       x27, x28, [sp], #" STRINGIZE_VALUE_OF(2 * PTR_SIZE) "\n"
-    "ldp       x29, x30, [sp], #" STRINGIZE_VALUE_OF(2 * PTR_SIZE) "\n"
+    "ldp       x27, x28, [sp], #" STRINGIZE_VALUE_OF(2 * GPREG_SIZE) "\n"
+    "ldp       x29, x30, [sp], #" STRINGIZE_VALUE_OF(2 * GPREG_SIZE) "\n"
     "ret" "\n"
 );
-#endif // COMPILER(GCC_OR_CLANG)
+#endif // COMPILER(GCC_COMPATIBLE)
 
 void MacroAssembler::probe(Probe::Function function, void* arg)
 {
@@ -517,7 +527,7 @@ void MacroAssembler::probe(Probe::Function function, void* arg)
     move(TrustedImmPtr(reinterpret_cast<void*>(Probe::executeProbe)), x28);
     move(TrustedImmPtr(reinterpret_cast<void*>(function)), x24);
     move(TrustedImmPtr(arg), x25);
-    m_assembler.blr(x26);
+    call(x26, CFunctionPtrTag);
 
     // ctiMasmProbeTrampoline should have restored every register except for lr and the sp.
     load64(Address(sp, offsetof(LRRestorationRecord, lr)), lr);
@@ -526,6 +536,36 @@ void MacroAssembler::probe(Probe::Function function, void* arg)
 
 #endif // ENABLE(MASM_PROBE)
 
+void MacroAssemblerARM64::collectCPUFeatures()
+{
+#if OS(LINUX)
+    static std::once_flag onceKey;
+    std::call_once(onceKey, [] {
+        // A register for describing ARM64 CPU features are only accessible in kernel mode.
+        // Thus, some kernel support is necessary to collect CPU features. In Linux, the
+        // kernel passes CPU feature flags in AT_HWCAP auxiliary vector which is passed
+        // when the process starts. While this may pose a bit conservative information
+        // (for example, the Linux kernel may add a flag for a feature after the feature
+        // is shipped and implemented in some CPUs. In that case, even if the CPU has
+        // that feature, the kernel does not tell it to users.), it is a stable approach.
+        // https://www.kernel.org/doc/Documentation/arm64/elf_hwcaps.txt
+        uint64_t hwcaps = getauxval(AT_HWCAP);
+
+#if !defined(HWCAP_JSCVT)
+#define HWCAP_JSCVT (1 << 13)
+#endif
+
+        s_jscvtCheckState = (hwcaps & HWCAP_JSCVT) ? CPUIDCheckState::Set : CPUIDCheckState::Clear;
+    });
+#elif HAVE(FJCVTZS_INSTRUCTION)
+    s_jscvtCheckState = CPUIDCheckState::Set;
+#else
+    s_jscvtCheckState = CPUIDCheckState::Clear;
+#endif
+}
+
+MacroAssemblerARM64::CPUIDCheckState MacroAssemblerARM64::s_jscvtCheckState = CPUIDCheckState::NotChecked;
+
 } // namespace JSC
 
 #endif // ENABLE(ASSEMBLER) && CPU(ARM64)
