@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,6 +35,7 @@
 #include "GetterSetter.h"
 #include "GetterSetterAccessCase.h"
 #include "HeapInlines.h"
+#include "InstanceOfAccessCase.h"
 #include "IntrinsicGetterAccessCase.h"
 #include "JSCJSValueInlines.h"
 #include "JSModuleEnvironment.h"
@@ -76,6 +77,7 @@ std::unique_ptr<AccessCase> AccessCase::create(VM& vm, JSCell* owner, AccessType
     case ScopedArgumentsLength:
     case ModuleNamespaceLoad:
     case Replace:
+    case InstanceOfGeneric:
         RELEASE_ASSERT(!prototypeAccessChain);
         break;
     default:
@@ -116,11 +118,30 @@ std::unique_ptr<AccessCase> AccessCase::fromStructureStubInfo(
     case CacheType::PutByIdReplace:
         return AccessCase::create(vm, owner, Replace, stubInfo.u.byIdSelf.offset, stubInfo.u.byIdSelf.baseObjectStructure.get());
 
+    case CacheType::InByIdSelf:
+        return AccessCase::create(vm, owner, InHit, stubInfo.u.byIdSelf.offset, stubInfo.u.byIdSelf.baseObjectStructure.get());
+
+    case CacheType::ArrayLength:
+        return AccessCase::create(vm, owner, AccessCase::ArrayLength);
+
+    case CacheType::StringLength:
+        return AccessCase::create(vm, owner, AccessCase::StringLength);
+
     default:
         return nullptr;
     }
 }
 
+bool AccessCase::hasAlternateBase() const
+{
+    return !conditionSet().isEmpty();
+}
+
+JSObject* AccessCase::alternateBase() const
+{
+    return conditionSet().slotBaseCondition().object();
+}
+
 std::unique_ptr<AccessCase> AccessCase::clone() const
 {
     std::unique_ptr<AccessCase> result(new AccessCase(*this));
@@ -138,10 +159,12 @@ Vector<WatchpointSet*, 2> AccessCase::commit(VM& vm, const Identifier& ident)
     Vector<WatchpointSet*, 2> result;
     Structure* structure = this->structure();
 
-    if ((structure && structure->needImpurePropertyWatchpoint())
-        || m_conditionSet.needImpurePropertyWatchpoint()
-        || (m_polyProtoAccessChain && m_polyProtoAccessChain->needImpurePropertyWatchpoint()))
-        result.append(vm.ensureWatchpointSetForImpureProperty(ident));
+    if (!ident.isNull()) {
+        if ((structure && structure->needImpurePropertyWatchpoint())
+            || m_conditionSet.needImpurePropertyWatchpoint()
+            || (m_polyProtoAccessChain && m_polyProtoAccessChain->needImpurePropertyWatchpoint()))
+            result.append(vm.ensureWatchpointSetForImpureProperty(ident));
+    }
 
     if (additionalSet())
         result.append(additionalSet());
@@ -173,6 +196,9 @@ bool AccessCase::guardedByStructureCheck() const
     case DirectArgumentsLength:
     case ScopedArgumentsLength:
     case ModuleNamespaceLoad:
+    case InstanceOfHit:
+    case InstanceOfMiss:
+    case InstanceOfGeneric:
         return false;
     default:
         return true;
@@ -211,6 +237,9 @@ bool AccessCase::canReplace(const AccessCase& other) const
 {
     // This puts in a good effort to try to figure out if 'other' is made superfluous by '*this'.
     // It's fine for this to return false if it's in doubt.
+    //
+    // Note that if A->guardedByStructureCheck() && B->guardedByStructureCheck() then
+    // A->canReplace(B) == B->canReplace(A).
 
     switch (type()) {
     case ArrayLength:
@@ -225,6 +254,25 @@ bool AccessCase::canReplace(const AccessCase& other) const
         auto& otherCase = this->as<ModuleNamespaceAccessCase>();
         return thisCase.moduleNamespaceObject() == otherCase.moduleNamespaceObject();
     }
+    case InstanceOfHit:
+    case InstanceOfMiss: {
+        if (other.type() != type())
+            return false;
+
+        if (this->as<InstanceOfAccessCase>().prototype() != other.as<InstanceOfAccessCase>().prototype())
+            return false;
+
+        return structure() == other.structure();
+    }
+    case InstanceOfGeneric:
+        switch (other.type()) {
+        case InstanceOfGeneric:
+        case InstanceOfHit:
+        case InstanceOfMiss:
+            return true;
+        default:
+            return false;
+        }
     default:
         if (other.type() != type())
             return false;
@@ -301,6 +349,9 @@ bool AccessCase::visitWeak(VM& vm) const
             return false;
         if (accessCase.moduleEnvironment() && !Heap::isMarked(accessCase.moduleEnvironment()))
             return false;
+    } else if (type() == InstanceOfHit || type() == InstanceOfMiss) {
+        if (as<InstanceOfAccessCase>().prototype() && !Heap::isMarked(as<InstanceOfAccessCase>().prototype()))
+            return false;
     }
 
     return true;
@@ -341,82 +392,16 @@ void AccessCase::generateWithGuard(
     m_state = Generated;
 
     CCallHelpers& jit = *state.jit;
+    StructureStubInfo& stubInfo = *state.stubInfo;
     VM& vm = state.m_vm;
     JSValueRegs valueRegs = state.valueRegs;
     GPRReg baseGPR = state.baseGPR;
+    GPRReg thisGPR = state.thisGPR != InvalidGPRReg ? state.thisGPR : baseGPR;
     GPRReg scratchGPR = state.scratchGPR;
 
     UNUSED_PARAM(vm);
 
-    switch (m_type) {
-    case ArrayLength: {
-        ASSERT(!viaProxy());
-        jit.load8(CCallHelpers::Address(baseGPR, JSCell::indexingTypeAndMiscOffset()), scratchGPR);
-        fallThrough.append(
-            jit.branchTest32(
-                CCallHelpers::Zero, scratchGPR, CCallHelpers::TrustedImm32(IsArray)));
-        fallThrough.append(
-            jit.branchTest32(
-                CCallHelpers::Zero, scratchGPR, CCallHelpers::TrustedImm32(IndexingShapeMask)));
-        break;
-    }
-
-    case StringLength: {
-        ASSERT(!viaProxy());
-        fallThrough.append(
-            jit.branch8(
-                CCallHelpers::NotEqual,
-                CCallHelpers::Address(baseGPR, JSCell::typeInfoTypeOffset()),
-                CCallHelpers::TrustedImm32(StringType)));
-        break;
-    }
-
-    case DirectArgumentsLength: {
-        ASSERT(!viaProxy());
-        fallThrough.append(
-            jit.branch8(
-                CCallHelpers::NotEqual,
-                CCallHelpers::Address(baseGPR, JSCell::typeInfoTypeOffset()),
-                CCallHelpers::TrustedImm32(DirectArgumentsType)));
-
-        fallThrough.append(
-            jit.branchTestPtr(
-                CCallHelpers::NonZero,
-                CCallHelpers::Address(baseGPR, DirectArguments::offsetOfMappedArguments())));
-        jit.load32(
-            CCallHelpers::Address(baseGPR, DirectArguments::offsetOfLength()),
-            valueRegs.payloadGPR());
-        jit.boxInt32(valueRegs.payloadGPR(), valueRegs);
-        state.succeed();
-        return;
-    }
-
-    case ScopedArgumentsLength: {
-        ASSERT(!viaProxy());
-        fallThrough.append(
-            jit.branch8(
-                CCallHelpers::NotEqual,
-                CCallHelpers::Address(baseGPR, JSCell::typeInfoTypeOffset()),
-                CCallHelpers::TrustedImm32(ScopedArgumentsType)));
-
-        fallThrough.append(
-            jit.branchTest8(
-                CCallHelpers::NonZero,
-                CCallHelpers::Address(baseGPR, ScopedArguments::offsetOfOverrodeThings())));
-        jit.load32(
-            CCallHelpers::Address(baseGPR, ScopedArguments::offsetOfTotalLength()),
-            valueRegs.payloadGPR());
-        jit.boxInt32(valueRegs.payloadGPR(), valueRegs);
-        state.succeed();
-        return;
-    }
-
-    case ModuleNamespaceLoad: {
-        this->as<ModuleNamespaceAccessCase>().emit(state, fallThrough);
-        return;
-    }
-
-    default: {
+    auto emitDefaultGuard = [&] () {
         if (m_polyProtoAccessChain) {
             GPRReg baseForAccessGPR = state.scratchGPR;
             jit.move(state.baseGPR, baseForAccessGPR);
@@ -457,31 +442,184 @@ void AccessCase::generateWithGuard(
                     }
                 }
             });
-        } else {
-            if (viaProxy()) {
-                fallThrough.append(
-                    jit.branch8(
-                        CCallHelpers::NotEqual,
-                        CCallHelpers::Address(baseGPR, JSCell::typeInfoTypeOffset()),
-                        CCallHelpers::TrustedImm32(PureForwardingProxyType)));
-
-                jit.loadPtr(CCallHelpers::Address(baseGPR, JSProxy::targetOffset()), scratchGPR);
-
-                fallThrough.append(
-                    jit.branchStructure(
-                        CCallHelpers::NotEqual,
-                        CCallHelpers::Address(scratchGPR, JSCell::structureIDOffset()),
-                        structure()));
-            } else {
-                fallThrough.append(
-                    jit.branchStructure(
-                        CCallHelpers::NotEqual,
-                        CCallHelpers::Address(baseGPR, JSCell::structureIDOffset()),
-                        structure()));
-            }
+            return;
         }
+
+        if (viaProxy()) {
+            fallThrough.append(
+                jit.branchIfNotType(baseGPR, PureForwardingProxyType));
+
+            jit.loadPtr(CCallHelpers::Address(baseGPR, JSProxy::targetOffset()), scratchGPR);
+
+            fallThrough.append(
+                jit.branchStructure(
+                    CCallHelpers::NotEqual,
+                    CCallHelpers::Address(scratchGPR, JSCell::structureIDOffset()),
+                    structure()));
+            return;
+        }
+
+        fallThrough.append(
+            jit.branchStructure(
+                CCallHelpers::NotEqual,
+                CCallHelpers::Address(baseGPR, JSCell::structureIDOffset()),
+                structure()));
+    };
+
+    switch (m_type) {
+    case ArrayLength: {
+        ASSERT(!viaProxy());
+        jit.load8(CCallHelpers::Address(baseGPR, JSCell::indexingTypeAndMiscOffset()), scratchGPR);
+        fallThrough.append(
+            jit.branchTest32(
+                CCallHelpers::Zero, scratchGPR, CCallHelpers::TrustedImm32(IsArray)));
+        fallThrough.append(
+            jit.branchTest32(
+                CCallHelpers::Zero, scratchGPR, CCallHelpers::TrustedImm32(IndexingShapeMask)));
         break;
-    } };
+    }
+
+    case StringLength: {
+        ASSERT(!viaProxy());
+        fallThrough.append(
+            jit.branchIfNotString(baseGPR));
+        break;
+    }
+
+    case DirectArgumentsLength: {
+        ASSERT(!viaProxy());
+        fallThrough.append(
+            jit.branchIfNotType(baseGPR, DirectArgumentsType));
+
+        fallThrough.append(
+            jit.branchTestPtr(
+                CCallHelpers::NonZero,
+                CCallHelpers::Address(baseGPR, DirectArguments::offsetOfMappedArguments())));
+        jit.load32(
+            CCallHelpers::Address(baseGPR, DirectArguments::offsetOfLength()),
+            valueRegs.payloadGPR());
+        jit.boxInt32(valueRegs.payloadGPR(), valueRegs);
+        state.succeed();
+        return;
+    }
+
+    case ScopedArgumentsLength: {
+        ASSERT(!viaProxy());
+        fallThrough.append(
+            jit.branchIfNotType(baseGPR, ScopedArgumentsType));
+
+        jit.loadPtr(
+            CCallHelpers::Address(baseGPR, ScopedArguments::offsetOfStorage()),
+            scratchGPR);
+        fallThrough.append(
+            jit.branchTest8(
+                CCallHelpers::NonZero,
+                CCallHelpers::Address(scratchGPR, ScopedArguments::offsetOfOverrodeThingsInStorage())));
+        jit.load32(
+            CCallHelpers::Address(scratchGPR, ScopedArguments::offsetOfTotalLengthInStorage()),
+            valueRegs.payloadGPR());
+        jit.boxInt32(valueRegs.payloadGPR(), valueRegs);
+        state.succeed();
+        return;
+    }
+
+    case ModuleNamespaceLoad: {
+        this->as<ModuleNamespaceAccessCase>().emit(state, fallThrough);
+        return;
+    }
+
+    case InstanceOfHit:
+    case InstanceOfMiss:
+        emitDefaultGuard();
+
+        fallThrough.append(
+            jit.branchPtr(
+                CCallHelpers::NotEqual, thisGPR,
+                CCallHelpers::TrustedImmPtr(as<InstanceOfAccessCase>().prototype())));
+        break;
+
+    case InstanceOfGeneric: {
+        // Legend: value = `base instanceof this`.
+
+        GPRReg valueGPR = valueRegs.payloadGPR();
+
+        ScratchRegisterAllocator allocator(stubInfo.patch.usedRegisters);
+        allocator.lock(baseGPR);
+        allocator.lock(valueGPR);
+        allocator.lock(thisGPR);
+        allocator.lock(scratchGPR);
+
+        GPRReg scratch2GPR = allocator.allocateScratchGPR();
+
+        if (!state.stubInfo->prototypeIsKnownObject)
+            state.failAndIgnore.append(jit.branchIfNotObject(thisGPR));
+
+        ScratchRegisterAllocator::PreservedState preservedState =
+            allocator.preserveReusedRegistersByPushing(
+                jit,
+                ScratchRegisterAllocator::ExtraStackSpace::NoExtraSpace);
+        CCallHelpers::Jump failAndIgnore;
+
+        jit.move(baseGPR, valueGPR);
+
+        CCallHelpers::Label loop(&jit);
+        failAndIgnore = jit.branchIfType(valueGPR, ProxyObjectType);
+
+        jit.emitLoadStructure(vm, valueGPR, scratch2GPR, scratchGPR);
+#if USE(JSVALUE64)
+        jit.load64(CCallHelpers::Address(scratch2GPR, Structure::prototypeOffset()), scratch2GPR);
+        CCallHelpers::Jump hasMonoProto = jit.branchTest64(CCallHelpers::NonZero, scratch2GPR);
+        jit.load64(
+            CCallHelpers::Address(valueGPR, offsetRelativeToBase(knownPolyProtoOffset)),
+            scratch2GPR);
+        hasMonoProto.link(&jit);
+#else
+        jit.load32(
+            CCallHelpers::Address(scratch2GPR, Structure::prototypeOffset() + TagOffset),
+            scratchGPR);
+        jit.load32(
+            CCallHelpers::Address(scratch2GPR, Structure::prototypeOffset() + PayloadOffset),
+            scratch2GPR);
+        CCallHelpers::Jump hasMonoProto = jit.branch32(
+            CCallHelpers::NotEqual, scratchGPR, CCallHelpers::TrustedImm32(JSValue::EmptyValueTag));
+        jit.load32(
+            CCallHelpers::Address(
+                valueGPR, offsetRelativeToBase(knownPolyProtoOffset) + PayloadOffset),
+            scratch2GPR);
+        hasMonoProto.link(&jit);
+#endif
+        jit.move(scratch2GPR, valueGPR);
+
+        CCallHelpers::Jump isInstance = jit.branchPtr(CCallHelpers::Equal, valueGPR, thisGPR);
+
+#if USE(JSVALUE64)
+        jit.branchIfCell(JSValueRegs(valueGPR)).linkTo(loop, &jit);
+#else
+        jit.branchTestPtr(CCallHelpers::NonZero, valueGPR).linkTo(loop, &jit);
+#endif
+
+        jit.boxBooleanPayload(false, valueGPR);
+        allocator.restoreReusedRegistersByPopping(jit, preservedState);
+        state.succeed();
+
+        isInstance.link(&jit);
+        jit.boxBooleanPayload(true, valueGPR);
+        allocator.restoreReusedRegistersByPopping(jit, preservedState);
+        state.succeed();
+
+        if (allocator.didReuseRegisters()) {
+            failAndIgnore.link(&jit);
+            allocator.restoreReusedRegistersByPopping(jit, preservedState);
+            state.failAndIgnore.append(jit.jump());
+        } else
+            state.failAndIgnore.append(failAndIgnore);
+        return;
+    }
+
+    default:
+        emitDefaultGuard();
+        break;
+    }
 
     generateImpl(state);
 }
@@ -517,7 +655,7 @@ void AccessCase::generateImpl(AccessGenerationState& state)
     for (const ObjectPropertyCondition& condition : m_conditionSet) {
         RELEASE_ASSERT(!m_polyProtoAccessChain);
 
-        Structure* structure = condition.object()->structure();
+        Structure* structure = condition.object()->structure(vm);
 
         if (condition.isWatchableAssumingImpurePropertyWatchpoint()) {
             structure->addTransitionWatchpoint(state.addWatchpoint(condition));
@@ -547,7 +685,7 @@ void AccessCase::generateImpl(AccessGenerationState& state)
     switch (m_type) {
     case InHit:
     case InMiss:
-        jit.boxBooleanPayload(m_type == InHit, valueRegs.payloadGPR());
+        jit.boxBoolean(m_type == InHit, valueRegs);
         state.succeed();
         return;
 
@@ -556,6 +694,12 @@ void AccessCase::generateImpl(AccessGenerationState& state)
         state.succeed();
         return;
 
+    case InstanceOfHit:
+    case InstanceOfMiss:
+        jit.boxBooleanPayload(m_type == InstanceOfHit, valueRegs.payloadGPR());
+        state.succeed();
+        return;
+
     case Load:
     case GetGetter:
     case Getter:
@@ -568,10 +712,10 @@ void AccessCase::generateImpl(AccessGenerationState& state)
 
         if (isValidOffset(m_offset)) {
             Structure* currStructure;
-            if (m_conditionSet.isEmpty())
+            if (!hasAlternateBase())
                 currStructure = structure();
             else
-                currStructure = m_conditionSet.slotBaseCondition().object()->structure();
+                currStructure = alternateBase()->structure(vm);
             currStructure->startWatchingPropertyForReplacements(vm, offset());
         }
 
@@ -599,7 +743,7 @@ void AccessCase::generateImpl(AccessGenerationState& state)
             // but it'd require emitting the same code to load the base twice.
             baseForAccessGPR = scratchGPR;
         } else {
-            if (!m_conditionSet.isEmpty()) {
+            if (hasAlternateBase()) {
                 jit.move(
                     CCallHelpers::TrustedImmPtr(alternateBase()), scratchGPR);
                 baseForAccessGPR = scratchGPR;
@@ -814,13 +958,13 @@ void AccessCase::generateImpl(AccessGenerationState& state)
 
             jit.addLinkTask([=, &vm] (LinkBuffer& linkBuffer) {
                 this->as<GetterSetterAccessCase>().callLinkInfo()->setCallLocations(
-                    CodeLocationLabel(linkBuffer.locationOfNearCall(slowPathCall)),
-                    CodeLocationLabel(linkBuffer.locationOf(addressOfLinkFunctionCheck)),
-                    linkBuffer.locationOfNearCall(fastPathCall));
+                    CodeLocationLabel<JSInternalPtrTag>(linkBuffer.locationOfNearCall<JSInternalPtrTag>(slowPathCall)),
+                    CodeLocationLabel<JSInternalPtrTag>(linkBuffer.locationOf<JSInternalPtrTag>(addressOfLinkFunctionCheck)),
+                    linkBuffer.locationOfNearCall<JSInternalPtrTag>(fastPathCall));
 
                 linkBuffer.link(
                     slowPathCall,
-                    CodeLocationLabel(vm.getCTIStub(linkCallThunkGenerator).code()));
+                    CodeLocationLabel<JITThunkPtrTag>(vm.getCTIStub(linkCallThunkGenerator).code()));
             });
         } else {
             ASSERT(m_type == CustomValueGetter || m_type == CustomAccessorGetter || m_type == CustomValueSetter || m_type == CustomAccessorSetter);
@@ -839,31 +983,20 @@ void AccessCase::generateImpl(AccessGenerationState& state)
             // FIXME: Remove this differences in custom values and custom accessors.
             // https://bugs.webkit.org/show_bug.cgi?id=158014
             GPRReg baseForCustom = m_type == CustomValueGetter || m_type == CustomValueSetter ? baseForAccessGPR : baseForCustomGetGPR;
-#if USE(JSVALUE64)
             if (m_type == CustomValueGetter || m_type == CustomAccessorGetter) {
-                jit.setupArgumentsWithExecState(
-                    baseForCustom,
-                    CCallHelpers::TrustedImmPtr(ident.impl()));
-            } else
-                jit.setupArgumentsWithExecState(baseForCustom, valueRegs.gpr());
-#else
-            if (m_type == CustomValueGetter || m_type == CustomAccessorGetter) {
-                jit.setupArgumentsWithExecState(
-                    EABI_32BIT_DUMMY_ARG baseForCustom,
-                    CCallHelpers::TrustedImm32(JSValue::CellTag),
+                jit.setupArguments<PropertySlot::GetValueFunc>(
+                    CCallHelpers::CellValue(baseForCustom),
                     CCallHelpers::TrustedImmPtr(ident.impl()));
             } else {
-                jit.setupArgumentsWithExecState(
-                    EABI_32BIT_DUMMY_ARG baseForCustom,
-                    CCallHelpers::TrustedImm32(JSValue::CellTag),
-                    valueRegs.payloadGPR(), valueRegs.tagGPR());
+                jit.setupArguments<PutPropertySlot::PutValueFunc>(
+                    CCallHelpers::CellValue(baseForCustom),
+                    valueRegs);
             }
-#endif
             jit.storePtr(GPRInfo::callFrameRegister, &vm.topCallFrame);
 
-            operationCall = jit.call();
+            operationCall = jit.call(OperationPtrTag);
             jit.addLinkTask([=] (LinkBuffer& linkBuffer) {
-                linkBuffer.link(operationCall, FunctionPtr(this->as<GetterSetterAccessCase>().m_customAccessor.opaque));
+                linkBuffer.link(operationCall, this->as<GetterSetterAccessCase>().m_customAccessor);
             });
 
             if (m_type == CustomValueGetter || m_type == CustomAccessorGetter)
@@ -885,14 +1018,6 @@ void AccessCase::generateImpl(AccessGenerationState& state)
     }
 
     case Replace: {
-        if (InferredType* type = structure()->inferredTypeFor(ident.impl())) {
-            if (AccessCaseInternal::verbose)
-                dataLog("Have type: ", type->descriptor(), "\n");
-            state.failAndRepatch.append(
-                jit.branchIfNotType(valueRegs, scratchGPR, type->descriptor()));
-        } else if (AccessCaseInternal::verbose)
-            dataLog("Don't have type.\n");
-
         if (isInlineOffset(m_offset)) {
             jit.storeValue(
                 valueRegs,
@@ -915,14 +1040,6 @@ void AccessCase::generateImpl(AccessGenerationState& state)
         // AccessCase::transition() should have returned null if this wasn't true.
         RELEASE_ASSERT(GPRInfo::numberOfRegisters >= 6 || !structure()->outOfLineCapacity() || structure()->outOfLineCapacity() == newStructure()->outOfLineCapacity());
 
-        if (InferredType* type = newStructure()->inferredTypeFor(ident.impl())) {
-            if (AccessCaseInternal::verbose)
-                dataLog("Have type: ", type->descriptor(), "\n");
-            state.failAndRepatch.append(
-                jit.branchIfNotType(valueRegs, scratchGPR, type->descriptor()));
-        } else if (AccessCaseInternal::verbose)
-            dataLog("Don't have type.\n");
-
         // NOTE: This logic is duplicated in AccessCase::doesCalls(). It's important that doesCalls() knows
         // exactly when this would make calls.
         bool allocating = newStructure()->outOfLineCapacity() != structure()->outOfLineCapacity();
@@ -932,7 +1049,7 @@ void AccessCase::generateImpl(AccessGenerationState& state)
         ScratchRegisterAllocator allocator(stubInfo.patch.usedRegisters);
         allocator.lock(baseGPR);
 #if USE(JSVALUE32_64)
-        allocator.lock(static_cast<GPRReg>(stubInfo.patch.baseTagGPR));
+        allocator.lock(stubInfo.patch.baseTagGPR);
 #endif
         allocator.lock(valueRegs);
         allocator.lock(scratchGPR);
@@ -945,7 +1062,7 @@ void AccessCase::generateImpl(AccessGenerationState& state)
         }
 
         ScratchRegisterAllocator::PreservedState preservedState =
-        allocator.preserveReusedRegistersByPushing(jit, ScratchRegisterAllocator::ExtraStackSpace::SpaceForCCall);
+            allocator.preserveReusedRegistersByPushing(jit, ScratchRegisterAllocator::ExtraStackSpace::SpaceForCCall);
 
         CCallHelpers::JumpList slowPath;
 
@@ -1003,25 +1120,25 @@ void AccessCase::generateImpl(AccessGenerationState& state)
                 jit.makeSpaceOnStackForCCall();
 
                 if (!reallocating) {
-                    jit.setupArgumentsWithExecState(baseGPR);
+                    jit.setupArguments<decltype(operationReallocateButterflyToHavePropertyStorageWithInitialCapacity)>(baseGPR);
 
-                    CCallHelpers::Call operationCall = jit.call();
+                    CCallHelpers::Call operationCall = jit.call(OperationPtrTag);
                     jit.addLinkTask([=] (LinkBuffer& linkBuffer) {
                         linkBuffer.link(
                             operationCall,
-                            FunctionPtr(operationReallocateButterflyToHavePropertyStorageWithInitialCapacity));
+                            FunctionPtr<OperationPtrTag>(operationReallocateButterflyToHavePropertyStorageWithInitialCapacity));
                     });
                 } else {
                     // Handle the case where we are reallocating (i.e. the old structure/butterfly
                     // already had out-of-line property storage).
-                    jit.setupArgumentsWithExecState(
+                    jit.setupArguments<decltype(operationReallocateButterflyToGrowPropertyStorage)>(
                         baseGPR, CCallHelpers::TrustedImm32(newSize / sizeof(JSValue)));
 
-                    CCallHelpers::Call operationCall = jit.call();
+                    CCallHelpers::Call operationCall = jit.call(OperationPtrTag);
                     jit.addLinkTask([=] (LinkBuffer& linkBuffer) {
                         linkBuffer.link(
                             operationCall,
-                            FunctionPtr(operationReallocateButterflyToGrowPropertyStorage));
+                            FunctionPtr<OperationPtrTag>(operationReallocateButterflyToGrowPropertyStorage));
                     });
                 }
 
@@ -1096,8 +1213,16 @@ void AccessCase::generateImpl(AccessGenerationState& state)
     }
 
     case StringLength: {
-        jit.load32(CCallHelpers::Address(baseGPR, JSString::offsetOfLength()), valueRegs.payloadGPR());
-        jit.boxInt32(valueRegs.payloadGPR(), valueRegs);
+        jit.loadPtr(CCallHelpers::Address(baseGPR, JSString::offsetOfValue()), scratchGPR);
+        auto isRope = jit.branchIfRopeStringImpl(scratchGPR);
+        jit.load32(CCallHelpers::Address(scratchGPR, StringImpl::lengthMemoryOffset()), scratchGPR);
+        auto done = jit.jump();
+
+        isRope.link(&jit);
+        jit.load32(CCallHelpers::Address(baseGPR, JSRopeString::offsetOfLength()), scratchGPR);
+
+        done.link(&jit);
+        jit.boxInt32(scratchGPR, valueRegs);
         state.succeed();
         return;
     }
@@ -1108,10 +1233,10 @@ void AccessCase::generateImpl(AccessGenerationState& state)
         // We need to ensure the getter value does not move from under us. Note that GetterSetters
         // are immutable so we just need to watch the property not any value inside it.
         Structure* currStructure;
-        if (m_conditionSet.isEmpty())
+        if (!hasAlternateBase())
             currStructure = structure();
         else
-            currStructure = m_conditionSet.slotBaseCondition().object()->structure();
+            currStructure = alternateBase()->structure(vm);
         currStructure->startWatchingPropertyForReplacements(vm, offset());
 
         this->as<IntrinsicGetterAccessCase>().emitIntrinsicGetter(state);
@@ -1121,6 +1246,7 @@ void AccessCase::generateImpl(AccessGenerationState& state)
     case DirectArgumentsLength:
     case ScopedArgumentsLength:
     case ModuleNamespaceLoad:
+    case InstanceOfGeneric:
         // These need to be handled by generateWithGuard(), since the guard is part of the
         // algorithm. We can be sure that nobody will call generate() directly for these since they
         // are not guarded by structure checks.
