@@ -65,11 +65,13 @@ public:
         case SetObjectUse:
         case WeakMapObjectUse:
         case WeakSetObjectUse:
+        case DataViewObjectUse:
         case ObjectOrOtherUse:
         case StringIdentUse:
         case StringUse:
         case StringOrOtherUse:
         case SymbolUse:
+        case BigIntUse:
         case StringObjectUse:
         case StringOrStringObjectUse:
         case NotStringVarUse:
@@ -106,6 +108,11 @@ public:
                 m_result = false;
             return;
 
+        case KnownOtherUse:
+            if (m_state.forNode(edge).m_type & ~SpecOther)
+                m_result = false;
+            return;
+
         case LastUseKind:
             RELEASE_ASSERT_NOT_REACHED();
             break;
@@ -133,14 +140,14 @@ private:
 // safely execute before that check, so long as that check continues to guard any
 // user-observable things done to the loaded value.
 template<typename AbstractStateType>
-bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
+bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node, bool ignoreEmptyChildren = false)
 {
     SafeToExecuteEdge<AbstractStateType> safeToExecuteEdge(state);
     DFG_NODE_DO_TO_CHILDREN(graph, node, safeToExecuteEdge);
     if (!safeToExecuteEdge.result())
         return false;
 
-    if (safeToExecuteEdge.maySeeEmptyChild()) {
+    if (!ignoreEmptyChildren && safeToExecuteEdge.maySeeEmptyChild()) {
         // We conservatively assume if the empty value flows into a node,
         // it might not be able to handle it (e.g, crash). In general, the bytecode generator
         // emits code in such a way that most node types don't need to worry about the empty value
@@ -170,6 +177,8 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case IdentityWithProfile:
     case ToThis:
     case CreateThis:
+    case ObjectCreate:
+    case ObjectKeys:
     case GetCallee:
     case SetCallee:
     case GetArgumentCountIncludingThis:
@@ -189,9 +198,10 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case Flush:
     case PhantomLocal:
     case SetArgument:
-    case BitAnd:
-    case BitOr:
-    case BitXor:
+    case ArithBitNot:
+    case ArithBitAnd:
+    case ArithBitOr:
+    case ArithBitXor:
     case BitLShift:
     case BitRShift:
     case BitURShift:
@@ -218,7 +228,14 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case ArithCeil:
     case ArithTrunc:
     case ArithUnary:
+    case ValueBitAnd:
+    case ValueBitXor:
+    case ValueBitOr:
+    case ValueNegate:
     case ValueAdd:
+    case ValueSub:
+    case ValueMul:
+    case ValueDiv:
     case TryGetById:
     case DeleteById:
     case DeleteByVal:
@@ -268,6 +285,7 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case RegExpExecNonGlobalOrSticky:
     case RegExpTest:
     case RegExpMatchFast:
+    case RegExpMatchFastGlobal:
     case CompareLess:
     case CompareLessEq:
     case CompareGreater:
@@ -277,6 +295,7 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case CompareEq:
     case CompareStrictEq:
     case CompareEqPtr:
+    case SameValue:
     case Call:
     case DirectCall:
     case TailCallInlinedCaller:
@@ -298,6 +317,7 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case NewArrayWithSpread:
     case Spread:
     case NewRegexp:
+    case NewSymbol:
     case ProfileType:
     case ProfileControlFlow:
     case CheckTypeInfoFlags:
@@ -307,6 +327,7 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case InstanceOfCustom:
     case IsEmpty:
     case IsUndefined:
+    case IsUndefinedOrNull:
     case IsBoolean:
     case IsNumber:
     case NumberIsInteger:
@@ -329,7 +350,8 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case CallStringConstructor:
     case NewStringObject:
     case MakeRope:
-    case In:
+    case InByVal:
+    case InById:
     case HasOwnProperty:
     case PushWithScope:
     case CreateActivation:
@@ -367,6 +389,7 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case Unreachable:
     case ExtractOSREntryLocal:
     case ExtractCatchLocal:
+    case ClearCatchLocals:
     case CheckTierUpInLoop:
     case CheckTierUpAtReturn:
     case CheckTierUpAndOSREnter:
@@ -427,6 +450,7 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case ResolveScope:
     case MapHash:
     case NormalizeMapKey:
+    case StringValueOf:
     case StringSlice:
     case ToLowerCase:
     case GetMapBucket:
@@ -449,6 +473,9 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case AtomicsXor:
     case AtomicsIsLockFree:
     case InitializeEntrypointArguments:
+    case MatchStructure:
+    case DataViewGetInt:
+    case DataViewGetFloat:
         return true;
 
     case ArraySlice:
@@ -474,6 +501,14 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
         // effect based memory model to rely solely on that right now.
         return false;
 
+    case FilterCallLinkStatus:
+    case FilterGetByIdStatus:
+    case FilterPutByIdStatus:
+    case FilterInByIdStatus:
+        // We don't want these to be moved anywhere other than where we put them, since we want them
+        // to capture "profiling" at the point in control flow here the user put them.
+        return false;
+
     case GetByVal:
     case GetIndexedPropertyStorage:
     case GetArrayLength:
@@ -483,9 +518,6 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case StringCharCodeAt:
         return node->arrayMode().alreadyChecked(graph, node, state.forNode(graph.child(node, 0)));
 
-    case GetArrayMask:
-        return state.forNode(node->child1()).isType(SpecObject);
-
     case ArrayPush:
         return node->arrayMode().alreadyChecked(graph, node, state.forNode(graph.varArgChild(node, 1)));
 
@@ -508,50 +540,23 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
     case PutByOffset: {
         StorageAccessData& data = node->storageAccessData();
         PropertyOffset offset = data.offset;
-        UniquedStringImpl* uid = graph.identifiers()[data.identifierNumber];
-
-        InferredType::Descriptor inferredType;
-        switch (node->op()) {
-        case GetByOffset:
-        case GetGetterSetterByOffset:
-            inferredType = data.inferredType;
-            break;
-        case PutByOffset:
-            // PutByOffset knows about inferred types because it's the enforcer of that type rather
-            // than the consumer of that type. Therefore, PutByOffset expects TOP for the purpose of
-            // safe-to-execute in the sense that it will be happy with anything as general as TOP
-            // (so any type).
-            inferredType = InferredType::Top;
-            break;
-        default:
-            DFG_CRASH(graph, node, "Bad opcode");
-            break;
-        }
-
         // Graph::isSafeToLoad() is all about proofs derived from PropertyConditions. Those don't
         // know anything about inferred types. But if we have a proof derived from watching a
         // structure that has a type proof, then the next case below will deal with it.
-        if (state.structureClobberState() == StructuresAreWatched
-            && inferredType.kind() == InferredType::Top) {
-            if (JSObject* knownBase = node->child1()->dynamicCastConstant<JSObject*>(graph.m_vm)) {
+        if (state.structureClobberState() == StructuresAreWatched) {
+            if (JSObject* knownBase = node->child2()->dynamicCastConstant<JSObject*>(graph.m_vm)) {
                 if (graph.isSafeToLoad(knownBase, offset))
                     return true;
             }
         }
 
-        StructureAbstractValue& value = state.forNode(node->child1()).m_structure;
+        StructureAbstractValue& value = state.forNode(node->child2()).m_structure;
         if (value.isInfinite())
             return false;
         for (unsigned i = value.size(); i--;) {
             Structure* thisStructure = value[i].get();
             if (!thisStructure->isValidOffset(offset))
                 return false;
-            if (inferredType.kind() != InferredType::Top) {
-                InferredType::Descriptor thisInferredType =
-                    graph.inferredTypeForProperty(thisStructure, uid);
-                if (!inferredType.subsumes(thisInferredType))
-                    return false;
-            }
         }
         return true;
     }
@@ -584,6 +589,9 @@ bool safeToExecute(AbstractStateType& state, Graph& graph, Node* node)
         return true;
     }
 
+    case DataViewSet:
+        return false;
+
     case SetAdd:
     case MapSet:
         return false;
