@@ -1,7 +1,7 @@
 /*
  *  Copyright (C) 1999-2000 Harri Porten (porten@kde.org)
  *  Copyright (C) 2001 Peter Kelly (pmk@post.com)
- *  Copyright (C) 2003-2017 Apple Inc. All rights reserved.
+ *  Copyright (C) 2003-2019 Apple Inc. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public
@@ -32,21 +32,19 @@
 #include "HandleSet.h"
 #include "HeapFinalizerCallback.h"
 #include "HeapObserver.h"
-#include "ListableHandler.h"
 #include "MarkedBlock.h"
 #include "MarkedSpace.h"
 #include "MutatorState.h"
 #include "Options.h"
 #include "StructureIDTable.h"
 #include "Synchronousness.h"
-#include "UnconditionalFinalizer.h"
 #include "WeakHandleOwner.h"
-#include "WeakReferenceHarvester.h"
 #include <wtf/AutomaticThread.h>
 #include <wtf/ConcurrentPtrHashSet.h>
 #include <wtf/Deque.h>
 #include <wtf/HashCountedSet.h>
 #include <wtf/HashSet.h>
+#include <wtf/Markable.h>
 #include <wtf/ParallelHelperPool.h>
 #include <wtf/Threading.h>
 
@@ -58,7 +56,6 @@ class CollectingScope;
 class ConservativeRoots;
 class GCDeferralContext;
 class EdenGCActivityCallback;
-class ExecutableBase;
 class FullGCActivityCallback;
 class GCActivityCallback;
 class GCAwareJITStubRoutine;
@@ -69,6 +66,7 @@ class IncrementalSweeper;
 class JITStubRoutine;
 class JITStubRoutineSet;
 class JSCell;
+class JSImmutableButterfly;
 class JSValue;
 class LLIntOffsetsExtractor;
 class MachineThreads;
@@ -84,16 +82,26 @@ class SlotVisitor;
 class SpaceTimeMutatorScheduler;
 class StopIfNecessaryTimer;
 class SweepingScope;
-class ThreadLocalCacheLayout;
 class VM;
 class WeakGCMapBase;
 struct CurrentThreadState;
 
+#if USE(GLIB)
+class JSCGLibWrapperObject;
+#endif
+
 namespace DFG {
 class SpeculativeJIT;
 class Worklist;
 }
 
+#if !ASSERT_DISABLED
+#define ENABLE_DFG_DOES_GC_VALIDATION 1
+#else
+#define ENABLE_DFG_DOES_GC_VALIDATION 0
+#endif
+constexpr bool validateDFGDoesGC = ENABLE_DFG_DOES_GC_VALIDATION;
+
 typedef HashCountedSet<JSCell*> ProtectCountSet;
 typedef HashCountedSet<const char*> TypeCountSet;
 
@@ -153,7 +161,7 @@ public:
     void removeObserver(HeapObserver* observer) { m_observers.removeFirst(observer); }
 
     MutatorState mutatorState() const { return m_mutatorState; }
-    std::optional<CollectionScope> collectionScope() const { return m_collectionScope; }
+    Optional<CollectionScope> collectionScope() const { return m_collectionScope; }
     bool hasHeapAccess() const;
     bool worldIsStopped() const;
     bool worldIsRunning() const { return !worldIsStopped(); }
@@ -164,7 +172,6 @@ public:
 
     typedef void (*Finalizer)(JSCell*);
     JS_EXPORT_PRIVATE void addFinalizer(JSCell*, Finalizer);
-    void addExecutable(ExecutableBase*);
 
     void notifyIsSafeToCollect();
     bool isSafeToCollect() const { return m_isSafeToCollect; }
@@ -178,8 +185,8 @@ public:
     bool shouldCollectHeuristic();
 
     // Queue up a collection. Returns immediately. This will not queue a collection if a collection
-    // of equal or greater strength exists. Full collections are stronger than std::nullopt collections
-    // and std::nullopt collections are stronger than Eden collections. std::nullopt means that the GC can
+    // of equal or greater strength exists. Full collections are stronger than WTF::nullopt collections
+    // and WTF::nullopt collections are stronger than Eden collections. WTF::nullopt means that the GC can
     // choose Eden or Full. This implies that if you request a GC while that GC is ongoing, nothing
     // will happen.
     JS_EXPORT_PRIVATE void collectAsync(GCRequest = GCRequest());
@@ -272,6 +279,9 @@ public:
 #if USE(FOUNDATION)
     template<typename T> void releaseSoon(RetainPtr<T>&&);
 #endif
+#if USE(GLIB)
+    void releaseSoon(std::unique_ptr<JSCGLibWrapperObject>&&);
+#endif
 
     JS_EXPORT_PRIVATE void registerWeakGCMap(WeakGCMapBase* weakGCMap);
     JS_EXPORT_PRIVATE void unregisterWeakGCMap(WeakGCMapBase* weakGCMap);
@@ -291,6 +301,16 @@ public:
     unsigned barrierThreshold() const { return m_barrierThreshold; }
     const unsigned* addressOfBarrierThreshold() const { return &m_barrierThreshold; }
 
+#if ENABLE(DFG_DOES_GC_VALIDATION)
+    bool expectDoesGC() const { return m_expectDoesGC; }
+    void setExpectDoesGC(bool value) { m_expectDoesGC = value; }
+    bool* addressOfExpectDoesGC() { return &m_expectDoesGC; }
+#else
+    bool expectDoesGC() const { UNREACHABLE_FOR_PLATFORM(); return true; }
+    void setExpectDoesGC(bool) { UNREACHABLE_FOR_PLATFORM(); }
+    bool* addressOfExpectDoesGC() { UNREACHABLE_FOR_PLATFORM(); return nullptr; }
+#endif
+
     // If true, the GC believes that the mutator is currently messing with the heap. We call this
     // "having heap access". The GC may block if the mutator is in this state. If false, the GC may
     // currently be doing things to the heap that make the heap unsafe to access for the mutator.
@@ -373,8 +393,11 @@ public:
 
     template<typename Func>
     void forEachSlotVisitor(const Func&);
+    unsigned numberOfSlotVisitors();
 
-    ThreadLocalCacheLayout& threadLocalCacheLayout() { return *m_threadLocalCacheLayout; }
+    Seconds totalGCTime() const { return m_totalGCTime; }
+
+    HashMap<JSImmutableButterfly*, JSString*> immutableButterflyToStringCache;
 
 private:
     friend class AllocatingScope;
@@ -400,7 +423,6 @@ private:
     friend class StochasticSpaceTimeMutatorScheduler;
     friend class SweepingScope;
     friend class IncrementalSweeper;
-    friend class HeapStatistics;
     friend class VM;
     friend class WeakSet;
 
@@ -504,7 +526,6 @@ private:
 
     void finalizeUnconditionalFinalizers();
 
-    void clearUnmarkedExecutables();
     void deleteUnmarkedCompiledCode();
     JS_EXPORT_PRIVATE void addToRememberedSet(const JSCell*);
     void updateAllocationLimits();
@@ -552,33 +573,41 @@ private:
 
     void setBonusVisitorTask(RefPtr<SharedTask<void(SlotVisitor&)>>);
 
+    static bool useGenerationalGC();
+    static bool shouldSweepSynchronously();
+
     const HeapType m_heapType;
+    MutatorState m_mutatorState { MutatorState::Running };
     const size_t m_ramSize;
     const size_t m_minBytesPerCycle;
-    size_t m_sizeAfterLastCollect;
-    size_t m_sizeAfterLastFullCollect;
-    size_t m_sizeBeforeLastFullCollect;
-    size_t m_sizeAfterLastEdenCollect;
-    size_t m_sizeBeforeLastEdenCollect;
+    size_t m_sizeAfterLastCollect { 0 };
+    size_t m_sizeAfterLastFullCollect { 0 };
+    size_t m_sizeBeforeLastFullCollect { 0 };
+    size_t m_sizeAfterLastEdenCollect { 0 };
+    size_t m_sizeBeforeLastEdenCollect { 0 };
 
-    size_t m_bytesAllocatedThisCycle;
-    size_t m_bytesAbandonedSinceLastFullCollect;
+    size_t m_bytesAllocatedThisCycle { 0 };
+    size_t m_bytesAbandonedSinceLastFullCollect { 0 };
     size_t m_maxEdenSize;
     size_t m_maxEdenSizeWhenCritical;
     size_t m_maxHeapSize;
-    bool m_shouldDoFullCollection;
-    size_t m_totalBytesVisited;
-    size_t m_totalBytesVisitedThisCycle;
+    size_t m_totalBytesVisited { 0 };
+    size_t m_totalBytesVisitedThisCycle { 0 };
     double m_incrementBalance { 0 };
 
-    std::optional<CollectionScope> m_collectionScope;
-    std::optional<CollectionScope> m_lastCollectionScope;
-    MutatorState m_mutatorState { MutatorState::Running };
+    bool m_shouldDoFullCollection { false };
+    Markable<CollectionScope, EnumMarkableTraits<CollectionScope>> m_collectionScope;
+    Markable<CollectionScope, EnumMarkableTraits<CollectionScope>> m_lastCollectionScope;
+    Lock m_raceMarkStackLock;
+#if ENABLE(DFG_DOES_GC_VALIDATION)
+    bool m_expectDoesGC { true };
+#endif
+
     StructureIDTable m_structureIDTable;
     MarkedSpace m_objectSpace;
     GCIncomingRefCountedSet<ArrayBuffer> m_arrayBuffers;
-    size_t m_extraMemorySize;
-    size_t m_deprecatedExtraMemorySize;
+    size_t m_extraMemorySize { 0 };
+    size_t m_deprecatedExtraMemorySize { 0 };
 
     HashSet<const JSCell*> m_copyingRememberedSet;
 
@@ -590,10 +619,7 @@ private:
     std::unique_ptr<SlotVisitor> m_collectorSlotVisitor;
     std::unique_ptr<SlotVisitor> m_mutatorSlotVisitor;
     std::unique_ptr<MarkStackArray> m_mutatorMarkStack;
-
-    Lock m_raceMarkStackLock;
     std::unique_ptr<MarkStackArray> m_raceMarkStack;
-
     std::unique_ptr<MarkingConstraintSet> m_constraintSet;
 
     // We pool the slot visitors used by parallel marking threads. It's useful to be able to
@@ -602,66 +628,57 @@ private:
     // them at the end.
     Vector<std::unique_ptr<SlotVisitor>> m_parallelSlotVisitors;
     Vector<SlotVisitor*> m_availableParallelSlotVisitors;
-    Lock m_parallelSlotVisitorLock;
 
     HandleSet m_handleSet;
     std::unique_ptr<CodeBlockSet> m_codeBlocks;
     std::unique_ptr<JITStubRoutineSet> m_jitStubRoutines;
     FinalizerOwner m_finalizerOwner;
 
-    bool m_isSafeToCollect;
+    Lock m_parallelSlotVisitorLock;
+    bool m_isSafeToCollect { false };
     bool m_isShuttingDown { false };
-
     bool m_mutatorShouldBeFenced { Options::forceFencedBarrier() };
+
     unsigned m_barrierThreshold { Options::forceFencedBarrier() ? tautologicalThreshold : blackThreshold };
 
     VM* m_vm;
-    Seconds m_lastFullGCLength;
-    Seconds m_lastEdenGCLength;
-
-    Vector<ExecutableBase*> m_executables;
+    Seconds m_lastFullGCLength { 10_ms };
+    Seconds m_lastEdenGCLength { 10_ms };
 
     Vector<WeakBlock*> m_logicallyEmptyWeakBlocks;
     size_t m_indexOfNextLogicallyEmptyWeakBlockToSweep { WTF::notFound };
 
     RefPtr<FullGCActivityCallback> m_fullActivityCallback;
     RefPtr<GCActivityCallback> m_edenActivityCallback;
-    RefPtr<IncrementalSweeper> m_sweeper;
-    RefPtr<StopIfNecessaryTimer> m_stopIfNecessaryTimer;
+    Ref<IncrementalSweeper> m_sweeper;
+    Ref<StopIfNecessaryTimer> m_stopIfNecessaryTimer;
 
     Vector<HeapObserver*> m_observers;
 
     Vector<HeapFinalizerCallback> m_heapFinalizerCallbacks;
 
-    unsigned m_deferralDepth;
-    bool m_didDeferGCWork { false };
-
     std::unique_ptr<HeapVerifier> m_verifier;
 
 #if USE(FOUNDATION)
     Vector<RetainPtr<CFTypeRef>> m_delayedReleaseObjects;
-    unsigned m_delayedReleaseRecursionCount;
+    unsigned m_delayedReleaseRecursionCount { 0 };
 #endif
+#if USE(GLIB)
+    Vector<std::unique_ptr<JSCGLibWrapperObject>> m_delayedReleaseObjects;
+    unsigned m_delayedReleaseRecursionCount { 0 };
+#endif
+    unsigned m_deferralDepth { 0 };
 
     HashSet<WeakGCMapBase*> m_weakGCMaps;
 
-    Lock m_visitRaceLock;
-
-    Lock m_markingMutex;
-    Condition m_markingConditionVariable;
     std::unique_ptr<MarkStackArray> m_sharedCollectorMarkStack;
     std::unique_ptr<MarkStackArray> m_sharedMutatorMarkStack;
     unsigned m_numberOfActiveParallelMarkers { 0 };
     unsigned m_numberOfWaitingParallelMarkers { 0 };
-    bool m_parallelMarkersShouldExit { false };
 
     ConcurrentPtrHashSet m_opaqueRoots;
-
     static const size_t s_blockFragmentLength = 32;
 
-    ListableHandler<WeakReferenceHarvester>::List m_weakReferenceHarvesters;
-    ListableHandler<UnconditionalFinalizer>::List m_unconditionalFinalizers;
-
     ParallelHelperClient m_helperClient;
     RefPtr<SharedTask<void(SlotVisitor&)>> m_bonusVisitorTask;
 
@@ -680,6 +697,10 @@ private:
     static const unsigned mutatorWaitingBit = 1u << 5u; // Allows the mutator to use this as a condition variable.
     Atomic<unsigned> m_worldState;
     bool m_worldIsStopped { false };
+    Lock m_visitRaceLock;
+    Lock m_markingMutex;
+    Condition m_markingConditionVariable;
+
     MonotonicTime m_beforeGC;
     MonotonicTime m_afterGC;
     MonotonicTime m_stopTime;
@@ -688,37 +709,42 @@ private:
     GCRequest m_currentRequest;
     Ticket m_lastServedTicket { 0 };
     Ticket m_lastGrantedTicket { 0 };
+
+    CollectorPhase m_lastPhase { CollectorPhase::NotRunning };
     CollectorPhase m_currentPhase { CollectorPhase::NotRunning };
     CollectorPhase m_nextPhase { CollectorPhase::NotRunning };
     bool m_threadShouldStop { false };
     bool m_threadIsStopping { false };
     bool m_mutatorDidRun { true };
+    bool m_didDeferGCWork { false };
+    bool m_shouldStopCollectingContinuously { false };
+
     uint64_t m_mutatorExecutionVersion { 0 };
     uint64_t m_phaseVersion { 0 };
     Box<Lock> m_threadLock;
-    RefPtr<AutomaticThreadCondition> m_threadCondition; // The mutator must not wait on this. It would cause a deadlock.
+    Ref<AutomaticThreadCondition> m_threadCondition; // The mutator must not wait on this. It would cause a deadlock.
     RefPtr<AutomaticThread> m_thread;
 
-#if PLATFORM(IOS)
-    unsigned m_precentAvailableMemoryCachedCallCount;
-    bool m_overCriticalMemoryThreshold;
-#endif
-
-    Lock m_collectContinuouslyLock;
-    Condition m_collectContinuouslyCondition;
-    bool m_shouldStopCollectingContinuously { false };
     RefPtr<WTF::Thread> m_collectContinuouslyThread { nullptr };
 
     MonotonicTime m_lastGCStartTime;
     MonotonicTime m_lastGCEndTime;
     MonotonicTime m_currentGCStartTime;
+    Seconds m_totalGCTime;
 
     uintptr_t m_barriersExecuted { 0 };
 
     CurrentThreadState* m_currentThreadState { nullptr };
     WTF::Thread* m_currentThread { nullptr }; // It's OK if this becomes a dangling pointer.
 
-    std::unique_ptr<ThreadLocalCacheLayout> m_threadLocalCacheLayout;
+#if PLATFORM(IOS_FAMILY)
+    unsigned m_precentAvailableMemoryCachedCallCount;
+    bool m_overCriticalMemoryThreshold;
+#endif
+
+    bool m_parallelMarkersShouldExit { false };
+    Lock m_collectContinuouslyLock;
+    Condition m_collectContinuouslyCondition;
 };
 
 } // namespace JSC
