@@ -171,6 +171,7 @@ void buildSizeClassTable(TableType& table, const SizeClassCons& cons, const Defa
             table[i] = entry;
         nextIndex = index + 1;
     }
+    ASSERT(MarkedSpace::sizeClassToIndex(MarkedSpace::largeCutoff - 1) < MarkedSpace::numSizeClasses);
     for (size_t i = nextIndex; i < MarkedSpace::numSizeClasses; ++i)
         table[i] = defaultCons(MarkedSpace::indexToSizeClass(i));
 }
@@ -196,8 +197,6 @@ void MarkedSpace::initializeSizeClassForStepSize()
 
 MarkedSpace::MarkedSpace(Heap* heap)
     : m_heap(heap)
-    , m_capacity(0)
-    , m_isIterating(false)
 {
     initializeSizeClassForStepSize();
 }
@@ -259,6 +258,7 @@ void MarkedSpace::sweepLargeAllocations()
 
 void MarkedSpace::prepareForAllocation()
 {
+    ASSERT(!mayBeGCThread() || m_heap->worldIsStopped());
     for (Subspace* subspace : m_subspaces)
         subspace->prepareForAllocation();
 
