@@ -41,10 +41,10 @@
 #include "ScriptCallStackFactory.h"
 #include "ScriptDebugServer.h"
 #include "ScriptObject.h"
-#include "ScriptValue.h"
 #include <wtf/JSONValues.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/Stopwatch.h>
+#include <wtf/text/StringConcatenateNumbers.h>
 #include <wtf/text/WTFString.h>
 
 namespace Inspector {
@@ -57,12 +57,11 @@ const char* InspectorDebuggerAgent::backtraceObjectGroup = "backtrace";
 // create objects in the same group.
 static String objectGroupForBreakpointAction(const ScriptBreakpointAction& action)
 {
-    static NeverDestroyed<String> objectGroup(MAKE_STATIC_STRING_IMPL("breakpoint-action-"));
-    return makeString(objectGroup.get(), String::number(action.identifier));
+    return makeString("breakpoint-action-", action.identifier);
 }
 
 InspectorDebuggerAgent::InspectorDebuggerAgent(AgentContext& context)
-    : InspectorAgentBase(ASCIILiteral("Debugger"))
+    : InspectorAgentBase("Debugger"_s)
     , m_injectedScriptManager(context.injectedScriptManager)
     , m_frontendDispatcher(std::make_unique<DebuggerFrontendDispatcher>(context.frontendRouter))
     , m_backendDispatcher(DebuggerBackendDispatcher::create(context.backendDispatcher, this))
@@ -144,7 +143,7 @@ void InspectorDebuggerAgent::setAsyncStackTraceDepth(ErrorString& errorString, i
         return;
 
     if (depth < 0) {
-        errorString = ASCIILiteral("depth must be a positive number.");
+        errorString = "depth must be a positive number."_s;
         return;
     }
 
@@ -174,7 +173,7 @@ void InspectorDebuggerAgent::setSuppressAllPauses(bool suppress)
 
 static RefPtr<JSON::Object> buildAssertPauseReason(const String& message)
 {
-    auto reason = Inspector::Protocol::Debugger::AssertPauseReason::create().release();
+    auto reason = Protocol::Debugger::AssertPauseReason::create().release();
     if (!message.isNull())
         reason->setMessage(message);
     return reason->openAccessors();
@@ -182,7 +181,7 @@ static RefPtr<JSON::Object> buildAssertPauseReason(const String& message)
 
 static RefPtr<JSON::Object> buildCSPViolationPauseReason(const String& directiveText)
 {
-    auto reason = Inspector::Protocol::Debugger::CSPViolationPauseReason::create()
+    auto reason = Protocol::Debugger::CSPViolationPauseReason::create()
         .setDirective(directiveText)
         .release();
     return reason->openAccessors();
@@ -195,7 +194,7 @@ RefPtr<JSON::Object> InspectorDebuggerAgent::buildBreakpointPauseReason(JSC::Bre
     if (it == m_debuggerBreakpointIdentifierToInspectorBreakpointIdentifier.end())
         return nullptr;
 
-    auto reason = Inspector::Protocol::Debugger::BreakpointPauseReason::create()
+    auto reason = Protocol::Debugger::BreakpointPauseReason::create()
         .setBreakpointId(it->value)
         .release();
     return reason->openAccessors();
@@ -309,7 +308,7 @@ void InspectorDebuggerAgent::didDispatchAsyncCall()
     auto& asyncStackTrace = it->value;
     asyncStackTrace->didDispatchAsyncCall();
 
-    m_currentAsyncCallIdentifier = std::nullopt;
+    m_currentAsyncCallIdentifier = WTF::nullopt;
 
     if (!asyncStackTrace->isPending())
         m_pendingAsyncCalls.remove(identifier);
@@ -318,16 +317,16 @@ void InspectorDebuggerAgent::didDispatchAsyncCall()
 static Ref<JSON::Object> buildObjectForBreakpointCookie(const String& url, int lineNumber, int columnNumber, const String& condition, RefPtr<JSON::Array>& actions, bool isRegex, bool autoContinue, unsigned ignoreCount)
 {
     Ref<JSON::Object> breakpointObject = JSON::Object::create();
-    breakpointObject->setString(ASCIILiteral("url"), url);
-    breakpointObject->setInteger(ASCIILiteral("lineNumber"), lineNumber);
-    breakpointObject->setInteger(ASCIILiteral("columnNumber"), columnNumber);
-    breakpointObject->setString(ASCIILiteral("condition"), condition);
-    breakpointObject->setBoolean(ASCIILiteral("isRegex"), isRegex);
-    breakpointObject->setBoolean(ASCIILiteral("autoContinue"), autoContinue);
-    breakpointObject->setInteger(ASCIILiteral("ignoreCount"), ignoreCount);
+    breakpointObject->setString("url"_s, url);
+    breakpointObject->setInteger("lineNumber"_s, lineNumber);
+    breakpointObject->setInteger("columnNumber"_s, columnNumber);
+    breakpointObject->setString("condition"_s, condition);
+    breakpointObject->setBoolean("isRegex"_s, isRegex);
+    breakpointObject->setBoolean("autoContinue"_s, autoContinue);
+    breakpointObject->setInteger("ignoreCount"_s, ignoreCount);
 
     if (actions)
-        breakpointObject->setArray(ASCIILiteral("actions"), actions);
+        breakpointObject->setArray("actions"_s, actions);
 
     return breakpointObject;
 }
@@ -343,19 +342,19 @@ static bool matches(const String& url, const String& pattern, bool isRegex)
 
 static bool breakpointActionTypeForString(const String& typeString, ScriptBreakpointActionType* output)
 {
-    if (typeString == Inspector::Protocol::InspectorHelpers::getEnumConstantValue(Inspector::Protocol::Debugger::BreakpointAction::Type::Log)) {
+    if (typeString == Protocol::InspectorHelpers::getEnumConstantValue(Protocol::Debugger::BreakpointAction::Type::Log)) {
         *output = ScriptBreakpointActionTypeLog;
         return true;
     }
-    if (typeString == Inspector::Protocol::InspectorHelpers::getEnumConstantValue(Inspector::Protocol::Debugger::BreakpointAction::Type::Evaluate)) {
+    if (typeString == Protocol::InspectorHelpers::getEnumConstantValue(Protocol::Debugger::BreakpointAction::Type::Evaluate)) {
         *output = ScriptBreakpointActionTypeEvaluate;
         return true;
     }
-    if (typeString == Inspector::Protocol::InspectorHelpers::getEnumConstantValue(Inspector::Protocol::Debugger::BreakpointAction::Type::Sound)) {
+    if (typeString == Protocol::InspectorHelpers::getEnumConstantValue(Protocol::Debugger::BreakpointAction::Type::Sound)) {
         *output = ScriptBreakpointActionTypeSound;
         return true;
     }
-    if (typeString == Inspector::Protocol::InspectorHelpers::getEnumConstantValue(Inspector::Protocol::Debugger::BreakpointAction::Type::Probe)) {
+    if (typeString == Protocol::InspectorHelpers::getEnumConstantValue(Protocol::Debugger::BreakpointAction::Type::Probe)) {
         *output = ScriptBreakpointActionTypeProbe;
         return true;
     }
@@ -377,29 +376,29 @@ bool InspectorDebuggerAgent::breakpointActionsFromProtocol(ErrorString& errorStr
         RefPtr<JSON::Value> value = actions->get(i);
         RefPtr<JSON::Object> object;
         if (!value->asObject(object)) {
-            errorString = ASCIILiteral("BreakpointAction of incorrect type, expected object");
+            errorString = "BreakpointAction of incorrect type, expected object"_s;
             return false;
         }
 
         String typeString;
-        if (!object->getString(ASCIILiteral("type"), typeString)) {
-            errorString = ASCIILiteral("BreakpointAction had type missing");
+        if (!object->getString("type"_s, typeString)) {
+            errorString = "BreakpointAction had type missing"_s;
             return false;
         }
 
         ScriptBreakpointActionType type;
         if (!breakpointActionTypeForString(typeString, &type)) {
-            errorString = ASCIILiteral("BreakpointAction had unknown type");
+            errorString = "BreakpointAction had unknown type"_s;
             return false;
         }
 
         // Specifying an identifier is optional. They are used to correlate probe samples
         // in the frontend across multiple backend probe actions and segregate object groups.
         int identifier = 0;
-        object->getInteger(ASCIILiteral("id"), identifier);
+        object->getInteger("id"_s, identifier);
 
         String data;
-        object->getString(ASCIILiteral("data"), data);
+        object->getString("data"_s, data);
 
         result->append(ScriptBreakpointAction(type, identifier, data));
     }
@@ -407,11 +406,11 @@ bool InspectorDebuggerAgent::breakpointActionsFromProtocol(ErrorString& errorStr
     return true;
 }
 
-static RefPtr<Inspector::Protocol::Debugger::Location> buildDebuggerLocation(const JSC::Breakpoint& breakpoint)
+static RefPtr<Protocol::Debugger::Location> buildDebuggerLocation(const JSC::Breakpoint& breakpoint)
 {
     ASSERT(breakpoint.resolved);
 
-    auto location = Inspector::Protocol::Debugger::Location::create()
+    auto location = Protocol::Debugger::Location::create()
         .setScriptId(String::number(breakpoint.sourceID))
         .setLineNumber(breakpoint.line)
         .release();
@@ -423,23 +422,23 @@ static RefPtr<Inspector::Protocol::Debugger::Location> buildDebuggerLocation(con
 static bool parseLocation(ErrorString& errorString, const JSON::Object& location, JSC::SourceID& sourceID, unsigned& lineNumber, unsigned& columnNumber)
 {
     String scriptIDStr;
-    if (!location.getString(ASCIILiteral("scriptId"), scriptIDStr) || !location.getInteger(ASCIILiteral("lineNumber"), lineNumber)) {
+    if (!location.getString("scriptId"_s, scriptIDStr) || !location.getInteger("lineNumber"_s, lineNumber)) {
         sourceID = JSC::noSourceID;
-        errorString = ASCIILiteral("scriptId and lineNumber are required.");
+        errorString = "scriptId and lineNumber are required."_s;
         return false;
     }
 
     sourceID = scriptIDStr.toIntPtr();
     columnNumber = 0;
-    location.getInteger(ASCIILiteral("columnNumber"), columnNumber);
+    location.getInteger("columnNumber"_s, columnNumber);
     return true;
 }
 
-void InspectorDebuggerAgent::setBreakpointByUrl(ErrorString& errorString, int lineNumber, const String* const optionalURL, const String* const optionalURLRegex, const int* const optionalColumnNumber, const JSON::Object* options, Inspector::Protocol::Debugger::BreakpointId* outBreakpointIdentifier, RefPtr<JSON::ArrayOf<Inspector::Protocol::Debugger::Location>>& locations)
+void InspectorDebuggerAgent::setBreakpointByUrl(ErrorString& errorString, int lineNumber, const String* optionalURL, const String* optionalURLRegex, const int* optionalColumnNumber, const JSON::Object* options, Protocol::Debugger::BreakpointId* outBreakpointIdentifier, RefPtr<JSON::ArrayOf<Protocol::Debugger::Location>>& locations)
 {
-    locations = JSON::ArrayOf<Inspector::Protocol::Debugger::Location>::create();
+    locations = JSON::ArrayOf<Protocol::Debugger::Location>::create();
     if (!optionalURL == !optionalURLRegex) {
-        errorString = ASCIILiteral("Either url or urlRegex must be specified.");
+        errorString = "Either url or urlRegex must be specified."_s;
         return;
     }
 
@@ -447,9 +446,9 @@ void InspectorDebuggerAgent::setBreakpointByUrl(ErrorString& errorString, int li
     int columnNumber = optionalColumnNumber ? *optionalColumnNumber : 0;
     bool isRegex = optionalURLRegex;
 
-    String breakpointIdentifier = (isRegex ? "/" + url + "/" : url) + ':' + String::number(lineNumber) + ':' + String::number(columnNumber);
+    String breakpointIdentifier = makeString(isRegex ? "/" : "", url, isRegex ? "/:" : ":", lineNumber, ':', columnNumber);
     if (m_javaScriptBreakpoints.contains(breakpointIdentifier)) {
-        errorString = ASCIILiteral("Breakpoint at specified location already exists.");
+        errorString = "Breakpoint at specified location already exists."_s;
         return;
     }
 
@@ -458,10 +457,10 @@ void InspectorDebuggerAgent::setBreakpointByUrl(ErrorString& errorString, int li
     unsigned ignoreCount = 0;
     RefPtr<JSON::Array> actions;
     if (options) {
-        options->getString(ASCIILiteral("condition"), condition);
-        options->getBoolean(ASCIILiteral("autoContinue"), autoContinue);
-        options->getArray(ASCIILiteral("actions"), actions);
-        options->getInteger(ASCIILiteral("ignoreCount"), ignoreCount);
+        options->getString("condition"_s, condition);
+        options->getBoolean("autoContinue"_s, autoContinue);
+        options->getArray("actions"_s, actions);
+        options->getInteger("ignoreCount"_s, ignoreCount);
     }
 
     BreakpointActions breakpointActions;
@@ -496,7 +495,7 @@ void InspectorDebuggerAgent::setBreakpointByUrl(ErrorString& errorString, int li
     *outBreakpointIdentifier = breakpointIdentifier;
 }
 
-void InspectorDebuggerAgent::setBreakpoint(ErrorString& errorString, const JSON::Object& location, const JSON::Object* options, Inspector::Protocol::Debugger::BreakpointId* outBreakpointIdentifier, RefPtr<Inspector::Protocol::Debugger::Location>& actualLocation)
+void InspectorDebuggerAgent::setBreakpoint(ErrorString& errorString, const JSON::Object& location, const JSON::Object* options, Protocol::Debugger::BreakpointId* outBreakpointIdentifier, RefPtr<Protocol::Debugger::Location>& actualLocation)
 {
     JSC::SourceID sourceID;
     unsigned lineNumber;
@@ -509,10 +508,10 @@ void InspectorDebuggerAgent::setBreakpoint(ErrorString& errorString, const JSON:
     unsigned ignoreCount = 0;
     RefPtr<JSON::Array> actions;
     if (options) {
-        options->getString(ASCIILiteral("condition"), condition);
-        options->getBoolean(ASCIILiteral("autoContinue"), autoContinue);
-        options->getArray(ASCIILiteral("actions"), actions);
-        options->getInteger(ASCIILiteral("ignoreCount"), ignoreCount);
+        options->getString("condition"_s, condition);
+        options->getBoolean("autoContinue"_s, autoContinue);
+        options->getArray("actions"_s, actions);
+        options->getInteger("ignoreCount"_s, ignoreCount);
     }
 
     BreakpointActions breakpointActions;
@@ -521,7 +520,7 @@ void InspectorDebuggerAgent::setBreakpoint(ErrorString& errorString, const JSON:
 
     auto scriptIterator = m_scripts.find(sourceID);
     if (scriptIterator == m_scripts.end()) {
-        errorString = ASCIILiteral("No script for id: ") + String::number(sourceID);
+        errorString = makeString("No script for id: "_s, sourceID);
         return;
     }
 
@@ -529,18 +528,18 @@ void InspectorDebuggerAgent::setBreakpoint(ErrorString& errorString, const JSON:
     JSC::Breakpoint breakpoint(sourceID, lineNumber, columnNumber, condition, autoContinue, ignoreCount);
     resolveBreakpoint(script, breakpoint);
     if (!breakpoint.resolved) {
-        errorString = ASCIILiteral("Could not resolve breakpoint");
+        errorString = "Could not resolve breakpoint"_s;
         return;
     }
 
     bool existing;
     setBreakpoint(breakpoint, existing);
     if (existing) {
-        errorString = ASCIILiteral("Breakpoint at specified location already exists");
+        errorString = "Breakpoint at specified location already exists"_s;
         return;
     }
 
-    String breakpointIdentifier = String::number(sourceID) + ':' + String::number(breakpoint.line) + ':' + String::number(breakpoint.column);
+    String breakpointIdentifier = makeString(sourceID, ':', breakpoint.line, ':', breakpoint.column);
     ScriptBreakpoint scriptBreakpoint(breakpoint.line, breakpoint.column, condition, breakpointActions, autoContinue, ignoreCount);
     didSetBreakpoint(breakpoint, breakpointIdentifier, scriptBreakpoint);
 
@@ -624,7 +623,7 @@ void InspectorDebuggerAgent::continueToLocation(ErrorString& errorString, const
     if (scriptIterator == m_scripts.end()) {
         m_scriptDebugServer.continueProgram();
         m_frontendDispatcher->resumed();
-        errorString = ASCIILiteral("No script for id: ") + String::number(sourceID);
+        errorString = makeString("No script for id: "_s, sourceID);
         return;
     }
 
@@ -637,7 +636,7 @@ void InspectorDebuggerAgent::continueToLocation(ErrorString& errorString, const
     if (!breakpoint.resolved) {
         m_scriptDebugServer.continueProgram();
         m_frontendDispatcher->resumed();
-        errorString = ASCIILiteral("Could not resolve breakpoint");
+        errorString = "Could not resolve breakpoint"_s;
         return;
     }
 
@@ -660,12 +659,12 @@ void InspectorDebuggerAgent::continueToLocation(ErrorString& errorString, const
     m_scriptDebugServer.continueProgram();
 }
 
-void InspectorDebuggerAgent::searchInContent(ErrorString& error, const String& scriptIDStr, const String& query, const bool* optionalCaseSensitive, const bool* optionalIsRegex, RefPtr<JSON::ArrayOf<Inspector::Protocol::GenericTypes::SearchMatch>>& results)
+void InspectorDebuggerAgent::searchInContent(ErrorString& error, const String& scriptIDStr, const String& query, const bool* optionalCaseSensitive, const bool* optionalIsRegex, RefPtr<JSON::ArrayOf<Protocol::GenericTypes::SearchMatch>>& results)
 {
     JSC::SourceID sourceID = scriptIDStr.toIntPtr();
     auto it = m_scripts.find(sourceID);
     if (it == m_scripts.end()) {
-        error = ASCIILiteral("No script for id: ") + scriptIDStr;
+        error = makeString("No script for id: "_s, scriptIDStr);
         return;
     }
 
@@ -681,14 +680,14 @@ void InspectorDebuggerAgent::getScriptSource(ErrorString& error, const String& s
     if (it != m_scripts.end())
         *scriptSource = it->value.source;
     else
-        error = ASCIILiteral("No script for id: ") + scriptIDStr;
+        error = makeString("No script for id: "_s, scriptIDStr);
 }
 
-void InspectorDebuggerAgent::getFunctionDetails(ErrorString& errorString, const String& functionId, RefPtr<Inspector::Protocol::Debugger::FunctionDetails>& details)
+void InspectorDebuggerAgent::getFunctionDetails(ErrorString& errorString, const String& functionId, RefPtr<Protocol::Debugger::FunctionDetails>& details)
 {
     InjectedScript injectedScript = m_injectedScriptManager.injectedScriptForObjectId(functionId);
     if (injectedScript.hasNoValue()) {
-        errorString = ASCIILiteral("Function object id is obsolete");
+        errorString = "Function object id is obsolete"_s;
         return;
     }
 
@@ -729,7 +728,7 @@ void InspectorDebuggerAgent::pause(ErrorString&)
 void InspectorDebuggerAgent::resume(ErrorString& errorString)
 {
     if (!m_pausedScriptState && !m_javaScriptPauseScheduled) {
-        errorString = ASCIILiteral("Was not paused or waiting to pause");
+        errorString = "Was not paused or waiting to pause"_s;
         return;
     }
 
@@ -813,13 +812,13 @@ void InspectorDebuggerAgent::setPauseOnExceptions(ErrorString& errorString, cons
     else if (stringPauseState == "uncaught")
         pauseState = JSC::Debugger::PauseOnUncaughtExceptions;
     else {
-        errorString = ASCIILiteral("Unknown pause on exceptions mode: ") + stringPauseState;
+        errorString = makeString("Unknown pause on exceptions mode: "_s, stringPauseState);
         return;
     }
 
     m_scriptDebugServer.setPauseOnExceptionsState(static_cast<JSC::Debugger::PauseOnExceptionsState>(pauseState));
     if (m_scriptDebugServer.pauseOnExceptionsState() != pauseState)
-        errorString = ASCIILiteral("Internal error. Could not change pause on exceptions state");
+        errorString = "Internal error. Could not change pause on exceptions state"_s;
 }
 
 void InspectorDebuggerAgent::setPauseOnAssertions(ErrorString&, bool enabled)
@@ -827,40 +826,34 @@ void InspectorDebuggerAgent::setPauseOnAssertions(ErrorString&, bool enabled)
     m_pauseOnAssertionFailures = enabled;
 }
 
-void InspectorDebuggerAgent::evaluateOnCallFrame(ErrorString& errorString, const String& callFrameId, const String& expression, const String* const objectGroup, const bool* const includeCommandLineAPI, const bool* const doNotPauseOnExceptionsAndMuteConsole, const bool* const returnByValue, const bool* generatePreview, const bool* saveResult, RefPtr<Inspector::Protocol::Runtime::RemoteObject>& result, Inspector::Protocol::OptOutput<bool>* out_wasThrown, Inspector::Protocol::OptOutput<int>* out_savedResultIndex)
+void InspectorDebuggerAgent::evaluateOnCallFrame(ErrorString& errorString, const String& callFrameId, const String& expression, const String* objectGroup, const bool* includeCommandLineAPI, const bool* doNotPauseOnExceptionsAndMuteConsole, const bool* returnByValue, const bool* generatePreview, const bool* saveResult, RefPtr<Protocol::Runtime::RemoteObject>& result, Optional<bool>& wasThrown, Optional<int>& savedResultIndex)
 {
-    if (m_currentCallStack.hasNoValue()) {
-        errorString = ASCIILiteral("Not paused");
+    if (!m_currentCallStack) {
+        errorString = "Not paused"_s;
         return;
     }
 
     InjectedScript injectedScript = m_injectedScriptManager.injectedScriptForObjectId(callFrameId);
     if (injectedScript.hasNoValue()) {
-        errorString = ASCIILiteral("Could not find InjectedScript for callFrameId");
+        errorString = "Could not find InjectedScript for callFrameId"_s;
         return;
     }
 
-    JSC::Debugger::PauseOnExceptionsState previousPauseOnExceptionsState = m_scriptDebugServer.pauseOnExceptionsState();
-    if (doNotPauseOnExceptionsAndMuteConsole ? *doNotPauseOnExceptionsAndMuteConsole : false) {
-        if (previousPauseOnExceptionsState != JSC::Debugger::DontPauseOnExceptions)
+    auto pauseState = m_scriptDebugServer.pauseOnExceptionsState();
+    bool pauseAndMute = doNotPauseOnExceptionsAndMuteConsole && *doNotPauseOnExceptionsAndMuteConsole;
+    if (pauseAndMute) {
+        if (pauseState != JSC::Debugger::DontPauseOnExceptions)
             m_scriptDebugServer.setPauseOnExceptionsState(JSC::Debugger::DontPauseOnExceptions);
         muteConsole();
     }
 
-    // FIXME: remove this bridging code when generated protocol commands no longer use OptOutput<T>.
-    bool wasThrown;
-    std::optional<int> savedResultIndex;
+    injectedScript.evaluateOnCallFrame(errorString, m_currentCallStack.get(), callFrameId, expression,
+        objectGroup ? *objectGroup : emptyString(), includeCommandLineAPI && *includeCommandLineAPI, returnByValue && *returnByValue, generatePreview && *generatePreview, saveResult && *saveResult,
+        result, wasThrown, savedResultIndex);
 
-    injectedScript.evaluateOnCallFrame(errorString, m_currentCallStack, callFrameId, expression, objectGroup ? *objectGroup : "", includeCommandLineAPI ? *includeCommandLineAPI : false, returnByValue ? *returnByValue : false, generatePreview ? *generatePreview : false, saveResult ? *saveResult : false, result, wasThrown, savedResultIndex);
-
-    *out_wasThrown = wasThrown;
-    if (savedResultIndex.has_value())
-        *out_savedResultIndex = savedResultIndex.value();
-
-    if (doNotPauseOnExceptionsAndMuteConsole ? *doNotPauseOnExceptionsAndMuteConsole : false) {
+    if (pauseAndMute) {
         unmuteConsole();
-        if (m_scriptDebugServer.pauseOnExceptionsState() != previousPauseOnExceptionsState)
-            m_scriptDebugServer.setPauseOnExceptionsState(previousPauseOnExceptionsState);
+        m_scriptDebugServer.setPauseOnExceptionsState(pauseState);
     }
 }
 
@@ -874,13 +867,13 @@ void InspectorDebuggerAgent::scriptExecutionBlockedByCSP(const String& directive
         breakProgram(DebuggerFrontendDispatcher::Reason::CSPViolation, buildCSPViolationPauseReason(directiveText));
 }
 
-Ref<JSON::ArrayOf<Inspector::Protocol::Debugger::CallFrame>> InspectorDebuggerAgent::currentCallFrames(const InjectedScript& injectedScript)
+Ref<JSON::ArrayOf<Protocol::Debugger::CallFrame>> InspectorDebuggerAgent::currentCallFrames(const InjectedScript& injectedScript)
 {
     ASSERT(!injectedScript.hasNoValue());
     if (injectedScript.hasNoValue())
-        return JSON::ArrayOf<Inspector::Protocol::Debugger::CallFrame>::create();
+        return JSON::ArrayOf<Protocol::Debugger::CallFrame>::create();
 
-    return injectedScript.wrapCallFrames(m_currentCallStack);
+    return injectedScript.wrapCallFrames(m_currentCallStack.get());
 }
 
 String InspectorDebuggerAgent::sourceMapURLForScript(const Script& script)
@@ -888,6 +881,17 @@ String InspectorDebuggerAgent::sourceMapURLForScript(const Script& script)
     return script.sourceMappingURL;
 }
 
+void InspectorDebuggerAgent::setPauseForInternalScripts(ErrorString&, bool shouldPause)
+{
+    if (shouldPause == m_pauseForInternalScripts)
+        return;
+
+    m_pauseForInternalScripts = shouldPause;
+
+    if (m_pauseForInternalScripts)
+        m_scriptDebugServer.clearBlacklist();
+}
+
 static bool isWebKitInjectedScript(const String& sourceURL)
 {
     return sourceURL.startsWith("__InjectedScript_") && sourceURL.endsWith(".js");
@@ -909,7 +913,7 @@ void InspectorDebuggerAgent::didParseSource(JSC::SourceID sourceID, const Script
 
     m_scripts.set(sourceID, script);
 
-    if (hasSourceURL && isWebKitInjectedScript(sourceURL))
+    if (hasSourceURL && isWebKitInjectedScript(sourceURL) && !m_pauseForInternalScripts)
         m_scriptDebugServer.addToBlacklist(sourceID);
 
     String scriptURLForBreakpoints = hasSourceURL ? script.sourceURL : script.url;
@@ -921,20 +925,20 @@ void InspectorDebuggerAgent::didParseSource(JSC::SourceID sourceID, const Script
 
         bool isRegex;
         String url;
-        breakpointObject->getBoolean(ASCIILiteral("isRegex"), isRegex);
-        breakpointObject->getString(ASCIILiteral("url"), url);
+        breakpointObject->getBoolean("isRegex"_s, isRegex);
+        breakpointObject->getString("url"_s, url);
         if (!matches(scriptURLForBreakpoints, url, isRegex))
             continue;
 
         ScriptBreakpoint scriptBreakpoint;
-        breakpointObject->getInteger(ASCIILiteral("lineNumber"), scriptBreakpoint.lineNumber);
-        breakpointObject->getInteger(ASCIILiteral("columnNumber"), scriptBreakpoint.columnNumber);
-        breakpointObject->getString(ASCIILiteral("condition"), scriptBreakpoint.condition);
-        breakpointObject->getBoolean(ASCIILiteral("autoContinue"), scriptBreakpoint.autoContinue);
-        breakpointObject->getInteger(ASCIILiteral("ignoreCount"), scriptBreakpoint.ignoreCount);
+        breakpointObject->getInteger("lineNumber"_s, scriptBreakpoint.lineNumber);
+        breakpointObject->getInteger("columnNumber"_s, scriptBreakpoint.columnNumber);
+        breakpointObject->getString("condition"_s, scriptBreakpoint.condition);
+        breakpointObject->getBoolean("autoContinue"_s, scriptBreakpoint.autoContinue);
+        breakpointObject->getInteger("ignoreCount"_s, scriptBreakpoint.ignoreCount);
         ErrorString errorString;
         RefPtr<JSON::Array> actions;
-        breakpointObject->getArray(ASCIILiteral("actions"), actions);
+        breakpointObject->getArray("actions"_s, actions);
         if (!breakpointActionsFromProtocol(errorString, actions, &scriptBreakpoint.actions)) {
             ASSERT_NOT_REACHED();
             continue;
@@ -1010,7 +1014,7 @@ void InspectorDebuggerAgent::didPause(JSC::ExecState& scriptState, JSC::JSValue
     m_conditionToDispatchResumed = ShouldDispatchResumed::No;
     m_enablePauseWhenIdle = false;
 
-    RefPtr<Inspector::Protocol::Console::StackTrace> asyncStackTrace;
+    RefPtr<Protocol::Console::StackTrace> asyncStackTrace;
     if (m_currentAsyncCallIdentifier) {
         auto it = m_pendingAsyncCalls.find(m_currentAsyncCallIdentifier.value());
         if (it != m_pendingAsyncCalls.end())
@@ -1046,7 +1050,7 @@ void InspectorDebuggerAgent::breakpointActionProbe(JSC::ExecState& scriptState,
         .setProbeId(action.identifier)
         .setBatchId(batchId)
         .setSampleId(sampleId)
-        .setTimestamp(m_injectedScriptManager.inspectorEnvironment().executionStopwatch()->elapsedTime())
+        .setTimestamp(m_injectedScriptManager.inspectorEnvironment().executionStopwatch()->elapsedTime().seconds())
         .setPayload(WTFMove(payload))
         .release();
     m_frontendDispatcher->didSampleProbe(WTFMove(result));
@@ -1126,7 +1130,7 @@ void InspectorDebuggerAgent::didClearGlobalObject()
 bool InspectorDebuggerAgent::assertPaused(ErrorString& errorString)
 {
     if (!m_pausedScriptState) {
-        errorString = ASCIILiteral("Can only perform operation while paused.");
+        errorString = "Can only perform operation while paused."_s;
         return false;
     }
 
@@ -1150,7 +1154,7 @@ void InspectorDebuggerAgent::clearExceptionValue()
 void InspectorDebuggerAgent::clearAsyncStackTraceData()
 {
     m_pendingAsyncCalls.clear();
-    m_currentAsyncCallIdentifier = std::nullopt;
+    m_currentAsyncCallIdentifier = WTF::nullopt;
 
     didClearAsyncStackTraceData();
 }
