@@ -34,11 +34,11 @@ require "risc"
 #  x2 => t2, a2, r2
 #  x3 => t3, a3, r3
 #  x6 =>            (callee-save scratch)
-#  x7 => cfr        (ARMv7 only)
+#  x7 => cfr
 #  x8 => t4         (callee-save)
 #  x9 => t5         (callee-save)
 # x10 =>            (callee-save scratch)
-# x11 => cfr        (ARM and ARMv7 traditional)
+# x11 => cfr, csr0  (callee-save, metadataTable)
 # x12 =>            (callee-save scratch)
 #  lr => lr
 #  sp => sp
@@ -55,28 +55,6 @@ require "risc"
 # d6 =>              (scratch)
 # d7 =>              (scratch)
 
-def isARMv7
-    case $activeBackend
-    when "ARMv7"
-        true
-    when "ARMv7_TRADITIONAL", "ARM"
-        false
-    else
-        raise "bad value for $activeBackend: #{$activeBackend}"
-    end
-end
-
-def isARMv7Traditional
-    case $activeBackend
-    when "ARMv7_TRADITIONAL"
-        true
-    when "ARMv7", "ARM"
-        false
-    else
-        raise "bad value for $activeBackend: #{$activeBackend}"
-    end
-end
-
 class Node
     def armSingle
         doubleOperand = armOperand
@@ -104,13 +82,11 @@ def armMoveImmediate(value, register)
         $asm.puts "mov #{register.armOperand}, \##{value}"
     elsif (~value) >= 0 && (~value) < 256
         $asm.puts "mvn #{register.armOperand}, \##{~value}"
-    elsif isARMv7 or isARMv7Traditional
+    else
         $asm.puts "movw #{register.armOperand}, \##{value & 0xffff}"
         if (value & 0xffff0000) != 0
             $asm.puts "movt #{register.armOperand}, \##{(value >> 16) & 0xffff}"
         end
-    else
-        $asm.puts "ldr #{register.armOperand}, =#{value}"
     end
 end
 
@@ -132,7 +108,9 @@ class RegisterID
         when "t5"
             "r9"
         when "cfr"
-            isARMv7 ?  "r7" : "r11"
+            "r7"
+        when "csr0"
+            "r11"
         when "lr"
             "lr"
         when "sp"
@@ -220,27 +198,96 @@ end
 # Actual lowering code follows.
 #
 
-class Sequence
-    def getModifiedListARM
-        raise unless $activeBackend == "ARM"
-        getModifiedListARMCommon
-    end
+def armOpcodeReversedOperands(opcode)
+    m = /\Ab[ipb]/.match(opcode)
 
+    operation =
+        case m.post_match
+        when "eq" then "eq"
+        when "neq" then "neq"
+        when "a" then "b"
+        when "aeq" then "beq"
+        when "b" then "a"
+        when "beq" then "aeq"
+        when "gt" then "lt"
+        when "gteq" then "lteq"
+        when "lt" then "gt"
+        when "lteq" then "gteq"
+        else
+            raise "unknown operation #{m.post_match}"
+        end
+
+    "#{m[0]}#{operation}"
+end
+
+def armLowerStackPointerInComparison(list)
+    newList = []
+    list.each {
+        | node |
+        if node.is_a? Instruction
+            case node.opcode
+            when "bieq", "bpeq", "bbeq",
+                "bineq", "bpneq", "bbneq",
+                "bia", "bpa", "bba",
+                "biaeq", "bpaeq", "bbaeq",
+                "bib", "bpb", "bbb",
+                "bibeq", "bpbeq", "bbbeq",
+                "bigt", "bpgt", "bbgt",
+                "bigteq", "bpgteq", "bbgteq",
+                "bilt", "bplt", "bblt",
+                "bilteq", "bplteq", "bblteq"
+                if node.operands[1].is_a?(RegisterID) && node.operands[1].name == "sp"
+                    newList << Instruction.new(codeOrigin, armOpcodeReversedOperands(node.opcode), [node.operands[1], node.operands[0]] + node.operands[2..-1])
+                else
+                    newList << node
+                end
+            else
+                newList << node
+            end
+        else
+            newList << node
+        end
+    }
+    newList
+end
+
+def armLowerLabelReferences(list)
+    newList = []
+    list.each {
+        | node |
+        if node.is_a? Instruction
+            case node.opcode
+            when "leai", "leap", "leaq"
+                labelRef = node.operands[0]
+                if labelRef.is_a? LabelReference
+                    raise unless labelRef.offset == 0
+                    tmp = Tmp.new(node.codeOrigin, :gpr)
+                    newList << Instruction.new(codeOrigin, "globaladdr", [LabelReference.new(node.codeOrigin, labelRef.label), node.operands[1], tmp])
+                else
+                    newList << node
+                end
+            else
+                newList << node
+            end
+        else
+            newList << node
+        end
+    }
+    newList
+end
+
+class Sequence
     def getModifiedListARMv7
         raise unless $activeBackend == "ARMv7"
         getModifiedListARMCommon
     end
 
-    def getModifiedListARMv7_TRADITIONAL
-        raise unless $activeBackend == "ARMv7_TRADITIONAL"
-        getModifiedListARMCommon
-    end
-
     def getModifiedListARMCommon
         result = @list
         result = riscLowerSimpleBranchOps(result)
         result = riscLowerHardBranchOps(result)
         result = riscLowerShiftOps(result)
+        result = armLowerLabelReferences(result)
         result = riscLowerMalformedAddresses(result) {
             | node, address |
             if address.is_a? BaseIndex
@@ -258,6 +305,7 @@ class Sequence
         result = riscLowerRegisterReuse(result)
         result = assignRegistersToTemporaries(result, :gpr, ARM_EXTRA_GPRS)
         result = assignRegistersToTemporaries(result, :fpr, ARM_EXTRA_FPRS)
+        result = armLowerStackPointerInComparison(result)
         return result
     end
 end
@@ -332,21 +380,11 @@ def emitArmTestSet(operands, code)
 end
 
 class Instruction
-    def lowerARM
-        raise unless $activeBackend == "ARM"
-        lowerARMCommon
-    end
-
     def lowerARMv7
         raise unless $activeBackend == "ARMv7"
         lowerARMCommon
     end
 
-    def lowerARMv7_TRADITIONAL
-        raise unless $activeBackend == "ARMv7_TRADITIONAL"
-        lowerARMCommon
-    end
-
     def lowerARMCommon
         case opcode
         when "addi", "addp", "addis", "addps"
@@ -406,7 +444,7 @@ class Instruction
             $asm.puts "str #{armOperands(operands)}"
         when "loadb"
             $asm.puts "ldrb #{armFlippedOperands(operands)}"
-        when "loadbs"
+        when "loadbs", "loadbsp"
             $asm.puts "ldrsb.w #{armFlippedOperands(operands)}"
         when "storeb"
             $asm.puts "strb #{armOperands(operands)}"
@@ -560,9 +598,6 @@ class Instruction
             else
                 $asm.puts "mov pc, #{operands[0].armOperand}"
             end
-            if not isARMv7 and not isARMv7Traditional
-                $asm.puts ".ltorg"
-            end
         when "call"
             if operands[0].label?
                 if OS_DARWIN
@@ -628,6 +663,28 @@ class Instruction
             $asm.puts "dmb sy"
         when "clrbp"
             $asm.puts "bic #{operands[2].armOperand}, #{operands[0].armOperand}, #{operands[1].armOperand}"
+        when "globaladdr"
+            labelRef = operands[0]
+            dest = operands[1]
+            temp = operands[2]
+
+            uid = $asm.newUID
+            gotLabel = Assembler.localLabelReference("offlineasm_arm_got_#{uid}")
+            offsetLabel = Assembler.localLabelReference("offlineasm_arm_got_offset_#{uid}")
+
+            $asm.puts "ldr #{dest.armOperand}, #{gotLabel}"
+            $asm.puts "ldr #{temp.armOperand}, #{gotLabel}+4"
+            $asm.puts "#{offsetLabel}:"
+            $asm.puts "add #{dest.armOperand}, pc, #{dest.armOperand}"
+            $asm.puts "ldr #{dest.armOperand}, [#{dest.armOperand}, #{temp.armOperand}]"
+
+            offset = 4
+
+            $asm.deferNextLabelAction {
+                $asm.puts "#{gotLabel}:"
+                $asm.puts ".word _GLOBAL_OFFSET_TABLE_-(#{offsetLabel}+#{offset})"
+                $asm.puts ".word #{labelRef.asmLabel}(GOT)"
+            }
         else
             lowerDefault
         end
