@@ -284,16 +284,16 @@ ALWAYS_INLINE uint32_t jsMapHash(ExecState* exec, VM& vm, JSValue value)
     return wangsInt64Hash(JSValue::encode(value));
 }
 
-ALWAYS_INLINE std::optional<uint32_t> concurrentJSMapHash(JSValue key)
+ALWAYS_INLINE Optional<uint32_t> concurrentJSMapHash(JSValue key)
 {
     key = normalizeMapKey(key);
     if (key.isString()) {
         JSString* string = asString(key);
         if (string->length() > 10 * 1024)
-            return std::nullopt;
+            return WTF::nullopt;
         const StringImpl* impl = string->tryGetValueImpl();
         if (!impl)
-            return std::nullopt;
+            return WTF::nullopt;
         return impl->concurrentHash();
     }
 
@@ -346,7 +346,7 @@ public:
 
     static void visitChildren(JSCell*, SlotVisitor&);
 
-    static size_t estimatedSize(JSCell*);
+    static size_t estimatedSize(JSCell*, VM&);
 
     HashMapImpl(VM& vm, Structure* structure)
         : Base(vm, structure)
