@@ -51,22 +51,20 @@ inline JSArrayBufferView* speciesConstruct(ExecState* exec, JSObject* exemplar,
     JSValue constructor = exemplar->get(exec, vm.propertyNames->constructor);
     RETURN_IF_EXCEPTION(scope, nullptr);
 
-    if (constructor.isUndefined()) {
-        scope.release();
-        return defaultConstructor();
-    }
+    if (constructor.isUndefined())
+        RELEASE_AND_RETURN(scope, defaultConstructor());
+
     if (!constructor.isObject()) {
-        throwTypeError(exec, scope, ASCIILiteral("constructor Property should not be null"));
+        throwTypeError(exec, scope, "constructor Property should not be null"_s);
         return nullptr;
     }
 
     JSValue species = constructor.get(exec, vm.propertyNames->speciesSymbol);
     RETURN_IF_EXCEPTION(scope, nullptr);
 
-    if (species.isUndefinedOrNull()) {
-        scope.release();
-        return defaultConstructor();
-    }
+    if (species.isUndefinedOrNull())
+        RELEASE_AND_RETURN(scope, defaultConstructor());
+
 
     JSValue result = construct(exec, species, args, "species is not a constructor");
     RETURN_IF_EXCEPTION(scope, nullptr);
@@ -75,11 +73,11 @@ inline JSArrayBufferView* speciesConstruct(ExecState* exec, JSObject* exemplar,
         if (!view->isNeutered())
             return view;
 
-        throwTypeError(exec, scope, ASCIILiteral(typedArrayBufferHasBeenDetachedErrorMessage));
+        throwTypeError(exec, scope, typedArrayBufferHasBeenDetachedErrorMessage);
         return nullptr;
     }
 
-    throwTypeError(exec, scope, ASCIILiteral("species constructor did not return a TypedArray View"));
+    throwTypeError(exec, scope, "species constructor did not return a TypedArray View"_s);
     return nullptr;
 }
 
@@ -106,7 +104,7 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewProtoFuncSet(VM& vm, ExecState
     ViewClass* thisObject = jsCast<ViewClass*>(exec->thisValue());
 
     if (UNLIKELY(!exec->argumentCount()))
-        return throwVMTypeError(exec, scope, ASCIILiteral("Expected at least one argument"));
+        return throwVMTypeError(exec, scope, "Expected at least one argument"_s);
 
     unsigned offset;
     if (exec->argumentCount() >= 2) {
@@ -123,7 +121,7 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewProtoFuncSet(VM& vm, ExecState
 
     JSObject* sourceArray = jsDynamicCast<JSObject*>(vm, exec->uncheckedArgument(0));
     if (UNLIKELY(!sourceArray))
-        return throwVMTypeError(exec, scope, ASCIILiteral("First argument should be an object"));
+        return throwVMTypeError(exec, scope, "First argument should be an object"_s);
 
     unsigned length;
     if (isTypedView(sourceArray->classInfo(vm)->typedArrayStorageType)) {
@@ -233,7 +231,7 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewProtoFuncIndexOf(VM& vm, ExecS
         return throwVMTypeError(exec, scope, typedArrayBufferHasBeenDetachedErrorMessage);
 
     if (!exec->argumentCount())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Expected at least one argument"));
+        return throwVMTypeError(exec, scope, "Expected at least one argument"_s);
 
     unsigned length = thisObject->length();
 
@@ -279,8 +277,7 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewProtoFuncJoin(VM& vm, ExecStat
             joiner.append(*exec, thisObject->getIndexQuickly(i));
             RETURN_IF_EXCEPTION(scope, encodedJSValue());
         }
-        scope.release();
-        return JSValue::encode(joiner.join(*exec));
+        RELEASE_AND_RETURN(scope, JSValue::encode(joiner.join(*exec)));
     };
 
     JSValue separatorValue = exec->argument(0);
@@ -310,7 +307,7 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewProtoFuncLastIndexOf(VM& vm, E
         return throwVMTypeError(exec, scope, typedArrayBufferHasBeenDetachedErrorMessage);
 
     if (!exec->argumentCount())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Expected at least one argument"));
+        return throwVMTypeError(exec, scope, "Expected at least one argument"_s);
 
     unsigned length = thisObject->length();
 
@@ -452,7 +449,7 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewProtoFuncSlice(VM& vm, ExecSta
     ASSERT(!args.hasOverflowed());
 
     JSArrayBufferView* result = speciesConstruct(exec, thisObject, args, [&]() {
-        Structure* structure = callee->globalObject()->typedArrayStructure(ViewClass::TypedArrayStorageType);
+        Structure* structure = callee->globalObject(vm)->typedArrayStructure(ViewClass::TypedArrayStorageType);
         return ViewClass::createUninitialized(exec, structure, length);
     });
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
@@ -548,20 +545,19 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewPrivateFuncSubarrayCreate(VM&v
 
     unsigned newByteOffset = thisObject->byteOffset() + offset * ViewClass::elementSize;
 
-    JSObject* defaultConstructor = callee->globalObject()->typedArrayConstructor(ViewClass::TypedArrayStorageType);
+    JSObject* defaultConstructor = callee->globalObject(vm)->typedArrayConstructor(ViewClass::TypedArrayStorageType);
     JSValue species = exec->uncheckedArgument(2);
     if (species == defaultConstructor) {
-        Structure* structure = callee->globalObject()->typedArrayStructure(ViewClass::TypedArrayStorageType);
+        Structure* structure = callee->globalObject(vm)->typedArrayStructure(ViewClass::TypedArrayStorageType);
 
-        scope.release();
-        return JSValue::encode(ViewClass::create(
+        RELEASE_AND_RETURN(scope, JSValue::encode(ViewClass::create(
             exec, structure, WTFMove(arrayBuffer),
             thisObject->byteOffset() + offset * ViewClass::elementSize,
-            length));
+            length)));
     }
 
     MarkedArgumentBuffer args;
-    args.append(vm.m_typedArrayController->toJS(exec, thisObject->globalObject(), arrayBuffer.get()));
+    args.append(vm.m_typedArrayController->toJS(exec, thisObject->globalObject(vm), arrayBuffer.get()));
     args.append(jsNumber(newByteOffset));
     args.append(jsNumber(length));
     ASSERT(!args.hasOverflowed());
@@ -572,7 +568,7 @@ EncodedJSValue JSC_HOST_CALL genericTypedArrayViewPrivateFuncSubarrayCreate(VM&v
     if (jsDynamicCast<JSArrayBufferView*>(vm, result))
         return JSValue::encode(result);
 
-    throwTypeError(exec, scope, ASCIILiteral("species constructor did not return a TypedArray View"));
+    throwTypeError(exec, scope, "species constructor did not return a TypedArray View"_s);
     return JSValue::encode(JSValue());
 }
 
