@@ -30,6 +30,7 @@
 #include "JSFunction.h"
 #include "JSGlobalObject.h"
 #include "JSGlobalObjectFunctions.h"
+#include "JSImmutableButterfly.h"
 #include "Lookup.h"
 #include "ObjectPrototype.h"
 #include "PropertyDescriptor.h"
@@ -73,20 +74,21 @@ const ClassInfo ObjectConstructor::s_info = { "Function", &InternalFunction::s_i
   getOwnPropertyDescriptors objectConstructorGetOwnPropertyDescriptors  DontEnum|Function 1
   getOwnPropertyNames       objectConstructorGetOwnPropertyNames        DontEnum|Function 1
   getOwnPropertySymbols     objectConstructorGetOwnPropertySymbols      DontEnum|Function 1
-  keys                      objectConstructorKeys                       DontEnum|Function 1
+  keys                      objectConstructorKeys                       DontEnum|Function 1 ObjectKeysIntrinsic
   defineProperty            objectConstructorDefineProperty             DontEnum|Function 3
   defineProperties          objectConstructorDefineProperties           DontEnum|Function 2
-  create                    objectConstructorCreate                     DontEnum|Function 2
+  create                    objectConstructorCreate                     DontEnum|Function 2 ObjectCreateIntrinsic
   seal                      objectConstructorSeal                       DontEnum|Function 1
   freeze                    objectConstructorFreeze                     DontEnum|Function 1
   preventExtensions         objectConstructorPreventExtensions          DontEnum|Function 1
   isSealed                  objectConstructorIsSealed                   DontEnum|Function 1
   isFrozen                  objectConstructorIsFrozen                   DontEnum|Function 1
   isExtensible              objectConstructorIsExtensible               DontEnum|Function 1
-  is                        objectConstructorIs                         DontEnum|Function 2
+  is                        objectConstructorIs                         DontEnum|Function 2 ObjectIsIntrinsic
   assign                    objectConstructorAssign                     DontEnum|Function 2
   values                    objectConstructorValues                     DontEnum|Function 1
   entries                   JSBuiltin                                   DontEnum|Function 1
+  fromEntries               JSBuiltin                                   DontEnum|Function 1
 @end
 */
 
@@ -115,9 +117,9 @@ void ObjectConstructor::finishCreation(VM& vm, JSGlobalObject* globalObject, Obj
 // ES 19.1.1.1 Object([value])
 static ALWAYS_INLINE JSObject* constructObject(ExecState* exec, JSValue newTarget)
 {
+    VM& vm = exec->vm();
     ObjectConstructor* objectConstructor = jsCast<ObjectConstructor*>(exec->jsCallee());
-    JSGlobalObject* globalObject = objectConstructor->globalObject();
-    VM& vm = globalObject->vm();
+    JSGlobalObject* globalObject = objectConstructor->globalObject(vm);
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     // We need to check newTarget condition in this caller side instead of InternalFunction::createSubclassStructure side.
@@ -138,8 +140,7 @@ static ALWAYS_INLINE JSObject* constructObject(ExecState* exec, JSValue newTarge
         return constructEmptyObject(exec, globalObject->objectStructureForObjectConstructor());
 
     // 3. Return ToObject(value).
-    scope.release();
-    return arg.toObject(exec, globalObject);
+    RELEASE_AND_RETURN(scope, arg.toObject(exec, globalObject));
 }
 
 static EncodedJSValue JSC_HOST_CALL constructWithObjectConstructor(ExecState* exec)
@@ -158,8 +159,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorGetPrototypeOf(ExecState* exec)
     auto scope = DECLARE_THROW_SCOPE(vm);
     JSObject* object = exec->argument(0).toObject(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
-    scope.release();
-    return JSValue::encode(object->getPrototype(vm, exec));
+    RELEASE_AND_RETURN(scope, JSValue::encode(object->getPrototype(vm, exec)));
 }
 
 EncodedJSValue JSC_HOST_CALL objectConstructorSetPrototypeOf(ExecState* exec)
@@ -169,11 +169,11 @@ EncodedJSValue JSC_HOST_CALL objectConstructorSetPrototypeOf(ExecState* exec)
 
     JSValue objectValue = exec->argument(0);
     if (objectValue.isUndefinedOrNull())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Cannot set prototype of undefined or null"));
+        return throwVMTypeError(exec, scope, "Cannot set prototype of undefined or null"_s);
 
     JSValue protoValue = exec->argument(1);
     if (!protoValue.isObject() && !protoValue.isNull())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Prototype value can only be an object or null"));
+        return throwVMTypeError(exec, scope, "Prototype value can only be an object or null"_s);
 
     JSObject* object = objectValue.toObject(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
@@ -189,10 +189,8 @@ JSValue objectConstructorGetOwnPropertyDescriptor(ExecState* exec, JSObject* obj
     VM& vm = exec->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
     PropertyDescriptor descriptor;
-    if (!object->getOwnPropertyDescriptor(exec, propertyName, descriptor)) {
-        scope.release();
-        return jsUndefined();
-    }
+    if (!object->getOwnPropertyDescriptor(exec, propertyName, descriptor))
+        RELEASE_AND_RETURN(scope, jsUndefined());
     RETURN_IF_EXCEPTION(scope, { });
 
     JSObject* result = constructObjectFromPropertyDescriptor(exec, descriptor);
@@ -242,8 +240,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorGetOwnPropertyDescriptor(ExecState
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
     auto propertyName = exec->argument(1).toPropertyKey(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
-    scope.release();
-    return JSValue::encode(objectConstructorGetOwnPropertyDescriptor(exec, object, propertyName));
+    RELEASE_AND_RETURN(scope, JSValue::encode(objectConstructorGetOwnPropertyDescriptor(exec, object, propertyName)));
 }
 
 EncodedJSValue JSC_HOST_CALL objectConstructorGetOwnPropertyDescriptors(ExecState* exec)
@@ -252,8 +249,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorGetOwnPropertyDescriptors(ExecStat
     auto scope = DECLARE_THROW_SCOPE(vm);
     JSObject* object = exec->argument(0).toObject(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
-    scope.release();
-    return JSValue::encode(objectConstructorGetOwnPropertyDescriptors(exec, object));
+    RELEASE_AND_RETURN(scope, JSValue::encode(objectConstructorGetOwnPropertyDescriptors(exec, object)));
 }
 
 // FIXME: Use the enumeration cache.
@@ -263,8 +259,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorGetOwnPropertyNames(ExecState* exe
     auto scope = DECLARE_THROW_SCOPE(vm);
     JSObject* object = exec->argument(0).toObject(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
-    scope.release();
-    return JSValue::encode(ownPropertyKeys(exec, object, PropertyNameMode::Strings, DontEnumPropertiesMode::Include));
+    RELEASE_AND_RETURN(scope, JSValue::encode(ownPropertyKeys(exec, object, PropertyNameMode::Strings, DontEnumPropertiesMode::Include)));
 }
 
 // FIXME: Use the enumeration cache.
@@ -274,19 +269,16 @@ EncodedJSValue JSC_HOST_CALL objectConstructorGetOwnPropertySymbols(ExecState* e
     auto scope = DECLARE_THROW_SCOPE(vm);
     JSObject* object = exec->argument(0).toObject(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
-    scope.release();
-    return JSValue::encode(ownPropertyKeys(exec, object, PropertyNameMode::Symbols, DontEnumPropertiesMode::Include));
+    RELEASE_AND_RETURN(scope, JSValue::encode(ownPropertyKeys(exec, object, PropertyNameMode::Symbols, DontEnumPropertiesMode::Include)));
 }
 
-// FIXME: Use the enumeration cache.
 EncodedJSValue JSC_HOST_CALL objectConstructorKeys(ExecState* exec)
 {
     VM& vm = exec->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
     JSObject* object = exec->argument(0).toObject(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
-    scope.release();
-    return JSValue::encode(ownPropertyKeys(exec, object, PropertyNameMode::Strings, DontEnumPropertiesMode::Exclude));
+    RELEASE_AND_RETURN(scope, JSValue::encode(ownPropertyKeys(exec, object, PropertyNameMode::Strings, DontEnumPropertiesMode::Exclude)));
 }
 
 EncodedJSValue JSC_HOST_CALL objectConstructorAssign(ExecState* exec)
@@ -296,10 +288,16 @@ EncodedJSValue JSC_HOST_CALL objectConstructorAssign(ExecState* exec)
 
     JSValue targetValue = exec->argument(0);
     if (targetValue.isUndefinedOrNull())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Object.assign requires that input parameter not be null or undefined"));
+        return throwVMTypeError(exec, scope, "Object.assign requires that input parameter not be null or undefined"_s);
     JSObject* target = targetValue.toObject(exec);
     RETURN_IF_EXCEPTION(scope, { });
 
+    // FIXME: Extend this for non JSFinalObject. For example, we would like to use this fast path for function objects too.
+    // https://bugs.webkit.org/show_bug.cgi?id=185358
+    bool targetCanPerformFastPut = jsDynamicCast<JSFinalObject*>(vm, target) && target->canPerformFastPutInlineExcludingProto(vm);
+
+    Vector<RefPtr<UniquedStringImpl>, 8> properties;
+    MarkedArgumentBuffer values;
     unsigned argsCount = exec->argumentCount();
     for (unsigned i = 1; i < argsCount; ++i) {
         JSValue sourceValue = exec->uncheckedArgument(i);
@@ -308,6 +306,73 @@ EncodedJSValue JSC_HOST_CALL objectConstructorAssign(ExecState* exec)
         JSObject* source = sourceValue.toObject(exec);
         RETURN_IF_EXCEPTION(scope, { });
 
+        if (targetCanPerformFastPut) {
+            if (!source->staticPropertiesReified(vm)) {
+                source->reifyAllStaticProperties(exec);
+                RETURN_IF_EXCEPTION(scope, { });
+            }
+
+            auto canPerformFastPropertyEnumerationForObjectAssign = [] (Structure* structure) {
+                if (structure->typeInfo().overridesGetOwnPropertySlot())
+                    return false;
+                if (structure->typeInfo().overridesGetPropertyNames())
+                    return false;
+                // FIXME: Indexed properties can be handled.
+                // https://bugs.webkit.org/show_bug.cgi?id=185358
+                if (hasIndexedProperties(structure->indexingType()))
+                    return false;
+                if (structure->hasGetterSetterProperties())
+                    return false;
+                if (structure->isUncacheableDictionary())
+                    return false;
+                // Cannot perform fast [[Put]] to |target| if the property names of the |source| contain "__proto__".
+                if (structure->hasUnderscoreProtoPropertyExcludingOriginalProto())
+                    return false;
+                return true;
+            };
+
+            if (canPerformFastPropertyEnumerationForObjectAssign(source->structure(vm))) {
+                // |source| Structure does not have any getters. And target can perform fast put.
+                // So enumerating properties and putting properties are non observable.
+
+                // FIXME: It doesn't seem like we should have to do this in two phases, but
+                // we're running into crashes where it appears that source is transitioning
+                // under us, and even ends up in a state where it has a null butterfly. My
+                // leading hypothesis here is that we fire some value replacement watchpoint
+                // that ends up transitioning the structure underneath us.
+                // https://bugs.webkit.org/show_bug.cgi?id=187837
+
+                // Do not clear since Vector::clear shrinks the backing store.
+                properties.resize(0);
+                values.clear();
+                source->structure(vm)->forEachProperty(vm, [&] (const PropertyMapEntry& entry) -> bool {
+                    if (entry.attributes & PropertyAttribute::DontEnum)
+                        return true;
+
+                    PropertyName propertyName(entry.key);
+                    if (propertyName.isPrivateName())
+                        return true;
+
+                    properties.append(entry.key);
+                    values.appendWithCrashOnOverflow(source->getDirect(entry.offset));
+
+                    return true;
+                });
+
+                for (size_t i = 0; i < properties.size(); ++i) {
+                    // FIXME: We could put properties in a batching manner to accelerate Object.assign more.
+                    // https://bugs.webkit.org/show_bug.cgi?id=185358
+                    PutPropertySlot putPropertySlot(target, true);
+                    target->putOwnDataProperty(vm, properties[i].get(), values.at(i), putPropertySlot);
+                }
+                continue;
+            }
+        }
+
+        // [[GetOwnPropertyNames]], [[Get]] etc. could modify target object and invalidate this assumption.
+        // For example, [[Get]] of source object could configure setter to target object. So disable the fast path.
+        targetCanPerformFastPut = false;
+
         PropertyNameArray properties(&vm, PropertyNameMode::StringsAndSymbols, PrivateSymbolMode::Exclude);
         source->methodTable(vm)->getOwnPropertyNames(source, exec, properties, EnumerationMode(DontEnumPropertiesMode::Include));
         RETURN_IF_EXCEPTION(scope, { });
@@ -368,7 +433,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorValues(ExecState* exec)
 
     JSValue targetValue = exec->argument(0);
     if (targetValue.isUndefinedOrNull())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Object.values requires that input parameter not be null or undefined"));
+        return throwVMTypeError(exec, scope, "Object.values requires that input parameter not be null or undefined"_s);
     JSObject* target = targetValue.toObject(exec);
     RETURN_IF_EXCEPTION(scope, { });
 
@@ -420,7 +485,7 @@ bool toPropertyDescriptor(ExecState* exec, JSValue in, PropertyDescriptor& desc)
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (!in.isObject()) {
-        throwTypeError(exec, scope, ASCIILiteral("Property description must be an object."));
+        throwTypeError(exec, scope, "Property description must be an object."_s);
         return false;
     }
     JSObject* description = asObject(in);
@@ -469,8 +534,8 @@ bool toPropertyDescriptor(ExecState* exec, JSValue in, PropertyDescriptor& desc)
         RETURN_IF_EXCEPTION(scope, false);
         if (!get.isUndefined()) {
             CallData callData;
-            if (getCallData(get, callData) == CallType::None) {
-                throwTypeError(exec, scope, ASCIILiteral("Getter must be a function."));
+            if (getCallData(vm, get, callData) == CallType::None) {
+                throwTypeError(exec, scope, "Getter must be a function."_s);
                 return false;
             }
         }
@@ -485,8 +550,8 @@ bool toPropertyDescriptor(ExecState* exec, JSValue in, PropertyDescriptor& desc)
         RETURN_IF_EXCEPTION(scope, false);
         if (!set.isUndefined()) {
             CallData callData;
-            if (getCallData(set, callData) == CallType::None) {
-                throwTypeError(exec, scope, ASCIILiteral("Setter must be a function."));
+            if (getCallData(vm, set, callData) == CallType::None) {
+                throwTypeError(exec, scope, "Setter must be a function."_s);
                 return false;
             }
         }
@@ -498,12 +563,12 @@ bool toPropertyDescriptor(ExecState* exec, JSValue in, PropertyDescriptor& desc)
         return true;
 
     if (desc.value()) {
-        throwTypeError(exec, scope, ASCIILiteral("Invalid property.  'value' present on property with getter or setter."));
+        throwTypeError(exec, scope, "Invalid property.  'value' present on property with getter or setter."_s);
         return false;
     }
 
     if (desc.writablePresent()) {
-        throwTypeError(exec, scope, ASCIILiteral("Invalid property.  'writable' present on property with getter or setter."));
+        throwTypeError(exec, scope, "Invalid property.  'writable' present on property with getter or setter."_s);
         return false;
     }
     return true;
@@ -515,7 +580,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorDefineProperty(ExecState* exec)
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (!exec->argument(0).isObject())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Properties can only be defined on Objects."));
+        return throwVMTypeError(exec, scope, "Properties can only be defined on Objects."_s);
     JSObject* obj = asObject(exec->argument(0));
     auto propertyName = exec->argument(1).toPropertyKey(exec);
     RETURN_IF_EXCEPTION(scope, encodedJSValue());
@@ -527,8 +592,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorDefineProperty(ExecState* exec)
     ASSERT((descriptor.attributes() & PropertyAttribute::Accessor) || (!descriptor.isAccessorDescriptor()));
     scope.assertNoException();
     obj->methodTable(vm)->defineOwnProperty(obj, exec, propertyName, descriptor, true);
-    scope.release();
-    return JSValue::encode(obj);
+    RELEASE_AND_RETURN(scope, JSValue::encode(obj));
 }
 
 static JSValue defineProperties(ExecState* exec, JSObject* object, JSObject* properties)
@@ -580,14 +644,13 @@ EncodedJSValue JSC_HOST_CALL objectConstructorDefineProperties(ExecState* exec)
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     if (!exec->argument(0).isObject())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Properties can only be defined on Objects."));
+        return throwVMTypeError(exec, scope, "Properties can only be defined on Objects."_s);
     JSObject* targetObj = asObject(exec->argument(0));
     JSObject* props = exec->argument(1).toObject(exec);
     EXCEPTION_ASSERT(!!scope.exception() == !props);
     if (UNLIKELY(!props))
         return encodedJSValue();
-    scope.release();
-    return JSValue::encode(defineProperties(exec, targetObj, props));
+    RELEASE_AND_RETURN(scope, JSValue::encode(defineProperties(exec, targetObj, props)));
 }
 
 EncodedJSValue JSC_HOST_CALL objectConstructorCreate(ExecState* exec)
@@ -597,16 +660,15 @@ EncodedJSValue JSC_HOST_CALL objectConstructorCreate(ExecState* exec)
 
     JSValue proto = exec->argument(0);
     if (!proto.isObject() && !proto.isNull())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Object prototype may only be an Object or null."));
+        return throwVMTypeError(exec, scope, "Object prototype may only be an Object or null."_s);
     JSObject* newObject = proto.isObject()
         ? constructEmptyObject(exec, asObject(proto))
         : constructEmptyObject(exec, exec->lexicalGlobalObject()->nullPrototypeObjectStructure());
     if (exec->argument(1).isUndefined())
         return JSValue::encode(newObject);
     if (!exec->argument(1).isObject())
-        return throwVMTypeError(exec, scope, ASCIILiteral("Property descriptor list must be an Object."));
-    scope.release();
-    return JSValue::encode(defineProperties(exec, newObject, asObject(exec->argument(1))));
+        return throwVMTypeError(exec, scope, "Property descriptor list must be an Object."_s);
+    RELEASE_AND_RETURN(scope, JSValue::encode(defineProperties(exec, newObject, asObject(exec->argument(1)))));
 }
 
 enum class IntegrityLevel {
@@ -704,6 +766,24 @@ bool testIntegrityLevel(ExecState* exec, VM& vm, JSObject* object)
     return true;
 }
 
+JSObject* objectConstructorSeal(ExecState* exec, JSObject* object)
+{
+    VM& vm = exec->vm();
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    if (jsDynamicCast<JSFinalObject*>(vm, object) && !hasIndexedProperties(object->indexingType())) {
+        object->seal(vm);
+        return object;
+    }
+
+    bool success = setIntegrityLevel<IntegrityLevel::Sealed>(exec, vm, object);
+    RETURN_IF_EXCEPTION(scope, nullptr);
+    if (UNLIKELY(!success))
+        return throwTypeError(exec, scope, "Unable to prevent extension in Object.seal"_s);
+
+    return object;
+}
+
 EncodedJSValue JSC_HOST_CALL objectConstructorSeal(ExecState* exec)
 {
     VM& vm = exec->vm();
@@ -713,21 +793,8 @@ EncodedJSValue JSC_HOST_CALL objectConstructorSeal(ExecState* exec)
     JSValue obj = exec->argument(0);
     if (!obj.isObject())
         return JSValue::encode(obj);
-    JSObject* object = asObject(obj);
 
-    if (isJSFinalObject(object) && !hasIndexedProperties(object->indexingType())) {
-        object->seal(vm);
-        return JSValue::encode(obj);
-    }
-
-    bool success = setIntegrityLevel<IntegrityLevel::Sealed>(exec, vm, object);
-    RETURN_IF_EXCEPTION(scope, encodedJSValue());
-    if (UNLIKELY(!success)) {
-        throwTypeError(exec, scope, ASCIILiteral("Unable to prevent extension in Object.seal"));
-        return encodedJSValue();
-    }
-
-    return JSValue::encode(obj);
+    RELEASE_AND_RETURN(scope, JSValue::encode(objectConstructorSeal(exec, asObject(obj))));
 }
 
 JSObject* objectConstructorFreeze(ExecState* exec, JSObject* object)
@@ -735,7 +802,7 @@ JSObject* objectConstructorFreeze(ExecState* exec, JSObject* object)
     VM& vm = exec->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-    if (isJSFinalObject(object) && !hasIndexedProperties(object->indexingType())) {
+    if (jsDynamicCast<JSFinalObject*>(vm, object) && !hasIndexedProperties(object->indexingType())) {
         object->freeze(vm);
         return object;
     }
@@ -743,7 +810,7 @@ JSObject* objectConstructorFreeze(ExecState* exec, JSObject* object)
     bool success = setIntegrityLevel<IntegrityLevel::Frozen>(exec, vm, object);
     RETURN_IF_EXCEPTION(scope, nullptr);
     if (!success)
-        return throwTypeError(exec, scope, ASCIILiteral("Unable to prevent extension in Object.freeze"));
+        return throwTypeError(exec, scope, "Unable to prevent extension in Object.freeze"_s);
     return object;
 }
 
@@ -782,7 +849,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorIsSealed(ExecState* exec)
     JSObject* object = asObject(obj);
 
     // Quick check for final objects.
-    if (isJSFinalObject(object) && !hasIndexedProperties(object->indexingType()))
+    if (jsDynamicCast<JSFinalObject*>(vm, object) && !hasIndexedProperties(object->indexingType()))
         return JSValue::encode(jsBoolean(object->isSealed(vm)));
 
     // 2. Return ? TestIntegrityLevel(O, "sealed").
@@ -800,7 +867,7 @@ EncodedJSValue JSC_HOST_CALL objectConstructorIsFrozen(ExecState* exec)
     JSObject* object = asObject(obj);
 
     // Quick check for final objects.
-    if (isJSFinalObject(object) && !hasIndexedProperties(object->indexingType()))
+    if (jsDynamicCast<JSFinalObject*>(vm, object) && !hasIndexedProperties(object->indexingType()))
         return JSValue::encode(jsBoolean(object->isFrozen(vm)));
 
     // 2. Return ? TestIntegrityLevel(O, "frozen").
@@ -825,11 +892,23 @@ EncodedJSValue JSC_HOST_CALL objectConstructorIs(ExecState* exec)
     return JSValue::encode(jsBoolean(sameValue(exec, exec->argument(0), exec->argument(1))));
 }
 
-// FIXME: Use the enumeration cache.
 JSArray* ownPropertyKeys(ExecState* exec, JSObject* object, PropertyNameMode propertyNameMode, DontEnumPropertiesMode dontEnumPropertiesMode)
 {
     VM& vm = exec->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
+
+    auto* globalObject = exec->lexicalGlobalObject();
+    bool isObjectKeys = propertyNameMode == PropertyNameMode::Strings && dontEnumPropertiesMode == DontEnumPropertiesMode::Exclude;
+    // We attempt to look up own property keys cache in Object.keys case.
+    if (isObjectKeys) {
+        if (LIKELY(!globalObject->isHavingABadTime())) {
+            if (auto* immutableButterfly = object->structure(vm)->cachedOwnKeys()) {
+                Structure* arrayStructure = globalObject->originalArrayStructureForIndexingType(immutableButterfly->indexingMode());
+                return JSArray::createWithButterfly(vm, nullptr, arrayStructure, immutableButterfly->toButterfly());
+            }
+        }
+    }
+
     PropertyNameArray properties(&vm, propertyNameMode, PrivateSymbolMode::Exclude);
     object->methodTable(vm)->getOwnPropertyNames(object, exec, properties, EnumerationMode(dontEnumPropertiesMode));
     RETURN_IF_EXCEPTION(scope, nullptr);
@@ -851,8 +930,30 @@ JSArray* ownPropertyKeys(ExecState* exec, JSObject* object, PropertyNameMode pro
     if (propertyNameMode != PropertyNameMode::StringsAndSymbols) {
         ASSERT(propertyNameMode == PropertyNameMode::Strings || propertyNameMode == PropertyNameMode::Symbols);
         if (!mustFilterProperty && properties.size() < MIN_SPARSE_ARRAY_INDEX) {
-            auto* globalObject = exec->lexicalGlobalObject();
             if (LIKELY(!globalObject->isHavingABadTime())) {
+                if (isObjectKeys) {
+                    Structure* structure = object->structure(vm);
+                    if (structure->canCacheOwnKeys()) {
+                        auto* cachedButterfly = structure->cachedOwnKeysIgnoringSentinel();
+                        if (cachedButterfly == StructureRareData::cachedOwnKeysSentinel()) {
+                            size_t numProperties = properties.size();
+                            auto* newButterfly = JSImmutableButterfly::create(vm, CopyOnWriteArrayWithContiguous, numProperties);
+                            for (size_t i = 0; i < numProperties; i++) {
+                                const auto& identifier = properties[i];
+                                ASSERT(!identifier.isSymbol());
+                                newButterfly->setIndex(vm, i, jsOwnedString(&vm, identifier.string()));
+                            }
+
+                            structure->setCachedOwnKeys(vm, newButterfly);
+                            Structure* arrayStructure = globalObject->originalArrayStructureForIndexingType(newButterfly->indexingMode());
+                            return JSArray::createWithButterfly(vm, nullptr, arrayStructure, newButterfly->toButterfly());
+                        }
+
+                        if (cachedButterfly == nullptr)
+                            structure->setCachedOwnKeys(vm, StructureRareData::cachedOwnKeysSentinel());
+                    }
+                }
+
                 size_t numProperties = properties.size();
                 JSArray* keys = JSArray::create(vm, globalObject->originalArrayStructureForIndexingType(ArrayWithContiguous), numProperties);
                 WriteBarrier<Unknown>* buffer = keys->butterfly()->contiguous().data();
