@@ -1145,22 +1145,22 @@ xmlSAX2AttributeInternal(void *ctx, const xmlChar *fullname,
 #endif
     {
 #ifdef LIBXML_VALID_ENABLED
-    /*
-     * Do the last stage of the attribute normalization
-     * Needed for HTML too:
-     *   http://www.w3.org/TR/html4/types.html#h-6.2
-     */
-    ctxt->vctxt.valid = 1;
-    nval = xmlValidCtxtNormalizeAttributeValue(&ctxt->vctxt,
-                                       ctxt->myDoc, ctxt->node,
-                       fullname, value);
-    if (ctxt->vctxt.valid != 1) {
-    ctxt->valid = 0;
-    }
-    if (nval != NULL)
-    value = nval;
+        /*
+         * Do the last stage of the attribute normalization
+         * Needed for HTML too:
+         *   http://www.w3.org/TR/html4/types.html#h-6.2
+         */
+        ctxt->vctxt.valid = 1;
+        nval = xmlValidCtxtNormalizeAttributeValue(&ctxt->vctxt,
+                                               ctxt->myDoc, ctxt->node,
+                                               fullname, value);
+        if (ctxt->vctxt.valid != 1) {
+            ctxt->valid = 0;
+        }
+        if (nval != NULL)
+            value = nval;
 #else
-    nval = NULL;
+        nval = NULL;
 #endif /* LIBXML_VALID_ENABLED */
     }
 
@@ -1305,24 +1305,24 @@ xmlSAX2AttributeInternal(void *ctx, const xmlChar *fullname,
     } else {
             xmlAttrPtr prop;
 
-    prop = ctxt->node->properties;
-    while (prop != NULL) {
-        if (prop->ns != NULL) {
-        if ((xmlStrEqual(name, prop->name)) &&
-            ((namespace == prop->ns) ||
-             (xmlStrEqual(namespace->href, prop->ns->href)))) {
-            xmlNsErrMsg(ctxt, XML_ERR_ATTRIBUTE_REDEFINED,
-                    "Attribute %s in %s redefined\n",
-                             name, namespace->href);
-            ctxt->wellFormed = 0;
-            if (ctxt->recovery == 0) ctxt->disableSAX = 1;
+            prop = ctxt->node->properties;
+            while (prop != NULL) {
+                if (prop->ns != NULL) {
+                    if ((xmlStrEqual(name, prop->name)) &&
+                        ((namespace == prop->ns) ||
+                         (xmlStrEqual(namespace->href, prop->ns->href)))) {
+                            xmlNsErrMsg(ctxt, XML_ERR_ATTRIBUTE_REDEFINED,
+                                    "Attribute %s in %s redefined\n",
+                                             name, namespace->href);
+                        ctxt->wellFormed = 0;
+                        if (ctxt->recovery == 0) ctxt->disableSAX = 1;
                         if (name != NULL)
                             xmlFree(name);
-            goto error;
-        }
-        }
-        prop = prop->next;
-    }
+                        goto error;
+                    }
+                }
+                prop = prop->next;
+            }
         }
     } else {
     namespace = NULL;
@@ -1665,7 +1665,11 @@ xmlSAX2StartElement(void *ctx, const xmlChar *fullname, const xmlChar **atts)
 #ifdef DEBUG_SAX_TREE
     xmlGenericError(xmlGenericErrorContext, "pushing(%s)\n", name);
 #endif
-    nodePush(ctxt, ret);
+    if (nodePush(ctxt, ret) < 0) {
+        xmlUnlinkNode(ret);
+        xmlFreeNode(ret);
+        return;
+    }
 
     /*
      * Link the child element
@@ -2257,6 +2261,7 @@ xmlSAX2StartElementNs(void *ctx,
     ctxt->freeElems = ret->next;
     ctxt->freeElemsNr--;
     memset(ret, 0, sizeof(xmlNode));
+        ret->doc = ctxt->myDoc;
     ret->type = XML_ELEMENT_NODE;
 
     if (ctxt->dictNames)
@@ -2336,7 +2341,11 @@ xmlSAX2StartElementNs(void *ctx,
     /*
      * We are parsing a new node.
      */
-    nodePush(ctxt, ret);
+    if (nodePush(ctxt, ret) < 0) {
+        xmlUnlinkNode(ret);
+        xmlFreeNode(ret);
+        return;
+    }
 
     /*
      * Link the child element
@@ -2373,9 +2382,9 @@ xmlSAX2StartElementNs(void *ctx,
         return;
         }
             if (prefix != NULL)
-        xmlNsWarnMsg(ctxt, XML_NS_ERR_UNDEFINED_NAMESPACE,
-            "Namespace prefix %s was not found\n",
-            prefix, NULL);
+                xmlNsWarnMsg(ctxt, XML_NS_ERR_UNDEFINED_NAMESPACE,
+                             "Namespace prefix %s was not found\n",
+                             prefix, NULL);
             else
                 xmlNsWarnMsg(ctxt, XML_NS_ERR_UNDEFINED_NAMESPACE,
                              "Namespace default prefix was not found\n",
@@ -2414,7 +2423,7 @@ xmlSAX2StartElementNs(void *ctx,
         }
         }
         xmlSAX2AttributeNs(ctxt, attributes[j], attributes[j+1],
-                           attributes[j+3], attributes[j+4]);
+                   attributes[j+3], attributes[j+4]);
     }
     }
 
