@@ -314,7 +314,7 @@ xmlParse3986Query(xmlURIPtr uri, const char **str)
  * @uri:  pointer to an URI structure
  * @str:  the string to analyze
  *
- * Parse a port  part and fills in the appropriate fields
+ * Parse a port part and fills in the appropriate fields
  * of the @uri structure
  *
  * port          = *DIGIT
@@ -591,10 +591,10 @@ xmlParse3986PathAbEmpty(xmlURIPtr uri, const char **str)
     if (uri != NULL) {
     if (uri->path != NULL) xmlFree(uri->path);
         if (*str != cur) {
-    if (uri->cleanup & 2)
-        uri->path = STRNDUP(*str, cur - *str);
-    else
-        uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
+            if (uri->cleanup & 2)
+                uri->path = STRNDUP(*str, cur - *str);
+            else
+                uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
         } else {
             uri->path = NULL;
         }
@@ -637,10 +637,10 @@ xmlParse3986PathAbsolute(xmlURIPtr uri, const char **str)
     if (uri != NULL) {
     if (uri->path != NULL) xmlFree(uri->path);
         if (cur != *str) {
-    if (uri->cleanup & 2)
-        uri->path = STRNDUP(*str, cur - *str);
-    else
-        uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
+            if (uri->cleanup & 2)
+                uri->path = STRNDUP(*str, cur - *str);
+            else
+                uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
         } else {
             uri->path = NULL;
         }
@@ -679,10 +679,10 @@ xmlParse3986PathRootless(xmlURIPtr uri, const char **str)
     if (uri != NULL) {
     if (uri->path != NULL) xmlFree(uri->path);
         if (cur != *str) {
-    if (uri->cleanup & 2)
-        uri->path = STRNDUP(*str, cur - *str);
-    else
-        uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
+            if (uri->cleanup & 2)
+                uri->path = STRNDUP(*str, cur - *str);
+            else
+                uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
         } else {
             uri->path = NULL;
         }
@@ -721,10 +721,10 @@ xmlParse3986PathNoScheme(xmlURIPtr uri, const char **str)
     if (uri != NULL) {
     if (uri->path != NULL) xmlFree(uri->path);
         if (cur != *str) {
-    if (uri->cleanup & 2)
-        uri->path = STRNDUP(*str, cur - *str);
-    else
-        uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
+            if (uri->cleanup & 2)
+                uri->path = STRNDUP(*str, cur - *str);
+            else
+                uri->path = xmlURIUnescapeString(*str, cur - *str, NULL);
         } else {
             uri->path = NULL;
         }
@@ -1086,7 +1086,7 @@ xmlSaveUri(xmlURIPtr uri) {
     if (len >= max) {
             temp = xmlSaveUriRealloc(ret, &max);
             if (temp == NULL) goto mem_error;
-        ret = temp;
+            ret = temp;
     }
     ret[len++] = ':';
     }
@@ -1096,7 +1096,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                 temp = xmlSaveUriRealloc(ret, &max);
                 if (temp == NULL) goto mem_error;
-        ret = temp;
+                ret = temp;
         }
         if (IS_RESERVED(*(p)) || IS_UNRESERVED(*(p)))
         ret[len++] = *p++;
@@ -1113,7 +1113,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                 temp = xmlSaveUriRealloc(ret, &max);
                 if (temp == NULL) goto mem_error;
-        ret = temp;
+                ret = temp;
         }
         ret[len++] = '/';
         ret[len++] = '/';
@@ -1123,7 +1123,7 @@ xmlSaveUri(xmlURIPtr uri) {
             if (len + 3 >= max) {
                         temp = xmlSaveUriRealloc(ret, &max);
                         if (temp == NULL) goto mem_error;
-            ret = temp;
+                        ret = temp;
             }
             if ((IS_UNRESERVED(*(p))) ||
             ((*(p) == ';')) || ((*(p) == ':')) ||
@@ -1142,34 +1142,34 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                     temp = xmlSaveUriRealloc(ret, &max);
                     if (temp == NULL) goto mem_error;
-            ret = temp;
+                    ret = temp;
         }
         ret[len++] = '@';
         }
         if (uri->server != NULL) {
         p = uri->server;
         while (*p != 0) {
-        if (len >= max) {
+            if (len >= max) {
             temp = xmlSaveUriRealloc(ret, &max);
             if (temp == NULL) goto mem_error;
             ret = temp;
-        }
-        ret[len++] = *p++;
+            }
+            ret[len++] = *p++;
         }
         if (uri->port > 0) {
-        if (len + 10 >= max) {
+            if (len + 10 >= max) {
             temp = xmlSaveUriRealloc(ret, &max);
             if (temp == NULL) goto mem_error;
             ret = temp;
-        }
-        len += snprintf((char *) &ret[len], max - len, ":%d", uri->port);
+            }
+            len += snprintf((char *) &ret[len], max - len, ":%d", uri->port);
         }
         }
     } else if (uri->authority != NULL) {
         if (len + 3 >= max) {
                 temp = xmlSaveUriRealloc(ret, &max);
                 if (temp == NULL) goto mem_error;
-            ret = temp;
+                ret = temp;
         }
         ret[len++] = '/';
         ret[len++] = '/';
@@ -1178,7 +1178,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                     temp = xmlSaveUriRealloc(ret, &max);
                     if (temp == NULL) goto mem_error;
-            ret = temp;
+                    ret = temp;
         }
         if ((IS_UNRESERVED(*(p))) ||
                     ((*(p) == '$')) || ((*(p) == ',')) || ((*(p) == ';')) ||
@@ -1197,7 +1197,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                 temp = xmlSaveUriRealloc(ret, &max);
                 if (temp == NULL) goto mem_error;
-            ret = temp;
+                ret = temp;
         }
     }
     if (uri->path != NULL) {
@@ -1225,7 +1225,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                     temp = xmlSaveUriRealloc(ret, &max);
                     if (temp == NULL) goto mem_error;
-            ret = temp;
+                    ret = temp;
         }
         if ((IS_UNRESERVED(*(p))) || ((*(p) == '/')) ||
                     ((*(p) == ';')) || ((*(p) == '@')) || ((*(p) == '&')) ||
@@ -1245,7 +1245,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 1 >= max) {
                 temp = xmlSaveUriRealloc(ret, &max);
                 if (temp == NULL) goto mem_error;
-            ret = temp;
+                ret = temp;
         }
         ret[len++] = '?';
         p = uri->query_raw;
@@ -1253,7 +1253,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 1 >= max) {
                     temp = xmlSaveUriRealloc(ret, &max);
                     if (temp == NULL) goto mem_error;
-            ret = temp;
+                    ret = temp;
         }
         ret[len++] = *p++;
         }
@@ -1261,7 +1261,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                 temp = xmlSaveUriRealloc(ret, &max);
                 if (temp == NULL) goto mem_error;
-            ret = temp;
+                ret = temp;
         }
         ret[len++] = '?';
         p = uri->query;
@@ -1269,7 +1269,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                     temp = xmlSaveUriRealloc(ret, &max);
                     if (temp == NULL) goto mem_error;
-            ret = temp;
+                    ret = temp;
         }
         if ((IS_UNRESERVED(*(p))) || (IS_RESERVED(*(p))))
             ret[len++] = *p++;
@@ -1295,7 +1295,7 @@ xmlSaveUri(xmlURIPtr uri) {
         if (len + 3 >= max) {
                 temp = xmlSaveUriRealloc(ret, &max);
                 if (temp == NULL) goto mem_error;
-            ret = temp;
+                ret = temp;
         }
         if ((IS_UNRESERVED(*(p))) || (IS_RESERVED(*(p))))
         ret[len++] = *p++;
@@ -1311,7 +1311,7 @@ xmlSaveUri(xmlURIPtr uri) {
     if (len >= max) {
         temp = xmlSaveUriRealloc(ret, &max);
         if (temp == NULL) goto mem_error;
-            ret = temp;
+        ret = temp;
     }
     ret[len] = 0;
     return(ret);
@@ -1532,9 +1532,9 @@ xmlNormalizeURIPath(char *path) {
           break;
         }
         /* Valgrind complained, strcpy(cur, segp + 3); */
-    /* string will overlap, do not use strcpy */
-    tmp = cur;
-    segp += 3;
+        /* string will overlap, do not use strcpy */
+        tmp = cur;
+        segp += 3;
         while ((*tmp++ = *segp++) != 0)
           ;
 
@@ -1748,8 +1748,8 @@ xmlURIEscape(const xmlChar * str)
 
 #define NULLCHK(p) if(!p) { \
          xmlURIErrMemory("escaping URI value\n"); \
-                        xmlFreeURI(uri); \
-                        return NULL; } \
+         xmlFreeURI(uri); \
+         return NULL; } \
 
     if (str == NULL)
         return (NULL);
@@ -2027,7 +2027,7 @@ xmlBuildURI(const xmlChar *URI, const xmlChar *base) {
     res->authority = xmlMemStrdup(bas->authority);
     else if ((bas->server != NULL) || (bas->port == -1)) {
     if (bas->server != NULL)
-    res->server = xmlMemStrdup(bas->server);
+        res->server = xmlMemStrdup(bas->server);
     if (bas->user != NULL)
         res->user = xmlMemStrdup(bas->user);
     res->port = bas->port;
@@ -2236,31 +2236,14 @@ xmlBuildRelativeURI (const xmlChar * URI, const xmlChar * base)
      * First we take care of the special case where either of the
      * two path components may be missing (bug 316224)
      */
-    if (bas->path == NULL) {
-    if (ref->path != NULL) {
-        uptr = (xmlChar *) ref->path;
-        if (*uptr == '/')
-        uptr++;
-        /* exception characters from xmlSaveUri */
-        val = xmlURIEscapeStr(uptr, BAD_CAST "/;&=+$,");
-    }
-    goto done;
-    }
     bptr = (xmlChar *)bas->path;
-    if (ref->path == NULL) {
-    for (ix = 0; bptr[ix] != 0; ix++) {
-        if (bptr[ix] == '/')
-        nbslash++;
-    }
-    uptr = NULL;
-    len = 1;    /* this is for a string terminator only */
-    } else {
+    {
         xmlChar *rptr = (xmlChar *) ref->path;
         int pos = 0;
 
-    /*
-     * Next we compare the two strings and find where they first differ
-     */
+        /*
+         * Next we compare the two strings and find where they first differ
+         */
     if ((*rptr == '.') && (rptr[1] == '/'))
             rptr += 2;
     if ((*bptr == '.') && (bptr[1] == '/'))
@@ -2280,30 +2263,28 @@ xmlBuildRelativeURI (const xmlChar * URI, const xmlChar * base)
      * beginning of the "unique" suffix of URI
      */
     ix = pos;
-    if ((rptr[ix] == '/') && (ix > 0))
-        ix--;
-    else if ((rptr[ix] == 0) && (ix > 1) && (rptr[ix - 1] == '/'))
-        ix -= 2;
     for (; ix > 0; ix--) {
-        if (rptr[ix] == '/')
+        if (rptr[ix - 1] == '/')
         break;
     }
-    if (ix == 0) {
-        uptr = (xmlChar *)rptr;
-    } else {
-        ix++;
-        uptr = (xmlChar *)&rptr[ix];
-    }
+    uptr = (xmlChar *)&rptr[ix];
 
     /*
      * In base, count the number of '/' from the differing point
      */
-    if (bptr[pos] != rptr[pos]) {/* check for trivial URI == base */
-        for (; bptr[ix] != 0; ix++) {
+    for (; bptr[ix] != 0; ix++) {
         if (bptr[ix] == '/')
-            nbslash++;
-        }
+        nbslash++;
     }
+
+    /*
+     * e.g: URI="foo/" base="foo/bar" -> "./"
+     */
+    if (nbslash == 0 && !uptr[0]) {
+        val = xmlStrdup(BAD_CAST "./");
+        goto done;
+    }
+
     len = xmlStrlen (uptr) + 1;
     }
 
