@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -23,8 +23,7 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef CheckedArithmetic_h
-#define CheckedArithmetic_h
+#pragma once
 
 #include <wtf/Assertions.h>
 
@@ -71,6 +70,49 @@ enum class CheckedState {
     DidNotOverflow
 };
 
+class AssertNoOverflow {
+public:
+    static NO_RETURN_DUE_TO_ASSERT void overflowed()
+    {
+        ASSERT_NOT_REACHED();
+    }
+
+    void clearOverflow() { }
+
+    static NO_RETURN_DUE_TO_CRASH void crash()
+    {
+        CRASH();
+    }
+
+public:
+    constexpr bool hasOverflowed() const { return false; }
+};
+
+class ConditionalCrashOnOverflow {
+public:
+    void overflowed()
+    {
+        m_overflowed = true;
+        if (m_shouldCrashOnOverflow)
+            crash();
+    }
+
+    bool shouldCrashOnOverflow() const { return m_shouldCrashOnOverflow; }
+    void setShouldCrashOnOverflow(bool value) { m_shouldCrashOnOverflow = value; }
+
+    bool hasOverflowed() const { return m_overflowed; }
+    void clearOverflow() { m_overflowed = false; }
+
+    static NO_RETURN_DUE_TO_CRASH void crash()
+    {
+        CRASH();
+    }
+
+private:
+    bool m_overflowed { false };
+    bool m_shouldCrashOnOverflow { true };
+};
+
 class CrashOnOverflow {
 public:
     static NO_RETURN_DUE_TO_CRASH void overflowed()
@@ -96,11 +138,6 @@ protected:
     {
     }
 
-    void overflowed()
-    {
-        m_overflowed = true;
-    }
-
     void clearOverflow()
     {
         m_overflowed = false;
@@ -113,6 +150,7 @@ protected:
 
 public:
     bool hasOverflowed() const { return m_overflowed; }
+    void overflowed() { m_overflowed = true; }
 
 private:
     unsigned char m_overflowed;
@@ -207,6 +245,11 @@ template <typename T> struct RemoveChecked {
     static const CleanType DefaultValue = 0;
 };
 
+template <typename T> struct RemoveChecked<Checked<T, ConditionalCrashOnOverflow>> {
+    using CleanType = typename RemoveChecked<T>::CleanType;
+    static const CleanType DefaultValue = 0;
+};
+
 template <typename T> struct RemoveChecked<Checked<T, CrashOnOverflow>> {
     typedef typename RemoveChecked<T>::CleanType CleanType;
     static const CleanType DefaultValue = 0;
@@ -270,6 +313,13 @@ template <typename LHS, typename RHS, typename ResultType> struct ArithmeticOper
 
     static inline bool add(LHS lhs, RHS rhs, ResultType& result) WARN_UNUSED_RETURN
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_add_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         if (signsMatch(lhs, rhs)) {
             if (lhs >= 0) {
                 if ((std::numeric_limits<ResultType>::max() - rhs) < lhs)
@@ -282,10 +332,18 @@ template <typename LHS, typename RHS, typename ResultType> struct ArithmeticOper
         } // if the signs do not match this operation can't overflow
         result = lhs + rhs;
         return true;
+#endif
     }
 
     static inline bool sub(LHS lhs, RHS rhs, ResultType& result) WARN_UNUSED_RETURN
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_sub_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         if (!signsMatch(lhs, rhs)) {
             if (lhs >= 0) {
                 if (lhs > std::numeric_limits<ResultType>::max() + rhs)
@@ -297,10 +355,18 @@ template <typename LHS, typename RHS, typename ResultType> struct ArithmeticOper
         } // if the signs match this operation can't overflow
         result = lhs - rhs;
         return true;
+#endif
     }
 
     static inline bool multiply(LHS lhs, RHS rhs, ResultType& result) WARN_UNUSED_RETURN
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_mul_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         if (signsMatch(lhs, rhs)) {
             if (lhs >= 0) {
                 if (lhs && (std::numeric_limits<ResultType>::max() / lhs) < rhs)
@@ -322,6 +388,7 @@ template <typename LHS, typename RHS, typename ResultType> struct ArithmeticOper
         }
         result = lhs * rhs;
         return true;
+#endif
     }
 
     static inline bool equals(LHS lhs, RHS rhs) { return lhs == rhs; }
@@ -332,24 +399,47 @@ template <typename LHS, typename RHS, typename ResultType> struct ArithmeticOper
     // LHS and RHS are unsigned types so bounds checks are nice and easy
     static inline bool add(LHS lhs, RHS rhs, ResultType& result) WARN_UNUSED_RETURN
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_add_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         ResultType temp = lhs + rhs;
         if (temp < lhs)
             return false;
         result = temp;
         return true;
+#endif
     }
 
     static inline bool sub(LHS lhs, RHS rhs, ResultType& result) WARN_UNUSED_RETURN
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_sub_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         ResultType temp = lhs - rhs;
         if (temp > lhs)
             return false;
         result = temp;
         return true;
+#endif
     }
 
     static inline bool multiply(LHS lhs, RHS rhs, ResultType& result) WARN_UNUSED_RETURN
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_mul_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         if (!lhs || !rhs) {
             result = 0;
             return true;
@@ -358,6 +448,7 @@ template <typename LHS, typename RHS, typename ResultType> struct ArithmeticOper
             return false;
         result = lhs * rhs;
         return true;
+#endif
     }
 
     static inline bool equals(LHS lhs, RHS rhs) { return lhs == rhs; }
@@ -367,6 +458,13 @@ template <typename LHS, typename RHS, typename ResultType> struct ArithmeticOper
 template <typename ResultType> struct ArithmeticOperations<int, unsigned, ResultType, true, false> {
     static inline bool add(int64_t lhs, int64_t rhs, ResultType& result)
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_add_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         int64_t temp = lhs + rhs;
         if (temp < std::numeric_limits<ResultType>::min())
             return false;
@@ -374,10 +472,18 @@ template <typename ResultType> struct ArithmeticOperations<int, unsigned, Result
             return false;
         result = static_cast<ResultType>(temp);
         return true;
+#endif
     }
 
     static inline bool sub(int64_t lhs, int64_t rhs, ResultType& result)
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_sub_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         int64_t temp = lhs - rhs;
         if (temp < std::numeric_limits<ResultType>::min())
             return false;
@@ -385,10 +491,18 @@ template <typename ResultType> struct ArithmeticOperations<int, unsigned, Result
             return false;
         result = static_cast<ResultType>(temp);
         return true;
+#endif
     }
 
     static inline bool multiply(int64_t lhs, int64_t rhs, ResultType& result)
     {
+#if COMPILER(GCC_COMPATIBLE)
+        ResultType temp;
+        if (__builtin_mul_overflow(lhs, rhs, &temp))
+            return false;
+        result = temp;
+        return true;
+#else
         int64_t temp = lhs * rhs;
         if (temp < std::numeric_limits<ResultType>::min())
             return false;
@@ -396,6 +510,7 @@ template <typename ResultType> struct ArithmeticOperations<int, unsigned, Result
             return false;
         result = static_cast<ResultType>(temp);
         return true;
+#endif
     }
 
     static inline bool equals(int lhs, unsigned rhs)
@@ -417,7 +532,7 @@ template <typename ResultType> struct ArithmeticOperations<unsigned, int, Result
 
     static inline bool multiply(int64_t lhs, int64_t rhs, ResultType& result)
     {
-        return ArithmeticOperations<int, unsigned, ResultType>::multiply(rhs, lhs, result);
+        return ArithmeticOperations<int, unsigned, ResultType>::multiply(lhs, rhs, result);
     }
 
     static inline bool equals(unsigned lhs, int rhs)
@@ -426,9 +541,25 @@ template <typename ResultType> struct ArithmeticOperations<unsigned, int, Result
     }
 };
 
+template <class OverflowHandler, typename = std::enable_if_t<!std::is_scalar<OverflowHandler>::value>>
+inline constexpr bool observesOverflow() { return true; }
+
+template <>
+inline constexpr bool observesOverflow<AssertNoOverflow>() { return !ASSERT_DISABLED; }
+
 template <typename U, typename V, typename R> static inline bool safeAdd(U lhs, V rhs, R& result)
 {
     return ArithmeticOperations<U, V, R>::add(lhs, rhs, result);
+    return true;
+}
+
+template <class OverflowHandler, typename U, typename V, typename R, typename = std::enable_if_t<!std::is_scalar<OverflowHandler>::value>>
+static inline bool safeAdd(U lhs, V rhs, R& result)
+{
+    if (observesOverflow<OverflowHandler>())
+        return safeAdd(lhs, rhs, result);
+    result = lhs + rhs;
+    return true;
 }
 
 template <typename U, typename V, typename R> static inline bool safeSub(U lhs, V rhs, R& result)
@@ -436,11 +567,29 @@ template <typename U, typename V, typename R> static inline bool safeSub(U lhs,
     return ArithmeticOperations<U, V, R>::sub(lhs, rhs, result);
 }
 
+template <class OverflowHandler, typename U, typename V, typename R, typename = std::enable_if_t<!std::is_scalar<OverflowHandler>::value>>
+static inline bool safeSub(U lhs, V rhs, R& result)
+{
+    if (observesOverflow<OverflowHandler>())
+        return safeSub(lhs, rhs, result);
+    result = lhs - rhs;
+    return true;
+}
+
 template <typename U, typename V, typename R> static inline bool safeMultiply(U lhs, V rhs, R& result)
 {
     return ArithmeticOperations<U, V, R>::multiply(lhs, rhs, result);
 }
 
+template <class OverflowHandler, typename U, typename V, typename R, typename = std::enable_if_t<!std::is_scalar<OverflowHandler>::value>>
+static inline bool safeMultiply(U lhs, V rhs, R& result)
+{
+    if (observesOverflow<OverflowHandler>())
+        return safeMultiply(lhs, rhs, result);
+    result = lhs * rhs;
+    return true;
+}
+
 template <typename U, typename V> static inline bool safeEquals(U lhs, V rhs)
 {
     return ArithmeticOperations<U, V>::equals(lhs, rhs);
@@ -560,11 +709,12 @@ public:
     }
 
     // Value accessors. unsafeGet() will crash if there's been an overflow.
-    T unsafeGet() const
+    template<typename U = T>
+    U unsafeGet() const
     {
         if (this->hasOverflowed())
             this->crash();
-        return m_value;
+        return static_cast<U>(m_value);
     }
 
     inline CheckedState safeGet(T& value) const WARN_UNUSED_RETURN
@@ -578,21 +728,21 @@ public:
     // Mutating assignment
     template <typename U> const Checked operator+=(U rhs)
     {
-        if (!safeAdd(m_value, rhs, m_value))
+        if (!safeAdd<OverflowHandler>(m_value, rhs, m_value))
             this->overflowed();
         return *this;
     }
 
     template <typename U> const Checked operator-=(U rhs)
     {
-        if (!safeSub(m_value, rhs, m_value))
+        if (!safeSub<OverflowHandler>(m_value, rhs, m_value))
             this->overflowed();
         return *this;
     }
 
     template <typename U> const Checked operator*=(U rhs)
     {
-        if (!safeMultiply(m_value, rhs, m_value))
+        if (!safeMultiply<OverflowHandler>(m_value, rhs, m_value))
             this->overflowed();
         return *this;
     }
@@ -716,7 +866,7 @@ template <typename U, typename V, typename OverflowHandler> static inline Checke
     V y = 0;
     bool overflowed = lhs.safeGet(x) == CheckedState::DidOverflow || rhs.safeGet(y) == CheckedState::DidOverflow;
     typename Result<U, V>::ResultType result = 0;
-    overflowed |= !safeAdd(x, y, result);
+    overflowed |= !safeAdd<OverflowHandler>(x, y, result);
     if (overflowed)
         return ResultOverflowed;
     return result;
@@ -728,7 +878,7 @@ template <typename U, typename V, typename OverflowHandler> static inline Checke
     V y = 0;
     bool overflowed = lhs.safeGet(x) == CheckedState::DidOverflow || rhs.safeGet(y) == CheckedState::DidOverflow;
     typename Result<U, V>::ResultType result = 0;
-    overflowed |= !safeSub(x, y, result);
+    overflowed |= !safeSub<OverflowHandler>(x, y, result);
     if (overflowed)
         return ResultOverflowed;
     return result;
@@ -740,7 +890,7 @@ template <typename U, typename V, typename OverflowHandler> static inline Checke
     V y = 0;
     bool overflowed = lhs.safeGet(x) == CheckedState::DidOverflow || rhs.safeGet(y) == CheckedState::DidOverflow;
     typename Result<U, V>::ResultType result = 0;
-    overflowed |= !safeMultiply(x, y, result);
+    overflowed |= !safeMultiply<OverflowHandler>(x, y, result);
     if (overflowed)
         return ResultOverflowed;
     return result;
@@ -832,6 +982,7 @@ template<typename T, typename... Args> bool productOverflows(Args... args)
 
 }
 
+using WTF::AssertNoOverflow;
 using WTF::Checked;
 using WTF::CheckedState;
 using WTF::CheckedInt8;
@@ -843,11 +994,10 @@ using WTF::CheckedUint32;
 using WTF::CheckedInt64;
 using WTF::CheckedUint64;
 using WTF::CheckedSize;
+using WTF::ConditionalCrashOnOverflow;
 using WTF::CrashOnOverflow;
 using WTF::RecordOverflow;
 using WTF::checkedSum;
 using WTF::differenceOverflows;
 using WTF::productOverflows;
 using WTF::sumOverflows;
-
-#endif
