@@ -22,13 +22,36 @@ void scheduleDispatchFunctionsOnMainThread()
     ASSERT(mid);
 
     env->CallStaticVoidMethod(jMainThreadCls, mid);
-    CheckAndClearException(env);
+    WTF::CheckAndClearException(env);
 }
 
 void initializeMainThreadPlatform()
 {
 }
 
+bool isMainThreadIfInitialized()
+{
+    return isMainThread();
+}
+
+bool isMainThread()
+{
+    AttachThreadAsNonDaemonToJavaEnv autoAttach;
+    JNIEnv* env = autoAttach.env();
+    static JGClass jMainThreadCls(env->FindClass("com/sun/webkit/MainThread"));
+
+    static jmethodID mid = env->GetStaticMethodID(
+            jMainThreadCls,
+            "fwkIsMainThread",
+            "()Z");
+
+    ASSERT(mid);
+
+    jboolean isMainThread = env->CallStaticBooleanMethod(jMainThreadCls, mid);
+    WTF::CheckAndClearException(env);
+    return isMainThread == JNI_TRUE;
+}
+
 extern "C" {
 
 /*
