@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2014-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2014-2019 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -23,8 +23,7 @@
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#ifndef StringView_h
-#define StringView_h
+#pragma once
 
 #include <limits.h>
 #include <unicode/utypes.h>
@@ -36,6 +35,7 @@
 #include <wtf/text/ConversionMode.h>
 #include <wtf/text/LChar.h>
 #include <wtf/text/StringCommon.h>
+#include <wtf/text/UTF8ConversionError.h>
 
 // FIXME: Enabling the StringView lifetime checking causes the MSVC build to fail. Figure out why.
 #if defined(NDEBUG) || COMPILER(MSVC)
@@ -97,16 +97,18 @@ public:
 
 #if USE(CF)
     // This function converts null strings to empty strings.
-    WTF_EXPORT_STRING_API RetainPtr<CFStringRef> createCFStringWithoutCopying() const;
+    WTF_EXPORT_PRIVATE RetainPtr<CFStringRef> createCFString() const;
+    WTF_EXPORT_PRIVATE RetainPtr<CFStringRef> createCFStringWithoutCopying() const;
 #endif
 
 #ifdef __OBJC__
     // These functions convert null strings to empty strings.
-    WTF_EXPORT_STRING_API RetainPtr<NSString> createNSString() const;
-    WTF_EXPORT_STRING_API RetainPtr<NSString> createNSStringWithoutCopying() const;
+    WTF_EXPORT_PRIVATE RetainPtr<NSString> createNSString() const;
+    WTF_EXPORT_PRIVATE RetainPtr<NSString> createNSStringWithoutCopying() const;
 #endif
 
-    WTF_EXPORT_STRING_API CString utf8(ConversionMode = LenientConversion) const;
+    WTF_EXPORT_PRIVATE Expected<CString, UTF8ConversionError> tryGetUtf8(ConversionMode = LenientConversion) const;
+    WTF_EXPORT_PRIVATE CString utf8(ConversionMode = LenientConversion) const;
 
     class UpconvertedCharacters;
     UpconvertedCharacters upconvertedCharacters() const;
@@ -123,31 +125,35 @@ public:
 
     class SplitResult;
     SplitResult split(UChar) const;
+    SplitResult splitAllowingEmptyEntries(UChar) const;
 
     size_t find(UChar, unsigned start = 0) const;
     size_t find(CodeUnitMatchFunction, unsigned start = 0) const;
 
-    WTF_EXPORT_STRING_API size_t find(StringView, unsigned start) const;
+    WTF_EXPORT_PRIVATE size_t find(StringView, unsigned start) const;
 
     size_t reverseFind(UChar, unsigned index = UINT_MAX) const;
 
-    WTF_EXPORT_STRING_API size_t findIgnoringASCIICase(const StringView&) const;
-    WTF_EXPORT_STRING_API size_t findIgnoringASCIICase(const StringView&, unsigned startOffset) const;
+    WTF_EXPORT_PRIVATE size_t findIgnoringASCIICase(const StringView&) const;
+    WTF_EXPORT_PRIVATE size_t findIgnoringASCIICase(const StringView&, unsigned startOffset) const;
+
+    WTF_EXPORT_PRIVATE String convertToASCIILowercase() const;
+    WTF_EXPORT_PRIVATE String convertToASCIIUppercase() const;
 
     bool contains(UChar) const;
-    WTF_EXPORT_STRING_API bool containsIgnoringASCIICase(const StringView&) const;
-    WTF_EXPORT_STRING_API bool containsIgnoringASCIICase(const StringView&, unsigned startOffset) const;
+    WTF_EXPORT_PRIVATE bool containsIgnoringASCIICase(const StringView&) const;
+    WTF_EXPORT_PRIVATE bool containsIgnoringASCIICase(const StringView&, unsigned startOffset) const;
 
-    WTF_EXPORT_STRING_API bool startsWith(const StringView&) const;
-    WTF_EXPORT_STRING_API bool startsWithIgnoringASCIICase(const StringView&) const;
+    WTF_EXPORT_PRIVATE bool startsWith(const StringView&) const;
+    WTF_EXPORT_PRIVATE bool startsWithIgnoringASCIICase(const StringView&) const;
 
-    WTF_EXPORT_STRING_API bool endsWith(const StringView&) const;
-    WTF_EXPORT_STRING_API bool endsWithIgnoringASCIICase(const StringView&) const;
+    WTF_EXPORT_PRIVATE bool endsWith(const StringView&) const;
+    WTF_EXPORT_PRIVATE bool endsWithIgnoringASCIICase(const StringView&) const;
 
     int toInt() const;
     int toInt(bool& isValid) const;
     int toIntStrict(bool& isValid) const;
-    std::optional<uint64_t> toUInt64Strict() const;
+    Optional<uint64_t> toUInt64Strict() const;
     float toFloat(bool& isValid) const;
 
     static void invalidate(const StringImpl&);
@@ -159,16 +165,14 @@ private:
 
     void initialize(const LChar*, unsigned length);
     void initialize(const UChar*, unsigned length);
-    void initialize(const LChar*, unsigned length, unsigned mask);
-    void initialize(const UChar*, unsigned length, unsigned mask);
 
     template<typename CharacterType, typename MatchedCharacterPredicate>
     StringView stripLeadingAndTrailingMatchedCharacters(const CharacterType*, const MatchedCharacterPredicate&);
 
 #if CHECK_STRINGVIEW_LIFETIME
-    WTF_EXPORT_STRING_API bool underlyingStringIsValid() const;
-    WTF_EXPORT_STRING_API void setUnderlyingString(const StringImpl*);
-    WTF_EXPORT_STRING_API void setUnderlyingString(const StringView&);
+    WTF_EXPORT_PRIVATE bool underlyingStringIsValid() const;
+    WTF_EXPORT_PRIVATE void setUnderlyingString(const StringImpl*);
+    WTF_EXPORT_PRIVATE void setUnderlyingString(const StringView&);
 #else
     bool underlyingStringIsValid() const { return true; }
     void setUnderlyingString(const StringImpl*) { }
@@ -178,7 +182,6 @@ private:
 
     const void* m_characters { nullptr };
     unsigned m_length { 0 };
-    unsigned m_mask { 0 };
     bool m_is8Bit { true };
 
 #if CHECK_STRINGVIEW_LIFETIME
@@ -190,8 +193,7 @@ private:
 template<typename CharacterType, size_t inlineCapacity> void append(Vector<CharacterType, inlineCapacity>&, StringView);
 
 bool equal(StringView, StringView);
-bool equal(StringView, const LChar*);
-bool equal(StringView, const char*);
+bool equal(StringView, const LChar* b);
 
 bool equalIgnoringASCIICase(StringView, StringView);
 bool equalIgnoringASCIICase(StringView, const char*);
@@ -199,16 +201,25 @@ bool equalIgnoringASCIICase(StringView, const char*);
 template<unsigned length> bool equalLettersIgnoringASCIICase(StringView, const char (&lowercaseLetters)[length]);
 
 inline bool operator==(StringView a, StringView b) { return equal(a, b); }
-inline bool operator==(StringView a, const LChar* b) { return equal(a, b); }
-inline bool operator==(StringView a, const char* b) { return equal(a, b); }
-inline bool operator==(const LChar* a, StringView b) { return equal(b, a); }
+inline bool operator==(StringView a, const LChar *b);
+inline bool operator==(StringView a, const char *b) { return equal(a, reinterpret_cast<const LChar*>(b)); }
 inline bool operator==(const char* a, StringView b) { return equal(b, a); }
 
 inline bool operator!=(StringView a, StringView b) { return !equal(a, b); }
 inline bool operator!=(StringView a, const LChar* b) { return !equal(a, b); }
 inline bool operator!=(StringView a, const char* b) { return !equal(a, b); }
-inline bool operator!=(const LChar* a, StringView b) { return !equal(b, a); }
-inline bool operator!=(const char* a, StringView b) { return !equal(b, a); }
+inline bool operator!=(const LChar*a, StringView b) { return !equal(b, a); }
+inline bool operator!=(const char*a, StringView b) { return !equal(b, a); }
+
+struct StringViewWithUnderlyingString;
+
+// This returns a StringView of the normalized result, and a String that is either
+// null, if the input was already normalized, or contains the normalized result
+// and needs to be kept around so the StringView remains valid. Typically the
+// easiest way to use it correctly is to put it into a local and use the StringView.
+WTF_EXPORT_PRIVATE StringViewWithUnderlyingString normalizedNFC(StringView);
+
+WTF_EXPORT_PRIVATE String normalizedNFC(const String&);
 
 }
 
@@ -217,12 +228,17 @@ inline bool operator!=(const char* a, StringView b) { return !equal(b, a); }
 
 namespace WTF {
 
+struct StringViewWithUnderlyingString {
+    StringView view;
+    String underlyingString;
+};
+
 inline StringView::StringView()
 {
-    // FIXME: It's peculiar that null strings are 16-bit and empty strings return 8-bit (according to the is8Bit function).
 }
 
 #if CHECK_STRINGVIEW_LIFETIME
+
 inline StringView::~StringView()
 {
     setUnderlyingString(nullptr);
@@ -231,7 +247,6 @@ inline StringView::~StringView()
 inline StringView::StringView(StringView&& other)
     : m_characters(other.m_characters)
     , m_length(other.m_length)
-    , m_mask(other.m_mask)
     , m_is8Bit(other.m_is8Bit)
 {
     ASSERT(other.underlyingStringIsValid());
@@ -245,7 +260,6 @@ inline StringView::StringView(StringView&& other)
 inline StringView::StringView(const StringView& other)
     : m_characters(other.m_characters)
     , m_length(other.m_length)
-    , m_mask(other.m_mask)
     , m_is8Bit(other.m_is8Bit)
 {
     ASSERT(other.underlyingStringIsValid());
@@ -259,7 +273,6 @@ inline StringView& StringView::operator=(StringView&& other)
 
     m_characters = other.m_characters;
     m_length = other.m_length;
-    m_mask = other.m_mask;
     m_is8Bit = other.m_is8Bit;
 
     other.clear();
@@ -276,38 +289,26 @@ inline StringView& StringView::operator=(const StringView& other)
 
     m_characters = other.m_characters;
     m_length = other.m_length;
-    m_mask = other.m_mask;
     m_is8Bit = other.m_is8Bit;
 
     setUnderlyingString(other);
 
     return *this;
 }
+
 #endif // CHECK_STRINGVIEW_LIFETIME
 
 inline void StringView::initialize(const LChar* characters, unsigned length)
 {
-    initialize(characters, length, maskForSize(length));
+    m_characters = characters;
+    m_length = length;
+    m_is8Bit = true;
 }
 
 inline void StringView::initialize(const UChar* characters, unsigned length)
-{
-    initialize(characters, length, maskForSize(length));
-}
-
-inline void StringView::initialize(const LChar* characters, unsigned length, unsigned mask)
 {
     m_characters = characters;
     m_length = length;
-    m_mask = mask;
-    m_is8Bit = true;
-}
-
-inline void StringView::initialize(const UChar* characters, unsigned length, unsigned mask)
-{
-    m_characters = characters;
-    m_length = length;
-    m_mask = mask;
     m_is8Bit = false;
 }
 
@@ -330,9 +331,9 @@ inline StringView::StringView(const StringImpl& string)
 {
     setUnderlyingString(&string);
     if (string.is8Bit())
-        initialize(string.characters8(), string.length(), string.mask());
+        initialize(string.characters8(), string.length());
     else
-        initialize(string.characters16(), string.length(), string.mask());
+        initialize(string.characters16(), string.length());
 }
 
 inline StringView::StringView(const StringImpl* string)
@@ -342,9 +343,9 @@ inline StringView::StringView(const StringImpl* string)
 
     setUnderlyingString(string);
     if (string->is8Bit())
-        initialize(string->characters8(), string->length(), string->mask());
+        initialize(string->characters8(), string->length());
     else
-        initialize(string->characters16(), string->length(), string->mask());
+        initialize(string->characters16(), string->length());
 }
 
 inline StringView::StringView(const String& string)
@@ -355,10 +356,10 @@ inline StringView::StringView(const String& string)
         return;
     }
     if (string.is8Bit()) {
-        initialize(string.characters8(), string.length(), string.mask());
+        initialize(string.characters8(), string.length());
         return;
     }
-    initialize(string.characters16(), string.length(), string.mask());
+    initialize(string.characters16(), string.length());
 }
 
 inline StringView::StringView(const AtomicString& atomicString)
@@ -370,7 +371,6 @@ inline void StringView::clear()
 {
     m_characters = nullptr;
     m_length = 0;
-    m_mask = 0;
     m_is8Bit = true;
 }
 
@@ -459,8 +459,8 @@ inline UChar StringView::operator[](unsigned index) const
 {
     ASSERT(index < length());
     if (is8Bit())
-        return characters8()[index & m_mask];
-    return characters16()[index & m_mask];
+        return characters8()[index];
+    return characters16()[index];
 }
 
 inline bool StringView::contains(UChar character) const
@@ -545,11 +545,11 @@ inline int StringView::toIntStrict(bool& isValid) const
     return charactersToIntStrict(characters16(), m_length, &isValid);
 }
 
-inline std::optional<uint64_t> StringView::toUInt64Strict() const
+inline Optional<uint64_t> StringView::toUInt64Strict() const
 {
     bool isValid;
     uint64_t result = is8Bit() ? charactersToUInt64Strict(characters8(), m_length, &isValid) : charactersToUInt64Strict(characters16(), m_length, &isValid);
-    return isValid ? std::make_optional(result) : std::nullopt;
+    return isValid ? makeOptional(result) : WTF::nullopt;
 }
 
 inline String StringView::toStringWithoutCopying() const
@@ -581,26 +581,23 @@ inline size_t StringView::reverseFind(UChar character, unsigned index) const
 }
 
 #if !CHECK_STRINGVIEW_LIFETIME
+
 inline void StringView::invalidate(const StringImpl&)
 {
 }
-#endif
 
-template<typename StringType, typename> class StringTypeAdapter;
+#endif
 
 template<> class StringTypeAdapter<StringView, void> {
 public:
     StringTypeAdapter(StringView string)
-        : m_string(string)
+        : m_string { string }
     {
     }
 
     unsigned length() { return m_string.length(); }
     bool is8Bit() { return m_string.is8Bit(); }
-    void writeTo(LChar* destination) { m_string.getCharactersWithUpconvert(destination); }
-    void writeTo(UChar* destination) { m_string.getCharactersWithUpconvert(destination); }
-
-    String toString() const { return m_string.toString(); }
+    template<typename CharacterType> void writeTo(CharacterType* destination) { m_string.getCharactersWithUpconvert(destination); }
 
 private:
     StringView m_string;
@@ -629,17 +626,16 @@ inline bool equal(StringView a, const LChar* b)
         return !a.isEmpty();
     if (a.isEmpty())
         return !b;
+
     unsigned aLength = a.length();
+    if (aLength != strlen(reinterpret_cast<const char*>(b)))
+        return false;
+
     if (a.is8Bit())
         return equal(a.characters8(), b, aLength);
     return equal(a.characters16(), b, aLength);
 }
 
-inline bool equal(StringView a, const char* b)
-{
-    return equal(a, reinterpret_cast<const LChar*>(b));
-}
-
 inline bool equalIgnoringASCIICase(StringView a, StringView b)
 {
     return equalIgnoringASCIICaseCommon(a, b);
@@ -652,7 +648,7 @@ inline bool equalIgnoringASCIICase(StringView a, const char* b)
 
 class StringView::SplitResult {
 public:
-    explicit SplitResult(StringView, UChar separator);
+    SplitResult(StringView, UChar separator, bool allowEmptyEntries);
 
     class Iterator;
     Iterator begin() const;
@@ -661,6 +657,7 @@ public:
 private:
     StringView m_string;
     UChar m_separator;
+    bool m_allowEmptyEntries;
 };
 
 class StringView::GraphemeClusters {
@@ -720,6 +717,7 @@ private:
     const SplitResult& m_result;
     unsigned m_position { 0 };
     unsigned m_length;
+    bool m_isDone;
 };
 
 class StringView::GraphemeClusters::Iterator {
@@ -758,7 +756,7 @@ public:
 
 private:
     std::reference_wrapper<const StringView> m_stringView;
-    std::optional<unsigned> m_nextCodePointOffset;
+    Optional<unsigned> m_nextCodePointOffset;
     UChar32 m_codePoint;
 };
 
@@ -823,7 +821,7 @@ inline auto StringView::CodePoints::Iterator::operator++() -> Iterator&
 {
     ASSERT(m_nextCodePointOffset);
     if (m_nextCodePointOffset.value() == m_stringView.get().length()) {
-        m_nextCodePointOffset = std::nullopt;
+        m_nextCodePointOffset = WTF::nullopt;
         return *this;
     }
     if (m_stringView.get().is8Bit())
@@ -914,12 +912,18 @@ inline auto StringView::CodeUnits::end() const -> Iterator
 
 inline auto StringView::split(UChar separator) const -> SplitResult
 {
-    return SplitResult { *this, separator };
+    return SplitResult { *this, separator, false };
 }
 
-inline StringView::SplitResult::SplitResult(StringView stringView, UChar separator)
+inline auto StringView::splitAllowingEmptyEntries(UChar separator) const -> SplitResult
+{
+    return SplitResult { *this, separator, true };
+}
+
+inline StringView::SplitResult::SplitResult(StringView stringView, UChar separator, bool allowEmptyEntries)
     : m_string { stringView }
     , m_separator { separator }
+    , m_allowEmptyEntries { allowEmptyEntries }
 {
 }
 
@@ -935,6 +939,7 @@ inline auto StringView::SplitResult::end() const -> Iterator
 
 inline StringView::SplitResult::Iterator::Iterator(const SplitResult& result)
     : m_result { result }
+    , m_isDone { result.m_string.isEmpty() && !result.m_allowEmptyEntries }
 {
     findNextSubstring();
 }
@@ -942,19 +947,20 @@ inline StringView::SplitResult::Iterator::Iterator(const SplitResult& result)
 inline StringView::SplitResult::Iterator::Iterator(const SplitResult& result, PositionTag)
     : m_result { result }
     , m_position { result.m_string.length() }
+    , m_isDone { true }
 {
 }
 
 inline StringView StringView::SplitResult::Iterator::operator*() const
 {
-    ASSERT(m_position < m_result.m_string.length());
+    ASSERT(m_position <= m_result.m_string.length() && !m_isDone);
     return m_result.m_string.substring(m_position, m_length);
 }
 
 inline bool StringView::SplitResult::Iterator::operator==(const Iterator& other) const
 {
     ASSERT(&m_result == &other.m_result);
-    return m_position == other.m_position;
+    return m_position == other.m_position && m_isDone == other.m_isDone;
 }
 
 inline bool StringView::SplitResult::Iterator::operator!=(const Iterator& other) const
@@ -1006,5 +1012,4 @@ template<unsigned length> inline bool equalLettersIgnoringASCIICase(StringView s
 using WTF::append;
 using WTF::equal;
 using WTF::StringView;
-
-#endif
+using WTF::StringViewWithUnderlyingString;
