@@ -148,6 +148,7 @@ namespace WebCore {
             PseudoClassFullScreenDocument,
             PseudoClassFullScreenAncestor,
             PseudoClassAnimatingFullScreenTransition,
+            PseudoClassFullScreenControlsHidden,
 #endif
             PseudoClassInRange,
             PseudoClassOutOfRange,
@@ -161,6 +162,9 @@ namespace WebCore {
 #endif
             PseudoClassHost,
             PseudoClassDefined,
+#if ENABLE(ATTACHMENT_ELEMENT)
+            PseudoClassHasAttachment,
+#endif
         };
 
         enum PseudoElementType {
@@ -182,7 +186,6 @@ namespace WebCore {
             PseudoElementScrollbarTrackPiece,
             PseudoElementSelection,
             PseudoElementSlotted,
-            PseudoElementUserAgentCustom,
             PseudoElementWebKitCustom,
 
             // WebKitCustom that appeared in an old prefixed form
@@ -241,16 +244,12 @@ namespace WebCore {
 
         void setValue(const AtomicString&, bool matchLowerCase = false);
 
-        // FIXME-NEWPARSER: These two methods can go away once the old parser is gone.
-        void setAttribute(const QualifiedName&, bool);
-        void setAttributeValueMatchingIsCaseInsensitive(bool);
         void setAttribute(const QualifiedName&, bool convertToLowercase, AttributeMatchType);
         void setNth(int a, int b);
         void setArgument(const AtomicString&);
         void setLangArgumentList(std::unique_ptr<Vector<AtomicString>>);
         void setSelectorList(std::unique_ptr<CSSSelectorList>);
 
-        bool parseNth() const;
         bool matchNth(int count) const;
         int nthA() const;
         int nthB() const;
@@ -325,7 +324,6 @@ namespace WebCore {
         unsigned m_relation              : 4; // enum RelationType.
         mutable unsigned m_match         : 4; // enum Match.
         mutable unsigned m_pseudoType    : 8; // PseudoType.
-        mutable unsigned m_parsedNth     : 1; // Used for :nth-*.
         unsigned m_isLastInSelectorList  : 1;
         unsigned m_isLastInTagHistory    : 1;
         unsigned m_hasRareData           : 1;
@@ -346,7 +344,6 @@ namespace WebCore {
             static Ref<RareData> create(AtomicString&& value) { return adoptRef(*new RareData(WTFMove(value))); }
             ~RareData();
 
-            bool parseNth();
             bool matchNth(int count);
 
             // For quirks mode, class and id are case-insensitive. In the case where uppercase
@@ -416,8 +413,7 @@ inline bool CSSSelector::isUnknownPseudoElement() const
 inline bool CSSSelector::isCustomPseudoElement() const
 {
     return match() == PseudoElement
-        && (pseudoElementType() == PseudoElementUserAgentCustom
-            || pseudoElementType() == PseudoElementWebKitCustom
+        && (pseudoElementType() == PseudoElementWebKitCustom
             || pseudoElementType() == PseudoElementWebKitCustomLegacyPrefixed);
 }
 
@@ -483,7 +479,6 @@ inline CSSSelector::CSSSelector()
     : m_relation(DescendantSpace)
     , m_match(Unknown)
     , m_pseudoType(0)
-    , m_parsedNth(false)
     , m_isLastInSelectorList(false)
     , m_isLastInTagHistory(true)
     , m_hasRareData(false)
@@ -501,7 +496,6 @@ inline CSSSelector::CSSSelector(const CSSSelector& o)
     : m_relation(o.m_relation)
     , m_match(o.m_match)
     , m_pseudoType(o.m_pseudoType)
-    , m_parsedNth(o.m_parsedNth)
     , m_isLastInSelectorList(o.m_isLastInSelectorList)
     , m_isLastInTagHistory(o.m_isLastInTagHistory)
     , m_hasRareData(o.m_hasRareData)
@@ -587,12 +581,6 @@ inline const AtomicString& CSSSelector::serializingValue() const
     return *reinterpret_cast<const AtomicString*>(&m_data.m_value);
 }
 
-inline void CSSSelector::setAttributeValueMatchingIsCaseInsensitive(bool isCaseInsensitive)
-{
-    ASSERT(isAttributeSelector() && match() != CSSSelector::Set);
-    m_caseInsensitiveAttributeValueMatching = isCaseInsensitive;
-}
-
 inline bool CSSSelector::attributeValueMatchingIsCaseInsensitive() const
 {
     return m_caseInsensitiveAttributeValueMatching;
