@@ -41,7 +41,7 @@
 namespace WebCore {
 
 // https://drafts.fxtf.org/geometry/#dom-dommatrixreadonly-dommatrixreadonly
-ExceptionOr<Ref<DOMMatrixReadOnly>> DOMMatrixReadOnly::create(ScriptExecutionContext& scriptExecutionContext, std::optional<Variant<String, Vector<double>>>&& init)
+ExceptionOr<Ref<DOMMatrixReadOnly>> DOMMatrixReadOnly::create(ScriptExecutionContext& scriptExecutionContext, Optional<Variant<String, Vector<double>>>&& init)
 {
     if (!init)
         return adoptRef(*new DOMMatrixReadOnly);
@@ -107,30 +107,30 @@ static bool sameValueZero(double a, double b)
 ExceptionOr<void> DOMMatrixReadOnly::validateAndFixup(DOMMatrix2DInit& init)
 {
     if (init.a && init.m11 && !sameValueZero(init.a.value(), init.m11.value()))
-        return Exception { TypeError, ASCIILiteral("init.a and init.m11 do not match") };
+        return Exception { TypeError, "init.a and init.m11 do not match"_s };
     if (init.b && init.m12 && !sameValueZero(init.b.value(), init.m12.value()))
-        return Exception { TypeError, ASCIILiteral("init.b and init.m12 do not match") };
+        return Exception { TypeError, "init.b and init.m12 do not match"_s };
     if (init.c && init.m21 && !sameValueZero(init.c.value(), init.m21.value()))
-        return Exception { TypeError, ASCIILiteral("init.c and init.m21 do not match") };
+        return Exception { TypeError, "init.c and init.m21 do not match"_s };
     if (init.d && init.m22 && !sameValueZero(init.d.value(), init.m22.value()))
-        return Exception { TypeError, ASCIILiteral("init.d and init.m22 do not match") };
+        return Exception { TypeError, "init.d and init.m22 do not match"_s };
     if (init.e && init.m41 && !sameValueZero(init.e.value(), init.m41.value()))
-        return Exception { TypeError, ASCIILiteral("init.e and init.m41 do not match") };
+        return Exception { TypeError, "init.e and init.m41 do not match"_s };
     if (init.f && init.m42 && !sameValueZero(init.f.value(), init.m42.value()))
-        return Exception { TypeError, ASCIILiteral("init.f and init.m42 do not match") };
+        return Exception { TypeError, "init.f and init.m42 do not match"_s };
 
     if (!init.m11)
-        init.m11 = init.a.value_or(1);
+        init.m11 = init.a.valueOr(1);
     if (!init.m12)
-        init.m12 = init.b.value_or(0);
+        init.m12 = init.b.valueOr(0);
     if (!init.m21)
-        init.m21 = init.c.value_or(0);
+        init.m21 = init.c.valueOr(0);
     if (!init.m22)
-        init.m22 = init.d.value_or(1);
+        init.m22 = init.d.valueOr(1);
     if (!init.m41)
-        init.m41 = init.e.value_or(0);
+        init.m41 = init.e.valueOr(0);
     if (!init.m42)
-        init.m42 = init.f.value_or(0);
+        init.m42 = init.f.valueOr(0);
 
     return { };
 }
@@ -143,25 +143,25 @@ ExceptionOr<void> DOMMatrixReadOnly::validateAndFixup(DOMMatrixInit& init)
 
     if (init.is2D && init.is2D.value()) {
         if (init.m13)
-            return Exception { TypeError, ASCIILiteral("m13 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m13 should be 0 for a 2D matrix"_s };
         if (init.m14)
-            return Exception { TypeError, ASCIILiteral("m14 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m14 should be 0 for a 2D matrix"_s };
         if (init.m23)
-            return Exception { TypeError, ASCIILiteral("m23 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m23 should be 0 for a 2D matrix"_s };
         if (init.m24)
-            return Exception { TypeError, ASCIILiteral("m24 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m24 should be 0 for a 2D matrix"_s };
         if (init.m31)
-            return Exception { TypeError, ASCIILiteral("m31 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m31 should be 0 for a 2D matrix"_s };
         if (init.m32)
-            return Exception { TypeError, ASCIILiteral("m32 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m32 should be 0 for a 2D matrix"_s };
         if (init.m34)
-            return Exception { TypeError, ASCIILiteral("m34 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m34 should be 0 for a 2D matrix"_s };
         if (init.m43)
-            return Exception { TypeError, ASCIILiteral("m43 should be 0 for a 2D matrix") };
+            return Exception { TypeError, "m43 should be 0 for a 2D matrix"_s };
         if (init.m33 != 1)
-            return Exception { TypeError, ASCIILiteral("m33 should be 1 for a 2D matrix") };
+            return Exception { TypeError, "m33 should be 1 for a 2D matrix"_s };
         if (init.m44 != 1)
-            return Exception { TypeError, ASCIILiteral("m44 should be 1 for a 2D matrix") };
+            return Exception { TypeError, "m44 should be 1 for a 2D matrix"_s };
     }
 
     if (!init.is2D) {
@@ -288,7 +288,7 @@ ExceptionOr<Ref<DOMMatrix>> DOMMatrixReadOnly::multiply(DOMMatrixInit&& other) c
     return matrix->multiplySelf(WTFMove(other));
 }
 
-Ref<DOMMatrix> DOMMatrixReadOnly::scale(double scaleX, std::optional<double> scaleY, double scaleZ, double originX, double originY, double originZ)
+Ref<DOMMatrix> DOMMatrixReadOnly::scale(double scaleX, Optional<double> scaleY, double scaleZ, double originX, double originY, double originZ)
 {
     auto matrix = cloneAsDOMMatrix();
     return matrix->scaleSelf(scaleX, scaleY, scaleZ, originX, originY, originZ);
@@ -300,7 +300,7 @@ Ref<DOMMatrix> DOMMatrixReadOnly::scale3d(double scale, double originX, double o
     return matrix->scale3dSelf(scale, originX, originY, originZ);
 }
 
-Ref<DOMMatrix> DOMMatrixReadOnly::rotate(double rotX, std::optional<double> rotY, std::optional<double> rotZ)
+Ref<DOMMatrix> DOMMatrixReadOnly::rotate(double rotX, Optional<double> rotY, Optional<double> rotZ)
 {
     auto matrix = cloneAsDOMMatrix();
     return matrix->rotateSelf(rotX, rotY, rotZ);
@@ -345,9 +345,9 @@ Ref<DOMPoint> DOMMatrixReadOnly::transformPoint(DOMPointInit&& pointInit)
 
 ExceptionOr<Ref<Float32Array>> DOMMatrixReadOnly::toFloat32Array() const
 {
-    auto array32 = Float32Array::createUninitialized(16);
+    auto array32 = Float32Array::tryCreateUninitialized(16);
     if (!array32)
-        return Exception { UnknownError, ASCIILiteral("Out of memory") };
+        return Exception { UnknownError, "Out of memory"_s };
 
     unsigned index = 0;
     array32->set(index++, m_matrix.m11());
@@ -371,9 +371,9 @@ ExceptionOr<Ref<Float32Array>> DOMMatrixReadOnly::toFloat32Array() const
 
 ExceptionOr<Ref<Float64Array>> DOMMatrixReadOnly::toFloat64Array() const
 {
-    auto array64 = Float64Array::createUninitialized(16);
+    auto array64 = Float64Array::tryCreateUninitialized(16);
     if (!array64)
-        return Exception { UnknownError, ASCIILiteral("Out of memory") };
+        return Exception { UnknownError, "Out of memory"_s };
 
     unsigned index = 0;
     array64->set(index++, m_matrix.m11());
@@ -399,55 +399,55 @@ ExceptionOr<Ref<Float64Array>> DOMMatrixReadOnly::toFloat64Array() const
 ExceptionOr<String> DOMMatrixReadOnly::toString() const
 {
     if (!m_matrix.containsOnlyFiniteValues())
-        return Exception { InvalidStateError, ASCIILiteral("Matrix contains non-finite values") };
+        return Exception { InvalidStateError, "Matrix contains non-finite values"_s };
 
     StringBuilder builder;
     if (is2D()) {
         builder.appendLiteral("matrix(");
-        builder.append(String::numberToStringECMAScript(m_matrix.a()));
+        builder.appendECMAScriptNumber(m_matrix.a());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.b()));
+        builder.appendECMAScriptNumber(m_matrix.b());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.c()));
+        builder.appendECMAScriptNumber(m_matrix.c());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.d()));
+        builder.appendECMAScriptNumber(m_matrix.d());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.e()));
+        builder.appendECMAScriptNumber(m_matrix.e());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.f()));
+        builder.appendECMAScriptNumber(m_matrix.f());
     } else {
         builder.appendLiteral("matrix3d(");
-        builder.append(String::numberToStringECMAScript(m_matrix.m11()));
+        builder.appendECMAScriptNumber(m_matrix.m11());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m12()));
+        builder.appendECMAScriptNumber(m_matrix.m12());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m13()));
+        builder.appendECMAScriptNumber(m_matrix.m13());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m14()));
+        builder.appendECMAScriptNumber(m_matrix.m14());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m21()));
+        builder.appendECMAScriptNumber(m_matrix.m21());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m22()));
+        builder.appendECMAScriptNumber(m_matrix.m22());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m23()));
+        builder.appendECMAScriptNumber(m_matrix.m23());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m24()));
+        builder.appendECMAScriptNumber(m_matrix.m24());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m31()));
+        builder.appendECMAScriptNumber(m_matrix.m31());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m32()));
+        builder.appendECMAScriptNumber(m_matrix.m32());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m33()));
+        builder.appendECMAScriptNumber(m_matrix.m33());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m34()));
+        builder.appendECMAScriptNumber(m_matrix.m34());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m41()));
+        builder.appendECMAScriptNumber(m_matrix.m41());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m42()));
+        builder.appendECMAScriptNumber(m_matrix.m42());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m43()));
+        builder.appendECMAScriptNumber(m_matrix.m43());
         builder.appendLiteral(", ");
-        builder.append(String::numberToStringECMAScript(m_matrix.m44()));
+        builder.appendECMAScriptNumber(m_matrix.m44());
     }
     builder.append(')');
     return builder.toString();
