@@ -28,6 +28,8 @@
 #include "CSSDeferredParser.h"
 #include "CSSParser.h"
 #include "CSSPendingSubstitutionValue.h"
+#include "CSSPropertyParser.h"
+#include "CSSTokenizer.h"
 #include "CSSValueKeywords.h"
 #include "CSSValueList.h"
 #include "CSSValuePool.h"
@@ -148,7 +150,7 @@ String StyleProperties::getPropertyValue(CSSPropertyID propertyID) const
     case CSSPropertyBackground:
         return getLayeredShorthandValue(backgroundShorthand());
     case CSSPropertyBorder:
-        return borderPropertyValue(OmitUncommonValues);
+        return borderPropertyValue(borderWidthShorthand(), borderStyleShorthand(), borderColorShorthand());
     case CSSPropertyBorderTop:
         return getShorthandValue(borderTopShorthand());
     case CSSPropertyBorderRight:
@@ -157,6 +159,30 @@ String StyleProperties::getPropertyValue(CSSPropertyID propertyID) const
         return getShorthandValue(borderBottomShorthand());
     case CSSPropertyBorderLeft:
         return getShorthandValue(borderLeftShorthand());
+    case CSSPropertyBorderBlock:
+        return borderPropertyValue(borderBlockWidthShorthand(), borderBlockStyleShorthand(), borderBlockColorShorthand());
+    case CSSPropertyBorderBlockColor:
+        return get2Values(borderBlockColorShorthand());
+    case CSSPropertyBorderBlockStyle:
+        return get2Values(borderBlockStyleShorthand());
+    case CSSPropertyBorderBlockWidth:
+        return get2Values(borderBlockWidthShorthand());
+    case CSSPropertyBorderBlockStart:
+        return getShorthandValue(borderBlockStartShorthand());
+    case CSSPropertyBorderBlockEnd:
+        return getShorthandValue(borderBlockEndShorthand());
+    case CSSPropertyBorderInline:
+        return borderPropertyValue(borderInlineWidthShorthand(), borderInlineStyleShorthand(), borderInlineColorShorthand());
+    case CSSPropertyBorderInlineColor:
+        return get2Values(borderInlineColorShorthand());
+    case CSSPropertyBorderInlineStyle:
+        return get2Values(borderInlineStyleShorthand());
+    case CSSPropertyBorderInlineWidth:
+        return get2Values(borderInlineWidthShorthand());
+    case CSSPropertyBorderInlineStart:
+        return getShorthandValue(borderInlineStartShorthand());
+    case CSSPropertyBorderInlineEnd:
+        return getShorthandValue(borderInlineEndShorthand());
     case CSSPropertyOutline:
         return getShorthandValue(outlineShorthand());
     case CSSPropertyBorderColor:
@@ -191,14 +217,28 @@ String StyleProperties::getPropertyValue(CSSPropertyID propertyID) const
         return getAlignmentShorthandValue(placeSelfShorthand());
     case CSSPropertyFont:
         return fontValue();
+    case CSSPropertyInset:
+        return get4Values(insetShorthand());
+    case CSSPropertyInsetBlock:
+        return get2Values(insetBlockShorthand());
+    case CSSPropertyInsetInline:
+        return get2Values(insetInlineShorthand());
     case CSSPropertyMargin:
         return get4Values(marginShorthand());
+    case CSSPropertyMarginBlock:
+        return get2Values(marginBlockShorthand());
+    case CSSPropertyMarginInline:
+        return get2Values(marginInlineShorthand());
     case CSSPropertyWebkitMarginCollapse:
         return getShorthandValue(webkitMarginCollapseShorthand());
     case CSSPropertyOverflow:
         return getCommonValue(overflowShorthand());
     case CSSPropertyPadding:
         return get4Values(paddingShorthand());
+    case CSSPropertyPaddingBlock:
+        return get2Values(paddingBlockShorthand());
+    case CSSPropertyPaddingInline:
+        return get2Values(paddingInlineShorthand());
     case CSSPropertyTransition:
         return getLayeredShorthandValue(transitionShorthand());
     case CSSPropertyListStyle:
@@ -238,11 +278,11 @@ String StyleProperties::getPropertyValue(CSSPropertyID propertyID) const
     }
 }
 
-std::optional<Color> StyleProperties::propertyAsColor(CSSPropertyID property) const
+Optional<Color> StyleProperties::propertyAsColor(CSSPropertyID property) const
 {
     auto colorValue = getPropertyCSSValue(property);
     if (!is<CSSPrimitiveValue>(colorValue))
-        return std::nullopt;
+        return WTF::nullopt;
 
     auto& primitiveColor = downcast<CSSPrimitiveValue>(*colorValue);
     return primitiveColor.isRGBColor() ? primitiveColor.color() : CSSParser::parseColor(colorValue->cssText());
@@ -346,6 +386,44 @@ String StyleProperties::fontValue() const
     return result.toString();
 }
 
+String StyleProperties::get2Values(const StylePropertyShorthand& shorthand) const
+{
+    // Assume the properties are in the usual order start, end.
+    int startValueIndex = findPropertyIndex(shorthand.properties()[0]);
+    int endValueIndex = findPropertyIndex(shorthand.properties()[1]);
+
+    if (startValueIndex == -1 || endValueIndex == -1)
+        return { };
+
+    auto start = propertyAt(startValueIndex);
+    auto end = propertyAt(endValueIndex);
+
+    // All 2 properties must be specified.
+    if (!start.value() || !end.value())
+        return { };
+
+    // Important flags must be the same
+    if (start.isImportant() != end.isImportant())
+        return { };
+
+    if (start.isInherited() && end.isInherited())
+        return getValueName(CSSValueInherit);
+
+    if (start.value()->isInitialValue() || end.value()->isInitialValue()) {
+        if (start.value()->isInitialValue() && end.value()->isInitialValue() && !start.isImplicit())
+            return getValueName(CSSValueInitial);
+        return { };
+    }
+
+    StringBuilder result;
+    result.append(start.value()->cssText());
+    if (!start.value()->equals(*end.value())) {
+        result.append(' ');
+        result.append(end.value()->cssText());
+    }
+    return result.toString();
+}
+
 String StyleProperties::get4Values(const StylePropertyShorthand& shorthand) const
 {
     // Assume the properties are in the usual order top, right, bottom, left.
@@ -366,6 +444,10 @@ String StyleProperties::get4Values(const StylePropertyShorthand& shorthand) cons
     if (!top.value() || !right.value() || !bottom.value() || !left.value())
         return String();
 
+    // Important flags must be the same
+    if (top.isImportant() != right.isImportant() || right.isImportant() != bottom.isImportant() || bottom.isImportant() != left.isImportant())
+        return String();
+
     if (top.isInherited() && right.isInherited() && bottom.isInherited() && left.isInherited())
         return getValueName(CSSValueInherit);
 
@@ -376,8 +458,6 @@ String StyleProperties::get4Values(const StylePropertyShorthand& shorthand) cons
         }
         return String();
     }
-    if (top.isImportant() != right.isImportant() || right.isImportant() != bottom.isImportant() || bottom.isImportant() != left.isImportant())
-        return String();
 
     bool showLeft = !right.value()->equals(*left.value());
     bool showBottom = !top.value()->equals(*bottom.value()) || showLeft;
@@ -609,22 +689,18 @@ String StyleProperties::getAlignmentShorthandValue(const StylePropertyShorthand&
     return value;
 }
 
-String StyleProperties::borderPropertyValue(CommonValueMode valueMode) const
+String StyleProperties::borderPropertyValue(const StylePropertyShorthand& width, const StylePropertyShorthand& style, const StylePropertyShorthand& color) const
 {
-    const StylePropertyShorthand properties[3] = { borderWidthShorthand(), borderStyleShorthand(), borderColorShorthand() };
+    const StylePropertyShorthand properties[3] = { width, style, color };
     String commonValue;
     StringBuilder result;
     for (size_t i = 0; i < WTF_ARRAY_LENGTH(properties); ++i) {
         String value = getCommonValue(properties[i]);
-        if (value.isNull()) {
-            if (valueMode == ReturnNullOnUncommonValues)
-                return String();
-            ASSERT(valueMode == OmitUncommonValues);
-            continue;
-        }
+        if (value.isNull())
+            return String();
         if (!i)
             commonValue = value;
-        else if (!commonValue.isNull() && commonValue != value)
+        else if (commonValue != value)
             commonValue = String();
         if (value == "initial")
             continue;
@@ -634,7 +710,7 @@ String StyleProperties::borderPropertyValue(CommonValueMode valueMode) const
     }
     if (isInitialOrInherit(commonValue))
         return commonValue;
-    return result.isEmpty() ? String() : result.toString();
+    return result.toString();
 }
 
 RefPtr<CSSValue> StyleProperties::getPropertyCSSValue(CSSPropertyID propertyID) const
@@ -755,6 +831,9 @@ bool StyleProperties::isPropertyImplicit(CSSPropertyID propertyID) const
 
 bool MutableStyleProperties::setProperty(CSSPropertyID propertyID, const String& value, bool important, CSSParserContext parserContext)
 {
+    if (!isEnabledCSSProperty(propertyID))
+        return false;
+
     // Setting the value to an empty string just removes the property in both IE and Gecko.
     // Setting it to null seems to produce less consistent results, but we treat it just the same.
     if (value.isEmpty())
@@ -773,7 +852,7 @@ bool MutableStyleProperties::setProperty(CSSPropertyID propertyID, const String&
     return setProperty(propertyID, value, important, parserContext);
 }
 
-bool MutableStyleProperties::setCustomProperty(const String& propertyName, const String& value, bool important, CSSParserContext parserContext)
+bool MutableStyleProperties::setCustomProperty(const Document* document, const String& propertyName, const String& value, bool important, CSSParserContext parserContext)
 {
     // Setting the value to an empty string just removes the property in both IE and Gecko.
     // Setting it to null seems to produce less consistent results, but we treat it just the same.
@@ -781,6 +860,17 @@ bool MutableStyleProperties::setCustomProperty(const String& propertyName, const
         return removeCustomProperty(propertyName);
 
     parserContext.mode = cssParserMode();
+
+    String syntax = "*";
+    auto* registered = document ? document->getCSSRegisteredCustomPropertySet().get(propertyName) : nullptr;
+
+    if (registered)
+        syntax = registered->syntax;
+
+    CSSTokenizer tokenizer(value);
+    if (!CSSPropertyParser::canParseTypedCustomPropertyValue(syntax, tokenizer.tokenRange(), parserContext))
+        return false;
+
     // When replacing an existing property value, this moves the property to the end of the list.
     // Firefox preserves the position, and MSIE moves the property to the beginning.
     return CSSParser::parseCustomPropertyValue(*this, propertyName, value, important, parserContext) == CSSParser::ParseResult::Changed;
@@ -890,7 +980,23 @@ String StyleProperties::asText() const
         CSSPropertyID propertyID = property.id();
         CSSPropertyID shorthandPropertyID = CSSPropertyInvalid;
         CSSPropertyID borderFallbackShorthandProperty = CSSPropertyInvalid;
+        CSSPropertyID borderBlockFallbackShorthandProperty = CSSPropertyInvalid;
+        CSSPropertyID borderInlineFallbackShorthandProperty = CSSPropertyInvalid;
         String value;
+        auto serializeBorderShorthand = [&] (const CSSPropertyID borderProperty, const CSSPropertyID fallbackProperty) {
+            // FIXME: Deal with cases where only some of border sides are specified.
+            ASSERT(borderProperty - firstCSSProperty < static_cast<CSSPropertyID>(shorthandPropertyAppeared.size()));
+            if (!shorthandPropertyAppeared[borderProperty - firstCSSProperty] && isEnabledCSSProperty(borderProperty)) {
+                value = getPropertyValue(borderProperty);
+                if (value.isNull())
+                    shorthandPropertyAppeared.set(borderProperty - firstCSSProperty);
+                else
+                    shorthandPropertyID = borderProperty;
+            } else if (shorthandPropertyUsed[borderProperty - firstCSSProperty])
+                shorthandPropertyID = borderProperty;
+            if (!shorthandPropertyID)
+                shorthandPropertyID = fallbackProperty;
+        };
 
         if (property.value() && property.value()->isPendingSubstitutionValue()) {
             auto& substitutionValue = downcast<CSSPendingSubstitutionValue>(*property.value());
@@ -940,19 +1046,39 @@ String StyleProperties::asText() const
             case CSSPropertyBorderLeftColor:
                 if (!borderFallbackShorthandProperty)
                     borderFallbackShorthandProperty = CSSPropertyBorderColor;
-
-                // FIXME: Deal with cases where only some of border-(top|right|bottom|left) are specified.
-                ASSERT(CSSPropertyBorder - firstCSSProperty < shorthandPropertyAppeared.size());
-                if (!shorthandPropertyAppeared[CSSPropertyBorder - firstCSSProperty]) {
-                    value = borderPropertyValue(ReturnNullOnUncommonValues);
-                    if (value.isNull())
-                        shorthandPropertyAppeared.set(CSSPropertyBorder - firstCSSProperty);
-                    else
-                        shorthandPropertyID = CSSPropertyBorder;
-                } else if (shorthandPropertyUsed[CSSPropertyBorder - firstCSSProperty])
-                    shorthandPropertyID = CSSPropertyBorder;
-                if (!shorthandPropertyID)
-                    shorthandPropertyID = borderFallbackShorthandProperty;
+                serializeBorderShorthand(CSSPropertyBorder, borderFallbackShorthandProperty);
+                break;
+            case CSSPropertyBorderBlockStartWidth:
+            case CSSPropertyBorderBlockEndWidth:
+                if (!borderBlockFallbackShorthandProperty)
+                    borderBlockFallbackShorthandProperty = CSSPropertyBorderBlockWidth;
+                FALLTHROUGH;
+            case CSSPropertyBorderBlockStartStyle:
+            case CSSPropertyBorderBlockEndStyle:
+                if (!borderBlockFallbackShorthandProperty)
+                    borderBlockFallbackShorthandProperty = CSSPropertyBorderBlockStyle;
+                FALLTHROUGH;
+            case CSSPropertyBorderBlockStartColor:
+            case CSSPropertyBorderBlockEndColor:
+                if (!borderBlockFallbackShorthandProperty)
+                    borderBlockFallbackShorthandProperty = CSSPropertyBorderBlockColor;
+                serializeBorderShorthand(CSSPropertyBorderBlock, borderBlockFallbackShorthandProperty);
+                break;
+            case CSSPropertyBorderInlineStartWidth:
+            case CSSPropertyBorderInlineEndWidth:
+                if (!borderInlineFallbackShorthandProperty)
+                    borderInlineFallbackShorthandProperty = CSSPropertyBorderInlineWidth;
+                FALLTHROUGH;
+            case CSSPropertyBorderInlineStartStyle:
+            case CSSPropertyBorderInlineEndStyle:
+                if (!borderInlineFallbackShorthandProperty)
+                    borderInlineFallbackShorthandProperty = CSSPropertyBorderInlineStyle;
+                FALLTHROUGH;
+            case CSSPropertyBorderInlineStartColor:
+            case CSSPropertyBorderInlineEndColor:
+                if (!borderInlineFallbackShorthandProperty)
+                    borderInlineFallbackShorthandProperty = CSSPropertyBorderInlineColor;
+                serializeBorderShorthand(CSSPropertyBorderInline, borderInlineFallbackShorthandProperty);
                 break;
             case CSSPropertyWebkitBorderHorizontalSpacing:
             case CSSPropertyWebkitBorderVerticalSpacing:
@@ -966,6 +1092,20 @@ String StyleProperties::asText() const
             case CSSPropertyFontWeight:
                 // Don't use CSSPropertyFont because old UAs can't recognize them but are important for editing.
                 break;
+            case CSSPropertyTop:
+            case CSSPropertyRight:
+            case CSSPropertyBottom:
+            case CSSPropertyLeft:
+                shorthandPropertyID = CSSPropertyInset;
+                break;
+            case CSSPropertyInsetBlockStart:
+            case CSSPropertyInsetBlockEnd:
+                shorthandPropertyID = CSSPropertyInsetBlock;
+                break;
+            case CSSPropertyInsetInlineStart:
+            case CSSPropertyInsetInlineEnd:
+                shorthandPropertyID = CSSPropertyInsetInline;
+                break;
             case CSSPropertyListStyleType:
             case CSSPropertyListStylePosition:
             case CSSPropertyListStyleImage:
@@ -977,6 +1117,14 @@ String StyleProperties::asText() const
             case CSSPropertyMarginLeft:
                 shorthandPropertyID = CSSPropertyMargin;
                 break;
+            case CSSPropertyMarginBlockStart:
+            case CSSPropertyMarginBlockEnd:
+                shorthandPropertyID = CSSPropertyMarginBlock;
+                break;
+            case CSSPropertyMarginInlineStart:
+            case CSSPropertyMarginInlineEnd:
+                shorthandPropertyID = CSSPropertyMarginInline;
+                break;
             case CSSPropertyOutlineWidth:
             case CSSPropertyOutlineStyle:
             case CSSPropertyOutlineColor:
@@ -992,6 +1140,14 @@ String StyleProperties::asText() const
             case CSSPropertyPaddingLeft:
                 shorthandPropertyID = CSSPropertyPadding;
                 break;
+            case CSSPropertyPaddingBlockStart:
+            case CSSPropertyPaddingBlockEnd:
+                shorthandPropertyID = CSSPropertyPaddingBlock;
+                break;
+            case CSSPropertyPaddingInlineStart:
+            case CSSPropertyPaddingInlineEnd:
+                shorthandPropertyID = CSSPropertyPaddingInline;
+                break;
 #if ENABLE(CSS_SCROLL_SNAP)
             case CSSPropertyScrollPaddingTop:
             case CSSPropertyScrollPaddingRight:
@@ -1047,7 +1203,7 @@ String StyleProperties::asText() const
         }
 
         unsigned shortPropertyIndex = shorthandPropertyID - firstCSSProperty;
-        if (shorthandPropertyID) {
+        if (shorthandPropertyID && isEnabledCSSProperty(shorthandPropertyID)) {
             ASSERT(shortPropertyIndex < shorthandPropertyUsed.size());
             if (shorthandPropertyUsed[shortPropertyIndex])
                 continue;
@@ -1227,8 +1383,7 @@ bool MutableStyleProperties::removePropertiesInSet(const CSSPropertyID* set, uns
         toRemove.add(set[i]);
 
     return m_propertyVector.removeAllMatching([&toRemove] (const CSSProperty& property) {
-        // Not quite sure if the isImportant test is needed but it matches the existing behavior.
-        return !property.isImportant() && toRemove.contains(property.id());
+        return toRemove.contains(property.id());
     }) > 0;
 }
 
