@@ -37,17 +37,18 @@
 #include "Document.h"
 #include "DocumentLoader.h"
 #include "FocusController.h"
+#include "Frame.h"
 #include "FrameLoader.h"
 #include "FrameLoaderClient.h"
 #include "FrameView.h"
 #include "HistoryController.h"
 #include "IgnoreOpensDuringUnloadCountIncrementer.h"
 #include "Logging.h"
-#include "MainFrame.h"
 #include "Page.h"
 #include "ScriptDisallowedScope.h"
 #include "Settings.h"
 #include "SubframeLoader.h"
+#include <pal/Logging.h>
 #include <wtf/MemoryPressureHandler.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/SetForScope.h>
@@ -84,6 +85,11 @@ static bool canCacheFrame(Frame& frame, DiagnosticLoggingClient& diagnosticLoggi
         return false;
     }
 
+    if (frame.isMainFrame() && frameLoader.stateMachine().isDisplayingInitialEmptyDocument()) {
+        PCLOG("   -MainFrame is displaying initial empty document");
+        return false;
+    }
+
     DocumentLoader* documentLoader = frameLoader.documentLoader();
     if (!documentLoader) {
         PCLOG("   -There is no DocumentLoader object");
@@ -202,7 +208,7 @@ static bool canCachePage(Page& page)
         logPageCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::isDisabledKey());
         isCacheable = false;
     }
-#if ENABLE(DEVICE_ORIENTATION) && !PLATFORM(IOS)
+#if ENABLE(DEVICE_ORIENTATION) && !PLATFORM(IOS_FAMILY)
     if (DeviceMotionController::isActiveAt(&page)) {
         PCLOG("   -Page is using DeviceMotion");
         logPageCacheFailureDiagnosticMessage(diagnosticLoggingClient, DiagnosticLoggingKeys::deviceMotionKey());
@@ -278,6 +284,25 @@ PageCache& PageCache::singleton()
     return globalPageCache;
 }
 
+PageCache::PageCache()
+{
+    static std::once_flag onceFlag;
+    std::call_once(onceFlag, [] {
+        PAL::registerNotifyCallback("com.apple.WebKit.showPageCache", [] {
+            PageCache::singleton().dump();
+        });
+    });
+}
+
+void PageCache::dump() const
+{
+    WTFLogAlways("\nPage Cache:");
+    for (auto& item : m_items) {
+        CachedPage& cachedPage = *item->m_cachedPage;
+        WTFLogAlways("  Page %p, document %p %s", &cachedPage.page(), cachedPage.document(), cachedPage.document() ? cachedPage.document()->url().string().utf8().data() : "");
+    }
+}
+
 bool PageCache::canCache(Page& page) const
 {
     if (!m_maxSize) {
@@ -371,7 +396,7 @@ static void setPageCacheState(Page& page, Document::PageCacheState pageCacheStat
 // When entering page cache, tear down the render tree before setting the in-cache flag.
 // This maintains the invariant that render trees are never present in the page cache.
 // Note that destruction happens bottom-up so that the main frame's tree dies last.
-static void destroyRenderTree(MainFrame& mainFrame)
+static void destroyRenderTree(Frame& mainFrame)
 {
     for (Frame* frame = mainFrame.tree().traversePrevious(CanWrap::Yes); frame; frame = frame->tree().traversePrevious(CanWrap::No)) {
         if (!frame->document())
@@ -400,13 +425,13 @@ static void firePageHideEventRecursively(Frame& frame)
         firePageHideEventRecursively(*child);
 }
 
-void PageCache::addIfCacheable(HistoryItem& item, Page* page)
+bool PageCache::addIfCacheable(HistoryItem& item, Page* page)
 {
     if (item.isInPageCache())
-        return;
+        return false;
 
     if (!page || !canCache(*page))
-        return;
+        return false;
 
     ASSERT_WITH_MESSAGE(!page->isUtilityPage(), "Utility pages such as SVGImage pages should never go into PageCache");
 
@@ -424,7 +449,7 @@ void PageCache::addIfCacheable(HistoryItem& item, Page* page)
     // could have altered the page in a way that could prevent caching.
     if (!canCache(*page)) {
         setPageCacheState(*page, Document::NotInPageCache);
-        return;
+        return false;
     }
 
     destroyRenderTree(page->mainFrame());
@@ -440,6 +465,7 @@ void PageCache::addIfCacheable(HistoryItem& item, Page* page)
         m_items.add(&item);
     }
     prune(PruningReason::ReachedMaxSize);
+    return true;
 }
 
 std::unique_ptr<CachedPage> PageCache::take(HistoryItem& item, Page* page)
