@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006, 2007, 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2018 Apple Inc. All rights reserved.
  * Copyright (C) 2010 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -58,6 +58,8 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(SliderThumbElement);
+WTF_MAKE_ISO_ALLOCATED_IMPL(SliderContainerElement);
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderSliderThumb);
 
 inline static Decimal sliderPosition(HTMLInputElement& element)
@@ -139,7 +141,7 @@ RenderBox::LogicalExtentComputedValues RenderSliderContainer::computeLogicalHeig
 #if ENABLE(DATALIST_ELEMENT)
     if (input.renderer()->isSlider() && !isVertical && input.list()) {
         int offsetFromCenter = theme().sliderTickOffsetFromTrackCenter();
-        LayoutUnit trackHeight = 0;
+        LayoutUnit trackHeight;
         if (offsetFromCenter < 0)
             trackHeight = -2 * offsetFromCenter;
         else {
@@ -163,13 +165,13 @@ void RenderSliderContainer::layout()
     ASSERT(element()->shadowHost());
     auto& input = downcast<HTMLInputElement>(*element()->shadowHost());
     bool isVertical = hasVerticalAppearance(input);
-    mutableStyle().setFlexDirection(isVertical ? FlowColumn : FlowRow);
+    mutableStyle().setFlexDirection(isVertical ? FlexDirection::Column : FlexDirection::Row);
     TextDirection oldTextDirection = style().direction();
     if (isVertical) {
         // FIXME: Work around rounding issues in RTL vertical sliders. We want them to
         // render identically to LTR vertical sliders. We can remove this work around when
         // subpixel rendering is enabled on all ports.
-        mutableStyle().setDirection(LTR);
+        mutableStyle().setDirection(TextDirection::LTR);
     }
 
     RenderBox* thumb = input.sliderThumbElement() ? input.sliderThumbElement()->renderBox() : nullptr;
@@ -205,11 +207,6 @@ void RenderSliderContainer::layout()
 
 SliderThumbElement::SliderThumbElement(Document& document)
     : HTMLDivElement(HTMLNames::divTag, document)
-    , m_inDragMode(false)
-#if ENABLE(IOS_TOUCH_EVENTS)
-    , m_exclusiveTouchIdentifier(NoIdentifier)
-    , m_isRegisteredAsTouchEventListener(false)
-#endif
 {
     setHasCustomStyleResolveCallbacks();
 }
@@ -249,7 +246,7 @@ void SliderThumbElement::dragFrom(const LayoutPoint& point)
 {
     Ref<SliderThumbElement> protectedThis(*this);
     setPositionFromPoint(point);
-#if !PLATFORM(IOS)
+#if !PLATFORM(IOS_FAMILY)
     startDragging();
 #endif
 }
@@ -301,7 +298,7 @@ void SliderThumbElement::setPositionFromPoint(const LayoutPoint& absolutePoint)
 #if ENABLE(DATALIST_ELEMENT)
     const LayoutUnit snappingThreshold = renderer()->theme().sliderTickSnappingThreshold();
     if (snappingThreshold > 0) {
-        if (std::optional<Decimal> closest = input->findClosestTickMarkValue(value)) {
+        if (Optional<Decimal> closest = input->findClosestTickMarkValue(value)) {
             double closestFraction = stepRange.proportionFromValue(*closest).toDouble();
             double closestRatio = isVertical || !isLeftToRightDirection ? 1.0 - closestFraction : closestFraction;
             LayoutUnit closestPosition = trackLength * closestRatio;
@@ -341,7 +338,7 @@ void SliderThumbElement::stopDragging()
         renderer()->setNeedsLayout();
 }
 
-#if !PLATFORM(IOS)
+#if !PLATFORM(IOS_FAMILY)
 void SliderThumbElement::defaultEventHandler(Event& event)
 {
     if (!is<MouseEvent>(event)) {
@@ -381,7 +378,8 @@ void SliderThumbElement::defaultEventHandler(Event& event)
 }
 #endif
 
-#if !PLATFORM(IOS)
+#if !PLATFORM(IOS_FAMILY)
+
 bool SliderThumbElement::willRespondToMouseMoveEvents()
 {
     const auto input = hostInput();
@@ -399,7 +397,8 @@ bool SliderThumbElement::willRespondToMouseClickEvents()
 
     return HTMLDivElement::willRespondToMouseClickEvents();
 }
-#endif // !PLATFORM(IOS)
+
+#endif // !PLATFORM(IOS_FAMILY)
 
 void SliderThumbElement::willDetachRenderers()
 {
@@ -413,6 +412,7 @@ void SliderThumbElement::willDetachRenderers()
 }
 
 #if ENABLE(IOS_TOUCH_EVENTS)
+
 unsigned SliderThumbElement::exclusiveTouchIdentifier() const
 {
     return m_exclusiveTouchIdentifier;
@@ -567,9 +567,10 @@ void SliderThumbElement::unregisterForTouchEvents()
     document().removeTouchEventHandler(*this);
     m_isRegisteredAsTouchEventListener = false;
 }
+
 #endif // ENABLE(IOS_TOUCH_EVENTS)
 
-void SliderThumbElement::disabledAttributeChanged()
+void SliderThumbElement::hostDisabledStateChanged()
 {
     if (isDisabledFormControl())
         stopDragging();
@@ -589,7 +590,7 @@ RefPtr<HTMLInputElement> SliderThumbElement::hostInput() const
     return downcast<HTMLInputElement>(shadowHost());
 }
 
-std::optional<ElementStyle> SliderThumbElement::resolveCustomStyle(const RenderStyle&, const RenderStyle* hostStyle)
+Optional<ElementStyle> SliderThumbElement::resolveCustomStyle(const RenderStyle&, const RenderStyle* hostStyle)
 {
     // This doesn't actually compute style. This is just a hack to pick shadow pseudo id when host style is known.
 
@@ -597,7 +598,7 @@ std::optional<ElementStyle> SliderThumbElement::resolveCustomStyle(const RenderS
     static NeverDestroyed<const AtomicString> mediaSliderThumbShadowPseudoId("-webkit-media-slider-thumb", AtomicString::ConstructFromLiteral);
 
     if (!hostStyle)
-        return std::nullopt;
+        return WTF::nullopt;
 
     switch (hostStyle->appearance()) {
     case MediaSliderPart:
@@ -612,7 +613,7 @@ std::optional<ElementStyle> SliderThumbElement::resolveCustomStyle(const RenderS
         m_shadowPseudoId = sliderThumbShadowPseudoId;
     }
 
-    return std::nullopt;
+    return WTF::nullopt;
 }
 
 const AtomicString& SliderThumbElement::shadowPseudoId() const
@@ -643,7 +644,7 @@ RenderPtr<RenderElement> SliderContainerElement::createElementRenderer(RenderSty
     return createRenderer<RenderSliderContainer>(*this, WTFMove(style));
 }
 
-std::optional<ElementStyle> SliderContainerElement::resolveCustomStyle(const RenderStyle&, const RenderStyle* hostStyle)
+Optional<ElementStyle> SliderContainerElement::resolveCustomStyle(const RenderStyle&, const RenderStyle* hostStyle)
 {
     // This doesn't actually compute style. This is just a hack to pick shadow pseudo id when host style is known.
 
@@ -651,7 +652,7 @@ std::optional<ElementStyle> SliderContainerElement::resolveCustomStyle(const Ren
     static NeverDestroyed<const AtomicString> sliderContainer("-webkit-slider-container", AtomicString::ConstructFromLiteral);
 
     if (!hostStyle)
-        return std::nullopt;
+        return WTF::nullopt;
 
     switch (hostStyle->appearance()) {
     case MediaSliderPart:
@@ -666,7 +667,7 @@ std::optional<ElementStyle> SliderContainerElement::resolveCustomStyle(const Ren
         m_shadowPseudoId = sliderContainer;
     }
 
-    return std::nullopt;
+    return WTF::nullopt;
 }
 
 const AtomicString& SliderContainerElement::shadowPseudoId() const
