@@ -34,12 +34,12 @@
 #include "EditorClient.h"
 #include "Element.h"
 #include "EmptyClients.h"
+#include "Frame.h"
 #include "FrameView.h"
 #include "GraphicsContext.h"
 #include "InspectorClient.h"
 #include "InspectorOverlayPage.h"
 #include "LibWebRTCProvider.h"
-#include "MainFrame.h"
 #include "Node.h"
 #include "Page.h"
 #include "PageConfiguration.h"
@@ -59,6 +59,9 @@
 #include <JavaScriptCore/InspectorProtocolObjects.h>
 #include <wtf/JSONValues.h>
 
+#if PLATFORM(MAC)
+#include "LocalDefaultSystemAppearance.h"
+#endif
 
 namespace WebCore {
 
@@ -177,8 +180,17 @@ void InspectorOverlay::paint(GraphicsContext& context)
     if (!shouldShowOverlay())
         return;
 
+    Page* overlayPage = this->overlayPage();
+    if (!overlayPage)
+        return;
+
     GraphicsContextStateSaver stateSaver(context);
-    FrameView* view = overlayPage()->mainFrame().view();
+    FrameView* view = overlayPage->mainFrame().view();
+
+#if PLATFORM(MAC)
+    LocalDefaultSystemAppearance localAppearance(view->useDarkAppearance());
+#endif
+
     view->updateLayoutAndStyleIfNeededRecursive();
     view->paint(context, IntRect(0, 0, view->width(), view->height()));
 }
@@ -259,16 +271,16 @@ void InspectorOverlay::setIndicating(bool indicating)
     m_indicating = indicating;
 
     if (m_indicating)
-        evaluateInOverlay(ASCIILiteral("showPageIndication"));
+        evaluateInOverlay("showPageIndication"_s);
     else
-        evaluateInOverlay(ASCIILiteral("hidePageIndication"));
+        evaluateInOverlay("hidePageIndication"_s);
 
     update();
 }
 
 bool InspectorOverlay::shouldShowOverlay() const
 {
-    return m_highlightNode || m_highlightNodeList || m_highlightQuad || m_indicating || m_showingPaintRects || !m_pausedInDebuggerMessage.isNull();
+    return m_highlightNode || m_highlightNodeList || m_highlightQuad || m_indicating || m_showingPaintRects || m_showRulers || !m_pausedInDebuggerMessage.isNull();
 }
 
 void InspectorOverlay::update()
@@ -282,26 +294,34 @@ void InspectorOverlay::update()
     if (!view)
         return;
 
-    FrameView* overlayView = overlayPage()->mainFrame().view();
-    IntSize viewportSize = view->sizeForVisibleContent();
-    IntSize frameViewFullSize = view->sizeForVisibleContent(ScrollableArea::IncludeScrollbars);
-    overlayView->resize(frameViewFullSize);
+    Page* overlayPage = this->overlayPage();
+    if (overlayPage) {
+        FrameView* overlayView = overlayPage->mainFrame().view();
+        IntSize frameViewFullSize = view->sizeForVisibleContent(ScrollableArea::IncludeScrollbars);
+        overlayView->resize(frameViewFullSize);
+    }
 
     // Clear canvas and paint things.
-    // FIXME: Remove extra parameter?
-    reset(viewportSize, IntSize());
+    IntSize viewportSize = view->sizeForVisibleContent();
+    IntPoint scrollOffset = view->scrollPosition();
+    reset(viewportSize, scrollOffset);
 
     // Include scrollbars to avoid masking them by the gutter.
-    drawGutter();
     drawNodeHighlight();
     drawQuadHighlight();
     drawPausedInDebuggerMessage();
     drawPaintRects();
 
+    if (m_showRulers)
+        drawRulers();
+
     // Position DOM elements.
-    overlayPage()->mainFrame().document()->resolveStyle(Document::ResolveStyleType::Rebuild);
-    if (overlayView->needsLayout())
-        overlayView->layoutContext().layout();
+    if (overlayPage) {
+        overlayPage->mainFrame().document()->resolveStyle(Document::ResolveStyleType::Rebuild);
+        FrameView* overlayView = overlayPage->mainFrame().view();
+        if (overlayView->needsLayout())
+            overlayView->layoutContext().layout();
+    }
 
     forcePaint();
 }
@@ -393,6 +413,16 @@ void InspectorOverlay::showPaintRect(const FloatRect& rect)
     forcePaint();
 }
 
+void InspectorOverlay::setShowRulers(bool showRulers)
+{
+    if (m_showRulers == showRulers)
+        return;
+
+    m_showRulers = showRulers;
+
+    update();
+}
+
 void InspectorOverlay::updatePaintRectsTimerFired()
 {
     MonotonicTime now = MonotonicTime::now();
@@ -417,12 +447,12 @@ void InspectorOverlay::drawPaintRects()
     for (const auto& pair : m_paintRects)
         arrayOfRects->addItem(buildObjectForRect(pair.second));
 
-    evaluateInOverlay(ASCIILiteral("updatePaintRects"), WTFMove(arrayOfRects));
+    evaluateInOverlay("updatePaintRects"_s, WTFMove(arrayOfRects));
 }
 
-void InspectorOverlay::drawGutter()
+void InspectorOverlay::drawRulers()
 {
-    evaluateInOverlay(ASCIILiteral("drawGutter"));
+    evaluateInOverlay("drawRulers"_s);
 }
 
 static RefPtr<JSON::ArrayOf<Inspector::Protocol::OverlayTypes::FragmentHighlightData>> buildArrayForRendererFragments(RenderObject* renderer, const HighlightConfig& config)
@@ -470,23 +500,23 @@ static void appendPathSegment(PathApplyInfo& pathApplyInfo, const PathElement& p
     switch (pathElement.type) {
     // The points member will contain 1 value.
     case PathElementMoveToPoint:
-        appendPathCommandAndPoints(pathApplyInfo, ASCIILiteral("M"), pathElement.points, 1);
+        appendPathCommandAndPoints(pathApplyInfo, "M"_s, pathElement.points, 1);
         break;
     // The points member will contain 1 value.
     case PathElementAddLineToPoint:
-        appendPathCommandAndPoints(pathApplyInfo, ASCIILiteral("L"), pathElement.points, 1);
+        appendPathCommandAndPoints(pathApplyInfo, "L"_s, pathElement.points, 1);
         break;
     // The points member will contain 3 values.
     case PathElementAddCurveToPoint:
-        appendPathCommandAndPoints(pathApplyInfo, ASCIILiteral("C"), pathElement.points, 3);
+        appendPathCommandAndPoints(pathApplyInfo, "C"_s, pathElement.points, 3);
         break;
     // The points member will contain 2 values.
     case PathElementAddQuadCurveToPoint:
-        appendPathCommandAndPoints(pathApplyInfo, ASCIILiteral("Q"), pathElement.points, 2);
+        appendPathCommandAndPoints(pathApplyInfo, "Q"_s, pathElement.points, 2);
         break;
     // The points member will contain no values.
     case PathElementCloseSubpath:
-        appendPathCommandAndPoints(pathApplyInfo, ASCIILiteral("Z"), nullptr, 0);
+        appendPathCommandAndPoints(pathApplyInfo, "Z"_s, nullptr, 0);
         break;
     }
 }
@@ -574,9 +604,9 @@ static RefPtr<Inspector::Protocol::OverlayTypes::ElementData> buildObjectForElem
     }
 
     if (node->isPseudoElement()) {
-        if (node->pseudoId() == BEFORE)
+        if (node->pseudoId() == PseudoId::Before)
             elementData->setPseudoElement("before");
-        else if (node->pseudoId() == AFTER)
+        else if (node->pseudoId() == PseudoId::After)
             elementData->setPseudoElement("after");
     }
 
@@ -682,16 +712,13 @@ void InspectorOverlay::drawPausedInDebuggerMessage()
 
 Page* InspectorOverlay::overlayPage()
 {
+#if PLATFORM(IOS_FAMILY)
+    return nullptr;
+#else
     if (m_overlayPage)
         return m_overlayPage.get();
 
-    PageConfiguration pageConfiguration(
-        createEmptyEditorClient(),
-        SocketProvider::create(),
-        LibWebRTCProvider::create(),
-        CacheStorageProvider::create()
-    );
-    fillWithEmptyClients(pageConfiguration);
+    auto pageConfiguration = pageConfigurationWithEmptyClients();
     m_overlayPage = std::make_unique<Page>(WTFMove(pageConfiguration));
     m_overlayPage->setDeviceScaleFactor(m_page.deviceScaleFactor());
 
@@ -717,10 +744,11 @@ Page* InspectorOverlay::overlayPage()
     frame.view()->setCanHaveScrollbars(false);
     frame.view()->setTransparent(true);
     ASSERT(loader.activeDocumentLoader());
-    loader.activeDocumentLoader()->writer().setMIMEType("text/html");
-    loader.activeDocumentLoader()->writer().begin();
-    loader.activeDocumentLoader()->writer().addData(reinterpret_cast<const char*>(InspectorOverlayPage_html), sizeof(InspectorOverlayPage_html));
-    loader.activeDocumentLoader()->writer().end();
+    auto& writer = loader.activeDocumentLoader()->writer();
+    writer.setMIMEType("text/html");
+    writer.begin();
+    writer.insertDataSynchronously(String(reinterpret_cast<const char*>(InspectorOverlayPage_html), sizeof(InspectorOverlayPage_html)));
+    writer.end();
 
 #if OS(WINDOWS)
     evaluateInOverlay("setPlatform", "windows");
@@ -731,6 +759,7 @@ Page* InspectorOverlay::overlayPage()
 #endif
 
     return m_overlayPage.get();
+#endif
 }
 
 void InspectorOverlay::forcePaint()
@@ -739,18 +768,25 @@ void InspectorOverlay::forcePaint()
     m_client->highlight();
 }
 
-void InspectorOverlay::reset(const IntSize& viewportSize, const IntSize& frameViewFullSize)
+void InspectorOverlay::reset(const IntSize& viewportSize, const IntPoint& scrollOffset)
 {
     auto configObject = Inspector::Protocol::OverlayTypes::OverlayConfiguration::create()
         .setDeviceScaleFactor(m_page.deviceScaleFactor())
         .setViewportSize(buildObjectForSize(viewportSize))
-        .setFrameViewFullSize(buildObjectForSize(frameViewFullSize))
+        .setPageScaleFactor(m_page.pageScaleFactor())
+        .setPageZoomFactor(m_page.mainFrame().pageZoomFactor())
+        .setScrollOffset(buildObjectForPoint(scrollOffset))
+        .setContentInset(buildObjectForSize(IntSize(0, m_page.mainFrame().view()->topContentInset(ScrollView::TopContentInsetType::WebCoreOrPlatformContentInset))))
+        .setShowRulers(m_showRulers)
         .release();
     evaluateInOverlay("reset", WTFMove(configObject));
 }
 
 static void evaluateCommandInOverlay(Page* page, Ref<JSON::Array>&& command)
 {
+    if (!page)
+        return;
+
     page->mainFrame().script().evaluate(ScriptSourceCode(makeString("dispatch(", command->toJSONString(), ')')));
 }
 
