@@ -30,14 +30,15 @@
 #include "SecurityOrigin.h"
 
 #include "BlobURL.h"
-#include "FileSystem.h"
-#include "URL.h"
 #include "SchemeRegistry.h"
 #include "SecurityPolicy.h"
+#include "TextEncoding.h"
 #include "ThreadableBlobRegistry.h"
+#include <wtf/FileSystem.h>
 #include <wtf/MainThread.h>
 #include <wtf/NeverDestroyed.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/URL.h>
 #include <wtf/text/StringBuilder.h>
 
 namespace WebCore {
@@ -99,7 +100,7 @@ static bool shouldTreatAsUniqueOrigin(const URL& url)
     return false;
 }
 
-static bool isLoopbackIPAddress(const String& host)
+static bool isLoopbackIPAddress(StringView host)
 {
     // The IPv6 loopback address is 0:0:0:0:0:0:0:1, which compresses to ::1.
     if (host == "[::1]")
@@ -137,19 +138,18 @@ static bool shouldTreatAsPotentiallyTrustworthy(const String& protocol, const St
 
 bool shouldTreatAsPotentiallyTrustworthy(const URL& url)
 {
-    return shouldTreatAsPotentiallyTrustworthy(url.protocol().toStringWithoutCopying(), url.host());
+    return shouldTreatAsPotentiallyTrustworthy(url.protocol().toStringWithoutCopying(), url.host().toStringWithoutCopying());
 }
 
 SecurityOrigin::SecurityOrigin(const URL& url)
-    : m_protocol { url.protocol().isNull() ? emptyString() : url.protocol().toString().convertToASCIILowercase() }
-    , m_host { url.host().isNull() ? emptyString() : url.host().convertToASCIILowercase() }
-    , m_port { url.port() }
+    : m_data(SecurityOriginData::fromURL(url))
+    , m_isLocal(SchemeRegistry::shouldTreatURLSchemeAsLocal(m_data.protocol))
 {
-    // document.domain starts as m_host, but can be set by the DOM.
-    m_domain = m_host;
+    // document.domain starts as m_data.host, but can be set by the DOM.
+    m_domain = m_data.host;
 
-    if (m_port && isDefaultPortForProtocol(m_port.value(), m_protocol))
-        m_port = std::nullopt;
+    if (m_data.port && WTF::isDefaultPortForProtocol(m_data.port.value(), m_data.protocol))
+        m_data.port = WTF::nullopt;
 
     // By default, only local SecurityOrigins can load local resources.
     m_canLoadLocalResources = isLocal();
@@ -157,25 +157,21 @@ SecurityOrigin::SecurityOrigin(const URL& url)
     if (m_canLoadLocalResources)
         m_filePath = url.fileSystemPath(); // In case enforceFilePathSeparation() is called.
 
-    if (!url.isValid())
-        m_isPotentiallyTrustworthy = IsPotentiallyTrustworthy::No;
+    m_isPotentiallyTrustworthy = shouldTreatAsPotentiallyTrustworthy(url);
 }
 
 SecurityOrigin::SecurityOrigin()
-    : m_protocol { emptyString() }
-    , m_host { emptyString() }
+    : m_data { emptyString(), emptyString(), WTF::nullopt }
     , m_domain { emptyString() }
     , m_isUnique { true }
-    , m_isPotentiallyTrustworthy { IsPotentiallyTrustworthy::Yes }
+    , m_isPotentiallyTrustworthy { true }
 {
 }
 
 SecurityOrigin::SecurityOrigin(const SecurityOrigin* other)
-    : m_protocol { other->m_protocol.isolatedCopy() }
-    , m_host { other->m_host.isolatedCopy() }
+    : m_data { other->m_data.isolatedCopy() }
     , m_domain { other->m_domain.isolatedCopy() }
     , m_filePath { other->m_filePath.isolatedCopy() }
-    , m_port { other->m_port }
     , m_isUnique { other->m_isUnique }
     , m_universalAccess { other->m_universalAccess }
     , m_domainWasSetInDOM { other->m_domainWasSetInDOM }
@@ -184,6 +180,7 @@ SecurityOrigin::SecurityOrigin(const SecurityOrigin* other)
     , m_enforcesFilePathSeparation { other->m_enforcesFilePathSeparation }
     , m_needsStorageAccessFromFileURLsQuirk { other->m_needsStorageAccessFromFileURLsQuirk }
     , m_isPotentiallyTrustworthy { other->m_isPotentiallyTrustworthy }
+    , m_isLocal { other->m_isLocal }
 {
 }
 
@@ -219,15 +216,6 @@ void SecurityOrigin::setDomainFromDOM(const String& newDomain)
     m_domain = newDomain.convertToASCIILowercase();
 }
 
-bool SecurityOrigin::isPotentiallyTrustworthy() const
-{
-    // This code is using an enum instead of an std::optional for thread-safety. Worst case scenario, several thread will read
-    // 'Unknown' value concurrently and they'll all call shouldTreatAsPotentiallyTrustworthy() and get the same result.
-    if (m_isPotentiallyTrustworthy == IsPotentiallyTrustworthy::Unknown)
-        m_isPotentiallyTrustworthy = shouldTreatAsPotentiallyTrustworthy(m_protocol, m_host) ? IsPotentiallyTrustworthy::Yes : IsPotentiallyTrustworthy::No;
-    return m_isPotentiallyTrustworthy == IsPotentiallyTrustworthy::Yes;
-}
-
 bool SecurityOrigin::isSecure(const URL& url)
 {
     // Invalid URLs are secure, as are URLs which have a secure protocol.
@@ -273,9 +261,9 @@ bool SecurityOrigin::canAccess(const SecurityOrigin& other) const
     // this is a security vulnerability.
 
     bool canAccess = false;
-    if (m_protocol == other.m_protocol) {
+    if (m_data.protocol == other.m_data.protocol) {
         if (!m_domainWasSetInDOM && !other.m_domainWasSetInDOM) {
-            if (m_host == other.m_host && m_port == other.m_port)
+            if (m_data.host == other.m_data.host && m_data.port == other.m_data.port)
                 canAccess = true;
         } else if (m_domainWasSetInDOM && other.m_domainWasSetInDOM) {
             if (m_domain == other.m_domain)
@@ -358,8 +346,8 @@ bool SecurityOrigin::canDisplay(const URL& url) const
     if (m_universalAccess)
         return true;
 
-#if !PLATFORM(IOS)
-    if (m_protocol == "file" && url.isLocalFile() && !FileSystem::filesHaveSameVolume(m_filePath, url.fileSystemPath()))
+#if !PLATFORM(IOS_FAMILY)
+    if (m_data.protocol == "file" && url.isLocalFile() && !FileSystem::filesHaveSameVolume(m_filePath, url.fileSystemPath()))
         return false;
 #endif
 
@@ -372,7 +360,7 @@ bool SecurityOrigin::canDisplay(const URL& url) const
         return canRequest(url);
 
     if (SchemeRegistry::shouldTreatURLSchemeAsDisplayIsolated(protocol))
-        return equalIgnoringASCIICase(m_protocol, protocol) || SecurityPolicy::isAccessToURLWhiteListed(this, url);
+        return equalIgnoringASCIICase(m_data.protocol, protocol) || SecurityPolicy::isAccessToURLWhiteListed(this, url);
 
     if (SecurityPolicy::restrictAccessToLocal() && SchemeRegistry::shouldTreatURLSchemeAsLocal(protocol))
         return canLoadLocalResources() || SecurityPolicy::isAccessToURLWhiteListed(this, url);
@@ -457,7 +445,7 @@ String SecurityOrigin::domainForCachePartition() const
     if (isHTTPFamily())
         return host();
 
-    if (SchemeRegistry::shouldPartitionCacheForURLScheme(m_protocol))
+    if (SchemeRegistry::shouldPartitionCacheForURLScheme(m_data.protocol))
         return host();
 
     return emptyString();
@@ -469,37 +457,18 @@ void SecurityOrigin::setEnforcesFilePathSeparation()
     m_enforcesFilePathSeparation = true;
 }
 
-bool SecurityOrigin::isLocal() const
-{
-    return SchemeRegistry::shouldTreatURLSchemeAsLocal(m_protocol);
-}
-
 String SecurityOrigin::toString() const
 {
     if (isUnique())
-        return ASCIILiteral("null");
-    if (m_protocol == "file" && m_enforcesFilePathSeparation)
-        return ASCIILiteral("null");
+        return "null"_s;
+    if (m_data.protocol == "file" && m_enforcesFilePathSeparation)
+        return "null"_s;
     return toRawString();
 }
 
 String SecurityOrigin::toRawString() const
 {
-    if (m_protocol == "file")
-        return ASCIILiteral("file://");
-
-    StringBuilder result;
-    result.reserveCapacity(m_protocol.length() + m_host.length() + 10);
-    result.append(m_protocol);
-    result.appendLiteral("://");
-    result.append(m_host);
-
-    if (m_port) {
-        result.append(':');
-        result.appendNumber(m_port.value());
-    }
-
-    return result.toString();
+    return m_data.toString();
 }
 
 static inline bool areOriginsMatching(const SecurityOrigin& origin1, const SecurityOrigin& origin2)
@@ -543,12 +512,12 @@ Ref<SecurityOrigin> SecurityOrigin::createFromString(const String& originString)
     return SecurityOrigin::create(URL(URL(), originString));
 }
 
-Ref<SecurityOrigin> SecurityOrigin::create(const String& protocol, const String& host, std::optional<uint16_t> port)
+Ref<SecurityOrigin> SecurityOrigin::create(const String& protocol, const String& host, Optional<uint16_t> port)
 {
     String decodedHost = decodeURLEscapeSequences(host);
     auto origin = create(URL(URL(), protocol + "://" + host + "/"));
-    if (port && !isDefaultPortForProtocol(*port, protocol))
-        origin->m_port = port;
+    if (port && !WTF::isDefaultPortForProtocol(*port, protocol))
+        origin->m_data.port = port;
     return origin;
 }
 
@@ -571,13 +540,7 @@ bool SecurityOrigin::equal(const SecurityOrigin* other) const
 
 bool SecurityOrigin::isSameSchemeHostPort(const SecurityOrigin& other) const
 {
-    if (m_host != other.m_host)
-        return false;
-
-    if (m_protocol != other.m_protocol)
-        return false;
-
-    if (m_port != other.m_port)
+    if (m_data != other.m_data)
         return false;
 
     if (isLocal() && !passesFileCheck(other))
@@ -586,7 +549,7 @@ bool SecurityOrigin::isSameSchemeHostPort(const SecurityOrigin& other) const
     return true;
 }
 
-bool SecurityOrigin::isLocalHostOrLoopbackIPAddress(const String& host)
+bool SecurityOrigin::isLocalHostOrLoopbackIPAddress(StringView host)
 {
     if (isLoopbackIPAddress(host))
         return true;
