@@ -41,7 +41,6 @@
 #include "RenderStyle.h"
 #include "RenderView.h"
 #include <algorithm>
-#include <wtf/CurrentTime.h>
 #include <wtf/Ref.h>
 
 namespace WebCore {
@@ -91,7 +90,7 @@ void AnimationBase::setNeedsStyleRecalc(Element* element)
         return;
 
     ASSERT(element->document().pageCacheState() == Document::NotInPageCache);
-    element->invalidateStyleAndLayerComposition();
+    element->invalidateStyle();
 }
 
 double AnimationBase::duration() const
@@ -101,7 +100,7 @@ double AnimationBase::duration() const
 
 bool AnimationBase::playStatePlaying() const
 {
-    return m_animation->playState() == AnimPlayStatePlaying;
+    return m_animation->playState() == AnimationPlayState::Playing;
 }
 
 bool AnimationBase::animationsMatch(const Animation& animation) const
@@ -159,26 +158,26 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
     // If we get AnimationStateInput::RestartAnimation then we force a new animation, regardless of state.
     if (input == AnimationStateInput::MakeNew) {
         if (m_animationState == AnimationState::StartWaitStyleAvailable)
-            m_compositeAnimation->animationController().removeFromAnimationsWaitingForStyle(this);
+            m_compositeAnimation->animationController().removeFromAnimationsWaitingForStyle(*this);
         LOG(Animations, "%p AnimationState %s -> New", this, nameForState(m_animationState));
         m_animationState = AnimationState::New;
-        m_startTime = std::nullopt;
-        m_pauseTime = std::nullopt;
+        m_startTime = WTF::nullopt;
+        m_pauseTime = WTF::nullopt;
         m_requestedStartTime = 0;
-        m_nextIterationDuration = std::nullopt;
+        m_nextIterationDuration = WTF::nullopt;
         endAnimation();
         return;
     }
 
     if (input == AnimationStateInput::RestartAnimation) {
         if (m_animationState == AnimationState::StartWaitStyleAvailable)
-            m_compositeAnimation->animationController().removeFromAnimationsWaitingForStyle(this);
+            m_compositeAnimation->animationController().removeFromAnimationsWaitingForStyle(*this);
         LOG(Animations, "%p AnimationState %s -> New", this, nameForState(m_animationState));
         m_animationState = AnimationState::New;
-        m_startTime = std::nullopt;
-        m_pauseTime = std::nullopt;
+        m_startTime = WTF::nullopt;
+        m_pauseTime = WTF::nullopt;
         m_requestedStartTime = 0;
-        m_nextIterationDuration = std::nullopt;
+        m_nextIterationDuration = WTF::nullopt;
         endAnimation();
 
         if (!paused())
@@ -188,7 +187,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
 
     if (input == AnimationStateInput::EndAnimation) {
         if (m_animationState == AnimationState::StartWaitStyleAvailable)
-            m_compositeAnimation->animationController().removeFromAnimationsWaitingForStyle(this);
+            m_compositeAnimation->animationController().removeFromAnimationsWaitingForStyle(*this);
         LOG(Animations, "%p AnimationState %s -> Done", this, nameForState(m_animationState));
         m_animationState = AnimationState::Done;
         endAnimation();
@@ -208,7 +207,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
     if (input == AnimationStateInput::ResumeOverride) {
         if (m_animationState == AnimationState::Looping || m_animationState == AnimationState::Ending) {
             // Start the animation
-            startAnimation(beginAnimationUpdateTime() - m_startTime.value_or(0));
+            startAnimation(beginAnimationUpdateTime() - m_startTime.valueOr(0));
         }
         return;
     }
@@ -226,13 +225,9 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
                 // We are pausing before we even started.
                 LOG(Animations, "%p AnimationState %s -> AnimationState::PausedNew", this, nameForState(m_animationState));
                 m_animationState = AnimationState::PausedNew;
-                m_pauseTime = std::nullopt;
+                m_pauseTime = WTF::nullopt;
             }
 
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-            if (m_animation->trigger() && m_animation->trigger()->isScrollAnimationTrigger())
-                m_compositeAnimation->animationController().addToAnimationsDependentOnScroll(this);
-#endif
             break;
         case AnimationState::StartWaitTimer:
             ASSERT(input == AnimationStateInput::StartTimerFired || input == AnimationStateInput::PlayStatePaused);
@@ -242,7 +237,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
                 // Start timer has fired, tell the animation to start and wait for it to respond with start time
                 LOG(Animations, "%p AnimationState %s -> StartWaitStyleAvailable (time is %f)", this, nameForState(m_animationState), param);
                 m_animationState = AnimationState::StartWaitStyleAvailable;
-                m_compositeAnimation->animationController().addToAnimationsWaitingForStyle(this);
+                m_compositeAnimation->animationController().addToAnimationsWaitingForStyle(*this);
 
                 // Trigger a render so we can start the animation
                 if (m_element)
@@ -280,7 +275,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
                         timeOffset = -m_animation->delay();
                     bool started = startAnimation(timeOffset);
 
-                    m_compositeAnimation->animationController().addToAnimationsWaitingForStartTimeResponse(this, started);
+                    m_compositeAnimation->animationController().addToAnimationsWaitingForStartTimeResponse(*this, started);
                     m_isAccelerated = started;
                 }
             } else {
@@ -318,7 +313,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
                 // We are pausing while waiting for a start response. Cancel the animation and wait. When
                 // we unpause, we will act as though the start timer just fired
                 m_pauseTime = beginAnimationUpdateTime();
-                pauseAnimation(beginAnimationUpdateTime() - m_startTime.value_or(0));
+                pauseAnimation(beginAnimationUpdateTime() - m_startTime.valueOr(0));
                 LOG(Animations, "%p AnimationState %s -> PausedWaitResponse", this, nameForState(m_animationState));
                 m_animationState = AnimationState::PausedWaitResponse;
             }
@@ -338,7 +333,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
             } else {
                 // We are pausing while running. Cancel the animation and wait
                 m_pauseTime = beginAnimationUpdateTime();
-                pauseAnimation(beginAnimationUpdateTime() - m_startTime.value_or(0));
+                pauseAnimation(beginAnimationUpdateTime() - m_startTime.valueOr(0));
                 LOG(Animations, "%p AnimationState %s -> PausedRun", this, nameForState(m_animationState));
                 m_animationState = AnimationState::PausedRun;
             }
@@ -370,7 +365,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
             } else {
                 // We are pausing while running. Cancel the animation and wait
                 m_pauseTime = beginAnimationUpdateTime();
-                pauseAnimation(beginAnimationUpdateTime() - m_startTime.value_or(0));
+                pauseAnimation(beginAnimationUpdateTime() - m_startTime.valueOr(0));
                 LOG(Animations, "%p AnimationState %s -> PausedRun", this, nameForState(m_animationState));
                 m_animationState = AnimationState::PausedRun;
             }
@@ -380,8 +375,8 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
             ASSERT(input == AnimationStateInput::PlayStateRunning);
             ASSERT(paused());
             // Update the times
-            m_startTime = m_startTime.value() + beginAnimationUpdateTime() - m_pauseTime.value_or(0);
-            m_pauseTime = std::nullopt;
+            m_startTime = m_startTime.valueOr(0) + beginAnimationUpdateTime() - m_pauseTime.valueOr(0);
+            m_pauseTime = WTF::nullopt;
 
             // we were waiting for the start timer to fire, go back and wait again
             LOG(Animations, "%p AnimationState %s -> New", this, nameForState(m_animationState));
@@ -396,7 +391,7 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
             // AnimationState::PausedWaitResponse, we don't yet have a valid startTime, so we send 0 to startAnimation.
             // When the AnimationStateInput::StartTimeSet comes in and we were in AnimationState::PausedRun, we will notice
             // that we have already set the startTime and will ignore it.
-            ASSERT(input == AnimationStateInput::PlayStatePaused || input == AnimationStateInput::PlayStateRunning || input == AnimationStateInput::StartTimeSet || input == AnimationStateInput::StyleAvailable || input == AnimationStateInput::StartAnimation);
+            ASSERT(input == AnimationStateInput::PlayStatePaused || input == AnimationStateInput::PlayStateRunning || input == AnimationStateInput::StartTimeSet || input == AnimationStateInput::StyleAvailable);
             ASSERT(paused());
 
             if (input == AnimationStateInput::PlayStateRunning) {
@@ -405,18 +400,18 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
                     // to start, so jump back to the New state and reset.
                     LOG(Animations, "%p AnimationState %s -> AnimationState::New", this, nameForState(m_animationState));
                     m_animationState = AnimationState::New;
-                    m_pauseTime = std::nullopt;
+                    m_pauseTime = WTF::nullopt;
                     updateStateMachine(input, param);
                     break;
                 }
 
                 // Update the times
                 if (m_animationState == AnimationState::PausedRun)
-                    m_startTime = m_startTime.value() + beginAnimationUpdateTime() - m_pauseTime.value_or(0);
+                    m_startTime = m_startTime.valueOr(0) + beginAnimationUpdateTime() - m_pauseTime.valueOr(0);
                 else
                     m_startTime = 0;
 
-                m_pauseTime = std::nullopt;
+                m_pauseTime = WTF::nullopt;
 
                 if (m_animationState == AnimationState::PausedWaitStyleAvailable) {
                     LOG(Animations, "%p AnimationState %s -> StartWaitStyleAvailable", this, nameForState(m_animationState));
@@ -435,8 +430,8 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
                         updateStateMachine(AnimationStateInput::StartTimeSet, beginAnimationUpdateTime());
                         m_isAccelerated = true;
                     } else {
-                        bool started = startAnimation(beginAnimationUpdateTime() - m_startTime.value_or(0));
-                        m_compositeAnimation->animationController().addToAnimationsWaitingForStartTimeResponse(this, started);
+                        bool started = startAnimation(beginAnimationUpdateTime() - m_startTime.valueOr(0));
+                        m_compositeAnimation->animationController().addToAnimationsWaitingForStartTimeResponse(*this, started);
                         m_isAccelerated = started;
                     }
                 }
@@ -452,11 +447,17 @@ void AnimationBase::updateStateMachine(AnimationStateInput input, double param)
                 m_animationState = AnimationState::PausedRun;
                 ASSERT(!m_startTime);
                 m_startTime = param;
-                m_pauseTime = m_pauseTime.value_or(0) + param;
+                m_pauseTime = m_pauseTime.valueOr(0) + param;
                 break;
             }
 
             ASSERT(m_animationState == AnimationState::PausedNew || m_animationState == AnimationState::PausedWaitStyleAvailable);
+
+            if (input == AnimationStateInput::PlayStatePaused)
+                break;
+
+            ASSERT(input == AnimationStateInput::StyleAvailable);
+
             // We are paused but we got the callback that notifies us that style has been updated.
             // We move to the AnimationState::PausedWaitResponse state
             LOG(Animations, "%p AnimationState %s -> PausedWaitResponse", this, nameForState(m_animationState));
@@ -488,30 +489,12 @@ void AnimationBase::fireAnimationEventsIfNeeded()
 
     // Check for start timeout
     if (m_animationState == AnimationState::StartWaitTimer) {
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-        if (m_animation->trigger() && m_animation->trigger()->isScrollAnimationTrigger()) {
-            if (m_element) {
-                float offset = m_compositeAnimation->animationController().scrollPosition();
-                auto& scrollTrigger = downcast<ScrollAnimationTrigger>(*m_animation->trigger());
-                if (offset > scrollTrigger.startValue().value())
-                    updateStateMachine(AnimationStateInput::StartTimerFired, 0);
-            }
-
-            return;
-        }
-#endif
         if (beginAnimationUpdateTime() - m_requestedStartTime >= m_animation->delay())
             updateStateMachine(AnimationStateInput::StartTimerFired, 0);
         return;
     }
 
-    double elapsedDuration = beginAnimationUpdateTime() - m_startTime.value_or(0);
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-    // If we are a triggered animation that depends on scroll, our elapsed
-    // time is determined by the scroll position.
-    if (m_animation->trigger() && m_animation->trigger()->isScrollAnimationTrigger())
-        elapsedDuration = getElapsedTime();
-#endif
+    double elapsedDuration = beginAnimationUpdateTime() - m_startTime.valueOr(0);
 
     // FIXME: we need to ensure that elapsedDuration is never < 0. If it is, this suggests that
     // we had a recalcStyle() outside of beginAnimationUpdate()/endAnimationUpdate().
@@ -547,7 +530,7 @@ void AnimationBase::fireAnimationEventsIfNeeded()
     }
 }
 
-void AnimationBase::updatePlayState(EAnimPlayState playState)
+void AnimationBase::updatePlayState(AnimationPlayState playState)
 {
     if (!m_compositeAnimation)
         return;
@@ -555,7 +538,7 @@ void AnimationBase::updatePlayState(EAnimPlayState playState)
     // When we get here, we can have one of 4 desired states: running, paused, suspended, paused & suspended.
     // The state machine can be in one of two states: running, paused.
     // Set the state machine to the desired state.
-    bool pause = playState == AnimPlayStatePaused || m_compositeAnimation->isSuspended();
+    bool pause = playState == AnimationPlayState::Paused || m_compositeAnimation->isSuspended();
 
     if (pause == paused() && !isNew())
         return;
@@ -563,25 +546,14 @@ void AnimationBase::updatePlayState(EAnimPlayState playState)
     updateStateMachine(pause ?  AnimationStateInput::PlayStatePaused : AnimationStateInput::PlayStateRunning, -1);
 }
 
-std::optional<Seconds> AnimationBase::timeToNextService()
+Optional<Seconds> AnimationBase::timeToNextService()
 {
-    // Returns the time at which next service is required. std::nullopt means no service is required. 0 means
+    // Returns the time at which next service is required. WTF::nullopt means no service is required. 0 means
     // service is required now, and > 0 means service is required that many seconds in the future.
     if (paused() || isNew() || postActive() || fillingForwards())
-        return std::nullopt;
+        return WTF::nullopt;
 
     if (m_animationState == AnimationState::StartWaitTimer) {
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-        if (m_animation->trigger()->isScrollAnimationTrigger()) {
-            if (m_element) {
-                float currentScrollPosition = m_element->document().view()->scrollPositionForFixedPosition().y().toFloat();
-                auto& scrollTrigger = downcast<ScrollAnimationTrigger>(*m_animation->trigger());
-                if (currentScrollPosition >= scrollTrigger.startValue().value() && (!scrollTrigger.hasEndValue() || currentScrollPosition <= scrollTrigger.endValue().value()))
-                    return 0_s;
-            }
-            return std::nullopt;
-        }
-#endif
         double timeFromNow = m_animation->delay() - (beginAnimationUpdateTime() - m_requestedStartTime);
         return std::max(Seconds { timeFromNow }, 0_s);
     }
@@ -659,9 +631,9 @@ double AnimationBase::progress(double scale, double offset, const TimingFunction
 void AnimationBase::getTimeToNextEvent(Seconds& time, bool& isLooping) const
 {
     // Decide when the end or loop event needs to fire
-    const double elapsedDuration = std::max(beginAnimationUpdateTime() - m_startTime.value_or(0), 0.0);
+    const double elapsedDuration = std::max(beginAnimationUpdateTime() - m_startTime.valueOr(0), 0.0);
     double durationLeft = 0;
-    double nextIterationTime = m_totalDuration.value_or(0);
+    double nextIterationTime = m_totalDuration.valueOr(0);
 
     if (!m_totalDuration || elapsedDuration < m_totalDuration.value()) {
         durationLeft = m_animation->duration() > 0 ? (m_animation->duration() - fmod(elapsedDuration, m_animation->duration())) : 0;
@@ -698,17 +670,17 @@ void AnimationBase::freezeAtTime(double t)
         // If we haven't started yet, make it as if we started.
         LOG(Animations, "%p AnimationState %s -> StartWaitResponse", this, nameForState(m_animationState));
         m_animationState = AnimationState::StartWaitResponse;
-        onAnimationStartResponse(monotonicallyIncreasingTime());
+        onAnimationStartResponse(MonotonicTime::now());
     }
 
     ASSERT(m_startTime); // If m_startTime is zero, we haven't started yet, so we'll get a bad pause time.
     if (t <= m_animation->delay())
-        m_pauseTime = m_startTime.value_or(0);
+        m_pauseTime = m_startTime.valueOr(0);
     else
-        m_pauseTime = m_startTime.value_or(0) + t - m_animation->delay();
+        m_pauseTime = m_startTime.valueOr(0) + t - m_animation->delay();
 
     if (auto* renderer = compositedRenderer())
-        renderer->suspendAnimations(m_pauseTime.value());
+        renderer->suspendAnimations(MonotonicTime::fromRawSeconds(m_pauseTime.value()));
 }
 
 double AnimationBase::beginAnimationUpdateTime() const
@@ -716,39 +688,23 @@ double AnimationBase::beginAnimationUpdateTime() const
     if (!m_compositeAnimation)
         return 0;
 
-    return m_compositeAnimation->animationController().beginAnimationUpdateTime();
+    return m_compositeAnimation->animationController().beginAnimationUpdateTime().secondsSinceEpoch().seconds();
 }
 
 double AnimationBase::getElapsedTime() const
 {
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-    if (m_animation->trigger() && m_animation->trigger()->isScrollAnimationTrigger()) {
-        auto& scrollTrigger = downcast<ScrollAnimationTrigger>(*m_animation->trigger());
-        if (scrollTrigger.hasEndValue() && m_element) {
-            float offset = m_compositeAnimation->animationController().scrollPosition();
-            float startValue = scrollTrigger.startValue().value();
-            if (offset < startValue)
-                return 0;
-            float endValue = scrollTrigger.endValue().value();
-            if (offset > endValue)
-                return m_animation->duration();
-            return m_animation->duration() * (offset - startValue) / (endValue - startValue);
-        }
-    }
-#endif
-
     if (paused()) {
         double delayOffset = (!m_startTime && m_animation->delay() < 0) ? m_animation->delay() : 0;
-        return m_pauseTime.value_or(0) - m_startTime.value_or(0) - delayOffset;
+        return m_pauseTime.valueOr(0) - m_startTime.valueOr(0) - delayOffset;
     }
 
     if (!m_startTime)
         return 0;
 
     if (postActive() || fillingForwards())
-        return m_totalDuration.value_or(0);
+        return m_totalDuration.valueOr(0);
 
-    return beginAnimationUpdateTime() - m_startTime.value_or(0);
+    return beginAnimationUpdateTime() - m_startTime.valueOr(0);
 }
 
 void AnimationBase::setElapsedTime(double time)
