@@ -64,14 +64,14 @@ void CompositeAnimation::clearElement()
         // Clear the renderers from all running animations, in case we are in the middle of
         // an animation callback (see https://bugs.webkit.org/show_bug.cgi?id=22052)
         for (auto& transition : m_transitions.values()) {
-            animationController().animationWillBeRemoved(transition.get());
+            animationController().animationWillBeRemoved(*transition);
             transition->clear();
         }
     }
     if (!m_keyframeAnimations.isEmpty()) {
         m_keyframeAnimations.checkConsistency();
         for (auto& animation : m_keyframeAnimations.values()) {
-            animationController().animationWillBeRemoved(animation.get());
+            animationController().animationWillBeRemoved(*animation);
             animation->clear();
         }
     }
@@ -109,12 +109,18 @@ void CompositeAnimation::updateTransitions(Element& element, const RenderStyle*
             for (int propertyIndex = 0; propertyIndex < CSSPropertyAnimation::getNumProperties(); ++propertyIndex) {
                 if (all) {
                     // Get the next property which is not a shorthand.
-                    bool isShorthand;
+                    Optional<bool> isShorthand;
                     prop = CSSPropertyAnimation::getPropertyAtIndex(propertyIndex, isShorthand);
-                    if (isShorthand)
+                    if (isShorthand && *isShorthand)
                         continue;
                 }
 
+                if (prop == CSSPropertyInvalid) {
+                    if (!all)
+                        break;
+                    continue;
+                }
+
                 // ImplicitAnimations are always hashed by actual properties, never animateAll.
                 ASSERT(prop >= firstCSSProperty && prop < (firstCSSProperty + numCSSProperties));
 
@@ -153,13 +159,13 @@ void CompositeAnimation::updateTransitions(Element& element, const RenderStyle*
                             implAnim->blendPropertyValueInStyle(prop, modifiedCurrentStyle.get());
                         }
                         LOG(Animations, "Removing existing ImplicitAnimation %p for property %s", implAnim, getPropertyName(prop));
-                        animationController().animationWillBeRemoved(implAnim);
+                        animationController().animationWillBeRemoved(*implAnim);
                         m_transitions.remove(prop);
                         equal = false;
                     }
                 } else {
                     // We need to start a transition if it is active and the properties don't match
-                    equal = !isActiveTransition || CSSPropertyAnimation::propertiesEqual(prop, fromStyle, &targetStyle);
+                    equal = !isActiveTransition || CSSPropertyAnimation::propertiesEqual(prop, fromStyle, &targetStyle) || !CSSPropertyAnimation::canPropertyBeInterpolated(prop, fromStyle, &targetStyle);
                 }
 
                 // We can be in this loop with an inactive transition (!isActiveTransition). We need
@@ -170,7 +176,7 @@ void CompositeAnimation::updateTransitions(Element& element, const RenderStyle*
                     // Add the new transition
                     auto implicitAnimation = ImplicitAnimation::create(animation, prop, element, *this, modifiedCurrentStyle ? *modifiedCurrentStyle : *fromStyle);
                     if (m_suspended && implicitAnimation->hasStyle())
-                        implicitAnimation->updatePlayState(AnimPlayStatePaused);
+                        implicitAnimation->updatePlayState(AnimationPlayState::Paused);
 
                     LOG(Animations, "Created ImplicitAnimation %p on element %p for property %s duration %.2f delay %.2f", implicitAnimation.ptr(), &element, getPropertyName(prop), animation.duration(), animation.delay());
                     m_transitions.set(prop, WTFMove(implicitAnimation));
@@ -187,7 +193,7 @@ void CompositeAnimation::updateTransitions(Element& element, const RenderStyle*
     Vector<int> toBeRemoved;
     for (auto& transition : m_transitions.values()) {
         if (!transition->active()) {
-            animationController().animationWillBeRemoved(transition.get());
+            animationController().animationWillBeRemoved(*transition);
             toBeRemoved.append(transition->animatingProperty());
             LOG(Animations, "Removing ImplicitAnimation %p from element %p for property %s", transition.get(), &element, getPropertyName(transition->animatingProperty()));
         }
@@ -209,10 +215,6 @@ void CompositeAnimation::updateKeyframeAnimations(Element& element, const Render
     if (currentStyle && currentStyle->hasAnimations() && targetStyle.hasAnimations() && *(currentStyle->animations()) == *(targetStyle.animations()))
         return;
 
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-    m_hasScrollTriggeredAnimation = false;
-#endif
-
     AnimationNameMap newAnimations;
 
     // Toss the animation order map.
@@ -238,11 +240,6 @@ void CompositeAnimation::updateKeyframeAnimations(Element& element, const Render
                 if (keyframeAnim->postActive())
                     continue;
 
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-                if (animation.trigger()->isScrollAnimationTrigger())
-                    m_hasScrollTriggeredAnimation = true;
-#endif
-
                 // Animations match, but play states may differ. Update if needed.
                 keyframeAnim->updatePlayState(animation.playState());
 
@@ -253,7 +250,7 @@ void CompositeAnimation::updateKeyframeAnimations(Element& element, const Render
                 LOG(Animations, "Creating KeyframeAnimation %p on element %p with keyframes %s, duration %.2f, delay %.2f, iterations %.2f", keyframeAnim.get(), &element, animation.name().utf8().data(), animation.duration(), animation.delay(), animation.iterationCount());
 
                 if (m_suspended) {
-                    keyframeAnim->updatePlayState(AnimPlayStatePaused);
+                    keyframeAnim->updatePlayState(AnimationPlayState::Paused);
                     LOG(Animations, "  (created in suspended/paused state)");
                 }
 #if !LOG_DISABLED
@@ -261,11 +258,6 @@ void CompositeAnimation::updateKeyframeAnimations(Element& element, const Render
                     LOG(Animations, "  property %s", getPropertyName(propertyID));
 #endif
 
-#if ENABLE(CSS_ANIMATIONS_LEVEL_2)
-                if (animation.trigger()->isScrollAnimationTrigger())
-                    m_hasScrollTriggeredAnimation = true;
-#endif
-
                 newAnimations.set(keyframeAnim->name().impl(), keyframeAnim);
             }
 
@@ -278,7 +270,7 @@ void CompositeAnimation::updateKeyframeAnimations(Element& element, const Render
     // Make a list of animations to be removed.
     for (auto& animation : m_keyframeAnimations.values()) {
         if (!newAnimations.contains(animation->name().impl())) {
-            animationController().animationWillBeRemoved(animation.get());
+            animationController().animationWillBeRemoved(*animation);
             animation->clear();
             LOG(Animations, "Removing KeyframeAnimation %p from element %p", animation.get(), &element);
         }
@@ -294,7 +286,7 @@ AnimationUpdate CompositeAnimation::animate(Element& element, const RenderStyle*
     updateKeyframeAnimations(element, currentStyle, targetStyle);
     m_keyframeAnimations.checkConsistency();
 
-    bool animationStateChanged = false;
+    bool animationChangeRequiresRecomposite = false;
     bool forceStackingContext = false;
 
     std::unique_ptr<RenderStyle> animatedStyle;
@@ -304,12 +296,11 @@ AnimationUpdate CompositeAnimation::animate(Element& element, const RenderStyle*
         // to fill in a RenderStyle*& only if needed.
         bool checkForStackingContext = false;
         for (auto& transition : m_transitions.values()) {
-            bool didBlendStyle = false;
-            if (transition->animate(*this, targetStyle, animatedStyle, didBlendStyle))
-                animationStateChanged = true;
-
-            if (didBlendStyle)
+            auto changes = transition->animate(*this, targetStyle, animatedStyle);
+            if (changes.contains(AnimateChange::StyleBlended))
                 checkForStackingContext |= WillChangeData::propertyCreatesStackingContext(transition->animatingProperty());
+
+            animationChangeRequiresRecomposite = changes.contains(AnimateChange::RunningStateChange) && transition->affectsAcceleratedProperty();
         }
 
         if (animatedStyle && checkForStackingContext) {
@@ -334,11 +325,9 @@ AnimationUpdate CompositeAnimation::animate(Element& element, const RenderStyle*
     for (auto& name : m_keyframeAnimationOrderMap) {
         RefPtr<KeyframeAnimation> keyframeAnim = m_keyframeAnimations.get(name);
         if (keyframeAnim) {
-            bool didBlendStyle = false;
-            if (keyframeAnim->animate(*this, targetStyle, animatedStyle, didBlendStyle))
-                animationStateChanged = true;
-
-            forceStackingContext |= didBlendStyle && keyframeAnim->triggersStackingContext();
+            auto changes = keyframeAnim->animate(*this, targetStyle, animatedStyle);
+            animationChangeRequiresRecomposite = changes.contains(AnimateChange::RunningStateChange) && keyframeAnim->affectsAcceleratedProperty();
+            forceStackingContext |= changes.contains(AnimateChange::StyleBlended) && keyframeAnim->triggersStackingContext();
             m_hasAnimationThatDependsOnLayout |= keyframeAnim->dependsOnLayout();
         }
     }
@@ -352,7 +341,7 @@ AnimationUpdate CompositeAnimation::animate(Element& element, const RenderStyle*
             animatedStyle->setZIndex(0);
     }
 
-    return { WTFMove(animatedStyle), animationStateChanged };
+    return { WTFMove(animatedStyle), animationChangeRequiresRecomposite };
 }
 
 std::unique_ptr<RenderStyle> CompositeAnimation::getAnimatedStyle() const
@@ -372,15 +361,15 @@ std::unique_ptr<RenderStyle> CompositeAnimation::getAnimatedStyle() const
     return resultStyle;
 }
 
-std::optional<Seconds> CompositeAnimation::timeToNextService() const
+Optional<Seconds> CompositeAnimation::timeToNextService() const
 {
-    // Returns the time at which next service is required. std::nullopt means no service is required. 0 means
+    // Returns the time at which next service is required. WTF::nullopt means no service is required. 0 means
     // service is required now, and > 0 means service is required that many seconds in the future.
-    std::optional<Seconds> minT;
+    Optional<Seconds> minT;
 
     if (!m_transitions.isEmpty()) {
         for (auto& transition : m_transitions.values()) {
-            std::optional<Seconds> t = transition->timeToNextService();
+            Optional<Seconds> t = transition->timeToNextService();
             if (!t)
                 continue;
             if (!minT || t.value() < minT.value())
@@ -392,7 +381,7 @@ std::optional<Seconds> CompositeAnimation::timeToNextService() const
     if (!m_keyframeAnimations.isEmpty()) {
         m_keyframeAnimations.checkConsistency();
         for (auto& animation : m_keyframeAnimations.values()) {
-            std::optional<Seconds> t = animation->timeToNextService();
+            Optional<Seconds> t = animation->timeToNextService();
             if (!t)
                 continue;
             if (!minT || t.value() < minT.value())
@@ -464,13 +453,13 @@ void CompositeAnimation::suspendAnimations()
     if (!m_keyframeAnimations.isEmpty()) {
         m_keyframeAnimations.checkConsistency();
         for (auto& animation : m_keyframeAnimations.values())
-            animation->updatePlayState(AnimPlayStatePaused);
+            animation->updatePlayState(AnimationPlayState::Paused);
     }
 
     if (!m_transitions.isEmpty()) {
         for (auto& transition : m_transitions.values()) {
             if (transition->hasStyle())
-                transition->updatePlayState(AnimPlayStatePaused);
+                transition->updatePlayState(AnimationPlayState::Paused);
         }
     }
 }
@@ -486,14 +475,14 @@ void CompositeAnimation::resumeAnimations()
         m_keyframeAnimations.checkConsistency();
         for (auto& animation : m_keyframeAnimations.values()) {
             if (animation->playStatePlaying())
-                animation->updatePlayState(AnimPlayStatePlaying);
+                animation->updatePlayState(AnimationPlayState::Playing);
         }
     }
 
     if (!m_transitions.isEmpty()) {
         for (auto& transition : m_transitions.values()) {
             if (transition->hasStyle())
-                transition->updatePlayState(AnimPlayStatePlaying);
+                transition->updatePlayState(AnimationPlayState::Playing);
         }
     }
 }
@@ -518,19 +507,19 @@ void CompositeAnimation::resumeOverriddenImplicitAnimations(CSSPropertyID proper
     }
 }
 
-bool CompositeAnimation::isAnimatingProperty(CSSPropertyID property, bool acceleratedOnly, AnimationBase::RunningState runningState) const
+bool CompositeAnimation::isAnimatingProperty(CSSPropertyID property, bool acceleratedOnly) const
 {
     if (!m_keyframeAnimations.isEmpty()) {
         m_keyframeAnimations.checkConsistency();
         for (auto& animation : m_keyframeAnimations.values()) {
-            if (animation->isAnimatingProperty(property, acceleratedOnly, runningState))
+            if (animation->isAnimatingProperty(property, acceleratedOnly))
                 return true;
         }
     }
 
     if (!m_transitions.isEmpty()) {
         for (auto& transition : m_transitions.values()) {
-            if (transition->isAnimatingProperty(property, acceleratedOnly, runningState))
+            if (transition->isAnimatingProperty(property, acceleratedOnly))
                 return true;
         }
     }
