@@ -35,7 +35,7 @@
 #include <wtf/HashSet.h>
 #include <wtf/WeakPtr.h>
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
 
 OBJC_CLASS WAKScrollView;
 OBJC_CLASS WAKView;
@@ -48,7 +48,7 @@ OBJC_CLASS WAKView;
 #define NSView WAKView
 #endif
 
-#endif // PLATFORM(IOS)
+#endif // PLATFORM(IOS_FAMILY)
 
 #if PLATFORM(COCOA) && defined __OBJC__
 @class NSScrollView;
@@ -74,9 +74,11 @@ public:
 
     virtual void notifyPageThatContentAreaWillPaint() const;
 
+    using Widget::weakPtrFactory;
+
     IntPoint locationOfContents() const;
 
-    // NOTE: This should only be called by the overriden setScrollOffset from ScrollableArea.
+    // NOTE: This should only be called by the overridden setScrollOffset from ScrollableArea.
     virtual void scrollTo(const ScrollPosition&);
 
     // The window thats hosts the ScrollView. The ScrollView will communicate scrolls and repaints to the
@@ -175,13 +177,13 @@ public:
     // contribute to painting but not to the scrollable area.
     // The unobscuredContentRect is the area that is not covered by UI elements.
     WEBCORE_EXPORT IntRect unobscuredContentRect(VisibleContentRectIncludesScrollbars = ExcludeScrollbars) const;
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
     IntRect unobscuredContentRectIncludingScrollbars() const { return unobscuredContentRect(IncludeScrollbars); }
 #else
     IntRect unobscuredContentRectIncludingScrollbars() const { return visibleContentRectIncludingScrollbars(); }
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
     // This is the area that is partially or fully exposed, and may extend under overlapping UI elements.
     WEBCORE_EXPORT FloatRect exposedContentRect() const;
 
@@ -233,7 +235,7 @@ public:
     IntPoint contentsScrollPosition() const;
     void setContentsScrollPosition(const IntPoint&);
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
     int actualScrollX() const { return unobscuredContentRect().x(); }
     int actualScrollY() const { return unobscuredContentRect().y(); }
     // FIXME: maybe fix scrollPosition() on iOS to return the actual scroll position.
@@ -281,6 +283,7 @@ public:
     WEBCORE_EXPORT IntPoint contentsToRootView(const IntPoint&) const;
     WEBCORE_EXPORT IntRect rootViewToContents(const IntRect&) const;
     WEBCORE_EXPORT IntRect contentsToRootView(const IntRect&) const;
+    WEBCORE_EXPORT FloatRect rootViewToContents(const FloatRect&) const;
 
     IntPoint viewToContents(const IntPoint&) const;
     IntPoint contentsToView(const IntPoint&) const;
@@ -288,6 +291,9 @@ public:
     IntRect viewToContents(IntRect) const;
     IntRect contentsToView(IntRect) const;
 
+    FloatRect viewToContents(FloatRect) const;
+    FloatRect contentsToView(FloatRect) const;
+
     IntPoint contentsToContainingViewContents(const IntPoint&) const;
     IntRect contentsToContainingViewContents(IntRect) const;
 
@@ -372,7 +378,7 @@ public:
     void setAllowsUnclampedScrollPositionForTesting(bool allowsUnclampedScrollPosition) { m_allowsUnclampedScrollPosition = allowsUnclampedScrollPosition; }
     bool allowsUnclampedScrollPosition() const { return m_allowsUnclampedScrollPosition; }
 
-    auto& weakPtrFactory() const { return m_weakPtrFactory; }
+    bool managesScrollbars() const;
 
 protected:
     ScrollView();
@@ -417,10 +423,18 @@ protected:
 
     virtual void scrollOffsetChangedViaPlatformWidgetImpl(const ScrollOffset&, const ScrollOffset&) = 0;
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
     virtual void unobscuredContentSizeChanged() = 0;
 #endif
 
+#if PLATFORM(COCOA) && defined __OBJC__
+public:
+    WEBCORE_EXPORT NSView* documentView() const;
+
+private:
+    NSScrollView<WebCoreFrameScrollView>* scrollView() const;
+#endif
+
 private:
     // Size available for view contents, excluding content insets. Not affected by zooming.
     IntSize sizeForUnobscuredContent(VisibleContentRectIncludesScrollbars = ExcludeScrollbars) const;
@@ -434,56 +448,6 @@ private:
 
     bool isScrollView() const final { return true; }
 
-    HashSet<Ref<Widget>> m_children;
-
-    RefPtr<Scrollbar> m_horizontalScrollbar;
-    RefPtr<Scrollbar> m_verticalScrollbar;
-    ScrollbarMode m_horizontalScrollbarMode { ScrollbarAuto };
-    ScrollbarMode m_verticalScrollbarMode { ScrollbarAuto };
-
-#if PLATFORM(IOS)
-    // FIXME: exposedContentRect is a very similar concept to fixedVisibleContentRect except it does not differentiate
-    // between exposed and unobscured areas. The two attributes should eventually be merged.
-    FloatRect m_exposedContentRect;
-    FloatSize m_unobscuredContentSize;
-    // This is only used for history scroll position restoration.
-#else
-    IntRect m_fixedVisibleContentRect;
-#endif
-    ScrollPosition m_scrollPosition;
-    IntPoint m_cachedScrollPosition;
-    IntSize m_fixedLayoutSize;
-    IntSize m_contentsSize;
-
-    std::optional<IntSize> m_deferredScrollDelta; // Needed for WebKit scrolling
-    std::optional<std::pair<ScrollOffset, ScrollOffset>> m_deferredScrollOffsets; // Needed for platform widget scrolling
-
-    IntPoint m_panScrollIconPoint;
-
-    WeakPtrFactory<ScrollView> m_weakPtrFactory;
-
-    bool m_horizontalScrollbarLock { false };
-    bool m_verticalScrollbarLock { false };
-
-    bool m_prohibitsScrolling { false };
-    bool m_allowsUnclampedScrollPosition { false };
-
-    // This bool is unused on Mac OS because we directly ask the platform widget
-    // whether it is safe to blit on scroll.
-    bool m_canBlitOnScroll { true };
-
-    bool m_scrollbarsSuppressed { false };
-
-    bool m_inUpdateScrollbars { false };
-    unsigned m_updateScrollbarsPass { 0 };
-
-    bool m_drawPanScrollIcon { false };
-    bool m_useFixedLayout { false };
-
-    bool m_paintsEntireContents { false };
-    bool m_delegatesScrolling { false };
-
-
     void init();
     void destroy();
 
@@ -512,19 +476,57 @@ private:
     void platformRepaintContentRectangle(const IntRect&);
     bool platformIsOffscreen() const;
     void platformSetScrollbarOverlayStyle(ScrollbarOverlayStyle);
-
     void platformSetScrollOrigin(const IntPoint&, bool updatePositionAtAll, bool updatePositionSynchronously);
 
     void calculateOverhangAreasForPainting(IntRect& horizontalOverhangRect, IntRect& verticalOverhangRect);
     void updateOverhangAreas();
 
-#if PLATFORM(COCOA) && defined __OBJC__
-public:
-    WEBCORE_EXPORT NSView* documentView() const;
+    HashSet<Ref<Widget>> m_children;
 
-private:
-    NSScrollView<WebCoreFrameScrollView>* scrollView() const;
+    RefPtr<Scrollbar> m_horizontalScrollbar;
+    RefPtr<Scrollbar> m_verticalScrollbar;
+    ScrollbarMode m_horizontalScrollbarMode { ScrollbarAuto };
+    ScrollbarMode m_verticalScrollbarMode { ScrollbarAuto };
+
+#if PLATFORM(IOS_FAMILY)
+    // FIXME: exposedContentRect is a very similar concept to fixedVisibleContentRect except it does not differentiate
+    // between exposed and unobscured areas. The two attributes should eventually be merged.
+    FloatRect m_exposedContentRect;
+    FloatSize m_unobscuredContentSize;
+    // This is only used for history scroll position restoration.
+#else
+    IntRect m_fixedVisibleContentRect;
 #endif
+    ScrollPosition m_scrollPosition;
+    IntPoint m_cachedScrollPosition;
+    IntSize m_fixedLayoutSize;
+    IntSize m_contentsSize;
+
+    Optional<IntSize> m_deferredScrollDelta; // Needed for WebKit scrolling
+    Optional<std::pair<ScrollOffset, ScrollOffset>> m_deferredScrollOffsets; // Needed for platform widget scrolling
+
+    IntPoint m_panScrollIconPoint;
+
+    unsigned m_updateScrollbarsPass { 0 };
+
+    bool m_horizontalScrollbarLock { false };
+    bool m_verticalScrollbarLock { false };
+
+    bool m_prohibitsScrolling { false };
+    bool m_allowsUnclampedScrollPosition { false };
+
+    // This bool is unused on Mac OS because we directly ask the platform widget
+    // whether it is safe to blit on scroll.
+    bool m_canBlitOnScroll { true };
+
+    bool m_scrollbarsSuppressed { false };
+    bool m_inUpdateScrollbars { false };
+
+    bool m_drawPanScrollIcon { false };
+    bool m_useFixedLayout { false };
+
+    bool m_paintsEntireContents { false };
+    bool m_delegatesScrolling { false };
 }; // class ScrollView
 
 } // namespace WebCore
