@@ -33,6 +33,7 @@
 #if PLATFORM(COCOA)
 #include <pal/spi/cocoa/CoreTextSPI.h>
 #endif
+#include "CharacterProperties.h"
 #include "FontCache.h"
 #include "FontCascade.h"
 #include "OpenTypeMathData.h"
@@ -56,10 +57,7 @@ const float smallCapsFontSizeMultiplier = 0.7f;
 const float emphasisMarkFontSizeMultiplier = 0.5f;
 
 Font::Font(const FontPlatformData& platformData, Origin origin, Interstitial interstitial, Visibility visibility, OrientationFallback orientationFallback)
-    : m_maxCharWidth(-1)
-    , m_avgCharWidth(-1)
-    , m_platformData(platformData)
-    , m_mathData(nullptr)
+    : m_platformData(platformData)
     , m_origin(origin)
     , m_visibility(visibility)
     , m_treatAsFixedPitch(false)
@@ -68,7 +66,7 @@ Font::Font(const FontPlatformData& platformData, Origin origin, Interstitial int
     , m_isBrokenIdeographFallback(false)
     , m_hasVerticalGlyphs(false)
     , m_isUsedInSystemFallbackCache(false)
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
     , m_shouldNotBeUsedForArabic(false)
 #endif
 {
@@ -76,7 +74,7 @@ Font::Font(const FontPlatformData& platformData, Origin origin, Interstitial int
     platformGlyphInit();
     platformCharWidthInit();
 #if ENABLE(OPENTYPE_VERTICAL)
-    if (platformData.orientation() == Vertical && orientationFallback == OrientationFallback::No) {
+    if (platformData.orientation() == FontOrientation::Vertical && orientationFallback == OrientationFallback::No) {
         m_verticalData = FontCache::singleton().verticalData(platformData);
         m_hasVerticalGlyphs = m_verticalData.get() && m_verticalData->hasVerticalMetrics();
     }
@@ -105,14 +103,20 @@ void Font::initCharWidths()
 
 void Font::platformGlyphInit()
 {
-    auto* glyphPageZero = glyphPage(0);
+#if USE(FREETYPE)
+    auto* glyphPageZeroWidthSpace = glyphPage(GlyphPage::pageNumberForCodePoint(zeroWidthSpace));
+    UChar32 zeroWidthSpaceCharacter = zeroWidthSpace;
+#else
+    // Ask for the glyph for 0 to avoid paging in ZERO WIDTH SPACE. Control characters, including 0,
+    // are mapped to the ZERO WIDTH SPACE glyph for non FreeType based ports.
+    auto* glyphPageZeroWidthSpace = glyphPage(0);
+    UChar32 zeroWidthSpaceCharacter = 0;
+#endif
     auto* glyphPageCharacterZero = glyphPage(GlyphPage::pageNumberForCodePoint('0'));
     auto* glyphPageSpace = glyphPage(GlyphPage::pageNumberForCodePoint(space));
 
-    // Ask for the glyph for 0 to avoid paging in ZERO WIDTH SPACE. Control characters, including 0,
-    // are mapped to the ZERO WIDTH SPACE glyph.
-    if (glyphPageZero)
-        m_zeroWidthSpaceGlyph = glyphPageZero->glyphDataForCharacter(0).glyph;
+    if (glyphPageZeroWidthSpace)
+        m_zeroWidthSpaceGlyph = glyphPageZeroWidthSpace->glyphDataForCharacter(zeroWidthSpaceCharacter).glyph;
 
     // Nasty hack to determine if we should round or ceil space widths.
     // If the font is monospace or fake monospace we ceil to ensure that
@@ -151,9 +155,168 @@ static bool fillGlyphPage(GlyphPage& pageToFill, UChar* buffer, unsigned bufferL
     return hasGlyphs;
 }
 
+static Optional<size_t> codePointSupportIndex(UChar32 codePoint)
+{
+    // FIXME: Consider reordering these so the most common ones are at the front.
+    // Doing this could cause the BitVector to fit inside inline storage and therefore
+    // be both a performance and a memory progression.
+    if (codePoint < 0x20)
+        return codePoint;
+    if (codePoint >= 0x7F && codePoint < 0xA0)
+        return codePoint - 0x7F + 0x20;
+    Optional<size_t> result;
+    switch (codePoint) {
+    case softHyphen:
+        result = 0x41;
+        break;
+    case newlineCharacter:
+        result = 0x42;
+        break;
+    case tabCharacter:
+        result = 0x43;
+        break;
+    case noBreakSpace:
+        result = 0x44;
+        break;
+    case narrowNoBreakSpace:
+        result = 0x45;
+        break;
+    case leftToRightMark:
+        result = 0x46;
+        break;
+    case rightToLeftMark:
+        result = 0x47;
+        break;
+    case leftToRightEmbed:
+        result = 0x48;
+        break;
+    case rightToLeftEmbed:
+        result = 0x49;
+        break;
+    case leftToRightOverride:
+        result = 0x4A;
+        break;
+    case rightToLeftOverride:
+        result = 0x4B;
+        break;
+    case leftToRightIsolate:
+        result = 0x4C;
+        break;
+    case rightToLeftIsolate:
+        result = 0x4D;
+        break;
+    case zeroWidthNonJoiner:
+        result = 0x4E;
+        break;
+    case zeroWidthJoiner:
+        result = 0x4F;
+        break;
+    case popDirectionalFormatting:
+        result = 0x50;
+        break;
+    case popDirectionalIsolate:
+        result = 0x51;
+        break;
+    case firstStrongIsolate:
+        result = 0x52;
+        break;
+    case objectReplacementCharacter:
+        result = 0x53;
+        break;
+    case zeroWidthNoBreakSpace:
+        result = 0x54;
+        break;
+    default:
+        result = WTF::nullopt;
+    }
+
+#ifndef NDEBUG
+    UChar32 codePointOrder[] = {
+        0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0A, 0x0B, 0x0C, 0x0D, 0x0E, 0x0F,
+        0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F,
+        0x7F,
+        0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 0x8F,
+        0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F,
+        softHyphen,
+        newlineCharacter,
+        tabCharacter,
+        noBreakSpace,
+        narrowNoBreakSpace,
+        leftToRightMark,
+        rightToLeftMark,
+        leftToRightEmbed,
+        rightToLeftEmbed,
+        leftToRightOverride,
+        rightToLeftOverride,
+        leftToRightIsolate,
+        rightToLeftIsolate,
+        zeroWidthNonJoiner,
+        zeroWidthJoiner,
+        popDirectionalFormatting,
+        popDirectionalIsolate,
+        firstStrongIsolate,
+        objectReplacementCharacter,
+        zeroWidthNoBreakSpace
+    };
+    bool found = false;
+    for (size_t i = 0; i < WTF_ARRAY_LENGTH(codePointOrder); ++i) {
+        if (codePointOrder[i] == codePoint) {
+            ASSERT(i == result);
+            found = true;
+        }
+    }
+    ASSERT(found == static_cast<bool>(result));
+#endif
+    return result;
+}
+
+#if !USE(FREETYPE)
+static void overrideControlCharacters(Vector<UChar>& buffer, unsigned start, unsigned end)
+{
+    auto overwriteCodePoints = [&](unsigned minimum, unsigned maximum, UChar newCodePoint) {
+        unsigned begin = std::max(start, minimum);
+        unsigned complete = std::min(end, maximum);
+        for (unsigned i = begin; i < complete; ++i) {
+            ASSERT(codePointSupportIndex(i));
+            buffer[i - start] = newCodePoint;
+        }
+    };
+
+    auto overwriteCodePoint = [&](UChar codePoint, UChar newCodePoint) {
+        ASSERT(codePointSupportIndex(codePoint));
+        if (codePoint >= start && codePoint < end)
+            buffer[codePoint - start] = newCodePoint;
+    };
+
+    // Code points 0x0 - 0x20 and 0x7F - 0xA0 are control character and shouldn't render. Map them to ZERO WIDTH SPACE.
+    overwriteCodePoints(0x0, 0x20, zeroWidthSpace);
+    overwriteCodePoints(0x7F, 0xA0, zeroWidthSpace);
+    overwriteCodePoint(softHyphen, zeroWidthSpace);
+    overwriteCodePoint('\n', space);
+    overwriteCodePoint('\t', space);
+    overwriteCodePoint(noBreakSpace, space);
+    overwriteCodePoint(narrowNoBreakSpace, zeroWidthSpace);
+    overwriteCodePoint(leftToRightMark, zeroWidthSpace);
+    overwriteCodePoint(rightToLeftMark, zeroWidthSpace);
+    overwriteCodePoint(leftToRightEmbed, zeroWidthSpace);
+    overwriteCodePoint(rightToLeftEmbed, zeroWidthSpace);
+    overwriteCodePoint(leftToRightOverride, zeroWidthSpace);
+    overwriteCodePoint(rightToLeftOverride, zeroWidthSpace);
+    overwriteCodePoint(leftToRightIsolate, zeroWidthSpace);
+    overwriteCodePoint(rightToLeftIsolate, zeroWidthSpace);
+    overwriteCodePoint(zeroWidthNonJoiner, zeroWidthSpace);
+    overwriteCodePoint(zeroWidthJoiner, zeroWidthSpace);
+    overwriteCodePoint(popDirectionalFormatting, zeroWidthSpace);
+    overwriteCodePoint(popDirectionalIsolate, zeroWidthSpace);
+    overwriteCodePoint(firstStrongIsolate, zeroWidthSpace);
+    overwriteCodePoint(objectReplacementCharacter, zeroWidthSpace);
+    overwriteCodePoint(zeroWidthNoBreakSpace, zeroWidthSpace);
+}
+#endif
+
 static RefPtr<GlyphPage> createAndFillGlyphPage(unsigned pageNumber, const Font& font)
 {
-#if PLATFORM(IOS)
+#if PLATFORM(IOS_FAMILY)
     // FIXME: Times New Roman contains Arabic glyphs, but Core Text doesn't know how to shape them. See <rdar://problem/9823975>.
     // Once we have the fix for <rdar://problem/9823975> then remove this code together with Font::shouldNotBeUsedForArabic()
     // in <rdar://problem/12096835>.
@@ -164,7 +327,6 @@ static RefPtr<GlyphPage> createAndFillGlyphPage(unsigned pageNumber, const Font&
     unsigned glyphPageSize = GlyphPage::sizeForPageNumber(pageNumber);
 
     unsigned start = GlyphPage::startingCodePointInPageNumber(pageNumber);
-    unsigned end = start + glyphPageSize;
     Vector<UChar> buffer(glyphPageSize * 2 + 2);
     unsigned bufferLength;
     // Fill in a buffer with the entire "page" of characters that we want to look up glyphs for.
@@ -173,41 +335,9 @@ static RefPtr<GlyphPage> createAndFillGlyphPage(unsigned pageNumber, const Font&
         for (unsigned i = 0; i < bufferLength; i++)
             buffer[i] = start + i;
 
-        // Code points 0x0 - 0x20 and 0x7F - 0xA0 are control character and shouldn't render. Map them to ZERO WIDTH SPACE.
-        auto overwriteCodePoints = [&](unsigned minimum, unsigned maximum, UChar newCodePoint) {
-            unsigned begin = std::max(start, minimum);
-            unsigned complete = std::min(end, maximum);
-            for (unsigned i = begin; i < complete; ++i)
-                buffer[i - start] = newCodePoint;
-        };
-
-        auto overwriteCodePoint = [&](UChar codePoint, UChar newCodePoint) {
-            if (codePoint >= start && codePoint < end)
-                buffer[codePoint - start] = newCodePoint;
-        };
-
-        overwriteCodePoints(0x0, 0x20, zeroWidthSpace);
-        overwriteCodePoints(0x7F, 0xA0, zeroWidthSpace);
-        overwriteCodePoint(softHyphen, zeroWidthSpace);
-        overwriteCodePoint('\n', space);
-        overwriteCodePoint('\t', space);
-        overwriteCodePoint(noBreakSpace, space);
-        overwriteCodePoint(narrowNoBreakSpace, zeroWidthSpace);
-        overwriteCodePoint(leftToRightMark, zeroWidthSpace);
-        overwriteCodePoint(rightToLeftMark, zeroWidthSpace);
-        overwriteCodePoint(leftToRightEmbed, zeroWidthSpace);
-        overwriteCodePoint(rightToLeftEmbed, zeroWidthSpace);
-        overwriteCodePoint(leftToRightOverride, zeroWidthSpace);
-        overwriteCodePoint(rightToLeftOverride, zeroWidthSpace);
-        overwriteCodePoint(leftToRightIsolate, zeroWidthSpace);
-        overwriteCodePoint(rightToLeftIsolate, zeroWidthSpace);
-        overwriteCodePoint(zeroWidthNonJoiner, zeroWidthSpace);
-        overwriteCodePoint(zeroWidthJoiner, zeroWidthSpace);
-        overwriteCodePoint(popDirectionalFormatting, zeroWidthSpace);
-        overwriteCodePoint(popDirectionalIsolate, zeroWidthSpace);
-        overwriteCodePoint(firstStrongIsolate, zeroWidthSpace);
-        overwriteCodePoint(objectReplacementCharacter, zeroWidthSpace);
-        overwriteCodePoint(zeroWidthNoBreakSpace, zeroWidthSpace);
+#if !USE(FREETYPE)
+        overrideControlCharacters(buffer, start, start + glyphPageSize);
+#endif
     } else {
         bufferLength = glyphPageSize * 2;
         for (unsigned i = 0; i < glyphPageSize; i++) {
@@ -272,7 +402,7 @@ const Font& Font::verticalRightOrientationFont() const
 {
     DerivedFonts& derivedFontData = ensureDerivedFontData();
     if (!derivedFontData.verticalRightOrientationFont) {
-        auto verticalRightPlatformData = FontPlatformData::cloneWithOrientation(m_platformData, Horizontal);
+        auto verticalRightPlatformData = FontPlatformData::cloneWithOrientation(m_platformData, FontOrientation::Horizontal);
         derivedFontData.verticalRightOrientationFont = create(verticalRightPlatformData, origin(), Interstitial::No, Visibility::Visible, OrientationFallback::Yes);
     }
     ASSERT(derivedFontData.verticalRightOrientationFont != this);
@@ -339,7 +469,7 @@ const Font& Font::brokenIdeographFont() const
     return *derivedFontData.brokenIdeographFont;
 }
 
-#ifndef NDEBUG
+#if !LOG_DISABLED
 String Font::description() const
 {
     if (origin() == Origin::Remote)
@@ -368,7 +498,6 @@ RefPtr<Font> Font::createScaledFont(const FontDescription& fontDescription, floa
 
 bool Font::applyTransforms(GlyphBufferGlyph* glyphs, GlyphBufferAdvance* advances, size_t glyphCount, bool enableKerning, bool requiresShaping) const
 {
-    // We need to handle transforms on SVG fonts internally, since they are rendered internally.
 #if PLATFORM(COCOA)
     CTFontTransformOptions options = (enableKerning ? kCTFontTransformApplyPositioning : 0) | (requiresShaping ? kCTFontTransformApplyShaping : 0);
     return CTFontTransformGlyphs(m_platformData.ctFont(), glyphs, reinterpret_cast<CGSize*>(advances), glyphCount, options);
@@ -388,10 +517,10 @@ public:
     {
     }
 
-    CharacterFallbackMapKey(const AtomicString& locale, UChar32 character, bool isForPlatformFont)
+    CharacterFallbackMapKey(const AtomicString& locale, UChar32 character, IsForPlatformFont isForPlatformFont)
         : locale(locale)
         , character(character)
-        , isForPlatformFont(isForPlatformFont)
+        , isForPlatformFont(isForPlatformFont == IsForPlatformFont::Yes)
     {
     }
 
@@ -446,13 +575,13 @@ static SystemFallbackCache& systemFallbackCache()
     return map.get();
 }
 
-RefPtr<Font> Font::systemFallbackFontForCharacter(UChar32 character, const FontDescription& description, bool isForPlatformFont) const
+RefPtr<Font> Font::systemFallbackFontForCharacter(UChar32 character, const FontDescription& description, IsForPlatformFont isForPlatformFont) const
 {
     auto fontAddResult = systemFallbackCache().add(this, CharacterFallbackMap());
 
     if (!character) {
         UChar codeUnit = 0;
-        return FontCache::singleton().systemFallbackForCharacters(description, this, isForPlatformFont, &codeUnit, 1);
+        return FontCache::singleton().systemFallbackForCharacters(description, this, isForPlatformFont, FontCache::PreferColoredFont::No, &codeUnit, 1);
     }
 
     auto key = CharacterFallbackMapKey(description.locale(), character, isForPlatformFont);
@@ -472,7 +601,7 @@ RefPtr<Font> Font::systemFallbackFontForCharacter(UChar32 character, const FontD
             codeUnitsLength = 2;
         }
 
-        fallbackFont = FontCache::singleton().systemFallbackForCharacters(description, this, isForPlatformFont, codeUnits, codeUnitsLength).get();
+        fallbackFont = FontCache::singleton().systemFallbackForCharacters(description, this, isForPlatformFont, FontCache::PreferColoredFont::No, codeUnits, codeUnitsLength).get();
         if (fallbackFont)
             fallbackFont->m_isUsedInSystemFallbackCache = true;
     }
@@ -498,7 +627,7 @@ void Font::removeFromSystemFallbackCache()
     }
 }
 
-#if !PLATFORM(COCOA)
+#if !PLATFORM(COCOA) && !USE(FREETYPE)
 bool Font::variantCapsSupportsCharacterForSynthesis(FontVariantCaps fontVariantCaps, UChar32) const
 {
     switch (fontVariantCaps) {
@@ -512,6 +641,68 @@ bool Font::variantCapsSupportsCharacterForSynthesis(FontVariantCaps fontVariantC
         return true;
     }
 }
+
+bool Font::platformSupportsCodePoint(UChar32 character, Optional<UChar32> variation) const
+{
+    return variation ? false : glyphForCharacter(character);
+}
 #endif
 
+bool Font::supportsCodePoint(UChar32 character) const
+{
+    // This is very similar to static_cast<bool>(glyphForCharacter(character))
+    // except that glyphForCharacter() maps certain code points to ZWS (because they
+    // shouldn't be visible). This function doesn't do that mapping, and instead is
+    // as honest as possible about what code points the font supports. This is so
+    // that we can accurately determine which characters are supported by this font
+    // so we know which boundaries to break strings when we send them to the complex
+    // text codepath. The complex text codepath is totally separate from this ZWS
+    // replacement logic (because CoreText handles those characters instead of WebKit).
+    if (auto index = codePointSupportIndex(character)) {
+        m_codePointSupport.ensureSize(2 * (*index + 1));
+        bool hasBeenSet = m_codePointSupport.quickSet(2 * *index);
+        if (!hasBeenSet && platformSupportsCodePoint(character))
+            m_codePointSupport.quickSet(2 * *index + 1);
+        return m_codePointSupport.quickGet(2 * *index + 1);
+    }
+    return glyphForCharacter(character);
+}
+
+bool Font::canRenderCombiningCharacterSequence(const UChar* characters, size_t length) const
+{
+    ASSERT(isMainThread());
+
+    auto codePoints = StringView(characters, length).codePoints();
+    auto it = codePoints.begin();
+    auto end = codePoints.end();
+    while (it != end) {
+        auto codePoint = *it;
+        ++it;
+
+        if (it != end && isVariationSelector(*it)) {
+            if (!platformSupportsCodePoint(codePoint, *it)) {
+                // Try the characters individually.
+                if (!supportsCodePoint(codePoint) || !supportsCodePoint(*it))
+                    return false;
+            }
+            ++it;
+            continue;
+        }
+
+        if (!supportsCodePoint(codePoint))
+            return false;
+    }
+    return true;
+}
+
+// Don't store the result of this! The hash map is free to rehash at any point, leaving this reference dangling.
+const Path& Font::pathForGlyph(Glyph glyph) const
+{
+    if (const auto& path = m_glyphPathMap.existingMetricsForGlyph(glyph))
+        return *path;
+    auto path = platformPathForGlyph(glyph);
+    m_glyphPathMap.setMetricsForGlyph(glyph, path);
+    return *m_glyphPathMap.existingMetricsForGlyph(glyph);
+}
+
 } // namespace WebCore
