@@ -78,7 +78,7 @@ bool GraphicsLayer::supportsLayerType(Type type)
     switch (type) {
     case Type::Normal:
     case Type::PageTiledBacking:
-    case Type::Scrolling:
+    case Type::ScrollContainer:
         return true;
     case Type::Shape:
         return false;
@@ -110,15 +110,23 @@ bool GraphicsLayer::supportsContentsTiling()
 }
 #endif
 
-GraphicsLayer::GraphicsLayer(Type type, GraphicsLayerClient& client)
-    : m_client(client)
-    , m_anchorPoint(0.5f, 0.5f, 0)
-    , m_opacity(1)
-    , m_zPosition(0)
-#if ENABLE(CSS_COMPOSITING)
-    , m_blendMode(BlendModeNormal)
-#endif
+// Singleton client used for layers on which clearClient has been called.
+class EmptyGraphicsLayerClient : public GraphicsLayerClient {
+    WTF_MAKE_FAST_ALLOCATED;
+public:
+    static EmptyGraphicsLayerClient& singleton();
+};
+
+EmptyGraphicsLayerClient& EmptyGraphicsLayerClient::singleton()
+{
+    static NeverDestroyed<EmptyGraphicsLayerClient> client;
+    return client;
+}
+
+GraphicsLayer::GraphicsLayer(Type type, GraphicsLayerClient& layerClient)
+    : m_client(&layerClient)
     , m_type(type)
+    , m_beingDestroyed(false)
     , m_contentsOpaque(false)
     , m_supportsSubpixelAntialiasedText(false)
     , m_preserves3D(false)
@@ -135,17 +143,9 @@ GraphicsLayer::GraphicsLayer(Type type, GraphicsLayerClient& client)
     , m_isTrackingDisplayListReplay(false)
     , m_userInteractionEnabled(true)
     , m_canDetachBackingStore(true)
-    , m_paintingPhase(GraphicsLayerPaintAllWithOverflowClip)
-    , m_contentsOrientation(CompositingCoordinatesTopDown)
-    , m_parent(nullptr)
-    , m_maskLayer(nullptr)
-    , m_replicaLayer(nullptr)
-    , m_replicatedLayer(nullptr)
-    , m_repaintCount(0)
-    , m_customAppearance(NoCustomAppearance)
 {
 #ifndef NDEBUG
-    m_client.verifyNotPainting();
+    client().verifyNotPainting();
 #endif
 }
 
@@ -155,16 +155,34 @@ GraphicsLayer::~GraphicsLayer()
     ASSERT(!m_parent); // willBeDestroyed should have been called already.
 }
 
+void GraphicsLayer::unparentAndClear(RefPtr<GraphicsLayer>& layer)
+{
+    if (layer) {
+        layer->removeFromParent();
+        layer->clearClient();
+        layer = nullptr;
+    }
+}
+
+void GraphicsLayer::clear(RefPtr<GraphicsLayer>& layer)
+{
+    if (layer) {
+        layer->clearClient();
+        layer = nullptr;
+    }
+}
+
 void GraphicsLayer::willBeDestroyed()
 {
+    m_beingDestroyed = true;
 #ifndef NDEBUG
-    m_client.verifyNotPainting();
+    client().verifyNotPainting();
 #endif
     if (m_replicaLayer)
-        m_replicaLayer->setReplicatedLayer(0);
+        m_replicaLayer->setReplicatedLayer(nullptr);
 
     if (m_replicatedLayer)
-        m_replicatedLayer->setReplicatedByLayer(0);
+        m_replicatedLayer->setReplicatedByLayer(nullptr);
 
     if (m_maskLayer) {
         m_maskLayer->setParent(nullptr);
@@ -175,6 +193,16 @@ void GraphicsLayer::willBeDestroyed()
     removeFromParent();
 }
 
+void GraphicsLayer::clearClient()
+{
+    m_client = &EmptyGraphicsLayerClient::singleton();
+}
+
+void GraphicsLayer::setClient(GraphicsLayerClient& client)
+{
+    m_client = &client;
+}
+
 void GraphicsLayer::setParent(GraphicsLayer* layer)
 {
     ASSERT(!layer || !layer->hasAncestor(this));
@@ -191,7 +219,7 @@ bool GraphicsLayer::hasAncestor(GraphicsLayer* ancestor) const
     return false;
 }
 
-bool GraphicsLayer::setChildren(const Vector<GraphicsLayer*>& newChildren)
+bool GraphicsLayer::setChildren(Vector<Ref<GraphicsLayer>>&& newChildren)
 {
     // If the contents of the arrays are the same, nothing to do.
     if (newChildren == m_children)
@@ -201,89 +229,82 @@ bool GraphicsLayer::setChildren(const Vector<GraphicsLayer*>& newChildren)
 
     size_t listSize = newChildren.size();
     for (size_t i = 0; i < listSize; ++i)
-        addChild(newChildren[i]);
+        addChild(WTFMove(newChildren[i]));
 
     return true;
 }
 
-void GraphicsLayer::addChild(GraphicsLayer* childLayer)
+void GraphicsLayer::addChild(Ref<GraphicsLayer>&& childLayer)
 {
-    ASSERT(childLayer != this);
-
-    if (childLayer->parent())
-        childLayer->removeFromParent();
+    ASSERT(childLayer.ptr() != this);
 
+    childLayer->removeFromParent();
     childLayer->setParent(this);
-    m_children.append(childLayer);
+    m_children.append(WTFMove(childLayer));
 }
 
-void GraphicsLayer::addChildAtIndex(GraphicsLayer* childLayer, int index)
+void GraphicsLayer::addChildAtIndex(Ref<GraphicsLayer>&& childLayer, int index)
 {
-    ASSERT(childLayer != this);
-
-    if (childLayer->parent())
-        childLayer->removeFromParent();
+    ASSERT(childLayer.ptr() != this);
 
+    childLayer->removeFromParent();
     childLayer->setParent(this);
-    m_children.insert(index, childLayer);
+    m_children.insert(index, WTFMove(childLayer));
 }
 
-void GraphicsLayer::addChildBelow(GraphicsLayer* childLayer, GraphicsLayer* sibling)
+void GraphicsLayer::addChildBelow(Ref<GraphicsLayer>&& childLayer, GraphicsLayer* sibling)
 {
-    ASSERT(childLayer != this);
+    ASSERT(childLayer.ptr() != this);
     childLayer->removeFromParent();
 
-    bool found = false;
+    childLayer->setParent(this);
+
     for (unsigned i = 0; i < m_children.size(); i++) {
-        if (sibling == m_children[i]) {
-            m_children.insert(i, childLayer);
-            found = true;
-            break;
+        if (sibling == m_children[i].ptr()) {
+            m_children.insert(i, WTFMove(childLayer));
+            return;
         }
     }
 
-    childLayer->setParent(this);
-
-    if (!found)
-        m_children.append(childLayer);
+    m_children.append(WTFMove(childLayer));
 }
 
-void GraphicsLayer::addChildAbove(GraphicsLayer* childLayer, GraphicsLayer* sibling)
+void GraphicsLayer::addChildAbove(Ref<GraphicsLayer>&& childLayer, GraphicsLayer* sibling)
 {
     childLayer->removeFromParent();
-    ASSERT(childLayer != this);
+    ASSERT(childLayer.ptr() != this);
+
+    childLayer->setParent(this);
 
-    bool found = false;
     for (unsigned i = 0; i < m_children.size(); i++) {
-        if (sibling == m_children[i]) {
-            m_children.insert(i+1, childLayer);
-            found = true;
-            break;
+        if (sibling == m_children[i].ptr()) {
+            m_children.insert(i + 1, WTFMove(childLayer));
+            return;
         }
     }
 
-    childLayer->setParent(this);
-
-    if (!found)
-        m_children.append(childLayer);
+    m_children.append(WTFMove(childLayer));
 }
 
-bool GraphicsLayer::replaceChild(GraphicsLayer* oldChild, GraphicsLayer* newChild)
+bool GraphicsLayer::replaceChild(GraphicsLayer* oldChild, Ref<GraphicsLayer>&& newChild)
 {
     ASSERT(!newChild->parent());
+
+    GraphicsLayer* rawNewChild = newChild.ptr();
+
     bool found = false;
     for (unsigned i = 0; i < m_children.size(); i++) {
-        if (oldChild == m_children[i]) {
-            m_children[i] = newChild;
+        if (oldChild == m_children[i].ptr()) {
+            m_children[i] = WTFMove(newChild);
             found = true;
             break;
         }
     }
     if (found) {
-        oldChild->setParent(0);
+        oldChild->setParent(nullptr);
 
-        newChild->removeFromParent();
-        newChild->setParent(this);
+        rawNewChild->removeFromParent();
+        rawNewChild->setParent(this);
         return true;
     }
     return false;
@@ -292,21 +313,52 @@ bool GraphicsLayer::replaceChild(GraphicsLayer* oldChild, GraphicsLayer* newChil
 void GraphicsLayer::removeAllChildren()
 {
     while (m_children.size()) {
-        GraphicsLayer* curLayer = m_children[0];
+        GraphicsLayer* curLayer = m_children[0].ptr();
         ASSERT(curLayer->parent());
         curLayer->removeFromParent();
+        // curLayer may be destroyed here.
     }
 }
 
 void GraphicsLayer::removeFromParent()
 {
     if (m_parent) {
-        m_parent->m_children.removeFirst(this);
+        GraphicsLayer* parent = m_parent;
         setParent(nullptr);
+        parent->m_children.removeFirstMatching([this](auto& layer) {
+            return layer.ptr() == this;
+        });
+        // |this| may be destroyed here.
     }
 }
 
-void GraphicsLayer::setMaskLayer(GraphicsLayer* layer)
+const TransformationMatrix& GraphicsLayer::transform() const
+{
+    return m_transform ? *m_transform : TransformationMatrix::identity;
+}
+
+void GraphicsLayer::setTransform(const TransformationMatrix& matrix)
+{
+    if (m_transform)
+        *m_transform = matrix;
+    else
+        m_transform = std::make_unique<TransformationMatrix>(matrix);
+}
+
+const TransformationMatrix& GraphicsLayer::childrenTransform() const
+{
+    return m_childrenTransform ? *m_childrenTransform : TransformationMatrix::identity;
+}
+
+void GraphicsLayer::setChildrenTransform(const TransformationMatrix& matrix)
+{
+    if (m_childrenTransform)
+        *m_childrenTransform = matrix;
+    else
+        m_childrenTransform = std::make_unique<TransformationMatrix>(matrix);
+}
+
+void GraphicsLayer::setMaskLayer(RefPtr<GraphicsLayer>&& layer)
 {
     if (layer == m_maskLayer)
         return;
@@ -320,7 +372,7 @@ void GraphicsLayer::setMaskLayer(GraphicsLayer* layer)
         m_maskLayer->setIsMaskLayer(false);
     }
 
-    m_maskLayer = layer;
+    m_maskLayer = WTFMove(layer);
 }
 
 Path GraphicsLayer::shapeLayerPath() const
@@ -346,7 +398,7 @@ WindRule GraphicsLayer::shapeLayerWindRule() const
 #if USE(CA)
     return m_shapeLayerWindRule;
 #else
-    return RULE_NONZERO;
+    return WindRule::NonZero;
 #endif
 }
 
@@ -369,10 +421,8 @@ void GraphicsLayer::noteDeviceOrPageScaleFactorChangedIncludingDescendants()
     if (m_replicaLayer)
         m_replicaLayer->noteDeviceOrPageScaleFactorChangedIncludingDescendants();
 
-    const Vector<GraphicsLayer*>& childLayers = children();
-    size_t numChildren = childLayers.size();
-    for (size_t i = 0; i < numChildren; ++i)
-        childLayers[i]->noteDeviceOrPageScaleFactorChangedIncludingDescendants();
+    for (auto& layer : children())
+        layer->noteDeviceOrPageScaleFactorChangedIncludingDescendants();
 }
 
 void GraphicsLayer::setIsInWindow(bool inWindow)
@@ -381,7 +431,7 @@ void GraphicsLayer::setIsInWindow(bool inWindow)
         tiledBacking->setIsInWindow(inWindow);
 }
 
-void GraphicsLayer::setReplicatedByLayer(GraphicsLayer* layer)
+void GraphicsLayer::setReplicatedByLayer(RefPtr<GraphicsLayer>&& layer)
 {
     if (m_replicaLayer == layer)
         return;
@@ -392,7 +442,7 @@ void GraphicsLayer::setReplicatedByLayer(GraphicsLayer* layer)
     if (layer)
         layer->setReplicatedLayer(this);
 
-    m_replicaLayer = layer;
+    m_replicaLayer = WTFMove(layer);
 }
 
 void GraphicsLayer::setOffsetFromRenderer(const FloatSize& offset, ShouldSetNeedsDisplay shouldSetNeedsDisplay)
@@ -407,6 +457,18 @@ void GraphicsLayer::setOffsetFromRenderer(const FloatSize& offset, ShouldSetNeed
         setNeedsDisplay();
 }
 
+void GraphicsLayer::setScrollOffset(const ScrollOffset& offset, ShouldSetNeedsDisplay shouldSetNeedsDisplay)
+{
+    if (offset == m_scrollOffset)
+        return;
+
+    m_scrollOffset = offset;
+
+    // If the compositing layer offset changes, we need to repaint.
+    if (shouldSetNeedsDisplay == SetNeedsDisplay)
+        setNeedsDisplay();
+}
+
 void GraphicsLayer::setSize(const FloatSize& size)
 {
     if (size == m_size)
@@ -425,13 +487,13 @@ void GraphicsLayer::setBackgroundColor(const Color& color)
 
 void GraphicsLayer::paintGraphicsLayerContents(GraphicsContext& context, const FloatRect& clip, GraphicsLayerPaintBehavior layerPaintBehavior)
 {
-    FloatSize offset = offsetFromRenderer();
+    FloatSize offset = offsetFromRenderer() - toFloatSize(scrollOffset());
     context.translate(-offset);
 
     FloatRect clipRect(clip);
     clipRect.move(offset);
 
-    m_client.paintContents(this, context, m_paintingPhase, clipRect, layerPaintBehavior);
+    client().paintContents(this, context, m_paintingPhase, clipRect, layerPaintBehavior);
 }
 
 String GraphicsLayer::animationNameForTransition(AnimatedPropertyID property)
@@ -444,7 +506,7 @@ String GraphicsLayer::animationNameForTransition(AnimatedPropertyID property)
     return id.toString();
 }
 
-void GraphicsLayer::suspendAnimations(double)
+void GraphicsLayer::suspendAnimations(MonotonicTime)
 {
 }
 
@@ -522,9 +584,8 @@ void GraphicsLayer::distributeOpacity(float accumulatedOpacity)
     setOpacityInternal(accumulatedOpacity);
 
     if (preserves3D()) {
-        size_t numChildren = children().size();
-        for (size_t i = 0; i < numChildren; ++i)
-            children()[i]->distributeOpacity(accumulatedOpacity);
+        for (auto& layer : children())
+            layer->distributeOpacity(accumulatedOpacity);
     }
 }
 
@@ -661,7 +722,7 @@ void GraphicsLayer::resetTrackedRepaints()
 
 void GraphicsLayer::addRepaintRect(const FloatRect& repaintRect)
 {
-    if (!m_client.isTrackingRepaints())
+    if (!client().isTrackingRepaints())
         return;
 
     FloatRect largestRepaintRect(FloatPoint(), m_size);
@@ -681,8 +742,8 @@ void GraphicsLayer::traverse(GraphicsLayer& layer, const WTF::Function<void (Gra
 {
     traversalFunc(layer);
 
-    for (auto* childLayer : layer.children())
-        traverse(*childLayer, traversalFunc);
+    for (auto& childLayer : layer.children())
+        traverse(childLayer.get(), traversalFunc);
 
     if (auto* replicaLayer = layer.replicaLayer())
         traverse(*replicaLayer, traversalFunc);
@@ -691,6 +752,12 @@ void GraphicsLayer::traverse(GraphicsLayer& layer, const WTF::Function<void (Gra
         traverse(*maskLayer, traversalFunc);
 }
 
+GraphicsLayer::EmbeddedViewID GraphicsLayer::nextEmbeddedViewID()
+{
+    static GraphicsLayer::EmbeddedViewID nextEmbeddedViewID;
+    return ++nextEmbeddedViewID;
+}
+
 void GraphicsLayer::dumpLayer(TextStream& ts, LayerTreeAsTextBehavior behavior) const
 {
     ts << indent << "(" << "GraphicsLayer";
@@ -705,11 +772,11 @@ void GraphicsLayer::dumpLayer(TextStream& ts, LayerTreeAsTextBehavior behavior)
     ts << indent << ")\n";
 }
 
-static void dumpChildren(TextStream& ts, const Vector<GraphicsLayer*>& children, unsigned& totalChildCount, LayerTreeAsTextBehavior behavior)
+static void dumpChildren(TextStream& ts, const Vector<Ref<GraphicsLayer>>& children, unsigned& totalChildCount, LayerTreeAsTextBehavior behavior)
 {
     totalChildCount += children.size();
-    for (auto* child : children) {
-        if ((behavior & LayerTreeAsTextDebug) || !child->client().shouldSkipLayerInDump(child, behavior)) {
+    for (auto& child : children) {
+        if ((behavior & LayerTreeAsTextDebug) || !child->client().shouldSkipLayerInDump(child.ptr(), behavior)) {
             TextStream::IndentScope indentScope(ts);
             child->dumpLayer(ts, behavior);
             continue;
@@ -726,6 +793,9 @@ void GraphicsLayer::dumpProperties(TextStream& ts, LayerTreeAsTextBehavior behav
     if (!m_offsetFromRenderer.isZero())
         ts << indent << "(offsetFromRenderer " << m_offsetFromRenderer << ")\n";
 
+    if (!m_scrollOffset.isZero())
+        ts << indent << "(scrollOffset " << m_scrollOffset << ")\n";
+
     if (m_position != FloatPoint())
         ts << indent << "(position " << m_position.x() << " " << m_position.y() << ")\n";
 
@@ -749,14 +819,14 @@ void GraphicsLayer::dumpProperties(TextStream& ts, LayerTreeAsTextBehavior behav
         ts << indent << "(opacity " << m_opacity << ")\n";
 
 #if ENABLE(CSS_COMPOSITING)
-    if (m_blendMode != BlendModeNormal)
+    if (m_blendMode != BlendMode::Normal)
         ts << indent << "(blendMode " << compositeOperatorName(CompositeSourceOver, m_blendMode) << ")\n";
 #endif
 
     if (type() == Type::Normal && tiledBacking())
         ts << indent << "(usingTiledLayer 1)\n";
 
-    bool needsIOSDumpRenderTreeMainFrameRenderViewLayerIsAlwaysOpaqueHack = m_client.needsIOSDumpRenderTreeMainFrameRenderViewLayerIsAlwaysOpaqueHack(*this);
+    bool needsIOSDumpRenderTreeMainFrameRenderViewLayerIsAlwaysOpaqueHack = client().needsIOSDumpRenderTreeMainFrameRenderViewLayerIsAlwaysOpaqueHack(*this);
     if (m_contentsOpaque || needsIOSDumpRenderTreeMainFrameRenderViewLayerIsAlwaysOpaqueHack)
         ts << indent << "(contentsOpaque " << (m_contentsOpaque || needsIOSDumpRenderTreeMainFrameRenderViewLayerIsAlwaysOpaqueHack) << ")\n";
 
@@ -766,7 +836,7 @@ void GraphicsLayer::dumpProperties(TextStream& ts, LayerTreeAsTextBehavior behav
     if (m_preserves3D)
         ts << indent << "(preserves3D " << m_preserves3D << ")\n";
 
-    if (m_drawsContent && m_client.shouldDumpPropertyForLayer(this, "drawsContent"))
+    if (m_drawsContent && client().shouldDumpPropertyForLayer(this, "drawsContent", behavior))
         ts << indent << "(drawsContent " << m_drawsContent << ")\n";
 
     if (!m_contentsVisible)
@@ -777,34 +847,34 @@ void GraphicsLayer::dumpProperties(TextStream& ts, LayerTreeAsTextBehavior behav
 
     if (behavior & LayerTreeAsTextDebug) {
         ts << indent << "(primary-layer-id " << primaryLayerID() << ")\n";
-        ts << indent << "(client " << static_cast<void*>(&m_client) << ")\n";
+        ts << indent << "(client " << static_cast<void*>(m_client) << ")\n";
     }
 
-    if (m_backgroundColor.isValid() && m_client.shouldDumpPropertyForLayer(this, "backgroundColor"))
+    if (m_backgroundColor.isValid() && client().shouldDumpPropertyForLayer(this, "backgroundColor", behavior))
         ts << indent << "(backgroundColor " << m_backgroundColor.nameForRenderTreeAsText() << ")\n";
 
     if (behavior & LayerTreeAsTextIncludeAcceleratesDrawing && m_acceleratesDrawing)
         ts << indent << "(acceleratesDrawing " << m_acceleratesDrawing << ")\n";
 
     if (behavior & LayerTreeAsTextIncludeBackingStoreAttached)
-        ts << indent << "(backingStoreAttached " << backingStoreAttached() << ")\n";
+        ts << indent << "(backingStoreAttached " << backingStoreAttachedForTesting() << ")\n";
 
-    if (!m_transform.isIdentity()) {
+    if (m_transform && !m_transform->isIdentity()) {
         ts << indent << "(transform ";
-        ts << "[" << m_transform.m11() << " " << m_transform.m12() << " " << m_transform.m13() << " " << m_transform.m14() << "] ";
-        ts << "[" << m_transform.m21() << " " << m_transform.m22() << " " << m_transform.m23() << " " << m_transform.m24() << "] ";
-        ts << "[" << m_transform.m31() << " " << m_transform.m32() << " " << m_transform.m33() << " " << m_transform.m34() << "] ";
-        ts << "[" << m_transform.m41() << " " << m_transform.m42() << " " << m_transform.m43() << " " << m_transform.m44() << "])\n";
+        ts << "[" << m_transform->m11() << " " << m_transform->m12() << " " << m_transform->m13() << " " << m_transform->m14() << "] ";
+        ts << "[" << m_transform->m21() << " " << m_transform->m22() << " " << m_transform->m23() << " " << m_transform->m24() << "] ";
+        ts << "[" << m_transform->m31() << " " << m_transform->m32() << " " << m_transform->m33() << " " << m_transform->m34() << "] ";
+        ts << "[" << m_transform->m41() << " " << m_transform->m42() << " " << m_transform->m43() << " " << m_transform->m44() << "])\n";
     }
 
     // Avoid dumping the sublayer transform on the root layer, because it's used for geometry flipping, whose behavior
     // differs between platforms.
-    if (parent() && !m_childrenTransform.isIdentity()) {
+    if (parent() && m_childrenTransform && !m_childrenTransform->isIdentity()) {
         ts << indent << "(childrenTransform ";
-        ts << "[" << m_childrenTransform.m11() << " " << m_childrenTransform.m12() << " " << m_childrenTransform.m13() << " " << m_childrenTransform.m14() << "] ";
-        ts << "[" << m_childrenTransform.m21() << " " << m_childrenTransform.m22() << " " << m_childrenTransform.m23() << " " << m_childrenTransform.m24() << "] ";
-        ts << "[" << m_childrenTransform.m31() << " " << m_childrenTransform.m32() << " " << m_childrenTransform.m33() << " " << m_childrenTransform.m34() << "] ";
-        ts << "[" << m_childrenTransform.m41() << " " << m_childrenTransform.m42() << " " << m_childrenTransform.m43() << " " << m_childrenTransform.m44() << "])\n";
+        ts << "[" << m_childrenTransform->m11() << " " << m_childrenTransform->m12() << " " << m_childrenTransform->m13() << " " << m_childrenTransform->m14() << "] ";
+        ts << "[" << m_childrenTransform->m21() << " " << m_childrenTransform->m22() << " " << m_childrenTransform->m23() << " " << m_childrenTransform->m24() << "] ";
+        ts << "[" << m_childrenTransform->m31() << " " << m_childrenTransform->m32() << " " << m_childrenTransform->m33() << " " << m_childrenTransform->m34() << "] ";
+        ts << "[" << m_childrenTransform->m41() << " " << m_childrenTransform->m42() << " " << m_childrenTransform->m43() << " " << m_childrenTransform->m44() << "])\n";
     }
 
     if (m_maskLayer) {
@@ -834,7 +904,7 @@ void GraphicsLayer::dumpProperties(TextStream& ts, LayerTreeAsTextBehavior behav
         ts << ")\n";
     }
 
-    if (behavior & LayerTreeAsTextIncludeRepaintRects && repaintRectMap().contains(this) && !repaintRectMap().get(this).isEmpty() && m_client.shouldDumpPropertyForLayer(this, "repaintRects")) {
+    if (behavior & LayerTreeAsTextIncludeRepaintRects && repaintRectMap().contains(this) && !repaintRectMap().get(this).isEmpty() && client().shouldDumpPropertyForLayer(this, "repaintRects", behavior)) {
         ts << indent << "(repaint rects\n";
         for (size_t i = 0; i < repaintRectMap().get(this).size(); ++i) {
             if (repaintRectMap().get(this)[i].isEmpty())
@@ -906,11 +976,11 @@ TextStream& operator<<(TextStream& ts, const Vector<GraphicsLayer::PlatformLayer
 TextStream& operator<<(TextStream& ts, const WebCore::GraphicsLayer::CustomAppearance& customAppearance)
 {
     switch (customAppearance) {
-    case GraphicsLayer::CustomAppearance::NoCustomAppearance: ts << "none"; break;
+    case GraphicsLayer::CustomAppearance::None: ts << "none"; break;
     case GraphicsLayer::CustomAppearance::ScrollingOverhang: ts << "scrolling-overhang"; break;
     case GraphicsLayer::CustomAppearance::ScrollingShadow: ts << "scrolling-shadow"; break;
-    case GraphicsLayer::CustomAppearance::LightBackdropAppearance: ts << "light-backdrop"; break;
-    case GraphicsLayer::CustomAppearance::DarkBackdropAppearance: ts << "dark-backdrop"; break;
+    case GraphicsLayer::CustomAppearance::LightBackdrop: ts << "light-backdrop"; break;
+    case GraphicsLayer::CustomAppearance::DarkBackdrop: ts << "dark-backdrop"; break;
     }
     return ts;
 }
@@ -931,7 +1001,7 @@ void showGraphicsLayerTree(const WebCore::GraphicsLayer* layer)
     if (!layer)
         return;
 
-    String output = layer->layerTreeAsText(WebCore::LayerTreeAsTextDebug | WebCore::LayerTreeAsTextIncludeVisibleRects | WebCore::LayerTreeAsTextIncludeTileCaches | WebCore::LayerTreeAsTextIncludeContentLayers);
-    fprintf(stderr, "%s\n", output.utf8().data());
+    String output = layer->layerTreeAsText(WebCore::LayerTreeAsTextShowAll);
+    WTFLogAlways("%s\n", output.utf8().data());
 }
 #endif
