@@ -99,8 +99,8 @@ size_t Item::sizeInBytes(const Item& item)
         return sizeof(downcast<DrawLine>(item));
     case ItemType::DrawLinesForText:
         return sizeof(downcast<DrawLinesForText>(item));
-    case ItemType::DrawLineForDocumentMarker:
-        return sizeof(downcast<DrawLineForDocumentMarker>(item));
+    case ItemType::DrawDotsForDocumentMarker:
+        return sizeof(downcast<DrawDotsForDocumentMarker>(item));
     case ItemType::DrawEllipse:
         return sizeof(downcast<DrawEllipse>(item));
     case ItemType::DrawPath:
@@ -371,7 +371,7 @@ inline GlyphBuffer DrawGlyphs::generateGlyphBuffer() const
 
 void DrawGlyphs::apply(GraphicsContext& context) const
 {
-    FontCascade::drawGlyphs(context, m_font, generateGlyphBuffer(), 0, m_glyphs.size(), anchorPoint(), m_smoothingMode);
+    context.drawGlyphs(m_font, generateGlyphBuffer(), 0, m_glyphs.size(), anchorPoint(), m_smoothingMode);
 }
 
 void DrawGlyphs::computeBounds()
@@ -391,7 +391,7 @@ void DrawGlyphs::computeBounds()
     }
 }
 
-std::optional<FloatRect> DrawGlyphs::localBounds(const GraphicsContext&) const
+Optional<FloatRect> DrawGlyphs::localBounds(const GraphicsContext&) const
 {
     FloatRect localBounds = m_bounds;
     localBounds.move(m_blockLocation.x(), m_blockLocation.y());
@@ -571,7 +571,7 @@ static TextStream& operator<<(TextStream& ts, const DrawRect& item)
     return ts;
 }
 
-std::optional<FloatRect> DrawLine::localBounds(const GraphicsContext&) const
+Optional<FloatRect> DrawLine::localBounds(const GraphicsContext&) const
 {
     FloatRect bounds;
     bounds.fitToPoints(m_point1, m_point2);
@@ -593,17 +593,17 @@ static TextStream& operator<<(TextStream& ts, const DrawLine& item)
 
 void DrawLinesForText::apply(GraphicsContext& context) const
 {
-    context.drawLinesForText(point(), m_widths, m_printing, m_doubleLines);
+    context.drawLinesForText(point(), m_thickness, m_widths, m_printing, m_doubleLines);
 }
 
-std::optional<FloatRect> DrawLinesForText::localBounds(const GraphicsContext&) const
+Optional<FloatRect> DrawLinesForText::localBounds(const GraphicsContext&) const
 {
     // This function needs to return a value equal to or enclosing what GraphicsContext::computeLineBoundsAndAntialiasingModeForText() returns.
 
     if (!m_widths.size())
         return FloatRect();
 
-    FloatRect result(point(), FloatSize(m_widths.last(), m_strokeWidth));
+    FloatRect result(point(), FloatSize(m_widths.last(), m_thickness));
     result.inflate(1); // Account for pixel snapping. FIXME: This isn't perfect, as it doesn't take the CTM into account.
     return result;
 }
@@ -614,6 +614,7 @@ static TextStream& operator<<(TextStream& ts, const DrawLinesForText& item)
     ts.dumpProperty("block-location", item.blockLocation());
     ts.dumpProperty("local-anchor", item.localAnchor());
     ts.dumpProperty("point", item.point());
+    ts.dumpProperty("thickness", item.thickness());
     ts.dumpProperty("double", item.doubleLines());
     ts.dumpProperty("widths", item.widths());
     ts.dumpProperty("is-printing", item.isPrinting());
@@ -621,25 +622,20 @@ static TextStream& operator<<(TextStream& ts, const DrawLinesForText& item)
     return ts;
 }
 
-void DrawLineForDocumentMarker::apply(GraphicsContext& context) const
+void DrawDotsForDocumentMarker::apply(GraphicsContext& context) const
 {
-    context.drawLineForDocumentMarker(m_point, m_width, m_style);
+    context.drawDotsForDocumentMarker(m_rect, m_style);
 }
 
-std::optional<FloatRect> DrawLineForDocumentMarker::localBounds(const GraphicsContext&) const
+Optional<FloatRect> DrawDotsForDocumentMarker::localBounds(const GraphicsContext&) const
 {
-    // This function needs to return a value equal to or enclosing what GraphicsContext::drawLineForDocumentMarker() returns.
-
-    FloatRect result(m_point, FloatSize(m_width, cMisspellingLineThickness));
-    result.inflate(cMisspellingLineThickness); // Account for "misspelling dot" snapping.
-    return result;
+    return m_rect;
 }
 
-static TextStream& operator<<(TextStream& ts, const DrawLineForDocumentMarker& item)
+static TextStream& operator<<(TextStream& ts, const DrawDotsForDocumentMarker& item)
 {
     ts << static_cast<const DrawingItem&>(item);
-    ts.dumpProperty("point", item.point());
-    ts.dumpProperty("width", item.width());
+    ts.dumpProperty("rect", item.rect());
     return ts;
 }
 
@@ -675,7 +671,7 @@ void DrawFocusRingPath::apply(GraphicsContext& context) const
     context.drawFocusRing(m_path, m_width, m_offset, m_color);
 }
 
-std::optional<FloatRect> DrawFocusRingPath::localBounds(const GraphicsContext&) const
+Optional<FloatRect> DrawFocusRingPath::localBounds(const GraphicsContext&) const
 {
     FloatRect result = m_path.fastBoundingRect();
     result.inflate(platformFocusRingWidth);
@@ -697,7 +693,7 @@ void DrawFocusRingRects::apply(GraphicsContext& context) const
     context.drawFocusRing(m_rects, m_width, m_offset, m_color);
 }
 
-std::optional<FloatRect> DrawFocusRingRects::localBounds(const GraphicsContext&) const
+Optional<FloatRect> DrawFocusRingRects::localBounds(const GraphicsContext&) const
 {
     FloatRect result;
     for (auto& rect : m_rects)
@@ -821,7 +817,7 @@ static TextStream& operator<<(TextStream& ts, const FillEllipse& item)
     return ts;
 }
 
-std::optional<FloatRect> StrokeRect::localBounds(const GraphicsContext&) const
+Optional<FloatRect> StrokeRect::localBounds(const GraphicsContext&) const
 {
     FloatRect bounds = m_rect;
     bounds.expand(m_lineWidth, m_lineWidth);
@@ -841,7 +837,7 @@ static TextStream& operator<<(TextStream& ts, const StrokeRect& item)
     return ts;
 }
 
-std::optional<FloatRect> StrokePath::localBounds(const GraphicsContext& context) const
+Optional<FloatRect> StrokePath::localBounds(const GraphicsContext& context) const
 {
     // FIXME: Need to take stroke thickness into account correctly, via CGPathByStrokingPath().
     float strokeThickness = context.strokeThickness();
@@ -863,7 +859,7 @@ static TextStream& operator<<(TextStream& ts, const StrokePath& item)
     return ts;
 }
 
-std::optional<FloatRect> StrokeEllipse::localBounds(const GraphicsContext& context) const
+Optional<FloatRect> StrokeEllipse::localBounds(const GraphicsContext& context) const
 {
     float strokeThickness = context.strokeThickness();
 
@@ -965,7 +961,7 @@ static TextStream& operator<<(TextStream& ts, const ItemType& type)
     case ItemType::DrawRect: ts << "draw-rect"; break;
     case ItemType::DrawLine: ts << "draw-line"; break;
     case ItemType::DrawLinesForText: ts << "draw-lines-for-text"; break;
-    case ItemType::DrawLineForDocumentMarker: ts << "draw-lines-for-document-marker"; break;
+    case ItemType::DrawDotsForDocumentMarker: ts << "draw-dots-for-document-marker"; break;
     case ItemType::DrawEllipse: ts << "draw-ellipse"; break;
     case ItemType::DrawPath: ts << "draw-path"; break;
     case ItemType::DrawFocusRingPath: ts << "draw-focus-ring-path"; break;
@@ -1072,8 +1068,8 @@ TextStream& operator<<(TextStream& ts, const Item& item)
     case ItemType::DrawLinesForText:
         ts << downcast<DrawLinesForText>(item);
         break;
-    case ItemType::DrawLineForDocumentMarker:
-        ts << downcast<DrawLineForDocumentMarker>(item);
+    case ItemType::DrawDotsForDocumentMarker:
+        ts << downcast<DrawDotsForDocumentMarker>(item);
         break;
     case ItemType::DrawEllipse:
         ts << downcast<DrawEllipse>(item);
