@@ -193,6 +193,19 @@ public:
         return static_cast<double>(std::abs(ideal - m_ideal.value())) / std::max(std::abs(ideal), std::abs(m_ideal.value()));
     }
 
+    double fitnessDistance(const Vector<ValueType>& discreteCapabilityValues) const
+    {
+        double minDistance = std::numeric_limits<double>::infinity();
+
+        for (auto& value : discreteCapabilityValues) {
+            auto distance = fitnessDistance(value, value);
+            if (distance < minDistance)
+                minDistance = distance;
+        }
+
+        return minDistance;
+    }
+
     bool validForRange(ValueType rangeMin, ValueType rangeMax) const
     {
         if (isEmpty())
@@ -240,9 +253,9 @@ public:
 
     ValueType valueForCapabilityRange(ValueType current, ValueType capabilityMin, ValueType capabilityMax) const
     {
-        ValueType value;
-        ValueType min = capabilityMin;
-        ValueType max = capabilityMax;
+        ValueType value { 0 };
+        ValueType min { capabilityMin };
+        ValueType max { capabilityMax };
 
         if (m_exact) {
             ASSERT(validForRange(capabilityMin, capabilityMax));
@@ -254,6 +267,8 @@ public:
             ASSERT(validForRange(value, capabilityMax));
             if (value > min)
                 min = value;
+            if (value < min)
+                value = min;
 
             // If there is no ideal, don't change if minimum is smaller than current.
             if (!m_ideal && value < current)
@@ -265,6 +280,8 @@ public:
             ASSERT(validForRange(capabilityMin, value));
             if (value < max)
                 max = value;
+            if (value > max)
+                value = max;
         }
 
         if (m_ideal)
@@ -273,6 +290,46 @@ public:
         return value;
     }
 
+    ValueType valueForDiscreteCapabilityValues(ValueType current, const Vector<ValueType>& discreteCapabilityValues) const
+    {
+        ValueType value { 0 };
+        Optional<ValueType> min;
+        Optional<ValueType> max;
+
+        if (m_exact) {
+            ASSERT(discreteCapabilityValues.contains(m_exact.value()));
+            return m_exact.value();
+        }
+
+        if (m_min) {
+            auto index = discreteCapabilityValues.findMatching([&](ValueType value) { return m_min.value() >= value; });
+            if (index != notFound) {
+                min = value = discreteCapabilityValues[index];
+
+                // If there is no ideal, don't change if minimum is smaller than current.
+                if (!m_ideal && value < current)
+                    value = current;
+            }
+        }
+
+        if (m_max && m_max.value() >= discreteCapabilityValues[0]) {
+            for (auto& discreteValue : discreteCapabilityValues) {
+                if (m_max.value() <= discreteValue)
+                    max = value = discreteValue;
+            }
+        }
+
+        if (m_ideal && discreteCapabilityValues.contains(m_ideal.value())) {
+            value = m_ideal.value();
+            if (max)
+                value = std::min(max.value(), value);
+            if (min)
+                value = std::max(min.value(), value);
+        }
+
+        return value;
+    }
+
     bool isEmpty() const { return !m_min && !m_max && !m_exact && !m_ideal; }
     bool isMandatory() const { return m_min || m_max || m_exact; }
 
@@ -337,10 +394,10 @@ protected:
         }
     }
 
-    std::optional<ValueType> m_min;
-    std::optional<ValueType> m_max;
-    std::optional<ValueType> m_exact;
-    std::optional<ValueType> m_ideal;
+    Optional<ValueType> m_min;
+    Optional<ValueType> m_max;
+    Optional<ValueType> m_exact;
+    Optional<ValueType> m_ideal;
 };
 
 class IntConstraint final : public NumericConstraint<int> {
@@ -470,8 +527,8 @@ public:
     }
 
 private:
-    std::optional<bool> m_exact;
-    std::optional<bool> m_ideal;
+    Optional<bool> m_exact;
+    Optional<bool> m_ideal;
 };
 
 class StringConstraint : public MediaConstraint {
@@ -553,6 +610,16 @@ public:
         return true;
     }
 
+    void removeEmptyStringConstraint()
+    {
+        m_exact.removeAllMatching([](auto& constraint) {
+            return constraint.isEmpty();
+        });
+        m_ideal.removeAllMatching([](auto& constraint) {
+            return constraint.isEmpty();
+        });
+    }
+
 private:
     Vector<String> m_exact;
     Vector<String> m_ideal;
@@ -578,27 +645,27 @@ public:
     bool isEmpty() const;
     WEBCORE_EXPORT size_t size() const;
 
-    WEBCORE_EXPORT void set(MediaConstraintType, std::optional<IntConstraint>&&);
-    WEBCORE_EXPORT void set(MediaConstraintType, std::optional<DoubleConstraint>&&);
-    WEBCORE_EXPORT void set(MediaConstraintType, std::optional<BooleanConstraint>&&);
-    WEBCORE_EXPORT void set(MediaConstraintType, std::optional<StringConstraint>&&);
+    WEBCORE_EXPORT void set(MediaConstraintType, Optional<IntConstraint>&&);
+    WEBCORE_EXPORT void set(MediaConstraintType, Optional<DoubleConstraint>&&);
+    WEBCORE_EXPORT void set(MediaConstraintType, Optional<BooleanConstraint>&&);
+    WEBCORE_EXPORT void set(MediaConstraintType, Optional<StringConstraint>&&);
 
-    std::optional<IntConstraint> width() const { return m_width; }
-    std::optional<IntConstraint> height() const { return m_height; }
-    std::optional<IntConstraint> sampleRate() const { return m_sampleRate; }
-    std::optional<IntConstraint> sampleSize() const { return m_sampleSize; }
+    Optional<IntConstraint> width() const { return m_width; }
+    Optional<IntConstraint> height() const { return m_height; }
+    Optional<IntConstraint> sampleRate() const { return m_sampleRate; }
+    Optional<IntConstraint> sampleSize() const { return m_sampleSize; }
 
-    std::optional<DoubleConstraint> aspectRatio() const { return m_aspectRatio; }
-    std::optional<DoubleConstraint> frameRate() const { return m_frameRate; }
-    std::optional<DoubleConstraint> volume() const { return m_volume; }
+    Optional<DoubleConstraint> aspectRatio() const { return m_aspectRatio; }
+    Optional<DoubleConstraint> frameRate() const { return m_frameRate; }
+    Optional<DoubleConstraint> volume() const { return m_volume; }
 
-    std::optional<BooleanConstraint> echoCancellation() const { return m_echoCancellation; }
-    std::optional<BooleanConstraint> displaySurface() const { return m_displaySurface; }
-    std::optional<BooleanConstraint> logicalSurface() const { return m_logicalSurface; }
+    Optional<BooleanConstraint> echoCancellation() const { return m_echoCancellation; }
+    Optional<BooleanConstraint> displaySurface() const { return m_displaySurface; }
+    Optional<BooleanConstraint> logicalSurface() const { return m_logicalSurface; }
 
-    std::optional<StringConstraint> facingMode() const { return m_facingMode; }
-    std::optional<StringConstraint> deviceId() const { return m_deviceId; }
-    std::optional<StringConstraint> groupId() const { return m_groupId; }
+    Optional<StringConstraint> facingMode() const { return m_facingMode; }
+    Optional<StringConstraint> deviceId() const { return m_deviceId; }
+    Optional<StringConstraint> groupId() const { return m_groupId; }
 
     template <class Encoder> void encode(Encoder& encoder) const
     {
@@ -620,59 +687,59 @@ public:
         encoder << m_groupId;
     }
 
-    template <class Decoder> static std::optional<MediaTrackConstraintSetMap> decode(Decoder& decoder)
+    template <class Decoder> static Optional<MediaTrackConstraintSetMap> decode(Decoder& decoder)
     {
         MediaTrackConstraintSetMap map;
         if (!decoder.decode(map.m_width))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_height))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_sampleRate))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_sampleSize))
-            return std::nullopt;
+            return WTF::nullopt;
 
         if (!decoder.decode(map.m_aspectRatio))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_frameRate))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_volume))
-            return std::nullopt;
+            return WTF::nullopt;
 
         if (!decoder.decode(map.m_echoCancellation))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_displaySurface))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_logicalSurface))
-            return std::nullopt;
+            return WTF::nullopt;
 
         if (!decoder.decode(map.m_facingMode))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_deviceId))
-            return std::nullopt;
+            return WTF::nullopt;
         if (!decoder.decode(map.m_groupId))
-            return std::nullopt;
+            return WTF::nullopt;
 
         return WTFMove(map);
     }
 
 private:
-    std::optional<IntConstraint> m_width;
-    std::optional<IntConstraint> m_height;
-    std::optional<IntConstraint> m_sampleRate;
-    std::optional<IntConstraint> m_sampleSize;
+    Optional<IntConstraint> m_width;
+    Optional<IntConstraint> m_height;
+    Optional<IntConstraint> m_sampleRate;
+    Optional<IntConstraint> m_sampleSize;
 
-    std::optional<DoubleConstraint> m_aspectRatio;
-    std::optional<DoubleConstraint> m_frameRate;
-    std::optional<DoubleConstraint> m_volume;
+    Optional<DoubleConstraint> m_aspectRatio;
+    Optional<DoubleConstraint> m_frameRate;
+    Optional<DoubleConstraint> m_volume;
 
-    std::optional<BooleanConstraint> m_echoCancellation;
-    std::optional<BooleanConstraint> m_displaySurface;
-    std::optional<BooleanConstraint> m_logicalSurface;
+    Optional<BooleanConstraint> m_echoCancellation;
+    Optional<BooleanConstraint> m_displaySurface;
+    Optional<BooleanConstraint> m_logicalSurface;
 
-    std::optional<StringConstraint> m_facingMode;
-    std::optional<StringConstraint> m_deviceId;
-    std::optional<StringConstraint> m_groupId;
+    Optional<StringConstraint> m_facingMode;
+    Optional<StringConstraint> m_deviceId;
+    Optional<StringConstraint> m_groupId;
 };
 
 class FlattenedConstraint {
@@ -822,7 +889,6 @@ struct MediaConstraints {
 
     MediaTrackConstraintSetMap mandatoryConstraints;
     Vector<MediaTrackConstraintSetMap> advancedConstraints;
-    String deviceIDHashSalt;
     bool isValid { false };
 };
 
