@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2012 Google Inc. All rights reserved.
- * Copyright (C) 2013-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2018 Apple Inc. All rights reserved.
  * Copyright (C) 2013 Nokia Corporation and/or its subsidiary(-ies).
  * Copyright (C) 2015 Ericsson AB. All rights reserved.
  *
@@ -36,40 +36,42 @@
 #if ENABLE(MEDIA_STREAM)
 #include "RealtimeMediaSource.h"
 
+#include "Logging.h"
 #include "MediaConstraints.h"
 #include "NotImplemented.h"
 #include "RealtimeMediaSourceCapabilities.h"
 #include "RealtimeMediaSourceCenter.h"
+#include <wtf/CompletionHandler.h>
 #include <wtf/MainThread.h>
 #include <wtf/UUID.h>
 #include <wtf/text/StringHash.h>
 
 namespace WebCore {
 
-RealtimeMediaSource::RealtimeMediaSource(const String& id, Type type, const String& name)
-    : m_id(id)
+RealtimeMediaSource::RealtimeMediaSource(Type type, String&& name, String&& deviceID, String&& hashSalt)
+    : m_idHashSalt(WTFMove(hashSalt))
+    , m_persistentID(WTFMove(deviceID))
     , m_type(type)
-    , m_name(name)
+    , m_name(WTFMove(name))
 {
-    // FIXME(147205): Need to implement fitness score for constraints
-
-    if (m_id.isEmpty())
-        m_id = createCanonicalUUIDString();
-    m_persistentID = m_id;
+    if (m_persistentID.isEmpty())
+        m_persistentID = createCanonicalUUIDString();
+    else
+        m_hashedID = RealtimeMediaSourceCenter::singleton().hashStringWithSalt(m_persistentID, m_idHashSalt);
 }
 
 void RealtimeMediaSource::addObserver(RealtimeMediaSource::Observer& observer)
 {
-    m_observers.append(observer);
+    auto locker = holdLock(m_observersLock);
+    m_observers.add(&observer);
 }
 
 void RealtimeMediaSource::removeObserver(RealtimeMediaSource::Observer& observer)
 {
-    m_observers.removeFirstMatching([&observer](auto anObserver) {
-        return &anObserver.get() == &observer;
-    });
+    auto locker = holdLock(m_observersLock);
 
-    if (!m_observers.size())
+    m_observers.remove(&observer);
+    if (m_observers.isEmpty())
         stop();
 }
 
@@ -109,43 +111,69 @@ void RealtimeMediaSource::notifyMutedChange(bool muted)
     notifyMutedObservers();
 }
 
-void RealtimeMediaSource::notifyMutedObservers() const
+void RealtimeMediaSource::setInterruptedForTesting(bool interrupted)
 {
-    for (Observer& observer : m_observers)
-        observer.sourceMutedChanged();
+    notifyMutedChange(interrupted);
 }
 
-void RealtimeMediaSource::settingsDidChange()
+void RealtimeMediaSource::forEachObserver(const WTF::Function<void(Observer&)>& apply) const
+{
+    Vector<Observer*> observersCopy;
+    {
+        auto locker = holdLock(m_observersLock);
+        observersCopy = copyToVector(m_observers);
+    }
+    for (auto* observer : observersCopy) {
+        auto locker = holdLock(m_observersLock);
+        // Make sure the observer has not been destroyed.
+        if (!m_observers.contains(observer))
+            continue;
+        apply(*observer);
+    }
+}
+
+void RealtimeMediaSource::notifyMutedObservers() const
+{
+    forEachObserver([](auto& observer) {
+        observer.sourceMutedChanged();
+    });
+}
+
+void RealtimeMediaSource::notifySettingsDidChangeObservers(OptionSet<RealtimeMediaSourceSettings::Flag> flags)
 {
     ASSERT(isMainThread());
 
+    settingsDidChange(flags);
+
     if (m_pendingSettingsDidChangeNotification)
         return;
-
     m_pendingSettingsDidChangeNotification = true;
 
     scheduleDeferredTask([this] {
         m_pendingSettingsDidChangeNotification = false;
-        for (Observer& observer : m_observers)
+        forEachObserver([](auto& observer) {
             observer.sourceSettingsChanged();
+        });
     });
 }
 
 void RealtimeMediaSource::videoSampleAvailable(MediaSample& mediaSample)
 {
-    for (Observer& observer : m_observers)
+    forEachObserver([&](auto& observer) {
         observer.videoSampleAvailable(mediaSample);
+    });
 }
 
 void RealtimeMediaSource::audioSamplesAvailable(const MediaTime& time, const PlatformAudioData& audioData, const AudioStreamDescription& description, size_t numberOfFrames)
 {
-    for (Observer& observer : m_observers)
+    forEachObserver([&](auto& observer) {
         observer.audioSamplesAvailable(time, audioData, description, numberOfFrames);
+    });
 }
 
 void RealtimeMediaSource::start()
 {
-    if (m_isProducingData)
+    if (m_isProducingData || m_isEnded)
         return;
 
     m_isProducingData = true;
@@ -154,8 +182,9 @@ void RealtimeMediaSource::start()
     if (!m_isProducingData)
         return;
 
-    for (Observer& observer : m_observers)
+    forEachObserver([](auto& observer) {
         observer.sourceStarted();
+    });
 }
 
 void RealtimeMediaSource::stop()
@@ -167,40 +196,50 @@ void RealtimeMediaSource::stop()
     stopProducingData();
 }
 
-void RealtimeMediaSource::requestStop(Observer* callingObserver)
+void RealtimeMediaSource::requestToEnd(Observer& callingObserver)
 {
     if (!m_isProducingData)
         return;
 
-    for (Observer& observer : m_observers) {
+    bool hasObserverPreventingStopping = false;
+    forEachObserver([&](auto& observer) {
         if (observer.preventSourceFromStopping())
-            return;
-    }
+            hasObserverPreventingStopping = true;
+    });
+    if (hasObserverPreventingStopping)
+        return;
+
+    auto protectedThis = makeRef(*this);
 
     stop();
+    m_isEnded = true;
+    hasEnded();
 
-    for (Observer& observer : m_observers) {
-        if (&observer != callingObserver)
+    forEachObserver([callingObserver](auto& observer) {
+        if (&observer != &callingObserver)
             observer.sourceStopped();
-    }
+    });
 }
 
 void RealtimeMediaSource::captureFailed()
 {
+    RELEASE_LOG_ERROR(MediaStream, "RealtimeMediaSource::captureFailed");
+
     m_isProducingData = false;
     m_captureDidFailed = true;
 
-    for (Observer& observer : m_observers)
+    forEachObserver([](auto& observer) {
         observer.sourceStopped();
+    });
 }
 
-bool RealtimeMediaSource::supportsSizeAndFrameRate(std::optional<int>, std::optional<int>, std::optional<double>)
+bool RealtimeMediaSource::supportsSizeAndFrameRate(Optional<int>, Optional<int>, Optional<double>)
 {
     // The size and frame rate are within the capability limits, so they are supported.
     return true;
 }
 
-bool RealtimeMediaSource::supportsSizeAndFrameRate(std::optional<IntConstraint> widthConstraint, std::optional<IntConstraint> heightConstraint, std::optional<DoubleConstraint> frameRateConstraint, String& badConstraint, double& distance)
+bool RealtimeMediaSource::supportsSizeAndFrameRate(Optional<IntConstraint> widthConstraint, Optional<IntConstraint> heightConstraint, Optional<DoubleConstraint> frameRateConstraint, String& badConstraint, double& distance)
 {
     if (!widthConstraint && !heightConstraint && !frameRateConstraint)
         return true;
@@ -209,7 +248,7 @@ bool RealtimeMediaSource::supportsSizeAndFrameRate(std::optional<IntConstraint>
 
     distance = std::numeric_limits<double>::infinity();
 
-    std::optional<int> width;
+    Optional<int> width;
     if (widthConstraint && capabilities.supportsWidth()) {
         double constraintDistance = fitnessDistance(*widthConstraint);
         if (std::isinf(constraintDistance)) {
@@ -218,11 +257,13 @@ bool RealtimeMediaSource::supportsSizeAndFrameRate(std::optional<IntConstraint>
         }
 
         distance = std::min(distance, constraintDistance);
-        auto range = capabilities.width();
-        width = widthConstraint->valueForCapabilityRange(size().width(), range.rangeMin().asInt, range.rangeMax().asInt);
+        if (widthConstraint->isMandatory()) {
+            auto range = capabilities.width();
+            width = widthConstraint->valueForCapabilityRange(size().width(), range.rangeMin().asInt, range.rangeMax().asInt);
+        }
     }
 
-    std::optional<int> height;
+    Optional<int> height;
     if (heightConstraint && capabilities.supportsHeight()) {
         double constraintDistance = fitnessDistance(*heightConstraint);
         if (std::isinf(constraintDistance)) {
@@ -231,11 +272,13 @@ bool RealtimeMediaSource::supportsSizeAndFrameRate(std::optional<IntConstraint>
         }
 
         distance = std::min(distance, constraintDistance);
-        auto range = capabilities.height();
-        height = heightConstraint->valueForCapabilityRange(size().height(), range.rangeMin().asInt, range.rangeMax().asInt);
+        if (heightConstraint->isMandatory()) {
+            auto range = capabilities.height();
+            height = heightConstraint->valueForCapabilityRange(size().height(), range.rangeMin().asInt, range.rangeMax().asInt);
+        }
     }
 
-    std::optional<double> frameRate;
+    Optional<double> frameRate;
     if (frameRateConstraint && capabilities.supportsFrameRate()) {
         double constraintDistance = fitnessDistance(*frameRateConstraint);
         if (std::isinf(constraintDistance)) {
@@ -244,8 +287,10 @@ bool RealtimeMediaSource::supportsSizeAndFrameRate(std::optional<IntConstraint>
         }
 
         distance = std::min(distance, constraintDistance);
-        auto range = capabilities.frameRate();
-        frameRate = frameRateConstraint->valueForCapabilityRange(this->frameRate(), range.rangeMin().asDouble, range.rangeMax().asDouble);
+        if (frameRateConstraint->isMandatory()) {
+            auto range = capabilities.frameRate();
+            frameRate = frameRateConstraint->valueForCapabilityRange(this->frameRate(), range.rangeMin().asDouble, range.rangeMax().asDouble);
+        }
     }
 
     // Each of the non-null values is supported individually, see if they all can be applied at the same time.
@@ -322,6 +367,9 @@ double RealtimeMediaSource::fitnessDistance(const MediaConstraint& constraint)
         if (!capabilities.supportsSampleRate())
             return 0;
 
+        if (auto discreteRates = discreteSampleRates())
+            return downcast<IntConstraint>(constraint).fitnessDistance(*discreteRates);
+
         auto range = capabilities.sampleRate();
         return downcast<IntConstraint>(constraint).fitnessDistance(range.rangeMin().asInt, range.rangeMax().asInt);
         break;
@@ -332,6 +380,9 @@ double RealtimeMediaSource::fitnessDistance(const MediaConstraint& constraint)
         if (!capabilities.supportsSampleSize())
             return 0;
 
+        if (auto discreteSizes = discreteSampleSizes())
+            return downcast<IntConstraint>(constraint).fitnessDistance(*discreteSizes);
+
         auto range = capabilities.sampleSize();
         return downcast<IntConstraint>(constraint).fitnessDistance(range.rangeMin().asInt, range.rangeMax().asInt);
         break;
@@ -387,19 +438,28 @@ double RealtimeMediaSource::fitnessDistance(const MediaConstraint& constraint)
 }
 
 template <typename ValueType>
-static void applyNumericConstraint(const NumericConstraint<ValueType>& constraint, ValueType current, ValueType capabilityMin, ValueType capabilityMax, RealtimeMediaSource* source, void (RealtimeMediaSource::*applier)(ValueType))
+static void applyNumericConstraint(const NumericConstraint<ValueType>& constraint, ValueType current, Optional<Vector<ValueType>> discreteCapabilityValues, ValueType capabilityMin, ValueType capabilityMax, RealtimeMediaSource& source, void (RealtimeMediaSource::*applier)(ValueType))
 {
+    if (discreteCapabilityValues) {
+        int value = constraint.valueForDiscreteCapabilityValues(current, *discreteCapabilityValues);
+        if (value != current)
+            (source.*applier)(value);
+        return;
+    }
+
     ValueType value = constraint.valueForCapabilityRange(current, capabilityMin, capabilityMax);
     if (value != current)
-        (source->*applier)(value);
+        (source.*applier)(value);
 }
 
-void RealtimeMediaSource::applySizeAndFrameRate(std::optional<int> width, std::optional<int> height, std::optional<double> frameRate)
+void RealtimeMediaSource::setSizeAndFrameRate(Optional<int> width, Optional<int> height, Optional<double> frameRate)
 {
+    IntSize size;
     if (width)
-        setWidth(width.value());
+        size.setWidth(width.value());
     if (height)
-        setHeight(height.value());
+        size.setHeight(height.value());
+    setSize(size);
     if (frameRate)
         setFrameRate(frameRate.value());
 }
@@ -408,35 +468,17 @@ void RealtimeMediaSource::applyConstraint(const MediaConstraint& constraint)
 {
     auto& capabilities = this->capabilities();
     switch (constraint.constraintType()) {
-    case MediaConstraintType::Width: {
-        ASSERT(constraint.isInt());
-        if (!capabilities.supportsWidth())
-            return;
-
-        auto range = capabilities.width();
-        applyNumericConstraint(downcast<IntConstraint>(constraint), size().width(), range.rangeMin().asInt, range.rangeMax().asInt, this, &RealtimeMediaSource::setWidth);
+    case MediaConstraintType::Width:
+        ASSERT_NOT_REACHED();
         break;
-    }
 
-    case MediaConstraintType::Height: {
-        ASSERT(constraint.isInt());
-        if (!capabilities.supportsHeight())
-            return;
-
-        auto range = capabilities.height();
-        applyNumericConstraint(downcast<IntConstraint>(constraint), size().height(), range.rangeMin().asInt, range.rangeMax().asInt, this, &RealtimeMediaSource::setHeight);
+    case MediaConstraintType::Height:
+        ASSERT_NOT_REACHED();
         break;
-    }
 
-    case MediaConstraintType::FrameRate: {
-        ASSERT(constraint.isDouble());
-        if (!capabilities.supportsFrameRate())
-            return;
-
-        auto range = capabilities.frameRate();
-        applyNumericConstraint(downcast<DoubleConstraint>(constraint), frameRate(), range.rangeMin().asDouble, range.rangeMax().asDouble, this, &RealtimeMediaSource::setFrameRate);
+    case MediaConstraintType::FrameRate:
+        ASSERT_NOT_REACHED();
         break;
-    }
 
     case MediaConstraintType::AspectRatio: {
         ASSERT(constraint.isDouble());
@@ -444,7 +486,7 @@ void RealtimeMediaSource::applyConstraint(const MediaConstraint& constraint)
             return;
 
         auto range = capabilities.aspectRatio();
-        applyNumericConstraint(downcast<DoubleConstraint>(constraint), aspectRatio(), range.rangeMin().asDouble, range.rangeMax().asDouble, this, &RealtimeMediaSource::setAspectRatio);
+        applyNumericConstraint(downcast<DoubleConstraint>(constraint), aspectRatio(), { }, range.rangeMin().asDouble, range.rangeMax().asDouble, *this, &RealtimeMediaSource::setAspectRatio);
         break;
     }
 
@@ -454,7 +496,7 @@ void RealtimeMediaSource::applyConstraint(const MediaConstraint& constraint)
             return;
 
         auto range = capabilities.volume();
-        applyNumericConstraint(downcast<DoubleConstraint>(constraint), volume(), range.rangeMin().asDouble, range.rangeMax().asDouble, this, &RealtimeMediaSource::setVolume);
+        applyNumericConstraint(downcast<DoubleConstraint>(constraint), volume(), { }, range.rangeMin().asDouble, range.rangeMax().asDouble, *this, &RealtimeMediaSource::setVolume);
         break;
     }
 
@@ -464,7 +506,7 @@ void RealtimeMediaSource::applyConstraint(const MediaConstraint& constraint)
             return;
 
         auto range = capabilities.sampleRate();
-        applyNumericConstraint(downcast<IntConstraint>(constraint), sampleRate(), range.rangeMin().asInt, range.rangeMax().asInt, this, &RealtimeMediaSource::setSampleRate);
+        applyNumericConstraint(downcast<IntConstraint>(constraint), sampleRate(), discreteSampleRates(), range.rangeMin().asInt, range.rangeMax().asInt, *this, &RealtimeMediaSource::setSampleRate);
         break;
     }
 
@@ -474,7 +516,7 @@ void RealtimeMediaSource::applyConstraint(const MediaConstraint& constraint)
             return;
 
         auto range = capabilities.sampleSize();
-        applyNumericConstraint(downcast<IntConstraint>(constraint), sampleSize(), range.rangeMin().asInt, range.rangeMax().asInt, this, &RealtimeMediaSource::setSampleSize);
+        applyNumericConstraint(downcast<IntConstraint>(constraint), sampleSize(), { }, range.rangeMin().asInt, range.rangeMax().asInt, *this, &RealtimeMediaSource::setSampleSize);
         break;
     }
 
@@ -574,10 +616,9 @@ bool RealtimeMediaSource::selectSettings(const MediaConstraints& constraints, Fl
                 return false;
 
             ASSERT(constraint.isString());
-            ASSERT(!constraints.deviceIDHashSalt.isEmpty());
+            ASSERT(!m_hashedID.isEmpty());
 
-            auto hashedID = RealtimeMediaSourceCenter::singleton().hashStringWithSalt(m_persistentID, constraints.deviceIDHashSalt);
-            double constraintDistance = downcast<StringConstraint>(constraint).fitnessDistance(hashedID);
+            double constraintDistance = downcast<StringConstraint>(constraint).fitnessDistance(m_hashedID);
             if (std::isinf(constraintDistance)) {
                 failedConstraint = constraint.name();
                 return true;
@@ -617,7 +658,19 @@ bool RealtimeMediaSource::selectSettings(const MediaConstraints& constraints, Fl
         double constraintDistance = 0;
         bool supported = false;
 
+        if (advancedConstraint.width() || advancedConstraint.height() || advancedConstraint.frameRate()) {
+            String dummy;
+            if (!supportsSizeAndFrameRate(advancedConstraint.width(), advancedConstraint.height(), advancedConstraint.frameRate(), dummy, constraintDistance))
+                continue;
+
+            supported = true;
+        }
+
         advancedConstraint.forEach([&](const MediaConstraint& constraint) {
+
+            if (constraint.constraintType() == MediaConstraintType::Width || constraint.constraintType() == MediaConstraintType::Height || constraint.constraintType() == MediaConstraintType::FrameRate)
+                return;
+
             distance = fitnessDistance(constraint);
             constraintDistance += distance;
             if (!std::isinf(distance))
@@ -653,7 +706,7 @@ bool RealtimeMediaSource::selectSettings(const MediaConstraints& constraints, Fl
     return true;
 }
 
-bool RealtimeMediaSource::supportsConstraint(const MediaConstraint& constraint) const
+bool RealtimeMediaSource::supportsConstraint(const MediaConstraint& constraint)
 {
     auto& capabilities = this->capabilities();
 
@@ -750,7 +803,7 @@ void RealtimeMediaSource::applyConstraints(const FlattenedConstraint& constraint
 
     auto& capabilities = this->capabilities();
 
-    std::optional<int> width;
+    Optional<int> width;
     if (const MediaConstraint* constraint = constraints.find(MediaConstraintType::Width)) {
         ASSERT(constraint->isInt());
         if (capabilities.supportsWidth()) {
@@ -759,7 +812,7 @@ void RealtimeMediaSource::applyConstraints(const FlattenedConstraint& constraint
         }
     }
 
-    std::optional<int> height;
+    Optional<int> height;
     if (const MediaConstraint* constraint = constraints.find(MediaConstraintType::Height)) {
         ASSERT(constraint->isInt());
         if (capabilities.supportsHeight()) {
@@ -768,7 +821,7 @@ void RealtimeMediaSource::applyConstraints(const FlattenedConstraint& constraint
         }
     }
 
-    std::optional<double> frameRate;
+    Optional<double> frameRate;
     if (const MediaConstraint* constraint = constraints.find(MediaConstraintType::FrameRate)) {
         ASSERT(constraint->isDouble());
         if (capabilities.supportsFrameRate()) {
@@ -777,10 +830,8 @@ void RealtimeMediaSource::applyConstraints(const FlattenedConstraint& constraint
         }
     }
 
-    // FIXME: applySizeAndFrameRate should take MediaConstraint* instead of std::optional<> so it can see if a constraint is an exact, min, max,
-    // or ideal, and choose the correct value for properties with non-discreet capabilities when necessary.
     if (width || height || frameRate)
-        applySizeAndFrameRate(WTFMove(width), WTFMove(height), WTFMove(frameRate));
+        setSizeAndFrameRate(WTFMove(width), WTFMove(height), WTFMove(frameRate));
 
     for (auto& variant : constraints) {
         if (variant.constraintType() == MediaConstraintType::Width || variant.constraintType() == MediaConstraintType::Height || variant.constraintType() == MediaConstraintType::FrameRate)
@@ -792,128 +843,144 @@ void RealtimeMediaSource::applyConstraints(const FlattenedConstraint& constraint
     commitConfiguration();
 }
 
-std::optional<std::pair<String, String>> RealtimeMediaSource::applyConstraints(const MediaConstraints& constraints)
+Optional<RealtimeMediaSource::ApplyConstraintsError> RealtimeMediaSource::applyConstraints(const MediaConstraints& constraints)
 {
     ASSERT(constraints.isValid);
 
     FlattenedConstraint candidates;
     String failedConstraint;
     if (!selectSettings(constraints, candidates, failedConstraint, SelectType::ForApplyConstraints))
-        return { { failedConstraint, ASCIILiteral("Constraint not supported") } };
+        return ApplyConstraintsError { failedConstraint, "Constraint not supported"_s };
 
     applyConstraints(candidates);
-    return std::nullopt;
+    return { };
 }
 
-void RealtimeMediaSource::applyConstraints(const MediaConstraints& constraints, SuccessHandler&& successHandler, FailureHandler&& failureHandler)
+void RealtimeMediaSource::applyConstraints(const MediaConstraints& constraints, ApplyConstraintsHandler&& completionHandler)
 {
-    auto result = applyConstraints(constraints);
-    if (!result && successHandler)
-        successHandler();
-    else if (result && failureHandler)
-        failureHandler(result.value().first, result.value().second);
+    completionHandler(applyConstraints(constraints));
 }
 
-void RealtimeMediaSource::setWidth(int width)
+void RealtimeMediaSource::setSize(const IntSize& size)
 {
-    if (width == m_size.width())
+    if (size == m_size)
         return;
 
-    int height = m_aspectRatio ? width / m_aspectRatio : m_size.height();
-    if (!applySize(IntSize(width, height)))
-        return;
-
-    m_size.setWidth(width);
-    if (m_aspectRatio)
-        m_size.setHeight(width / m_aspectRatio);
-
-    settingsDidChange();
+    m_size = size;
+    notifySettingsDidChangeObservers({ RealtimeMediaSourceSettings::Flag::Width, RealtimeMediaSourceSettings::Flag::Height });
 }
 
-void RealtimeMediaSource::setHeight(int height)
+const IntSize RealtimeMediaSource::size() const
 {
-    if (height == m_size.height())
+    auto size = m_size;
+
+    if (size.isEmpty() && !m_intrinsicSize.isEmpty()) {
+        if (size.isZero())
+            size = m_intrinsicSize;
+        else if (size.width())
+            size.setHeight(size.width() * (m_intrinsicSize.height() / static_cast<double>(m_intrinsicSize.width())));
+        else if (size.height())
+            size.setWidth(size.height() * (m_intrinsicSize.width() / static_cast<double>(m_intrinsicSize.height())));
+    }
+
+    return size;
+}
+
+void RealtimeMediaSource::setIntrinsicSize(const IntSize& size)
+{
+    if (m_intrinsicSize == size)
         return;
 
-    int width = m_aspectRatio ? height * m_aspectRatio : m_size.width();
-    if (!applySize(IntSize(width, height)))
-        return;
+    auto currentSize = this->size();
+    m_intrinsicSize = size;
 
-    if (m_aspectRatio)
-        m_size.setWidth(width);
-    m_size.setHeight(height);
+    if (currentSize != this->size())
+        notifySettingsDidChangeObservers({ RealtimeMediaSourceSettings::Flag::Width, RealtimeMediaSourceSettings::Flag::Height });
+}
 
-    settingsDidChange();
+const IntSize RealtimeMediaSource::intrinsicSize() const
+{
+    return m_intrinsicSize;
 }
 
 void RealtimeMediaSource::setFrameRate(double rate)
 {
-    if (m_frameRate == rate || !applyFrameRate(rate))
+    if (m_frameRate == rate)
         return;
 
     m_frameRate = rate;
-    settingsDidChange();
+    notifySettingsDidChangeObservers(RealtimeMediaSourceSettings::Flag::FrameRate);
 }
 
 void RealtimeMediaSource::setAspectRatio(double ratio)
 {
-    if (m_aspectRatio == ratio || !applyAspectRatio(ratio))
+    if (m_aspectRatio == ratio)
         return;
 
     m_aspectRatio = ratio;
     m_size.setHeight(m_size.width() / ratio);
-    settingsDidChange();
+    notifySettingsDidChangeObservers({ RealtimeMediaSourceSettings::Flag::AspectRatio, RealtimeMediaSourceSettings::Flag::Height });
 }
 
 void RealtimeMediaSource::setFacingMode(RealtimeMediaSourceSettings::VideoFacingMode mode)
 {
-    if (m_facingMode == mode || !applyFacingMode(mode))
+    if (m_facingMode == mode)
         return;
 
     m_facingMode = mode;
-    settingsDidChange();
+    notifySettingsDidChangeObservers(RealtimeMediaSourceSettings::Flag::FacingMode);
 }
 
 void RealtimeMediaSource::setVolume(double volume)
 {
-    if (m_volume == volume || !applyVolume(volume))
+    if (m_volume == volume)
         return;
 
     m_volume = volume;
-    settingsDidChange();
+    notifySettingsDidChangeObservers(RealtimeMediaSourceSettings::Flag::Volume);
 }
 
 void RealtimeMediaSource::setSampleRate(int rate)
 {
-    if (m_sampleRate == rate || !applySampleRate(rate))
+    if (m_sampleRate == rate)
         return;
 
     m_sampleRate = rate;
-    settingsDidChange();
+    notifySettingsDidChangeObservers(RealtimeMediaSourceSettings::Flag::SampleRate);
+}
+
+Optional<Vector<int>> RealtimeMediaSource::discreteSampleRates() const
+{
+    return WTF::nullopt;
 }
 
 void RealtimeMediaSource::setSampleSize(int size)
 {
-    if (m_sampleSize == size || !applySampleSize(size))
+    if (m_sampleSize == size)
         return;
 
     m_sampleSize = size;
-    settingsDidChange();
+    notifySettingsDidChangeObservers(RealtimeMediaSourceSettings::Flag::SampleSize);
+}
+
+Optional<Vector<int>> RealtimeMediaSource::discreteSampleSizes() const
+{
+    return WTF::nullopt;
 }
 
 void RealtimeMediaSource::setEchoCancellation(bool echoCancellation)
 {
-    if (m_echoCancellation == echoCancellation || !applyEchoCancellation(echoCancellation))
+    if (m_echoCancellation == echoCancellation)
         return;
 
     m_echoCancellation = echoCancellation;
-    settingsDidChange();
+    notifySettingsDidChangeObservers(RealtimeMediaSourceSettings::Flag::EchoCancellation);
 }
 
 void RealtimeMediaSource::scheduleDeferredTask(WTF::Function<void()>&& function)
 {
     ASSERT(function);
-    callOnMainThread([weakThis = createWeakPtr(), function = WTFMove(function)] {
+    callOnMainThread([weakThis = makeWeakPtr(*this), function = WTFMove(function)] {
         if (!weakThis)
             return;
 
@@ -921,6 +988,21 @@ void RealtimeMediaSource::scheduleDeferredTask(WTF::Function<void()>&& function)
     });
 }
 
+const String& RealtimeMediaSource::hashedId() const
+{
+    ASSERT(!m_hashedID.isEmpty());
+    return m_hashedID;
+}
+
+String RealtimeMediaSource::deviceIDHashSalt() const
+{
+    return m_idHashSalt;
+}
+
+RealtimeMediaSource::Observer::~Observer()
+{
+}
+
 } // namespace WebCore
 
 #endif // ENABLE(MEDIA_STREAM)
