@@ -29,10 +29,11 @@
 #include "TextCodecASCIIFastPath.h"
 #include <wtf/text/CString.h>
 #include <wtf/text/StringBuffer.h>
+#include <wtf/text/WTFString.h>
 #include <wtf/unicode/CharacterNames.h>
 
 namespace WebCore {
-using namespace WTF;
+
 using namespace WTF::Unicode;
 
 const int nonCharacter = -1;
