@@ -20,11 +20,15 @@
 
 #pragma once
 
+#include "ColorHash.h"
 #include "ControlStates.h"
+#include "GraphicsContext.h"
 #include "PaintInfo.h"
 #include "PopupMenuStyle.h"
 #include "ScrollTypes.h"
+#include "StyleColor.h"
 #include "ThemeTypes.h"
+#include <wtf/HashMap.h>
 
 namespace WebCore {
 
@@ -34,6 +38,7 @@ class FileList;
 class FillLayer;
 class HTMLInputElement;
 class Icon;
+class Page;
 class RenderAttachment;
 class RenderBox;
 class RenderMeter;
@@ -53,7 +58,7 @@ public:
     // appropriate platform theme.
     WEBCORE_EXPORT static RenderTheme& singleton();
 
-    virtual void purgeCaches() { }
+    virtual void purgeCaches();
 
     // This method is called whenever style has been computed for an element and the appearance
     // property has been set to a value other than "none".  The theme should map in all of the appropriate
@@ -91,6 +96,12 @@ public:
 #if ENABLE(SERVICE_CONTROLS)
     virtual String imageControlsStyleSheet() const { return String(); }
 #endif
+#if ENABLE(DATALIST_ELEMENT)
+    String dataListStyleSheet() const;
+#endif
+#if ENABLE(INPUT_TYPE_COLOR)
+    String colorInputStyleSheet() const;
+#endif
 
     // A method to obtain the baseline position for a "leaf" control.  This will only be used if a baseline
     // position cannot be determined by examining child content. Checkboxes and radio buttons are examples of
@@ -132,25 +143,26 @@ public:
     virtual bool supportsDataListUI(const AtomicString&) const { return false; }
 
     // Text selection colors.
-    Color activeSelectionBackgroundColor() const;
-    Color inactiveSelectionBackgroundColor() const;
-    Color activeSelectionForegroundColor() const;
-    Color inactiveSelectionForegroundColor() const;
+    Color activeSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    Color inactiveSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    virtual Color transformSelectionBackgroundColor(const Color&, OptionSet<StyleColor::Options>) const;
+    Color activeSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
+    Color inactiveSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
 
     // List box selection colors
-    Color activeListBoxSelectionBackgroundColor() const;
-    Color activeListBoxSelectionForegroundColor() const;
-    Color inactiveListBoxSelectionBackgroundColor() const;
-    Color inactiveListBoxSelectionForegroundColor() const;
+    Color activeListBoxSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    Color activeListBoxSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
+    Color inactiveListBoxSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    Color inactiveListBoxSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
 
-    // Highlighting colors for TextMatches.
-    virtual Color platformActiveTextSearchHighlightColor() const;
-    virtual Color platformInactiveTextSearchHighlightColor() const;
+    // Highlighting colors for search matches.
+    Color activeTextSearchHighlightColor(OptionSet<StyleColor::Options>) const;
+    Color inactiveTextSearchHighlightColor(OptionSet<StyleColor::Options>) const;
 
     virtual Color disabledTextColor(const Color& textColor, const Color& backgroundColor) const;
 
-    static Color focusRingColor();
-    virtual Color platformFocusRingColor() const { return Color(0, 0, 0); }
+    Color focusRingColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformFocusRingColor(OptionSet<StyleColor::Options>) const { return Color(0, 0, 0); }
     static void setCustomFocusRingColor(const Color&);
     static float platformFocusRingWidth() { return 3; }
     static float platformFocusRingOffset(float outlineWidth) { return std::max<float>(outlineWidth - platformFocusRingWidth(), 0); }
@@ -164,7 +176,7 @@ public:
 
     // System fonts and colors for CSS.
     void systemFont(CSSValueID, FontCascadeDescription&) const;
-    virtual Color systemColor(CSSValueID) const;
+    virtual Color systemColor(CSSValueID, OptionSet<StyleColor::Options>) const;
 
     virtual int minimumMenuListSize(const RenderStyle&) const { return 0; }
 
@@ -179,7 +191,7 @@ public:
     // Returns the repeat interval of the animation for the progress bar.
     virtual Seconds animationRepeatIntervalForProgressBar(RenderProgress&) const;
     // Returns the duration of the animation for the progress bar.
-    virtual double animationDurationForProgressBar(RenderProgress&) const;
+    virtual Seconds animationDurationForProgressBar(RenderProgress&) const;
     virtual IntRect progressBarRectForBounds(const RenderObject&, const IntRect&) const;
 
 #if ENABLE(VIDEO)
@@ -245,23 +257,31 @@ public:
     virtual bool shouldMockBoldSystemFontForAccessibility() const { return false; }
     virtual void setShouldMockBoldSystemFontForAccessibility(bool) { }
 
+#if USE(SYSTEM_PREVIEW)
+    virtual void paintSystemPreviewBadge(Image&, const PaintInfo&, const FloatRect&);
+#endif
+
 protected:
     virtual FontCascadeDescription& cachedSystemFontDescription(CSSValueID systemFontID) const;
     virtual void updateCachedSystemFontDescription(CSSValueID systemFontID, FontCascadeDescription&) const = 0;
 
     // The platform selection color.
-    virtual Color platformActiveSelectionBackgroundColor() const;
-    virtual Color platformInactiveSelectionBackgroundColor() const;
-    virtual Color platformActiveSelectionForegroundColor() const;
-    virtual Color platformInactiveSelectionForegroundColor() const;
+    virtual Color platformActiveSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformInactiveSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformActiveSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformInactiveSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
 
-    virtual Color platformActiveListBoxSelectionBackgroundColor() const;
-    virtual Color platformInactiveListBoxSelectionBackgroundColor() const;
-    virtual Color platformActiveListBoxSelectionForegroundColor() const;
-    virtual Color platformInactiveListBoxSelectionForegroundColor() const;
+    virtual Color platformActiveListBoxSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformInactiveListBoxSelectionBackgroundColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformActiveListBoxSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformInactiveListBoxSelectionForegroundColor(OptionSet<StyleColor::Options>) const;
 
-    virtual bool supportsSelectionForegroundColors() const { return true; }
-    virtual bool supportsListBoxSelectionForegroundColors() const { return true; }
+    // The platform highlighting colors for search matches.
+    virtual Color platformActiveTextSearchHighlightColor(OptionSet<StyleColor::Options>) const;
+    virtual Color platformInactiveTextSearchHighlightColor(OptionSet<StyleColor::Options>) const;
+
+    virtual bool supportsSelectionForegroundColors(OptionSet<StyleColor::Options>) const { return true; }
+    virtual bool supportsListBoxSelectionForegroundColors(OptionSet<StyleColor::Options>) const { return true; }
 
 #if !USE(NEW_THEME)
     // Methods for each appearance value.
@@ -321,6 +341,10 @@ protected:
     virtual bool paintAttachment(const RenderObject&, const PaintInfo&, const IntRect&);
 #endif
 
+#if ENABLE(DATALIST_ELEMENT)
+    virtual void adjustListButtonStyle(StyleResolver&, RenderStyle&, const Element*) const;
+#endif
+
     virtual void adjustProgressBarStyle(StyleResolver&, RenderStyle&, const Element*) const;
     virtual bool paintProgressBar(const RenderObject&, const PaintInfo&, const IntRect&) { return true; }
 
@@ -386,19 +410,39 @@ public:
     bool isSpinUpButtonPartPressed(const RenderObject&) const;
     bool isHovered(const RenderObject&) const;
     bool isSpinUpButtonPartHovered(const RenderObject&) const;
+    bool isPresenting(const RenderObject&) const;
     bool isReadOnlyControl(const RenderObject&) const;
     bool isDefault(const RenderObject&) const;
 
-private:
-    mutable Color m_activeSelectionBackgroundColor;
-    mutable Color m_inactiveSelectionBackgroundColor;
-    mutable Color m_activeSelectionForegroundColor;
-    mutable Color m_inactiveSelectionForegroundColor;
+protected:
+    struct ColorCache {
+        HashMap<int, Color> systemStyleColors;
 
-    mutable Color m_activeListBoxSelectionBackgroundColor;
-    mutable Color m_inactiveListBoxSelectionBackgroundColor;
-    mutable Color m_activeListBoxSelectionForegroundColor;
-    mutable Color m_inactiveListBoxSelectionForegroundColor;
+        Color systemLinkColor;
+        Color systemActiveLinkColor;
+        Color systemVisitedLinkColor;
+        Color systemFocusRingColor;
+        Color systemControlAccentColor;
+
+        Color activeSelectionBackgroundColor;
+        Color inactiveSelectionBackgroundColor;
+        Color activeSelectionForegroundColor;
+        Color inactiveSelectionForegroundColor;
+
+        Color activeListBoxSelectionBackgroundColor;
+        Color inactiveListBoxSelectionBackgroundColor;
+        Color activeListBoxSelectionForegroundColor;
+        Color inactiveListBoxSelectionForegroundColor;
+
+        Color activeTextSearchHighlightColor;
+        Color inactiveTextSearchHighlightColor;
+    };
+
+    virtual ColorCache& colorCache(OptionSet<StyleColor::Options>) const;
+
+private:
+    mutable ColorCache m_colorCache;
+    mutable ColorCache m_darkColorCache;
 };
 
 } // namespace WebCore
