@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2013, 2015 Apple Inc.  All rights reserved.
+# Copyright (C) 2007-2018 Apple Inc.  All rights reserved.
 # Copyright (C) 2009, 2010 Chris Jerdonek (chris.jerdonek@gmail.com)
 # Copyright (C) 2010, 2011 Research In Motion Limited. All rights reserved.
 # Copyright (C) 2012 Daniel Bates (dbates@intudata.com)
@@ -82,6 +82,7 @@ BEGIN {
         &parseDiffStartLine
         &parseFirstEOL
         &parsePatch
+        &parseSvnDiffStartLine
         &pathRelativeToSVNRepositoryRootForPath
         &possiblyColored
         &prepareParsedPatch
@@ -723,14 +724,14 @@ sub isExecutable($)
 # Parses an SVN or Git diff header start line.
 #
 # Args:
-#   $line: "Index: " line or "diff --git" line
+#   $line: "Index: " line or "diff --git" line.
 #
 # Returns the path of the target file or undef if the $line is unrecognized.
 sub parseDiffStartLine($)
 {
     my ($line) = @_;
-    return $1 if $line =~ /$svnDiffStartRegEx/;
     return parseGitDiffStartLine($line) if $line =~ /$gitDiffStartRegEx/;
+    return parseSvnDiffStartLine($line);
 }
 
 # Parse the Git diff header start line.
@@ -738,6 +739,9 @@ sub parseDiffStartLine($)
 # Args:
 #   $line: "diff --git" line.
 #
+# Prerequisites:
+#   $line argument matches /$gitDiffStartRegEx/.
+#
 # Returns the path of the target file.
 sub parseGitDiffStartLine($)
 {
@@ -752,13 +756,26 @@ sub parseGitDiffStartLine($)
         die("Could not find '/' in \"diff --git\" line: \"$line\"; only non-prefixed git diffs (i.e. not generated with --no-prefix) that move a top-level directory file are supported.");
     }
     my $pathPrefix = $1;
-    if (!/^diff --git \Q$pathPrefix\E.+ (\Q$pathPrefix\E.+)$/) {
+    if (!/^diff --git \Q$pathPrefix\E.+ (\Q$pathPrefix\E[^\r\n]+)/) {
         # FIXME: Moving a file through sub directories of top directory is not supported (e.g diff --git A/B.txt C/B.txt).
         die("Could not find '/' in \"diff --git\" line: \"$line\"; only non-prefixed git diffs (i.e. not generated with --no-prefix) that move a file between top-level directories are supported.");
     }
     return $1;
 }
 
+# Parses an SVN diff header start line.
+#
+# Args:
+#   $line: "Index: " line.
+#
+# Returns the path of the target file or undef if the $line is unrecognized.
+sub parseSvnDiffStartLine($)
+{
+    my ($line) = @_;
+    return $1 if $line =~ /$svnDiffStartRegEx/;
+    return undef;
+}
+
 # Parse the next Git diff header from the given file handle, and advance
 # the handle so the last line read is the first line after the header.
 #
@@ -938,6 +955,7 @@ sub parseSvnDiffHeader($$)
     my $copiedFromPath;
     my $foundHeaderEnding;
     my $isBinary;
+    my $isDeletion;
     my $isNew;
     my $sourceRevision;
     my $svnConvertedText;
@@ -992,7 +1010,15 @@ sub parseSvnDiffHeader($$)
     }
 
     if (!$foundHeaderEnding) {
-        die("Did not find end of header block corresponding to index path \"$indexPath\".");
+        if (-z $indexPath) {
+            # Delete an empty file.
+            $isDeletion = 1;
+        } elsif (! -e $indexPath) {
+            # Add an empty file.
+            $isNew = 1;
+        } else {
+            die "Did not find end of header block corresponding to index path \"$indexPath\".";
+        }
     }
 
     my %header;
@@ -1000,6 +1026,7 @@ sub parseSvnDiffHeader($$)
     $header{copiedFromPath} = $copiedFromPath if $copiedFromPath;
     $header{indexPath} = $indexPath;
     $header{isBinary} = $isBinary if $isBinary;
+    $header{isDeletion} = $isDeletion if $isDeletion;
     $header{isNew} = $isNew if $isNew;
     $header{sourceRevision} = $sourceRevision if $sourceRevision;
     $header{svnConvertedText} = $svnConvertedText;
